@@ -1,12 +1,12 @@
 ;; liquidsoap-mode.el -- Liquidsoap major mode
-;; Copyright (C) 2003-2019 Samuel Mimram
+;; Copyright (C) 2003-2021 Samuel Mimram
 
 (defvar liquidsoap-font-lock-keywords
  '(
    ("#.*" . 'font-lock-comment-face)
-   ("^\\(%ifdef .*\\|%ifndef .*\\|%ifencoder .*\\|%ifnencoder .*\\|%endif\\|%include\\|%define\\)" . 'font-lock-preprocessor-face)
-   ("\\<\\(fun\\|def\\|rec\\|begin\\|end\\|if\\|then\\|else\\|elsif\\|let\\)\\>\\|->\\|;" . font-lock-keyword-face)
-   ("\\<\\(and\\|or\\|not\\|mod\\|ref\\)\\>\\|:=" . font-lock-builtin-face)
+   ("^\\(%ifdef .*\\|%ifndef .*\\|%ifencoder .*\\|%ifnencoder .*\\|%ifversion .*\\|%else .*\\|%endif\\|%include\\|%define\\)" . 'font-lock-preprocessor-face)
+   ("\\<\\(fun\\|def\\|rec\\|replaces\\|begin\\|end\\|if\\|then\\|else\\|elsif\\|let\\|try\\|catch\\|while\\|for\\|in\\|to\\|do\\|open\\)\\>\\|->\\|;" . font-lock-keyword-face)
+   ("\\<\\(and\\|or\\|not\\|mod\\|??\\)\\>\\|:=" . font-lock-builtin-face)
    ("\\<\\(true\\|false\\)\\>" . font-lock-constant-face)
    ("\\<def[ \t]+\\([^ (]*\\)" 1 'font-lock-function-name-face)
   )
@@ -29,13 +29,13 @@
   "Indent current Liquidsoap line"
   (interactive)
   (beginning-of-line)
-  ; At begining, no indentation
+  ; At beginning, no indentation
   (if (bobp) (indent-line-to 0)
     ; not-indented is a boolean saying we found a match looking backward
     ; cur-indent is the current indetation
     (let ((not-indented t) cur-indent)
       ; De-indent after end
-      (if (looking-at "^[ \t]*\\(end\\|else\\|elsif\\|then\\|%endif\\)")
+      (if (looking-at "^[ \t]*\\(end\\|else\\|elsif\\|then\\|%endif\\|try\\|catch\\)")
         (progn
           (save-excursion
             (forward-line -1)
@@ -50,7 +50,7 @@
                   (setq cur-indent (current-indentation))
                   (setq not-indented nil))
               ; Increment if we find that we are in a block
-              (if (looking-at "^[ \t]*\\(def\\|if\\|then\\|elsif\\|%ifdef\\|.*=$\\)")
+              (if (looking-at "^[ \t]*\\(def\\|if\\|then\\|else\\|elsif\\|%ifdef\\|.*=$\\|try\\|catch\\|for\\|while\\)")
                   (progn
                     (setq cur-indent (+ (current-indentation) liquidsoap-tab-width))
                     (setq not-indented nil))
