@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -58,4 +58,4 @@ let alsa_buffer =
 (** A dedicated clock for all ALSA I/O operators, to make sure other
   * blocking I/O inteferes with them. In the future, we might even want
   * to have different clocks for different ALSA devices. *)
-let get_clock = Tutils.lazy_cell (fun () -> new Clock.self_sync "alsa")
+let get_clock = Tutils.lazy_cell (fun () -> new Clock.clock "alsa")
