@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -32,6 +32,24 @@ let log = Log.make ["clock"]
 let conf_clock =
   Dtools.Conf.void ~p:(Configure.conf#plug "clock") "Clock settings"
 
+module Time : Liq_time.T = (val !Liq_time.implementation)
+open Time
+
+let time_unit = Time.of_float 1.
+let time_zero = Time.of_float 0.
+
+let sleep d =
+  let cur = time () in
+  try sleep d
+  with Unix.Unix_error (Unix.EINTR, _, _) ->
+    let diff = d |-| time () |+| cur in
+    if time_zero |<| diff then sleep diff
+
+let () =
+  Lifecycle.on_init (fun () ->
+      log#important "Using %s implementation for latency control"
+        Time.implementation)
+
 (** [started] indicates that the application has loaded and started
   * its initial configuration; it is set after the first collect.
   * It is mostly intended to allow different behaviors on error:
@@ -78,22 +96,50 @@ let allow_streaming_errors =
         "use at your own risk!";
       ]
 
+let conf_log_delay =
+  Dtools.Conf.float
+    ~p:(conf_clock#plug "log_delay")
+    ~d:1. "How often (in seconds) we should indicate catchup errors."
+
 (** Leave a source, ignoring errors *)
 
-let leave (s : active_source) =
-  try s#leave (s :> source)
+let leave ?failed_to_start (s : active_source) =
+  try s#leave ?failed_to_start (s :> source)
   with e ->
-    log#severe "Error when leaving output %s: %s!" s#id (Printexc.to_string e);
-    List.iter (log#important "%s")
-      (Pcre.split ~pat:"\n" (Printexc.get_backtrace ()))
+    let bt = Printexc.get_backtrace () in
+    Utils.log_exception ~log ~bt
+      (Printf.sprintf "Error when leaving output %s: %s!" s#id
+         (Printexc.to_string e))
 
-(** Base clock class. *)
-class clock id =
+(** {1 Clock implementation}
+  * One could think of several clocks for isolated parts of a script.
+  * One can also think of alsa-clocks, etc. *)
+
+let conf =
+  Dtools.Conf.void ~p:(Configure.conf#plug "root") "Streaming clock settings"
+
+let conf_max_latency =
+  Dtools.Conf.float ~p:(conf#plug "max_latency") ~d:60.
+    "Maximum latency in seconds"
+    ~comments:
+      [
+        "If the latency gets higher than this value, the outputs will be reset,";
+        "instead of trying to catch it up second by second.";
+        "The reset is typically only useful to reconnect icecast mounts.";
+      ]
+
+(** Timing stuff, make sure the frame rate is correct. *)
+
+let sync_descr = function
+  | `Auto -> "auto-sync"
+  | `CPU -> "CPU sync"
+  | `None -> "no sync"
+
+class clock ?(start = true) ?(sync = `Auto) id =
   object (self)
     initializer Clocks.add clocks (self :> Source.clock)
-
     method id = id
-
+    method sync_mode : Source.sync = sync
     val log = Log.make ["clock"; id]
 
     (** List of outputs, together with a flag indicating their status:
@@ -121,13 +167,12 @@ class clock id =
                     | `New -> outputs
                     | `Active -> (`Old, s) :: outputs
                     | `Starting -> (`Aborted, s) :: outputs
-                    | `Old | `Aborted -> (flag, s) :: outputs )
+                    | `Old | `Aborted -> (flag, s) :: outputs)
                 else (flag, s) :: outputs)
               [] outputs)
         ()
 
     val mutable sub_clocks : Source.clock_variable list = []
-
     method sub_clocks = sub_clocks
 
     method attach_clock c =
@@ -138,11 +183,96 @@ class clock id =
       sub_clocks <- List.filter (fun c' -> c <> c') sub_clocks
 
     val mutable round = 0
-
     method get_tick = round
+    val mutable running = false
 
-    (** This is the main streaming step
-    * All clocks (wallclock, soundcard, stretch, etc) run this #end_tick *)
+    val do_running =
+      let lock = Mutex.create () in
+      fun f -> Tutils.mutexify lock f ()
+
+    val mutable self_sync = None
+    val mutable t0 = time ()
+    val mutable ticks = 0L
+
+    method private self_sync =
+      let new_val =
+        match sync with
+          | `Auto ->
+              List.exists
+                (fun (state, s) ->
+                  state = `Active && snd s#self_sync && s#is_ready)
+                outputs
+          | `CPU -> false
+          | `None -> true
+      in
+      begin
+        match (self_sync, new_val) with
+        | None, false | Some true, false ->
+            log#important "Delegating synchronisation to CPU clock";
+            t0 <- time ();
+            ticks <- 0L
+        | None, true | Some false, true ->
+            log#important "Delegating synchronisation to active sources"
+        | _ -> ()
+      end;
+      self_sync <- Some new_val;
+      new_val
+
+    method private run =
+      let acc = ref 0 in
+      let log_delay = Time.of_float conf_log_delay#get in
+      let max_latency = Time.of_float (-.conf_max_latency#get) in
+      let last_latency_log = ref (time ()) in
+      t0 <- time ();
+      ticks <- 0L;
+      let frame_duration = Time.of_float (Lazy.force Frame.duration) in
+      let delay () =
+        t0
+        |+| (frame_duration
+            |*| Time.of_float (Int64.to_float (Int64.add ticks 1L)))
+        |-| time ()
+      in
+      log#important "Streaming loop starts in %s mode" (sync_descr sync);
+      let rec loop () =
+        (* Stop running if there is no output. *)
+        if outputs = [] then ()
+        else (
+          let self_sync = self#self_sync in
+          let rem = if self_sync then time_zero else delay () in
+          (* Sleep a while or worry about the latency *)
+          if self_sync || time_zero |<| rem then (
+            acc := 0;
+            if time_zero |<| rem then sleep rem)
+          else (
+            incr acc;
+            if rem |<| max_latency then (
+              log#severe "Too much latency! Resetting active sources...";
+              List.iter (function `Active, s -> s#reset | _ -> ()) outputs;
+              t0 <- time ();
+              ticks <- 0L;
+              acc := 0)
+            else if
+              (rem |<=| (time_zero |-| time_unit) || !acc >= 100)
+              && !last_latency_log |+| log_delay |<| time ()
+            then (
+              last_latency_log := time ();
+              log#severe "We must catchup %.2f seconds%s!"
+                (Time.to_float (time_zero |-| rem))
+                (if !acc <= 100 then ""
+                else " (we've been late for 100 rounds)");
+              acc := 0));
+          ticks <- Int64.add ticks 1L;
+          (* This is where the streaming actually happens: *)
+          self#end_tick;
+          loop ())
+      in
+      loop ();
+      do_running (fun () -> running <- false);
+      log#important "Streaming loop stopped."
+
+    val thread_name = "clock_" ^ id
+
+    (** This is the main streaming step *)
     method end_tick =
       let leaving, active =
         Tutils.mutexify lock
@@ -161,6 +291,7 @@ class clock id =
           ()
       in
       List.iter (fun (s : active_source) -> leave s) leaving;
+      List.iter (fun s -> s#before_output) active;
       let error, active =
         List.fold_left
           (fun (e, a) s ->
@@ -168,20 +299,21 @@ class clock id =
               s#output;
               (e, s :: a)
             with exn ->
-              log#severe "Source %s failed while streaming: %s!" s#id
-                (Printexc.to_string exn);
-              List.iter (log#important "%s")
-                (Pcre.split ~pat:"\n" (Printexc.get_backtrace ()));
-              leave s;
+              let bt = Printexc.get_backtrace () in
+              Utils.log_exception ~log ~bt
+                (Printf.sprintf "Source %s failed while streaming: %s!" s#id
+                   (Printexc.to_string exn));
+              leave ~failed_to_start:true s;
               (s :: e, a))
           ([], []) active
       in
-      if error <> [] then begin
+      if error <> [] then (
         Tutils.mutexify lock
           (fun () ->
             outputs <-
               List.filter (fun (_, s) -> not (List.mem s error)) outputs)
           ();
+
         (* To stop this clock it would be enough to detach all sources
          * and let things stop by themselves. We stop all sources by
          * calling Tutils.shutdown, which calls Clock.stop, stopping
@@ -189,8 +321,7 @@ class clock id =
          * In any case, we can't just raise an exception here, otherwise
          * the streaming thread (method private run) will die and won't
          * be able to leave all sources. *)
-        if not allow_streaming_errors#get then Tutils.shutdown ()
-      end;
+        if not allow_streaming_errors#get then Tutils.shutdown 1);
       round <- round + 1;
       List.iter (fun s -> s#after_output) active
 
@@ -226,32 +357,16 @@ class clock id =
             (fun (s : active_source) ->
               try
                 s#get_ready [(s :> source)];
-                `Woken_up s
+                `Started s
               with e ->
-                log#severe "Error when starting %s: %s!" s#id
-                  (Printexc.to_string e);
-                List.iter (log#important "%s")
-                  (Pcre.split ~pat:"\n" (Printexc.get_backtrace ()));
-                leave s;
+                let bt = Printexc.get_backtrace () in
+                Utils.log_exception ~log ~bt
+                  (Printf.sprintf "Error when starting %s: %s!" s#id
+                     (Printexc.to_string e));
+                leave ~failed_to_start:true s;
                 `Error s)
             to_start
         in
-        let to_start =
-          List.map
-            (function
-              | `Error s -> `Error s
-              | `Woken_up (s : active_source) -> (
-                  try
-                    s#output_get_ready;
-                    `Started s
-                  with e ->
-                    let bt = Printexc.get_backtrace () in
-                    log#severe "Error when starting output %s: %s!\n%s" s#id
-                      (Printexc.to_string e) bt;
-                    leave s;
-                    `Error s ))
-            to_start
-        in
         (* Now mark the started sources as `Active,
          * unless they have been deactivating in the meantime (`Aborted)
          * in which case they have to be cleanly stopped. *)
@@ -265,12 +380,12 @@ class clock id =
                       match flag with
                         | `Starting -> ((`Active, s) :: outputs, leaving, errors)
                         | `Aborted -> (outputs, s :: leaving, errors)
-                        | `New | `Active | `Old -> assert false )
+                        | `New | `Active | `Old -> assert false)
                     else if List.mem (`Error s) to_start then (
                       match flag with
                         | `Starting -> (outputs, leaving, s :: errors)
                         | `Aborted -> (outputs, leaving, s :: errors)
-                        | `New | `Active | `Old -> assert false )
+                        | `New | `Active | `Old -> assert false)
                     else ((flag, s) :: outputs, leaving, errors))
                   ([], [], []) outputs
               in
@@ -278,164 +393,21 @@ class clock id =
               (leaving, errors))
             ()
         in
-        if !started <> `Yes && errors <> [] then Tutils.shutdown ();
-        if leaving <> [] then begin
+        if !started <> `Yes && errors <> [] then Tutils.shutdown 1;
+        if leaving <> [] then (
           log#info "Stopping %d sources..." (List.length leaving);
-          List.iter (fun (s : active_source) -> leave s) leaving
-        end;
-        errors
-  end
-
-(** {1 Wallclock implementation}
-  * This was formerly known as the Root.
-  * One could think of several wallclocks for isolated parts of a script.
-  * One can also think of alsa-clocks, etc. *)
-
-open Dtools
-
-let conf = Conf.void ~p:(Configure.conf#plug "root") "Streaming clock settings"
-
-let conf_max_latency =
-  Conf.float ~p:(conf#plug "max_latency") ~d:60. "Maximum latency in seconds"
-    ~comments:
-      [
-        "If the latency gets higher than this value, the outputs will be reset,";
-        "instead of trying to catch it up second by second.";
-        "The reset is typically only useful to reconnect icecast mounts.";
-      ]
-
-(** Timing stuff, make sure the frame rate is correct. *)
-
-let time = Unix.gettimeofday
-
-let usleep d =
-  (* In some implementations,
-   * Thread.delay uses Unix.select which can raise EINTR.
-   * A really good implementation would keep track of the elapsed time and then
-   * trigger another Thread.delay for the remaining time.
-   * This cheap thing does the job for now.. *)
-  try Thread.delay d with Unix.Unix_error (Unix.EINTR, _, _) -> ()
-
-class wallclock ?(sync = true) id =
-  object (self)
-    inherit clock ("wallclock_" ^ id) as super
-
-    (** Main loop. *)
-
-    val mutable running = false
-
-    val do_running =
-      let lock = Mutex.create () in
-      fun f -> Tutils.mutexify lock f ()
-
-    val mutable sync = sync
-
-    method private run =
-      let acc = ref 0 in
-      let max_latency = -.conf_max_latency#get in
-      let last_latency_log = ref (time ()) in
-      let t0 = ref (time ()) in
-      let ticks = ref 0L in
-      let frame_duration = Lazy.force Frame.duration in
-      let delay () =
-        !t0
-        +. (frame_duration *. Int64.to_float (Int64.add !ticks 1L))
-        -. time ()
-      in
-      if sync then
-        log#important "Streaming loop starts, synchronized with wallclock."
-      else
-        log#important "Streaming loop starts, synchronized by active sources.";
-      let rec loop () =
-        (* Stop running if there is no output. *)
-        if outputs = [] then ()
-        else (
-          let rem = if not sync then 0. else delay () in
-          (* Sleep a while or worry about the latency *)
-          if (not sync) || rem > 0. then begin
-            acc := 0;
-            usleep rem
-          end
-          else begin
-            incr acc;
-            if rem < max_latency then begin
-              log#severe "Too much latency! Resetting active sources...";
-              List.iter
-                (function
-                  | `Active, s when s#is_active -> s#output_reset | _ -> ())
-                outputs;
-              t0 := time ();
-              ticks := 0L;
-              acc := 0
-            end
-            else if
-              (rem <= -1. || !acc >= 100) && !last_latency_log +. 1. < time ()
-            then begin
-              last_latency_log := time ();
-              log#severe "We must catchup %.2f seconds%s!" (-.rem)
-                (if !acc <= 100 then "" else " (we've been late for 100 rounds)");
-              acc := 0
-            end
-          end;
-          ticks := Int64.add !ticks 1L;
-          (* This is where the streaming actually happens: *)
-          super#end_tick;
-          loop () )
-      in
-      loop ();
-      do_running (fun () -> running <- false);
-      log#important "Streaming loop stopped."
-
-    val thread_name = "wallclock_" ^ id
-
-    method start_outputs filter =
-      let f = super#start_outputs filter in
-      fun () ->
-        let errors = f () in
-        if List.exists (function `Active, _ -> true | _ -> false) outputs then
+          List.iter (fun (s : active_source) -> leave s) leaving);
+        if
+          start
+          && List.exists (function `Active, _ -> true | _ -> false) outputs
+        then
           do_running (fun () ->
-              if not running then begin
+              if not running then (
                 running <- true;
-                ignore (Tutils.create (fun () -> self#run) () thread_name)
-              end);
+                ignore (Tutils.create (fun () -> self#run) () thread_name)));
         errors
   end
 
-(** {1 Self-sync wallclock}
-  * Special kind of clock for self-synched devices,
-  * that only does synchronization when all input/outputs are stopped
-  * (a normal non-synched wallclock goes 100% CPU when blocking I/O
-  * stops). *)
-
-class self_sync id =
-  object
-    inherit wallclock ~sync:true id
-
-    val mutable blocking_sources = 0
-
-    val bs_lock = Mutex.create ()
-
-    method register_blocking_source =
-      Tutils.mutexify bs_lock
-        (fun () ->
-          if blocking_sources = 0 then begin
-            log#info "Delegating clock to active sources.";
-            sync <- false
-          end;
-          blocking_sources <- blocking_sources + 1)
-        ()
-
-    method unregister_blocking_source =
-      Tutils.mutexify bs_lock
-        (fun () ->
-          blocking_sources <- blocking_sources - 1;
-          if blocking_sources = 0 then begin
-            sync <- true;
-            log#info "All active sources stopped, synching with wallclock."
-          end)
-        ()
-  end
-
 (** {1 Global clock management} *)
 
 (** When created, sources have a clock variable, which gets unified
@@ -477,7 +449,7 @@ class self_sync id =
   * only the first part is done within critical section.
   *
   * The last trick is that we start with a fake task (after_collect_tasks=1)
-  * to avoid that the initial parsing of files triggers collect and thus
+  * to make sure that the initial parsing of files does not triggers collect and thus
   * a too early initialization of outputs (before daemonization). Main is
   * in charge of finishing that virtual task and trigger the initial
   * collect. *)
@@ -488,17 +460,16 @@ let lock = Mutex.create ()
 (** We might not need a default clock, so we use a lazy clock value.
   * We don't use Lazy because we need a thread-safe mechanism. *)
 let get_default =
-  Tutils.lazy_cell (fun () -> (new wallclock "main" :> Source.clock))
+  Tutils.lazy_cell (fun () -> (new clock "main" :> Source.clock))
 
 (** A function displaying the varying number of allocated clocks. *)
 let gc_alarm =
   let last_displayed = ref (-1) in
   fun () ->
     let nb_clocks = Clocks.count clocks in
-    if nb_clocks <> !last_displayed then begin
+    if nb_clocks <> !last_displayed then (
       log#info "Currently %d clocks allocated." nb_clocks;
-      last_displayed := nb_clocks
-    end
+      last_displayed := nb_clocks)
 
 let () = ignore (Gc.create_alarm gc_alarm)
 
@@ -508,12 +479,13 @@ let () = ignore (Gc.create_alarm gc_alarm)
   * and stop those that need stopping. *)
 let collect ~must_lock =
   if must_lock then Mutex.lock lock;
+
   (* If at least one task is engaged it will take care of collection later.
    * Otherwise, prepare a collection while in critical section
    * (to avoid harvesting sources created by a task) and run it
    * outside of critical section (to avoid all sorts of shit). *)
   if !after_collect_tasks > 0 then Mutex.unlock lock
-  else begin
+  else (
     Source.iterate_new_outputs (fun o ->
         if not (is_known o#clock) then
           ignore (unify o#clock (create_known (get_default ()))));
@@ -524,7 +496,7 @@ let collect ~must_lock =
     in
     let start =
       if !started <> `No then ignore
-      else begin
+      else (
         (* Avoid that some other collection takes up the task
          * to set started := true. Typically they would be
          * trivial (empty) collections terminating before us,
@@ -532,13 +504,11 @@ let collect ~must_lock =
         started := `Soon;
         fun () ->
           log#info "Main phase starts.";
-          started := `Yes
-      end
+          started := `Yes)
     in
     Mutex.unlock lock;
     List.iter (fun f -> ignore (f ())) collects;
-    start ()
-  end
+    start ())
 
 let collect_after f =
   Mutex.lock lock;
