@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,8 +20,12 @@
 
  *****************************************************************************)
 
+open Mm
+
 (** Samplerate converter using libsamplerate *)
 
+let log = Log.make ["audio"; "converter"; "libsamplerate"]
+
 let samplerate_conf =
   Dtools.Conf.void
     ~p:(Audio_converter.Samplerate.samplerate_conf#plug "libsamplerate")
@@ -48,7 +52,7 @@ let quality_of_string v =
     | "linear" -> Samplerate.Conv_linear
     | _ ->
         raise
-          (Lang_errors.Invalid_value
+          (Error.Invalid_value
              ( Lang.string v,
                "libsamplerate quality must be one of: \"best\", \"medium\", \
                 \"fast\", \"zero_order\", \"linear\"." ))
@@ -56,8 +60,20 @@ let quality_of_string v =
 let samplerate_converter () =
   let quality = quality_of_string quality_conf#get in
   let converter = Samplerate.create quality 1 in
-  let convert ratio b ofs len =
-    Samplerate.process_alloc converter ratio b ofs len
+  let convert ratio b =
+    let inlen = Audio.Mono.length b in
+    let outlen = int_of_float (float inlen *. ratio) in
+    let buf = Audio.Mono.create outlen in
+    let i, o = Samplerate.process_ba converter ratio b buf in
+    if i < inlen then
+      log#debug "Could not convert all the input buffer (%d instead of %d)." i
+        inlen;
+    if o < outlen then
+      log#debug "Unexpected output length (%d instead of %d)." o outlen;
+
+    (* TODO: the following would solve the issue but apparently messes up buffers *)
+    (* Audio.Mono.sub buf 0 o *)
+    buf
   in
   convert
 
