@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
 
  *****************************************************************************)
 
+open Mm
+
 (** Native audio converters *)
 
 let samplerate_conf =
@@ -39,7 +41,7 @@ let quality_of_string = function
   | "linear" -> `Linear
   | s ->
       raise
-        (Lang_errors.Invalid_value
+        (Error.Invalid_value
            ( Lang.string s,
              "Native resampling quality must either be \"nearest\" or \
               \"linear\"." ))
@@ -50,3 +52,14 @@ let samplerate_converter () =
 
 let () =
   Audio_converter.Samplerate.converters#register "native" samplerate_converter
+
+let channel_layout_converter src dst =
+  assert (src <> dst);
+  match dst with
+    | `Mono -> fun data -> [| Audio.to_mono data |]
+    | `Stereo -> fun data -> [| data.(0); data.(0) |]
+    | _ -> raise Audio_converter.Channel_layout.Unsupported
+
+let () =
+  Audio_converter.Channel_layout.converters#register "native"
+    channel_layout_converter
