@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
 
  *****************************************************************************)
 
+open Mm
+
 (** External audio samplerate conversion utilities. *)
 
 let log = Log.make ["audio"; "converter"]
@@ -40,10 +42,9 @@ let converter_conf =
 module Samplerate = struct
   exception Invalid_data
 
-  (** A converter takes a convertion ratio (output samplerate / input
-      samplerate), an audio buffer, an offset in the buffer, a duration in the
-      buffer and returns a resampled buffer. *)
-  type converter = float -> float array -> int -> int -> float array
+  (** A converter takes a conversion ratio (output samplerate / input
+      samplerate), an audio buffer, and returns a resampled buffer. *)
+  type converter = float -> Audio.Mono.buffer -> Audio.Mono.buffer
 
   type converter_plug = unit -> converter
   type t = converter array
@@ -73,7 +74,7 @@ module Samplerate = struct
     let converter =
       let rec f = function
         | conv :: l -> (
-            match converters#get conv with Some v -> v | None -> f l )
+            match converters#get conv with Some v -> v | None -> f l)
         | [] ->
             (* This should never come up since the native converter is always
                available. *)
@@ -83,22 +84,85 @@ module Samplerate = struct
     in
     Array.init channels (fun _ -> converter ())
 
-  let resample conv ratio data ofs len =
+  let resample conv ratio data =
     if Array.length conv <> Array.length data then raise Invalid_data;
-    let convert i b =
-      if ratio = 1. then Array.sub b ofs len else conv.(i) ratio b ofs len
-    in
-    if Array.length data.(0) = 0 then data else Array.mapi convert data
+    let convert i b = if ratio = 1. then b else conv.(i) ratio b in
+    Array.mapi convert data
 
   (** Log which converter is used at start. *)
   let () =
-    ignore
-      (Dtools.Init.at_start (fun () ->
-           let rec f = function
-             | conv :: _ when converters#get conv <> None ->
-                 log#important "Using samplerate converter: %s." conv
-             | _ :: l -> f l
-             | [] -> assert false
-           in
-           f converters_conf#get))
+    Lifecycle.before_start (fun () ->
+        let rec f = function
+          | conv :: _ when converters#get conv <> None ->
+              log#important "Using samplerate converter: %s." conv
+          | _ :: l -> f l
+          | [] -> assert false
+        in
+        f converters_conf#get)
+end
+
+module Channel_layout = struct
+  exception Unsupported
+  exception Invalid_data
+
+  type layout = [ `Mono | `Stereo | `Five_point_one ]
+
+  type converter =
+    layout -> layout -> Frame_content.Audio.data -> Frame_content.Audio.data
+
+  type t = {
+    src : layout;
+    converter : Frame_content.Audio.data -> Frame_content.Audio.data;
+  }
+
+  let channel_layout_conf =
+    Dtools.Conf.void
+      ~p:(converter_conf#plug "channel_layout")
+      "Channel layout conversion settings"
+      ~comments:["Options related to channel layout conversion."]
+
+  let converters_conf =
+    Dtools.Conf.list
+      ~p:(channel_layout_conf#plug "converters")
+      "Preferred samplerate converters" ~d:["native"]
+      ~comments:
+        [
+          "Preferred channel layout converter. The native converter is always \
+           available.";
+        ]
+
+  let converters : converter Plug.plug =
+    Plug.create "channel layout converters"
+      ~doc:"Methods for converting channel layouts."
+
+  let create src dst =
+    let converter =
+      if src = dst then fun _ _ x -> x
+      else (
+        let rec f = function
+          | conv :: l -> (
+              match converters#get conv with Some v -> v | None -> f l)
+          | [] ->
+              (* This should never come up since the native converter is always
+                 available. *)
+              assert false
+        in
+        f converters_conf#get)
+    in
+    { src; converter = converter src dst }
+
+  let channels_of_layout = function
+    | `Mono -> 1
+    | `Stereo -> 2
+    | `Five_point_one -> 6
+
+  let layout_of_channels = function
+    | 1 -> `Mono
+    | 2 -> `Stereo
+    | 6 -> `Five_point_one
+    | _ -> raise Unsupported
+
+  let convert { src; converter } input =
+    if Array.length input != channels_of_layout src then raise Invalid_data;
+    converter input
 end
