@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,9 @@
 
  *****************************************************************************)
 
- (* Video format converters *)
+open Mm
+
+(* Video format converters *)
 
 (** Plugin to add video-related configuration keys. *)
 val video_converter_conf : Dtools.Conf.ut
@@ -30,10 +32,14 @@ val video_converter_conf : Dtools.Conf.ut
   * raises Not_found if no conversion routine
   * was found *)
 type converter = proportional:bool -> Image.Generic.t -> Image.Generic.t -> unit
+
 (** A converter plugin is a name, a list of input formats,
   * a list of output formats,
-  * a fonction to create a converter. *)
-type converter_plug = (Image.Generic.Pixel.format list)*(Image.Generic.Pixel.format list)*(unit->converter)
+  * a function to create a converter. *)
+type converter_plug =
+  Image.Generic.Pixel.format list
+  * Image.Generic.Pixel.format list
+  * (unit -> converter)
 
 (** Plugin to register new converters. *)
 val video_converters : converter_plug Plug.plug
@@ -41,7 +47,14 @@ val video_converters : converter_plug Plug.plug
 (** [find_converter source destination] tries
   * to find a converter from source format
   * to destination format. Proportional scale
-  * is implicitely set via global configuration key
+  * is implicitly set via global configuration key
   * for now. Returns a conversion function: frame -> frame -> unit. *)
-val find_converter : Image.Generic.Pixel.format -> Image.Generic.Pixel.format -> (Image.Generic.t -> Image.Generic.t -> unit)
+val find_converter :
+  Image.Generic.Pixel.format ->
+  Image.Generic.Pixel.format ->
+  Image.Generic.t ->
+  Image.Generic.t ->
+  unit
 
+val scaler :
+  unit -> ?proportional:bool -> Video.Image.t -> Video.Image.t -> unit
