@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 
  *****************************************************************************)
 
+open Mm
 module Img = Image.RGBA32
 module P = Image.Generic.Pixel
 
@@ -45,10 +46,10 @@ let load_image filename =
       | OImages.Index16 img -> rgba32_p img#to_rgba32
       | OImages.Cmyk32 _ -> failwith "CMYK32 images are not supported for now."
   in
-  let img = Img.create width height in
+  let img = Video.Image.create width height in
   for j = 0 to height - 1 do
     for i = 0 to width - 1 do
-      Img.set_pixel img i j (p i j)
+      Video.Image.set_pixel_rgba img i j (p i j)
     done
   done;
   img
