@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -38,35 +38,50 @@ let file_extensions =
     ~p:(Decoder.conf_file_extensions#plug "taglib")
     "File extensions used for decoding metadata using TAGLIB" ~d:["mp3"]
 
+let int_tags = [("year", Taglib.tag_year); ("tracknumber", Taglib.tag_track)]
+let tag_aliases = [("track", "tracknumber")]
+
 (** We used to force the format. However, now that we check extensions, taglib's
   * automatic format detection should work. *)
 let get_tags fname =
   try
     if
       not
-        (Decoder.test_file ~mimes:mime_types#get ~extensions:file_extensions#get
-           ~log fname)
+        (Decoder.test_file ~log ~mimes:mime_types#get
+           ~extensions:file_extensions#get fname)
     then raise Invalid_file;
     let f = Taglib.File.open_file `Autodetect fname in
     Tutils.finalize
       ~k:(fun () -> Taglib.File.close_file f)
       (fun () ->
         let tags =
-          try [("year", string_of_int (Taglib.tag_year f))]
-          with Not_found -> []
+          List.fold_left
+            (fun cur (lbl, fn) ->
+              try
+                let v = fn f in
+                if v = 0 then cur else (lbl, string_of_int v) :: cur
+              with Not_found -> cur)
+            [] int_tags
         in
         Hashtbl.fold
           (fun key (values : string list) tags ->
-            if values = [] then tags
+            let key = String.lowercase_ascii key in
+            let key = try List.assoc key tag_aliases with _ -> key in
+            if List.mem_assoc key tags || values = [] then tags
             else (
               let v = List.hd values in
-              if v = "" then tags else (key, v) :: tags ))
+              match v with
+                | "0" when List.mem_assoc key int_tags -> tags
+                | "" -> tags
+                | _ -> (key, v) :: tags))
           (Taglib.File.properties f) tags)
   with
     | Invalid_file -> []
     | e ->
-        log#info "Error while decoding file tags: %s" (Printexc.to_string e);
-        log#info "Backtrace:\n%s" (Printexc.get_backtrace ());
+        let bt = Printexc.get_backtrace () in
+        Utils.log_exception ~log ~bt
+          (Printf.sprintf "Error while decoding file tags: %s"
+             (Printexc.to_string e));
         raise Not_found
 
 let () = Request.mresolvers#register "TAGLIB" get_tags
