@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,49 +20,51 @@
 
  *****************************************************************************)
 
+open Mm
+
 (** AVI encoder *)
 
 open Avi_format
-module Img = Image.RGBA32
 
 let encode_frame ~channels ~samplerate ~converter frame start len =
   let ratio = float samplerate /. float (Lazy.force Frame.audio_rate) in
-  let content =
-    Frame.content_of_type frame start
-      { Frame.audio = channels; video = 1; midi = 0 }
-  in
   let audio =
-    let astart = Frame.audio_of_master start in
-    let alen = Frame.audio_of_master len in
-    let pcm = content.Frame.audio in
+    let astart = Frame.audio_of_main start in
+    let alen = Frame.audio_of_main len in
+    let pcm = AFrame.pcm frame in
     (* Resample if needed. *)
     let pcm, astart, alen =
       if ratio = 1. then (pcm, astart, alen)
       else (
         let pcm =
-          Audio_converter.Samplerate.resample converter ratio pcm astart alen
+          Audio_converter.Samplerate.resample converter ratio
+            (Audio.sub pcm astart alen)
         in
-        (pcm, 0, Array.length pcm.(0)) )
+        (pcm, 0, Audio.length pcm))
     in
     let data = Bytes.create (2 * channels * alen) in
-    Audio.S16LE.of_audio pcm astart data 0 alen;
+    Audio.S16LE.of_audio (Audio.sub pcm astart alen) data 0;
     Avi.audio_chunk (Bytes.unsafe_to_string data)
   in
   let video =
-    let vbuf = content.Frame.video in
-    let vbuf = vbuf.(0) in
-    let vstart = Frame.video_of_master start in
-    let vlen = Frame.video_of_master len in
-    let data = ref "" in
+    let vbuf = VFrame.yuva420p frame in
+    let vstart = Frame.video_of_main start in
+    let vlen = Frame.video_of_main len in
+    let data = Strings.Mutable.empty () in
     for i = vstart to vstart + vlen - 1 do
-      (* TODO: mplayer needs flipping, but not vlc or GStreamer... *)
-      (* Img.Effect.flip vbuf.(i); *)
-      Img.swap_rb vbuf.(i);
-      data := !data ^ Img.to_RGB24_string vbuf.(i)
+      let img = Video.get vbuf i in
+      (* TODO: change stride otherwise *)
+      let width = Image.YUV420.width img in
+      assert (Image.YUV420.y_stride img = width);
+      assert (Image.YUV420.uv_stride img = width / 2);
+      let y, u, v = Image.YUV420.data img in
+      Strings.Mutable.add data (Image.Data.to_string y);
+      Strings.Mutable.add data (Image.Data.to_string u);
+      Strings.Mutable.add data (Image.Data.to_string v)
     done;
-    Avi.video_chunk !data
+    Avi.video_chunk_strings data
   in
-  video ^ audio
+  Strings.add video audio
 
 let encoder avi =
   let channels = avi.channels in
@@ -75,14 +77,24 @@ let encoder avi =
     let ans = encode_frame ~channels ~samplerate ~converter frame start len in
     if !need_header then (
       need_header := false;
-      header ^ ans )
+      Strings.dda header ans)
     else ans
   in
+  let hls =
+    {
+      Encoder.init_encode = (fun f o l -> (None, encode f o l));
+      split_encode = (fun f o l -> `Ok (Strings.empty, encode f o l));
+      codec_attrs = (fun () -> None);
+      bitrate = (fun () -> None);
+      video_size = (fun () -> None);
+    }
+  in
   {
     Encoder.insert_metadata = (fun _ -> ());
+    hls;
     encode;
-    header = Some header;
-    stop = (fun () -> "");
+    header = Strings.of_string header;
+    stop = (fun () -> Strings.empty);
   }
 
 let () =
