@@ -1,7 +1,7 @@
 (*****************************************************************************
   
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
   
  *****************************************************************************)
 
+open Mm
+
 (** FDK-AAC encoder *)
 
 module type Fdkaac_t = sig
@@ -86,15 +88,34 @@ module Register (Fdkaac : Fdkaac_t) = struct
         `Transmux params.Fdkaac_format.transmux;
         `Afterburner params.Fdkaac_format.afterburner;
       ]
-      @ ( if params.Fdkaac_format.aot = `Mpeg_4 `AAC_ELD then
-          [`Sbr_mode params.Fdkaac_format.sbr_mode]
-        else [] )
+      @ (if params.Fdkaac_format.aot = `Mpeg_4 `AAC_ELD then
+         [`Sbr_mode params.Fdkaac_format.sbr_mode]
+        else [])
       @
       match params.Fdkaac_format.bitrate_mode with
         | `Variable vbr -> [`Bitrate_mode (`Variable vbr)]
         | `Constant -> [`Bitrate (params.Fdkaac_format.bitrate * 1000)]
     in
-    List.iter (Fdkaac.Encoder.set encoder) params;
+    let string_of_param = function
+      | `Aot _ -> "aot"
+      | `Afterburner _ -> "afterburner"
+      | `Bandwidth _ -> "bandwidth"
+      | `Bitrate _ -> "bitrate"
+      | `Bitrate_mode _ -> "bitrate mode"
+      | `Granule_length _ -> "granule length"
+      | `Samplerate _ -> "samplerate"
+      | `Sbr_mode _ -> "sbr mode"
+      | `Transmux _ -> "transmux"
+    in
+    let set p =
+      try Fdkaac.Encoder.set encoder p with
+        | Fdkaac.Encoder.Invalid_config ->
+            failwith ("Invalid configuration: " ^ string_of_param p)
+        | Fdkaac.Encoder.Unsupported_parameter ->
+            failwith ("Unsupported parameter: " ^ string_of_param p)
+        | e -> raise e
+    in
+    List.iter set params;
     encoder
 
   let encoder aac =
@@ -104,43 +125,58 @@ module Register (Fdkaac : Fdkaac_t) = struct
     let samplerate_converter = Audio_converter.Samplerate.create channels in
     let src_freq = float (Frame.audio_of_seconds 1.) in
     let dst_freq = float samplerate in
-    let n = 1024 in
-    let buf = Buffer.create n in
+    let n = Utils.pagesize in
+    let buf = Strings.Mutable.empty () in
     let encode frame start len =
-      let start = Frame.audio_of_master start in
-      let b = AFrame.content_of_type ~channels frame start in
-      let len = Frame.audio_of_master len in
+      let b = AFrame.pcm frame in
+      let len = Frame.audio_of_main len in
       let b, start, len =
         if src_freq <> dst_freq then (
           let b =
             Audio_converter.Samplerate.resample samplerate_converter
-              (dst_freq /. src_freq) b start len
+              (dst_freq /. src_freq) (Audio.sub b start len)
           in
-          (b, 0, Array.length b.(0)) )
+          (b, 0, Audio.length b))
         else (b, start, len)
       in
-      let encoded = Buffer.create n in
-      Buffer.add_string buf (Audio.S16LE.make b start len);
-      let len = Buffer.length buf in
-      let rec f start =
-        if start + n > len then begin
-          Utils.buffer_drop buf start;
-          Buffer.contents encoded
-        end
-        else begin
-          let data = Buffer.sub buf start n in
-          Buffer.add_string encoded (Fdkaac.Encoder.encode enc data 0 n);
-          f (start + n)
-        end
-      in
-      f 0
+      let encoded = Strings.Mutable.empty () in
+      Strings.Mutable.add buf (Audio.S16LE.make (Audio.sub b start len));
+      while Strings.Mutable.length buf >= n do
+        let data = Bytes.create n in
+        Strings.blit (Strings.sub (Strings.Mutable.to_strings buf) 0 n) data 0;
+        let data = Bytes.unsafe_to_string data in
+        Strings.Mutable.drop buf n;
+        Strings.Mutable.add encoded (Fdkaac.Encoder.encode enc data 0 n)
+      done;
+      Strings.Mutable.to_strings encoded
     in
     let stop () =
-      let rem = Buffer.contents buf in
-      let s = Fdkaac.Encoder.encode enc rem 0 (String.length rem) in
-      s ^ Fdkaac.Encoder.flush enc
+      let rem =
+        Strings.Mutable.map
+          (fun rem ofs len ->
+            let rem = Fdkaac.Encoder.encode enc rem ofs len in
+            (rem, 0, String.length rem))
+          buf
+      in
+      Strings.Mutable.add rem (Fdkaac.Encoder.flush enc);
+      Strings.Mutable.to_strings rem
     in
-    { Encoder.insert_metadata = (fun _ -> ()); header = None; encode; stop }
+    let hls =
+      {
+        Encoder.init_encode = (fun f o l -> (None, encode f o l));
+        split_encode = (fun f o l -> `Ok (Strings.empty, encode f o l));
+        codec_attrs = (fun () -> None);
+        bitrate = (fun () -> None);
+        video_size = (fun () -> None);
+      }
+    in
+    {
+      Encoder.insert_metadata = (fun _ -> ());
+      header = Strings.empty;
+      hls;
+      encode;
+      stop;
+    }
 
   let register_encoder name =
     Encoder.plug#register name (function
