@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
 
  *****************************************************************************)
 
+open Mm
+
 (** FLAC encoder *)
 
 let encoder flac meta =
@@ -38,39 +40,47 @@ let encoder flac meta =
       total_samples = None;
     }
   in
-  let buf = Buffer.create 1024 in
-  let write chunk = Buffer.add_bytes buf chunk in
+  let buf = Strings.Mutable.empty () in
+  let write chunk = Strings.Mutable.add_bytes buf chunk in
   let cb = Flac.Encoder.get_callbacks write in
   let enc = Flac.Encoder.create ~comments p cb in
   let enc = ref enc in
   let encode frame start len =
-    let start = Frame.audio_of_master start in
-    let b = AFrame.content_of_type ~channels frame start in
-    let len = Frame.audio_of_master len in
+    let b = AFrame.pcm frame in
+    let len = Frame.audio_of_main len in
     let b, start, len =
       if src_freq <> dst_freq then (
         let b =
           Audio_converter.Samplerate.resample samplerate_converter
-            (dst_freq /. src_freq) b start len
+            (dst_freq /. src_freq) (Audio.sub b start len)
         in
-        (b, 0, Array.length b.(0)) )
+        (b, 0, Audio.length b))
       else (b, start, len)
     in
-    let b = Array.map (fun x -> Array.sub x start len) b in
+    let b = Audio.sub b start len in
+    let b = Audio.to_array b in
     Flac.Encoder.process !enc cb b;
-    let ret = Buffer.contents buf in
-    Buffer.reset buf;
-    ret
+    Strings.Mutable.flush buf
   in
   let stop () =
     Flac.Encoder.finish !enc cb;
-    Buffer.contents buf
+    Strings.Mutable.flush buf
+  in
+  let hls =
+    {
+      Encoder.init_encode = (fun f o l -> (None, encode f o l));
+      split_encode = (fun f o l -> `Ok (Strings.empty, encode f o l));
+      codec_attrs = (fun () -> None);
+      bitrate = (fun () -> None);
+      video_size = (fun () -> None);
+    }
   in
   {
     Encoder.insert_metadata = (fun _ -> ());
     (* Flac encoder do not support header
      * for now. It will probably never do.. *)
-    header = None;
+    header = Strings.empty;
+    hls;
     encode;
     stop;
   }
