@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
 
  *****************************************************************************)
 
+open Mm
+
 (** MP3 encoder *)
 
 open Encoder
@@ -73,8 +75,11 @@ module type Lame_t = sig
   exception Psychoacoustic_problem
   exception Unknown_error of int
 
-  val encode_buffer_float_part :
-    encoder -> float array -> float array -> int -> int -> string
+  val encode_buffer_float_ba :
+    encoder ->
+    (float, Bigarray.float32_elt, Bigarray.c_layout) Bigarray.Array1.t ->
+    (float, Bigarray.float32_elt, Bigarray.c_layout) Bigarray.Array1.t ->
+    string
 
   val encode_flush : encoder -> string
 end
@@ -86,7 +91,7 @@ let bit_at s pos =
   (byte land (1 lsl bit_pos)) lsr bit_pos == 1
 
 module Register (Lame : Lame_t) = struct
-  type id3v2 = Waiting | Rendered of string | Done
+  type id3v2 = Waiting | Rendered of Strings.t | Done
 
   (* Notation: XYZ; X: copyright bit, Y: original bit, Z: private bit 
    *           !: negation
@@ -134,15 +139,17 @@ module Register (Lame : Lame_t) = struct
       (* Input settings *)
       Lame.set_in_samplerate enc (Lazy.force Frame.audio_rate);
       Lame.set_num_channels enc (if mp3.Mp3_format.stereo then 2 else 1);
+
       (* Internal quality *)
       Lame.set_quality enc mp3.Mp3_format.internal_quality;
+
       (* Output settings *)
       if not mp3.Mp3_format.stereo then Lame.set_mode enc Lame.Mono
       else (
         match mp3.Mp3_format.stereo_mode with
           | Mp3_format.Default -> ()
           | Mp3_format.Stereo -> Lame.set_mode enc Lame.Stereo
-          | Mp3_format.Joint_stereo -> Lame.set_mode enc Lame.Joint_stereo );
+          | Mp3_format.Joint_stereo -> Lame.set_mode enc Lame.Joint_stereo);
       begin
         let apply_constaints enc
             {
@@ -188,11 +195,10 @@ module Register (Lame : Lame_t) = struct
       sync enc;
       let channels = if mp3.Mp3_format.stereo then 2 else 1 in
       let encode frame start len =
-        let start = Frame.audio_of_master start in
-        let b = AFrame.content_of_type ~channels frame start in
-        let len = Frame.audio_of_master len in
+        let b = AFrame.pcm frame in
+        let len = Frame.audio_of_main len in
         position := !position + len;
-        if mp3.Mp3_format.msg <> "" && !position > msg_interval then begin
+        if mp3.Mp3_format.msg <> "" && !position > msg_interval then (
           match !is_sync with
             | false ->
                 sync enc;
@@ -202,21 +208,35 @@ module Register (Lame : Lame_t) = struct
                 bset enc (bit_at msg !msg_position)
                   (bit_at msg (!msg_position + 1));
                 msg_position := (!msg_position + 2) mod msg_len;
-                if !msg_position mod 8 = 0 then is_sync := false
-        end;
+                if !msg_position mod 8 = 0 then is_sync := false);
         let encoded () =
           has_started := true;
-          if channels = 1 then
-            Lame.encode_buffer_float_part enc b.(0) b.(0) start len
-          else Lame.encode_buffer_float_part enc b.(0) b.(1) start len
+
+          (* Yes, lame requires this absurd scaling... *)
+          let scale buf =
+            let len = Audio.Mono.length buf in
+            let sbuf = Audio.Mono.create len in
+            for i = 0 to len - 1 do
+              Bigarray.Array1.unsafe_set sbuf i
+                (Bigarray.Array1.unsafe_get buf i *. 32768.)
+            done;
+            sbuf
+          in
+          if channels = 1 then (
+            let buf = scale (Audio.Mono.sub b.(0) start len) in
+            Lame.encode_buffer_float_ba enc buf buf)
+          else (
+            let bufl = scale (Audio.Mono.sub b.(0) start len) in
+            let bufr = scale (Audio.Mono.sub b.(1) start len) in
+            Lame.encode_buffer_float_ba enc bufl bufr)
         in
         match !id3v2 with
           | Rendered s when not !has_started ->
               id3v2 := Done;
-              Printf.sprintf "%s%s" s (encoded ())
-          | _ -> encoded ()
+              Strings.add s (encoded ())
+          | _ -> Strings.of_string (encoded ())
       in
-      let stop () = Lame.encode_flush enc in
+      let stop () = Strings.of_string (Lame.encode_flush enc) in
       let insert_metadata =
         match mp3.id3v2 with
           | Some f -> (
@@ -225,13 +245,22 @@ module Register (Lame : Lame_t) = struct
                 match !id3v2 with
                   | Waiting ->
                       if not (Meta_format.is_empty m) then
-                        id3v2 := Rendered (f m)
-                  | _ -> () )
+                        id3v2 := Rendered (Strings.of_string (f m))
+                  | _ -> ())
           | None -> fun _ -> ()
       in
       (* Try to insert initial metadata now.. *)
       insert_metadata metadata;
-      { insert_metadata; encode; header = None; stop }
+      let hls =
+        {
+          Encoder.init_encode = (fun f o l -> (None, encode f o l));
+          split_encode = (fun f o l -> `Ok (Strings.empty, encode f o l));
+          codec_attrs = (fun () -> None);
+          bitrate = (fun () -> None);
+          video_size = (fun () -> None);
+        }
+      in
+      { insert_metadata; hls; encode; header = Strings.empty; stop }
     in
     Encoder.plug#register name (function
       | Encoder.MP3 mp3 -> Some (fun _ meta -> mp3_encoder mp3 meta)
