@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
 
  *****************************************************************************)
 
+open Mm
+
 (** Fixed-point MP3 encoder *)
 
 open Shine_format
@@ -39,33 +41,48 @@ let encoder shine =
   let samples = Shine.samples_per_pass enc in
   let data = Audio.create channels samples in
   let buf = G.create () in
-  let encoded = Buffer.create 1024 in
+  let encoded = Strings.Mutable.empty () in
   let encode frame start len =
-    let start = Frame.audio_of_master start in
-    let b = AFrame.content_of_type ~channels frame start in
-    let len = Frame.audio_of_master len in
+    let b = AFrame.pcm frame in
+    let len = Frame.audio_of_main len in
     let b, start, len =
       if src_freq <> dst_freq then (
         let b =
           Audio_converter.Samplerate.resample samplerate_converter
-            (dst_freq /. src_freq) b start len
+            (dst_freq /. src_freq) (Audio.sub b start len)
         in
-        (b, 0, Array.length b.(0)) )
+        (b, 0, Audio.length b))
       else (Audio.copy b, start, len)
     in
     G.put buf b start len;
     while G.length buf > samples do
       let l = G.get buf samples in
-      let f (b, o, o', l) = Audio.blit b o data o' l in
+      let f (b, o, o', l) =
+        Audio.blit (Audio.sub b o l) (Audio.sub data o' l)
+      in
       List.iter f l;
-      Buffer.add_string encoded (Shine.encode_buffer enc data)
+      Strings.Mutable.add encoded
+        (Shine.encode_buffer enc (Audio.to_array data))
     done;
-    let ret = Buffer.contents encoded in
-    Buffer.reset encoded;
-    ret
+    Strings.Mutable.flush encoded
   in
-  let stop () = Shine.flush enc in
-  { Encoder.insert_metadata = (fun _ -> ()); header = None; encode; stop }
+  let stop () = Strings.of_string (Shine.flush enc) in
+  let hls =
+    {
+      Encoder.init_encode = (fun f o l -> (None, encode f o l));
+      split_encode = (fun f o l -> `Ok (Strings.empty, encode f o l));
+      codec_attrs = (fun () -> None);
+      bitrate = (fun () -> None);
+      video_size = (fun () -> None);
+    }
+  in
+  {
+    Encoder.insert_metadata = (fun _ -> ());
+    header = Strings.empty;
+    hls;
+    encode;
+    stop;
+  }
 
 let () =
   Encoder.plug#register "SHINE" (function
