@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -27,12 +27,12 @@ let conf =
     ~comments:["Settings for the encoder"]
 
 let conf_meta =
-  Dtools.Conf.void ~p:(conf#plug "encoder") "Metadata settings"
+  Dtools.Conf.void ~p:(conf#plug "metadata") "Metadata settings"
     ~comments:["Settings for the encoded metadata."]
 
 (** The list of metadata fields that should be exported when encoding. *)
 let conf_export_metadata =
-  Dtools.Conf.list ~p:(conf_meta#plug "export") "Exported metdata"
+  Dtools.Conf.list ~p:(conf_meta#plug "export") "Exported metadata"
     ~d:
       [
         "artist";
@@ -46,6 +46,9 @@ let conf_export_metadata =
         "year";
         "dj";
         "next";
+        "apic";
+        "metadata_block_picture";
+        "coverart";
       ]
     ~comments:["The list of labels of exported metadata."]
 
