@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -31,13 +31,12 @@ type t = {
   log : int;
 }
 
-let audio_channels m = if m.audio = None then 0 else m.channels
-let video_channels m = if m.video = None || not m.has_video then 0 else 1
+let audio_channels { channels; audio } = if audio <> None then channels else 0
 
 let to_string m =
   let pipeline l name value =
-    Utils.some_or l
-      (Utils.maybe (fun value -> Printf.sprintf "%s=%S" name value :: l) value)
+    Option.value ~default:l
+      (Option.map (fun value -> Printf.sprintf "%s=%S" name value :: l) value)
   in
   Printf.sprintf "%%gstreamer(%s,metadata=%S,has_video=%b,%slog=%d)"
     (String.concat ","
@@ -49,5 +48,6 @@ let to_string m =
              "video" m.video)
           "muxer" m.muxer))
     m.metadata m.has_video
-    (Utils.some_or "" (Utils.maybe (Printf.sprintf "pipeline=%S,") m.pipeline))
+    (Option.value ~default:""
+       (Option.map (Printf.sprintf "pipeline=%S,") m.pipeline))
     m.log
