@@ -1,4 +1,3 @@
-(* -*- mode: tuareg; -*- *)
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
@@ -30,7 +29,7 @@ module type Transport_t = sig
 
   val name : string
   val file_descr_of_socket : socket -> Unix.file_descr
-  val read : socket -> int -> bytes * int
+  val read : socket -> bytes -> int -> int -> int
   val accept : Unix.file_descr -> socket * Unix.sockaddr
   val close : socket -> unit
 
@@ -54,13 +53,8 @@ module Unix_transport = struct
 
   let name = "unix"
   let file_descr_of_socket socket = socket
-
-  let read socket len =
-    let buf = Bytes.create len in
-    let n = Unix.read socket buf 0 len in
-    (buf, n)
-
-  let accept fd = Unix.accept fd
+  let read = Unix.read
+  let accept fd = Unix.accept ~cloexec:true fd
   let close = Unix.close
 
   module Duppy = Duppy
@@ -82,7 +76,7 @@ module type T = sig
   (* Generic *)
 
   val file_descr_of_socket : socket -> Unix.file_descr
-  val read : socket -> int -> Bytes.t * int
+  val read : socket -> bytes -> int -> int -> int
   val close : socket -> unit
 
   (* Http Server *)
@@ -112,31 +106,31 @@ module type T = sig
   (* Source input *)
 
   class virtual source :
-    kind:Frame.content_kind
+    kind:Source.Kind.t
     -> object
          inherit Source.source
 
          method virtual relay :
            string ->
            (string * string) list ->
-           ?read:(socket -> int -> Bytes.t * int) ->
+           ?read:(socket -> bytes -> int -> int -> int) ->
            socket ->
            unit
 
          method virtual insert_metadata : (string, string) Hashtbl.t -> unit
 
-         method virtual login : string * (string -> string -> bool)
+         method virtual login :
+           string * (socket:socket -> string -> string -> bool)
 
          method virtual icy_charset : string option
-
          method virtual meta_charset : string option
-
          method virtual get_mime_type : string option
        end
 
   val http_auth_check :
     ?args:(string, string) Hashtbl.t ->
-    login:string * (string -> string -> bool) ->
+    login:string * (socket:socket -> string -> string -> bool) ->
+    socket ->
     (string * string) list ->
     (unit, reply) Duppy.Monad.t
 
@@ -176,18 +170,17 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
       method virtual relay
           : string ->
             (string * string) list ->
-            ?read:(socket -> int -> Bytes.t * int) ->
+            ?read:(socket -> bytes -> int -> int -> int) ->
             socket ->
             unit
 
       method virtual insert_metadata : (string, string) Hashtbl.t -> unit
 
-      method virtual login : string * (string -> string -> bool)
+      method virtual login
+          : string * (socket:socket -> string -> string -> bool)
 
       method virtual icy_charset : string option
-
       method virtual meta_charset : string option
-
       method virtual get_mime_type : string option
     end
 
@@ -329,10 +322,11 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
         Duppy.Monad.Io.exec ~priority:Tutils.Maybe_blocking h
           (let user, auth_f = s#login in
            let user = if requested_user = "" then user else requested_user in
-           if auth_f user password then Duppy.Monad.return (`Shout, "/", `Icy)
+           if auth_f ~socket:h.Duppy.Monad.Io.socket user password then
+             Duppy.Monad.return (`Shout, "/", `Icy)
            else (
              log#info "ICY error: invalid password";
-             simple_reply "Invalid password\r\n\r\n" )))
+             simple_reply "Invalid password\r\n\r\n")))
 
   let parse_http_request_line r =
     try
@@ -371,16 +365,16 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
       display_headers;
     headers
 
-  let auth_check ~auth_f user pass =
+  let auth_check ~auth_f socket user pass =
     (* OK *)
     if conf_pass_verbose#get then
       log#info "Requested username: %s, password: %s." user pass
     else ();
-    if not (auth_f user pass) then raise Not_authenticated else ();
+    if not (auth_f ~socket user pass) then raise Not_authenticated else ();
     log#info "Client logged in.";
     Duppy.Monad.return ()
 
-  let http_auth_check ?args ~login headers =
+  let http_auth_check ?args ~login socket headers =
     (* 401 error model *)
     let http_reply s =
       simple_reply
@@ -400,22 +394,22 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
                 let auth_data = Pcre.split ~pat:":" (Utils.decode64 x) in
                 match auth_data with
                   | x :: y :: _ -> (x, y)
-                  | _ -> raise Not_found )
+                  | _ -> raise Not_found)
             | _ -> raise Not_found
         with Not_found -> (
           match args with
             | Some args ->
                 (* ICY updates are done with
-                           * password sent in GET args
-                           * and user being valid_user
-                           * or user, if given. *)
+                         * password sent in GET args
+                         * and user being valid_user
+                         * or user, if given. *)
                 let user =
                   try Hashtbl.find args "user" with Not_found -> valid_user
                 in
                 (user, Hashtbl.find args "pass")
-            | _ -> raise Not_found )
+            | _ -> raise Not_found)
       in
-      auth_check ~auth_f user pass
+      auth_check ~auth_f socket user pass
     with
       | Not_authenticated ->
           log#info "Returned 401: wrong auth.";
@@ -426,7 +420,7 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
 
   let exec_http_auth_check ?args ~login h headers =
     Duppy.Monad.Io.exec ~priority:Tutils.Maybe_blocking h
-      (http_auth_check ?args ~login headers)
+      (http_auth_check ?args ~login h.Duppy.Monad.Io.socket headers)
 
   (* We do not implement anything with this handler for now. *)
   let handle_asterisk_options_request ~hprotocol:_ ~headers:_ _ =
@@ -448,9 +442,9 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
     in
     Duppy.Monad.bind __pa_duppy_0 (fun s ->
         Duppy.Monad.bind
-          ( if (* ICY and Xaudiocast auth check was done before.. *)
-               not auth then exec_http_auth_check ~login:s#login h headers
-          else Duppy.Monad.return () )
+          (if (* ICY and Xaudiocast auth check was done before.. *)
+              not auth then exec_http_auth_check ~login:s#login h headers
+          else Duppy.Monad.return ())
           (fun () ->
             try
               let sproto =
@@ -474,13 +468,19 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
               in
               let read =
                 if chunked then (
-                  let read connection =
-                    let s, len =
-                      Http.read_chunked ~timeout:conf_timeout#get connection
-                    in
-                    (Bytes.of_string s, len)
+                  let buf = Buffer.create Utils.pagesize in
+                  let read connection b ofs len =
+                    if Buffer.length buf < len then (
+                      let s, len =
+                        Http.read_chunked ~timeout:conf_timeout#get connection
+                      in
+                      Buffer.add_substring buf s 0 len);
+                    let len = min len (Buffer.length buf) in
+                    Buffer.blit buf 0 b ofs len;
+                    Utils.buffer_drop buf len;
+                    len
                   in
-                  Some (fun connection _ -> read connection) )
+                  Some read)
                 else None
               in
               let f () = s#relay ?read stype headers h.Duppy.Monad.Io.socket in
@@ -539,22 +539,23 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
       in
       (packet_type, data)
     in
+    let websocket_read = Websocket.read () in
     let read_hello s =
       let error () = simple_reply (websocket_error 1002 "Invalid hello.") in
       try
-        match Websocket.read s with
+        match websocket_read s with
           | `Text s -> (
               log#debug "Hello packet: %s\n%!" s;
               match extract_packet s with
                 | "hello", data ->
-                    let data = Utils.get_some data in
+                    let data = Option.get data in
                     let mime = json_string_of (List.assoc "mime" data) in
                     let user = json_string_of (List.assoc "user" data) in
                     let password =
                       json_string_of (List.assoc "password" data)
                     in
                     Duppy.Monad.return (mime, mount, user, password)
-                | _ -> error () )
+                | _ -> error ())
           | _ -> error ()
       with _ -> error ()
     in
@@ -580,21 +581,21 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
             Duppy.Monad.bind __pa_duppy_0 (fun source ->
                 let _, auth_f = source#login in
                 let __pa_duppy_0 =
-                  try auth_check ~auth_f user password
+                  try auth_check ~auth_f h.Duppy.Monad.Io.socket user password
                   with Not_authenticated ->
                     log#info "Authentication failed!";
                     simple_reply (websocket_error 1011 "Authentication failed.")
                 in
                 Duppy.Monad.bind __pa_duppy_0 (fun () ->
-                    let binary_data = Buffer.create 1024 in
+                    let binary_data = Buffer.create Utils.pagesize in
                     let read_socket socket =
-                      match Websocket.read socket with
+                      match websocket_read socket with
                         | `Binary buf -> Buffer.add_string binary_data buf
                         | `Text s -> (
                             match extract_packet s with
                               | "metadata", data ->
                                   log#debug "Metadata packet: %s\n%!" s;
-                                  let data = Utils.get_some data in
+                                  let data = Option.get data in
                                   let m =
                                     List.map
                                       (fun (l, v) -> (l, json_string_of v))
@@ -612,17 +613,17 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
                                   in
                                   source#insert_metadata m;
                                   raise Retry
-                              | _ -> raise Retry )
+                              | _ -> raise Retry)
                         | `Close _ -> raise Websocket_closed
                         | _ -> raise Retry
                     in
-                    let read socket len =
+                    let read socket buf ofs len =
                       if Buffer.length binary_data = 0 then read_socket socket
                       else ();
                       let len = min (Buffer.length binary_data) len in
-                      let data = Buffer.sub binary_data 0 len in
+                      Buffer.blit binary_data 0 buf ofs len;
                       Utils.buffer_drop binary_data len;
-                      (Bytes.of_string data, len)
+                      len
                     in
                     let f () =
                       source#relay stype headers ~read h.Duppy.Monad.Io.socket
@@ -669,20 +670,20 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
                       (exec_http_auth_check ~args ~login:s#login h headers)
                       (fun () ->
                         Duppy.Monad.bind
-                          ( if
-                            not
-                              (List.mem
-                                 (Utils.get_some s#get_mime_type)
-                                 conf_icy_metadata#get)
+                          (if
+                           not
+                             (List.mem
+                                (Option.value ~default:"" s#get_mime_type)
+                                conf_icy_metadata#get)
                           then (
-                            log#info
-                              "Returned 405 for '%s': Source format does not \
-                               support ICY metadata update"
-                              uri;
-                            simple_reply
-                              (http_error_page 405 "Method Not Allowed"
-                                 "Method Not Allowed") )
-                          else Duppy.Monad.return () )
+                           log#info
+                             "Returned 405 for '%s': Source format does not \
+                              support ICY metadata update"
+                             uri;
+                           simple_reply
+                             (http_error_page 405 "Method Not Allowed"
+                                "Method Not Allowed"))
+                          else Duppy.Monad.return ())
                           (fun () ->
                             Hashtbl.remove args "mount";
                             Hashtbl.remove args "mode";
@@ -735,26 +736,34 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
     in
     log#info "%s %s request on %s." protocol_name smethod base_uri;
     let args = Http.args_split args in
+    (try
+       if
+         assoc_uppercase "CONTENT-TYPE" headers
+         = "application/x-www-form-urlencoded"
+       then Hashtbl.iter (Hashtbl.add args) (Http.args_split data)
+     with Not_found -> ());
     (* Filter out password *)
     let log_args =
       if conf_pass_verbose#get then args
       else (
         let log_args = Hashtbl.copy args in
         Hashtbl.remove log_args "pass";
-        log_args )
+        log_args)
     in
     Hashtbl.iter (log#info "%s Arg: %s, value: %s." protocol_name) log_args;
+
     (* First, try with a registered handler. *)
     let handler, _ = find_handler port in
     let f (verb, reg_uri) handler =
       let rex = Pcre.regexp reg_uri in
       if (verb :> verb) = hmethod && Pcre.pmatch ~rex uri then (
         log#info "Found handler '%s %s' on port %d." smethod reg_uri port;
-        raise (Handled handler) )
+        raise (Handled handler))
       else ()
     in
     try
       Hashtbl.iter f handler.http;
+
       (* Otherwise, try with a standard handler. *)
       let is_admin s = try String.sub s 0 6 = "/admin" with _ -> false in
       match base_uri with
@@ -780,9 +789,9 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
       Duppy.Monad.Io.read ?timeout:(Some conf_timeout#get)
         ~priority:Tutils.Non_blocking
         ~marker:
-          ( match icy with
+          (match icy with
             | true -> Duppy.Io.Split "[\r]?\n"
-            | false -> Duppy.Io.Split "[\r]?\n[\r]?\n" )
+            | false -> Duppy.Io.Split "[\r]?\n[\r]?\n")
         h
     in
     Duppy.Monad.bind __pa_duppy_0 (fun s ->
@@ -820,7 +829,7 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
             let handle_source smethod =
               let __pa_duppy_0 =
                 (* X-audiocast sends lines of the form:
-                   * [SOURCE password path] *)
+                   [SOURCE password path] *)
                 match hprotocol with
                   | `Xaudiocast_uri uri ->
                       let password = huri in
@@ -838,11 +847,14 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
                           Duppy.Monad.Io.exec
                             ~priority:
                               (* ICY = true means that authentication has already
-                                 * hapenned *)
+                                 happened *)
                               Tutils.Maybe_blocking h
                             (let valid_user, auth_f = s#login in
-                             if not (auth_f valid_user password) then
-                               simple_reply "Invalid password!"
+                             if
+                               not
+                                 (auth_f ~socket:h.Duppy.Monad.Io.socket
+                                    valid_user password)
+                             then simple_reply "Invalid password!"
                              else Duppy.Monad.return (true, uri, `Xaudiocast)))
                   | _ -> Duppy.Monad.return (false, huri, smethod)
               in
@@ -912,13 +924,14 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
         let unix_socket = T.file_descr_of_socket socket in
         Unix.setsockopt unix_socket Unix.TCP_NODELAY true;
         let on_error e =
-          ( match e with
+          (match e with
             | Duppy.Io.Io_error -> log#info "Client %s disconnected" ip
             | Duppy.Io.Timeout ->
                 log#info "Timeout while communicating with client %s." ip
             | Duppy.Io.Unix (c, p, m) ->
                 log#info "%s" (Printexc.to_string (Unix.Unix_error (c, p, m)))
-            | Duppy.Io.Unknown e -> log#info "%s" (Printexc.to_string e) );
+            | Duppy.Io.Unknown e -> log#info "%s" (Printexc.to_string e));
+
           (* Sending an HTTP response in case of timeout
            * even though ICY connections are not HTTP.. *)
           if e = Duppy.Io.Timeout then
@@ -966,7 +979,7 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
             (function `Read s -> Unix.close s | _ -> assert false)
             events;
           []
-        with _ -> [] )
+        with _ -> [])
       else (
         let get_sock = function `Read sock -> sock | _ -> assert false in
         List.iter process_client (List.map get_sock e);
@@ -976,13 +989,13 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
             events;
             handler = incoming ~port ~icy events out_s;
           };
-        ] )
+        ])
     in
     let open_socket port bind_addr =
       let bind_addr_inet = Unix.inet_addr_of_string bind_addr in
       let bind_addr = Unix.ADDR_INET (bind_addr_inet, port) in
       let domain = Unix.domain_of_sockaddr bind_addr in
-      let sock = Unix.socket domain Unix.SOCK_STREAM 0 in
+      let sock = Unix.socket ~cloexec:true domain Unix.SOCK_STREAM 0 in
       (* Set TCP_NODELAY on the socket *)
       Unix.setsockopt sock Unix.SO_REUSEADDR true;
       Unix.setsockopt sock Unix.TCP_NODELAY true;
@@ -995,7 +1008,7 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
         (fun cur bind_addr -> if bind_addr <> "" then bind_addr :: cur else cur)
         [] conf_harbor_bind_addrs#get
     in
-    let in_s, out_s = Unix.pipe () in
+    let in_s, out_s = Unix.pipe ~cloexec:true () in
     let events = `Read in_s :: List.map (open_socket port) bind_addrs in
     Task.add Tutils.scheduler
       {
@@ -1014,7 +1027,7 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
        * we need to open a second one. *)
       if List.length socks = 1 && icy then (
         let socks = open_port ~icy (port + 1) :: socks in
-        Hashtbl.replace opened_ports port (h, socks) )
+        Hashtbl.replace opened_ports port (h, socks))
       else ();
       h
     with Not_found ->
@@ -1050,7 +1063,7 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
         Unix.close in_s
       in
       List.iter f socks;
-      Hashtbl.remove opened_ports port )
+      Hashtbl.remove opened_ports port)
     else ()
 
   (* Add http_handler... *)
@@ -1078,7 +1091,7 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
         Unix.close in_s
       in
       List.iter f socks;
-      Hashtbl.remove opened_ports port )
+      Hashtbl.remove opened_ports port)
     else ()
 end
 
