@@ -2,7 +2,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@ let conf_harbor_secure_transport_private_key =
 let conf_harbor_secure_transport_password =
   Conf.string
     ~p:(conf_harbor_secure_transport#plug "password")
-    ~d:"" "Path to the server's SSL password. (optional, blank if omited)"
+    ~d:"" "Path to the server's SSL password. (optional, blank if omitted)"
 
 module Monad = Duppy.Monad
 
@@ -78,14 +78,10 @@ module Transport = struct
 
   let name = "secure_transport"
   let file_descr_of_socket { sock } = sock
-
-  let read { ctx } len =
-    let buf = Bytes.create len in
-    let n = SecureTransport.read ctx buf 0 len in
-    (buf, n)
+  let read { ctx } buf ofs len = SecureTransport.read ctx buf ofs len
 
   let accept sock =
-    let sock, caller = Unix.accept sock in
+    let sock, caller = Unix.accept ~cloexec:true sock in
     let ctx =
       SecureTransport.init SecureTransport.Server SecureTransport.Stream
     in
