@@ -2,7 +2,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@ let conf_harbor_ssl_private_key =
 let conf_harbor_ssl_password =
   Conf.string
     ~p:(conf_harbor_ssl#plug "password")
-    ~d:"" "Path to the server's SSL password. (optional, blank if omited)"
+    ~d:"" "Path to the server's SSL password. (optional, blank if omitted)"
 
 let conf_harbor_ssl_read_timeout =
   Conf.float
@@ -107,15 +107,11 @@ module Transport = struct
 
   let name = "ssl"
   let file_descr_of_socket = Ssl.file_descr_of_socket
-
-  let read socket len =
-    let buf = Bytes.create len in
-    let n = Ssl.read socket buf 0 len in
-    (buf, n)
+  let read = Ssl.read
 
   let accept sock =
     let ctx = get_ctx () in
-    let s, caller = Unix.accept sock in
+    let s, caller = Unix.accept ~cloexec:true sock in
     set_socket_default s;
     let ssl_s = Ssl.embed_socket s ctx in
     Ssl.accept ssl_s;
