@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,13 +20,13 @@
 
  *****************************************************************************)
 
+open Mm
 open Extralib
 open Gstreamer
 module GU = Gstreamer_utils
-module Img = Image.RGBA32
 
 let log = Log.make ["io"; "gstreamer"]
-let gst_clock = Tutils.lazy_cell (fun () -> new Clock.self_sync "gstreamer")
+let gst_clock = Tutils.lazy_cell (fun () -> new Clock.clock "gstreamer")
 
 let string_of_state_change = function
   | Element.State_change_success -> "success"
@@ -47,21 +47,13 @@ type ('a, 'b) element = {
 class virtual ['a, 'b] element_factory ~on_error =
   object (self)
     val restart_m = Mutex.create ()
-
     val mutable restarting = false
-
     val mutable retry_in = -1.
-
     val task_m = Mutex.create ()
-
     val mutable task = None
-
     val mutable element_m = Mutex.create ()
-
     val mutable element = None
-
     method virtual log : Log.t
-
     method virtual make_element : ('a, 'b) element
 
     method private get_element =
@@ -79,16 +71,15 @@ class virtual ['a, 'b] element_factory ~on_error =
       let should_run =
         Tutils.mutexify restart_m
           (fun () ->
-            if not restarting then begin
+            if not restarting then (
               restarting <- true;
-              true
-            end
+              true)
             else false)
           ()
       in
       if should_run then (
         try
-          self#log#important "Restarting pipeline";
+          self#log#important "Restarting pipeline.";
           Tutils.mutexify element_m
             (fun () ->
               begin
@@ -110,18 +101,19 @@ class virtual ['a, 'b] element_factory ~on_error =
           Tutils.mutexify restart_m (fun () -> restarting <- false) ();
           if retry_in >= 0. then
             self#log#info
-              "An error occured while restarting pipeline, will retry in %.02f"
+              "An error occurred while restarting pipeline, will retry in %.02f"
               retry_in
           else self#log#info "Done restarting pipeline";
           retry_in
         with exn ->
-          self#log#important "Error while restarting pipeline: %s"
-            (Printexc.to_string exn);
-          self#log#info "Backtrace: %s" (Printexc.get_backtrace ());
+          let bt = Printexc.get_backtrace () in
+          Utils.log_exception ~log:self#log ~bt
+            (Printf.sprintf "Error while restarting pipeline: %s"
+               (Printexc.to_string exn));
           retry_in <- on_error exn;
           self#log#important "Will retry again in %.02f" retry_in;
           Tutils.mutexify restart_m (fun () -> restarting <- false) ();
-          retry_in )
+          retry_in)
       else -1.
 
     method private register_task ~priority scheduler =
@@ -179,7 +171,6 @@ class output ~kind ~clock_safe ~on_error ~infallible ~on_start ~on_stop
       | Some video_pipeline -> (true, video_pipeline)
       | None -> (false, "")
   in
-  let channels = (Frame.type_of_kind kind).Frame.audio in
   object (self)
     inherit
       Output.output
@@ -187,6 +178,8 @@ class output ~kind ~clock_safe ~on_error ~infallible ~on_start ~on_stop
           ~name:"output.gstreamer" ~output_kind:"gstreamer" source start as super
 
     inherit [App_src.t, App_src.t] element_factory ~on_error
+    val mutable started = false
+    method self_sync = (`Dynamic, started)
 
     method private set_clock =
       super#set_clock;
@@ -194,18 +187,21 @@ class output ~kind ~clock_safe ~on_error ~infallible ~on_start ~on_stop
         Clock.unify self#clock
           (Clock.create_known (gst_clock () :> Clock.clock))
 
-    method output_start =
+    method start =
       let el = self#get_element in
+      self#log#info "Playing.";
+      started <- true;
       ignore (Element.set_state el.bin Element.State_playing);
+
       (* Don't uncomment the following line, it locks the program. I guess that
          GStreamer is waiting for some data before answering that we are
          playing. *)
       (* ignore (Element.get_state el.bin); *)
-      self#register_task ~priority:Tutils.Blocking Tutils.scheduler;
-      if clock_safe then (gst_clock ())#register_blocking_source
+      self#register_task ~priority:Tutils.Blocking Tutils.scheduler
 
-    method output_stop =
+    method stop =
       self#stop_task;
+      started <- false;
       let todo =
         Tutils.mutexify element_m
           (fun () ->
@@ -215,22 +211,22 @@ class output ~kind ~clock_safe ~on_error ~infallible ~on_start ~on_stop
                   element <- None;
                   fun () ->
                     if has_audio then
-                      App_src.end_of_stream (Utils.get_some el.audio);
+                      App_src.end_of_stream (Option.get el.audio);
                     if has_video then
-                      App_src.end_of_stream (Utils.get_some el.video);
+                      App_src.end_of_stream (Option.get el.video);
                     ignore (Element.set_state el.bin Element.State_null);
                     ignore (Element.get_state el.bin);
                     GU.flush ~log:self#log el.bin)
           ()
       in
-      todo ();
-      if clock_safe then (gst_clock ())#unregister_blocking_source
+      todo ()
 
     method private make_element =
       let pipeline =
         if has_audio then
           Printf.sprintf "%s ! %s %s"
-            (GU.Pipeline.audio_src ~channels ~block:blocking "audio_src")
+            (GU.Pipeline.audio_src ~channels:self#audio_channels ~block:blocking
+               "audio_src")
             audio_pipeline pipeline
         else pipeline
       in
@@ -241,14 +237,14 @@ class output ~kind ~clock_safe ~on_error ~infallible ~on_start ~on_stop
             video_pipeline pipeline
         else pipeline
       in
-      self#log#debug "GStreamer pipeline: %s" pipeline;
+      self#log#info "GStreamer pipeline: %s" pipeline;
       let bin = Pipeline.parse_launch pipeline in
       let audio_src =
         if has_audio then (
           let audio_src =
             App_src.of_element (Bin.get_by_name bin "audio_src")
           in
-          Some audio_src )
+          Some audio_src)
         else None
       in
       let video_src =
@@ -256,49 +252,55 @@ class output ~kind ~clock_safe ~on_error ~infallible ~on_start ~on_stop
           let video_src =
             App_src.of_element (Bin.get_by_name bin "video_src")
           in
-          Some video_src )
+          Some video_src)
         else None
       in
       { bin; audio = audio_src; video = video_src }
 
     val mutable presentation_time = Int64.zero
 
-    method output_send frame =
+    method send_frame frame =
       let el = self#get_element in
       try
         if not (Frame.is_partial frame) then (
-          let _, content = Frame.content frame 0 in
           let len = Lazy.force Frame.size in
-          let duration = Gstreamer_utils.time_of_master len in
+          let duration = Gstreamer_utils.time_of_main len in
           if has_audio then (
-            let pcm = content.Frame.audio in
-            assert (Array.length pcm = channels);
-            let len = Frame.audio_of_master len in
-            let data = Bytes.create (2 * channels * len) in
-            Audio.S16LE.of_audio pcm 0 data 0 len;
+            let pcm = AFrame.pcm frame in
+            assert (Array.length pcm = self#audio_channels);
+            let len = Frame.audio_of_main len in
+            let data = Bytes.create (2 * self#audio_channels * len) in
+            Audio.S16LE.of_audio pcm data 0;
             Gstreamer.App_src.push_buffer_bytes ~duration ~presentation_time
-              (Utils.get_some el.audio) data 0 (Bytes.length data) );
+              (Option.get el.audio) data 0 (Bytes.length data));
           if has_video then (
-            let buf = content.Frame.video.(0) in
-            for i = 0 to Array.length buf - 1 do
-              let data = Img.data buf.(i) in
-              Gstreamer.App_src.push_buffer_data ~duration ~presentation_time
-                (Utils.get_some el.video) data 0 (Bigarray.Array1.dim data)
-            done );
+            let buf = VFrame.yuva420p frame in
+            for i = 0 to Video.length buf - 1 do
+              let img = Video.get buf i in
+              let y, u, v = Image.YUV420.data img in
+              let buf =
+                Gstreamer.Buffer.of_data_list
+                  (List.map (fun d -> (d, 0, Image.Data.length d)) [y; u; v])
+              in
+              Gstreamer.Buffer.set_duration buf duration;
+              Gstreamer.Buffer.set_presentation_time buf presentation_time;
+              Gstreamer.App_src.push_buffer (Option.get el.video) buf
+            done);
           presentation_time <- Int64.add presentation_time duration;
           GU.flush ~log:self#log
             ~on_error:(fun err -> raise (Flushing_error err))
-            el.bin )
+            el.bin)
       with e ->
-        self#log#important "Error while processing output data: %s"
-          (Printexc.to_string e);
-        self#log#info "Stacktrace: %s" (Printexc.get_backtrace ());
+        let bt = Printexc.get_backtrace () in
+        Utils.log_exception ~log:self#log ~bt
+          (Printf.sprintf "Error while processing output data: %s"
+             (Printexc.to_string e));
         self#on_error e
 
-    method output_reset = ()
+    method reset = ()
   end
 
-let output_proto ~kind ~pipeline =
+let output_proto ~return_t ~pipeline =
   Output.proto
   @ [
       ( "clock_safe",
@@ -307,86 +309,92 @@ let output_proto ~kind ~pipeline =
         Some "Use the dedicated GStreamer clock." );
       ( "on_error",
         Lang.fun_t [(false, "", Lang.string_t)] Lang.float_t,
-        Some (Lang.val_cst_fun [("", Lang.string_t, None)] (Lang.float 3.)),
+        Some (Lang.val_cst_fun [("", None)] (Lang.float 3.)),
         Some
           "Callback executed when an error happens. The callback receives a \
-           string representation of the error that occured and returns a \
+           string representation of the error that occurred and returns a \
            float. If returned value is positive, connection will be tried \
            again after this amount of time (in seconds)." );
       ( "pipeline",
         Lang.string_t,
         Some (Lang.string pipeline),
         Some "GStreamer pipeline for sink." );
-      ("", Lang.source_t kind, None, None);
+      ("", Lang.source_t return_t, None, None);
     ]
 
 let () =
-  let kind = Lang.any_fixed_with ~audio:1 () in
-  let kind = Lang.kind_type_of_kind_format ~fresh:1 kind in
-  Lang.add_operator "output.gstreamer.audio" ~active:true
-    (output_proto ~kind ~pipeline:"autoaudiosink")
-    ~category:Lang.Output ~descr:"Output stream to a GStreamer pipeline."
-    ~kind:(Lang.Unconstrained kind) (fun p kind ->
+  Lang.add_module "output.gstreamer";
+  Lang.add_module "input.gstreamer"
+
+let () =
+  let kind = { Frame.audio = Frame.audio_pcm; video = `Any; midi = `Any } in
+  let return_t = Lang.kind_type_of_kind_format kind in
+  Lang.add_operator "output.gstreamer.audio"
+    (output_proto ~return_t ~pipeline:"autoaudiosink")
+    ~category:`Output ~meth:Output.meth
+    ~descr:"Output stream to a GStreamer pipeline." ~return_t (fun p ->
       let clock_safe = Lang.to_bool (List.assoc "clock_safe" p) in
       let pipeline = Lang.to_string (List.assoc "pipeline" p) in
       let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
       let on_error = List.assoc "on_error" p in
       let on_error error =
         let msg = Printexc.to_string error in
-        Lang.to_float
-          (Lang.apply ~t:Lang.unit_t on_error [("", Lang.string msg)])
+        Lang.to_float (Lang.apply on_error [("", Lang.string msg)])
       in
       let start = Lang.to_bool (List.assoc "start" p) in
       let on_start =
         let f = List.assoc "on_start" p in
-        fun () -> ignore (Lang.apply ~t:Lang.unit_t f [])
+        fun () -> ignore (Lang.apply f [])
       in
       let on_stop =
         let f = List.assoc "on_stop" p in
-        fun () -> ignore (Lang.apply ~t:Lang.unit_t f [])
+        fun () -> ignore (Lang.apply f [])
       in
       let source = List.assoc "" p in
-      ( new output
-          ~kind ~clock_safe ~on_error ~infallible ~on_start ~on_stop source
-          start ("", Some pipeline, None)
-        :> Source.source ))
+      let kind = Source.Kind.of_kind kind in
+      (new output
+         ~kind ~clock_safe ~on_error ~infallible ~on_start ~on_stop source start
+         ("", Some pipeline, None)
+        :> Output.output))
 
 let () =
-  let kind = Lang.any_fixed_with ~video:1 () in
-  let kind = Lang.kind_type_of_kind_format ~fresh:1 kind in
-  Lang.add_operator "output.gstreamer.video" ~active:true
-    (output_proto ~kind ~pipeline:"videoconvert ! autovideosink")
-    ~category:Lang.Output ~descr:"Output stream to a GStreamer pipeline."
-    ~kind:(Lang.Unconstrained kind) (fun p kind ->
+  let kind = { Frame.audio = Frame.audio_pcm; video = `Any; midi = `Any } in
+  let return_t = Lang.kind_type_of_kind_format kind in
+  Lang.add_operator "output.gstreamer.video"
+    (output_proto ~return_t ~pipeline:"videoconvert ! autovideosink")
+    ~category:`Output ~meth:Output.meth
+    ~descr:"Output stream to a GStreamer pipeline." ~return_t (fun p ->
       let clock_safe = Lang.to_bool (List.assoc "clock_safe" p) in
       let pipeline = Lang.to_string (List.assoc "pipeline" p) in
       let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
       let on_error = List.assoc "on_error" p in
       let on_error error =
         let msg = Printexc.to_string error in
-        Lang.to_float
-          (Lang.apply ~t:Lang.unit_t on_error [("", Lang.string msg)])
+        Lang.to_float (Lang.apply on_error [("", Lang.string msg)])
       in
       let start = Lang.to_bool (List.assoc "start" p) in
       let on_start =
         let f = List.assoc "on_start" p in
-        fun () -> ignore (Lang.apply ~t:Lang.unit_t f [])
+        fun () -> ignore (Lang.apply f [])
       in
       let on_stop =
         let f = List.assoc "on_stop" p in
-        fun () -> ignore (Lang.apply ~t:Lang.unit_t f [])
+        fun () -> ignore (Lang.apply f [])
       in
       let source = List.assoc "" p in
-      ( new output
-          ~kind ~clock_safe ~infallible ~on_error ~on_start ~on_stop source
-          start ("", None, Some pipeline)
-        :> Source.source ))
+      let kind = Source.Kind.of_kind kind in
+      (new output
+         ~kind ~clock_safe ~infallible ~on_error ~on_start ~on_stop source start
+         ("", None, Some pipeline)
+        :> Output.output))
 
 let () =
-  let kind = Lang.any_fixed_with ~audio:1 ~video:1 () in
-  let kind = Lang.kind_type_of_kind_format ~fresh:1 kind in
-  Lang.add_operator "output.gstreamer.audio_video" ~active:true
-    ( output_proto ~kind ~pipeline:""
+  let kind =
+    { Frame.audio = Frame.audio_pcm; video = Frame.video_yuva420p; midi = `Any }
+  in
+  let return_t = Lang.kind_type_of_kind_format kind in
+  Lang.add_operator "output.gstreamer.audio_video"
+    (output_proto ~return_t ~pipeline:""
     @ [
         ( "audio_pipeline",
           Lang.string_t,
@@ -400,10 +408,10 @@ let () =
           Lang.bool_t,
           Some (Lang.bool true),
           Some "Pushing buffers is blocking." );
-      ] )
-    ~category:Lang.Output ~descr:"Output stream to a GStreamer pipeline."
-    ~kind:(Lang.Unconstrained kind)
-    (fun p kind ->
+      ])
+    ~category:`Output ~meth:Output.meth
+    ~descr:"Output stream to a GStreamer pipeline." ~return_t
+    (fun p ->
       let clock_safe = Lang.to_bool (List.assoc "clock_safe" p) in
       let pipeline = Lang.to_string (List.assoc "pipeline" p) in
       let audio_pipeline = Lang.to_string (List.assoc "audio_pipeline" p) in
@@ -412,25 +420,25 @@ let () =
       let on_error = List.assoc "on_error" p in
       let on_error error =
         let msg = Printexc.to_string error in
-        Lang.to_float
-          (Lang.apply ~t:Lang.unit_t on_error [("", Lang.string msg)])
+        Lang.to_float (Lang.apply on_error [("", Lang.string msg)])
       in
       let start = Lang.to_bool (List.assoc "start" p) in
       let blocking = Lang.to_bool (List.assoc "blocking" p) in
       let on_start =
         let f = List.assoc "on_start" p in
-        fun () -> ignore (Lang.apply ~t:Lang.unit_t f [])
+        fun () -> ignore (Lang.apply f [])
       in
       let on_stop =
         let f = List.assoc "on_stop" p in
-        fun () -> ignore (Lang.apply ~t:Lang.unit_t f [])
+        fun () -> ignore (Lang.apply f [])
       in
       let source = List.assoc "" p in
-      ( new output
-          ~kind ~clock_safe ~infallible ~on_error ~on_start ~on_stop ~blocking
-          source start
-          (pipeline, Some audio_pipeline, Some video_pipeline)
-        :> Source.source ))
+      let kind = Source.Kind.of_kind kind in
+      (new output
+         ~kind ~clock_safe ~infallible ~on_error ~on_start ~on_stop ~blocking
+         source start
+         (pipeline, Some audio_pipeline, Some video_pipeline)
+        :> Output.output))
 
 (***** Input *****)
 
@@ -442,11 +450,12 @@ type 'a sink = { pending : unit -> int; pull : unit -> 'a }
 
 class audio_video_input p kind (pipeline, audio_pipeline, video_pipeline) =
   let max = Lang.to_float (List.assoc "max" p) in
-  let max_ticks = Frame.master_of_seconds max in
+  let log_overfull = Lang.to_bool (List.assoc "log_overfull" p) in
+  let max_ticks = Frame.main_of_seconds max in
   let on_error = List.assoc "on_error" p in
   let on_error error =
     let msg = Printexc.to_string error in
-    Lang.to_float (Lang.apply ~t:Lang.unit_t on_error [("", Lang.string msg)])
+    Lang.to_float (Lang.apply on_error [("", Lang.string msg)])
   in
   let restart = Lang.to_bool (List.assoc "restart" p) in
   let content, has_audio, has_video =
@@ -457,38 +466,19 @@ class audio_video_input p kind (pipeline, audio_pipeline, video_pipeline) =
       | None, None ->
           failwith "There should be at least one audio or video pipeline!"
   in
-  let channels = (Frame.type_of_kind kind).Frame.audio in
   let width = Lazy.force Frame.video_width in
   let height = Lazy.force Frame.video_height in
   let rlog = ref (fun _ -> ()) in
-  let gen = Generator.create ~log:(fun x -> !rlog x) ~kind content in
+  let gen = Generator.create ~log_overfull ~log:(fun x -> !rlog x) content in
   object (self)
     inherit Source.source ~name:"input.gstreamer.audio_video" kind as super
-
     inherit [string sink, Gstreamer.data sink] element_factory ~on_error
-
-    initializer
-    (rlog := fun s -> self#log#important "%s" s);
-    let change_state s _ =
-      try
-        Printf.sprintf "Done. State change returned: %s"
-          (string_of_state_change (Element.set_state self#get_element.bin s))
-      with e ->
-        Printf.sprintf "Error while changing state: %s\n" (Printexc.to_string e)
-    in
-    self#register_command "pause"
-      ~descr:"Set gstreamer pipeline state to paused"
-      (change_state Element.State_paused);
-    self#register_command "play"
-      ~descr:"Set gstreamer pipeline state to playing"
-      (change_state Element.State_playing);
-    self#register_command "restart"
-      ~descr:"Restart gstreamer pipeline state to paused" (fun _ ->
-        self#restart;
-        "Done. Task will complete asynchronously.")
-
+    initializer rlog := fun s -> self#log#important "%s" s
+    method set_state s = ignore (Element.set_state self#get_element.bin s)
+    method pause_cmd = self#set_state Element.State_paused
+    method play_cmd = self#set_state Element.State_playing
+    method restart_cmd = self#restart
     method stype = Source.Fallible
-
     method remaining = -1
 
     (* Source is ready when ready = true and gst has some audio or some video. *)
@@ -501,14 +491,15 @@ class audio_video_input p kind (pipeline, audio_pipeline, video_pipeline) =
       in
       try
         ready
-        && ( Generator.length gen > 0
+        && (Generator.length gen > 0
            || pending self#get_element.audio
-           || pending self#get_element.video )
+           || pending self#get_element.video)
       with e ->
         log#info "Error when trying to check if ready: %s"
           (Printexc.to_string e);
         false
 
+    method self_sync = (`Dynamic, self#is_ready)
     method abort_track = ()
 
     method wake_up activations =
@@ -544,15 +535,15 @@ class audio_video_input p kind (pipeline, audio_pipeline, video_pipeline) =
       let pipeline =
         if has_audio then
           Printf.sprintf "%s %s ! %s ! %s" (pipeline ())
-            (Utils.get_some audio_pipeline ())
+            (Option.get audio_pipeline ())
             (GU.Pipeline.decode_audio ())
-            (GU.Pipeline.audio_sink ~channels "audio_sink")
+            (GU.Pipeline.audio_sink ~channels:self#audio_channels "audio_sink")
         else pipeline ()
       in
       let pipeline =
         if has_video then
           Printf.sprintf "%s %s ! %s ! %s" pipeline
-            (Utils.get_some video_pipeline ())
+            (Option.get video_pipeline ())
             (GU.Pipeline.decode_video ())
             (GU.Pipeline.video_sink "video_sink")
         else pipeline
@@ -574,40 +565,44 @@ class audio_video_input p kind (pipeline, audio_pipeline, video_pipeline) =
         { pending; pull }
       in
       let audio_sink =
-        if has_audio then begin
+        if has_audio then (
           let sink = App_sink.of_element (Bin.get_by_name bin "audio_sink") in
-          Some (wrap_sink sink Gstreamer.App_sink.pull_buffer_string)
-        end
+          Some (wrap_sink sink Gstreamer.App_sink.pull_buffer_string))
         else None
       in
       let video_sink =
-        if has_video then begin
+        if has_video then (
           let sink = App_sink.of_element (Bin.get_by_name bin "video_sink") in
-          Some (wrap_sink sink Gstreamer.App_sink.pull_buffer_data)
-        end
+          Some (wrap_sink sink Gstreamer.App_sink.pull_buffer_data))
         else None
       in
       { bin; audio = audio_sink; video = video_sink }
 
-    method is_active = true
-
     method private is_generator_at_max = Generator.length gen >= max_ticks
 
     method private fill_audio audio =
       while audio.pending () > 0 && not self#is_generator_at_max do
         let b = audio.pull () in
-        let len = String.length b / (2 * channels) in
-        let buf = Audio.create channels len in
-        Audio.S16LE.to_audio b 0 buf 0 len;
-        Generator.put_audio gen buf 0 len
+        let len = String.length b / (2 * self#audio_channels) in
+        let buf = Audio.create self#audio_channels len in
+        Audio.S16LE.to_audio b 0 (Audio.sub buf 0 len);
+        Generator.put_audio gen
+          (Frame_content.Audio.lift_data buf)
+          0 (Frame.main_of_audio len)
       done
 
     method private fill_video video =
       while video.pending () > 0 && not self#is_generator_at_max do
         let b = video.pull () in
-        let img = Img.make width height b in
-        let stream = [| img |] in
-        Generator.put_video gen [| stream |] 0 (Array.length stream)
+        let img =
+          Image.YUV420.make_data width height b (Image.Data.round 4 width)
+            (Image.Data.round 4 (width / 2))
+        in
+        let stream = Video.single img in
+        Generator.put_video gen
+          (Frame_content.Video.lift_data stream)
+          0
+          (Frame.main_of_video (Video.length stream))
       done
 
     method get_frame frame =
@@ -629,11 +624,12 @@ class audio_video_input p kind (pipeline, audio_pipeline, video_pipeline) =
             ready <- false;
             if restart then (
               self#log#info "Restarting.";
-              self#restart )
+              self#restart)
         | exn ->
-            self#log#important "Error while processing input data: %s"
-              (Printexc.to_string exn);
-            self#log#info "Stacktrace: %s" (Printexc.get_backtrace ());
+            let bt = Printexc.get_backtrace () in
+            Utils.log_exception ~log:self#log ~bt
+              (Printf.sprintf "Error while processing input data: %s"
+                 (Printexc.to_string exn));
             self#on_error exn
   end
 
@@ -641,10 +637,10 @@ let input_proto =
   [
     ( "on_error",
       Lang.fun_t [(false, "", Lang.string_t)] Lang.float_t,
-      Some (Lang.val_cst_fun [("", Lang.string_t, None)] (Lang.float 3.)),
+      Some (Lang.val_cst_fun [("", None)] (Lang.float 3.)),
       Some
         "Callback executed when an error happens. The callback receives a \
-         string representation of the error that occured and returns a float. \
+         string representation of the error that occurred and returns a float. \
          If returned value is positive, connection will be tried again after \
          this amount of time (in seconds)." );
     ( "restart",
@@ -655,38 +651,63 @@ let input_proto =
       Lang.float_t,
       Some (Lang.float 10.),
       Some "Maximum duration of the buffered data." );
+    ( "log_overfull",
+      Lang.bool_t,
+      Some (Lang.bool true),
+      Some "Log when the source's buffer is overfull." );
   ]
 
 let () =
-  let k =
-    Lang.kind_type_of_kind_format ~fresh:1
-      (Lang.Constrained
-         {
-           Frame.audio (* TODO: be more flexible on audio *) = Lang.Fixed 2;
-           video = Lang.Fixed 1;
-           midi = Lang.Fixed 0;
-         })
+  let kind =
+    (* TODO: be more flexible on audio *)
+    Frame.{ audio = audio_stereo; video = video_yuva420p; midi = none }
   in
+  let return_t = Lang.kind_type_of_kind_format kind in
   let proto =
     input_proto
     @ [
         ( "pipeline",
-          Lang.string_getter_t 1,
+          Lang.getter_t Lang.string_t,
           Some (Lang.string ""),
           Some "Main GStreamer pipeline." );
         ( "audio_pipeline",
-          Lang.string_getter_t 2,
+          Lang.getter_t Lang.string_t,
           Some (Lang.string "audiotestsrc"),
           Some "Audio pipeline to input from." );
         ( "video_pipeline",
-          Lang.string_getter_t 3,
+          Lang.getter_t Lang.string_t,
           Some (Lang.string "videotestsrc"),
           Some "Video pipeline to input from." );
       ]
   in
-  Lang.add_operator "input.gstreamer.audio_video" proto
-    ~kind:(Lang.Unconstrained k) ~category:Lang.Input ~flags:[]
-    ~descr:"Stream audio+video from a GStreamer pipeline." (fun p kind ->
+  Lang.add_operator "input.gstreamer.audio_video" proto ~return_t
+    ~category:`Input ~flags:[]
+    ~meth:
+      [
+        ( "pause",
+          ([], Lang.fun_t [] Lang.unit_t),
+          "Pause input.",
+          fun s ->
+            Lang.val_fun [] (fun _ ->
+                s#pause_cmd;
+                Lang.unit) );
+        ( "play",
+          ([], Lang.fun_t [] Lang.unit_t),
+          "Play input.",
+          fun s ->
+            Lang.val_fun [] (fun _ ->
+                s#play_cmd;
+                Lang.unit) );
+        ( "restart",
+          ([], Lang.fun_t [] Lang.unit_t),
+          "Restart input.",
+          fun s ->
+            Lang.val_fun [] (fun _ ->
+                s#restart_cmd;
+                Lang.unit) );
+      ]
+    ~descr:"Stream audio+video from a GStreamer pipeline."
+    (fun p ->
       let pipeline = Lang.to_string_getter (List.assoc "pipeline" p) in
       let audio_pipeline =
         Lang.to_string_getter (List.assoc "audio_pipeline" p)
@@ -694,43 +715,45 @@ let () =
       let video_pipeline =
         Lang.to_string_getter (List.assoc "video_pipeline" p)
       in
-      ( new audio_video_input
-          p kind
-          (pipeline, Some audio_pipeline, Some video_pipeline)
-        :> Source.source ))
+      let kind = Source.Kind.of_kind kind in
+      new audio_video_input
+        p kind
+        (pipeline, Some audio_pipeline, Some video_pipeline))
 
 let () =
-  let k = Lang.kind_type_of_kind_format ~fresh:1 Lang.audio_any in
+  let kind = Lang.audio_pcm in
+  let return_t = Lang.kind_type_of_kind_format kind in
   let proto =
     input_proto
     @ [
         ( "pipeline",
-          Lang.string_getter_t 1,
+          Lang.getter_t Lang.string_t,
           Some (Lang.string "audiotestsrc"),
           Some "GStreamer pipeline to input from." );
       ]
   in
-  Lang.add_operator "input.gstreamer.audio" proto ~kind:(Lang.Unconstrained k)
-    ~category:Lang.Input ~flags:[]
-    ~descr:"Stream audio from a GStreamer pipeline." (fun p kind ->
+  Lang.add_operator "input.gstreamer.audio" proto ~return_t ~category:`Input
+    ~flags:[] ~descr:"Stream audio from a GStreamer pipeline." (fun p ->
       let pipeline = Lang.to_string_getter (List.assoc "pipeline" p) in
-      ( new audio_video_input p kind ((fun () -> ""), Some pipeline, None)
-        :> Source.source ))
+      let kind = Source.Kind.of_kind kind in
+      (new audio_video_input p kind ((fun () -> ""), Some pipeline, None)
+        :> Source.source))
 
 let () =
-  let k = Lang.kind_type_of_kind_format ~fresh:1 (Lang.video_n 1) in
+  let kind = Lang.video_yuva420p in
+  let return_t = Lang.kind_type_of_kind_format kind in
   let proto =
     input_proto
     @ [
         ( "pipeline",
-          Lang.string_getter_t 1,
+          Lang.getter_t Lang.string_t,
           Some (Lang.string "videotestsrc"),
           Some "GStreamer pipeline to input from." );
       ]
   in
-  Lang.add_operator "input.gstreamer.video" proto ~kind:(Lang.Unconstrained k)
-    ~category:Lang.Input ~flags:[]
-    ~descr:"Stream video from a GStreamer pipeline." (fun p kind ->
+  Lang.add_operator "input.gstreamer.video" proto ~return_t ~category:`Input
+    ~flags:[] ~descr:"Stream video from a GStreamer pipeline." (fun p ->
       let pipeline = Lang.to_string_getter (List.assoc "pipeline" p) in
-      ( new audio_video_input p kind ((fun () -> ""), None, Some pipeline)
-        :> Source.source ))
+      let kind = Source.Kind.of_kind kind in
+      (new audio_video_input p kind ((fun () -> ""), None, Some pipeline)
+        :> Source.source))
