@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
 
  *****************************************************************************)
 
+open Mm
+
 external set_format : Unix.file_descr -> int -> int = "caml_oss_dsp_setfmt"
 external set_channels : Unix.file_descr -> int -> int = "caml_oss_dsp_channels"
 external set_rate : Unix.file_descr -> int -> int = "caml_oss_dsp_speed"
@@ -32,18 +34,17 @@ let force f fd x =
   if x <> x' then failwith "cannot obtain desired OSS settings"
 
 (** Dedicated clock. *)
-let get_clock = Tutils.lazy_cell (fun () -> new Clock.self_sync "OSS")
+let get_clock = Tutils.lazy_cell (fun () -> new Clock.clock "OSS")
 
 class output ~kind ~clock_safe ~on_start ~on_stop ~infallible ~start dev
   val_source =
-  let channels = (Frame.type_of_kind kind).Frame.audio in
   let samples_per_second = Lazy.force Frame.audio_rate in
   let name = Printf.sprintf "oss_out(%s)" dev in
   object (self)
     inherit
       Output.output
-        ~infallible ~on_stop ~on_start ~content_kind:kind ~name
-          ~output_kind:"output.oss" val_source start as super
+        ~infallible ~on_stop ~on_start ~content_kind:(Source.Kind.of_kind kind)
+          ~name ~output_kind:"output.oss" val_source start as super
 
     method private set_clock =
       super#set_clock;
@@ -52,12 +53,13 @@ class output ~kind ~clock_safe ~on_start ~on_stop ~infallible ~start dev
           (Clock.create_known (get_clock () :> Clock.clock))
 
     val mutable fd = None
+    method self_sync = (`Dynamic, fd <> None)
 
     method open_device =
-      let descr = Unix.openfile dev [Unix.O_WRONLY] 0o200 in
+      let descr = Unix.openfile dev [Unix.O_WRONLY; Unix.O_CLOEXEC] 0o200 in
       fd <- Some descr;
       force set_format descr 16;
-      force set_channels descr channels;
+      force set_channels descr self#audio_channels;
       force set_rate descr samples_per_second
 
     method close_device =
@@ -67,89 +69,65 @@ class output ~kind ~clock_safe ~on_start ~on_stop ~infallible ~start dev
             Unix.close x;
             fd <- None
 
-    method output_start =
-      if clock_safe then (get_clock ())#register_blocking_source;
-      self#open_device
+    method start = self#open_device
+    method stop = self#close_device
 
-    method output_stop =
-      if clock_safe then (get_clock ())#unregister_blocking_source;
-      self#close_device
-
-    method output_reset =
-      self#close_device;
-      self#open_device
-
-    method output_send memo =
-      let fd = Utils.get_some fd in
-      let buf = AFrame.content memo 0 in
-      let r = Audio.S16LE.length (Audio.channels buf) (Audio.duration buf) in
+    method send_frame memo =
+      let fd = Option.get fd in
+      let buf = AFrame.pcm memo in
+      let r = Audio.S16LE.size (Audio.channels buf) (Audio.length buf) in
       let s = Bytes.create r in
-      Audio.S16LE.of_audio buf 0 s 0 (Audio.duration buf);
+      Audio.S16LE.of_audio buf s 0;
       let w = Unix.write fd s 0 r in
       assert (w = r)
   end
 
 class input ~kind ~clock_safe ~start ~on_stop ~on_start ~fallible dev =
-  let channels = (Frame.type_of_kind kind).Frame.audio in
   let samples_per_second = Lazy.force Frame.audio_rate in
   object (self)
     inherit
-      Start_stop.input
-        ~content_kind:kind ~source_kind:"oss"
+      Start_stop.active_source
+        ~get_clock ~clock_safe ~content_kind:(Source.Kind.of_kind kind)
         ~name:(Printf.sprintf "oss_in(%s)" dev)
-        ~on_start ~on_stop ~fallible ~autostart:start as super
-
-    method private set_clock =
-      super#set_clock;
-      if clock_safe then
-        Clock.unify self#clock
-          (Clock.create_known (get_clock () :> Clock.clock))
+        ~on_start ~on_stop ~fallible ~autostart:start ()
 
     val mutable fd = None
-
-    method private start =
-      if clock_safe then (get_clock ())#register_blocking_source;
-      self#open_device
+    method self_sync = (`Dynamic, fd <> None)
+    method abort_track = ()
+    method remaining = -1
+    method private start = self#open_device
 
     method private open_device =
-      let descr = Unix.openfile dev [Unix.O_RDONLY] 0o400 in
+      let descr = Unix.openfile dev [Unix.O_RDONLY; Unix.O_CLOEXEC] 0o400 in
       fd <- Some descr;
       force set_format descr 16;
-      force set_channels descr channels;
+      force set_channels descr self#audio_channels;
       force set_rate descr samples_per_second
 
-    method private stop =
-      if clock_safe then (get_clock ())#unregister_blocking_source;
-      self#close_device
+    method private stop = self#close_device
 
     method private close_device =
-      Unix.close (Utils.get_some fd);
+      Unix.close (Option.get fd);
       fd <- None
 
-    method output_reset =
-      self#close_device;
-      self#open_device
-
-    method input frame =
+    method get_frame frame =
       assert (0 = AFrame.position frame);
-      let fd = Utils.get_some fd in
-      let buf = AFrame.content_of_type ~channels frame 0 in
-      let len = 2 * Array.length buf * Array.length buf.(0) in
+      let fd = Option.get fd in
+      let buf = AFrame.pcm frame in
+      let len = 2 * Array.length buf * Audio.Mono.length buf.(0) in
       let s = Bytes.create len in
       let r = Unix.read fd s 0 len in
       (* TODO: recursive read ? *)
       assert (len = r);
-      Audio.S16LE.to_audio (Bytes.unsafe_to_string s) 0 buf 0
-        (Audio.duration buf);
+      Audio.S16LE.to_audio (Bytes.unsafe_to_string s) 0 buf;
       AFrame.add_break frame (AFrame.size ())
   end
 
 let () =
-  let k =
-    Lang.kind_type_of_kind_format ~fresh:1 (Lang.any_fixed_with ~audio:1 ())
-  in
-  Lang.add_operator "output.oss" ~active:true
-    ( Output.proto
+  let kind = Lang.audio_pcm in
+  let k = Lang.kind_type_of_kind_format kind in
+  Lang.add_operator "output.oss"
+    (Output.proto
     @ [
         ( "clock_safe",
           Lang.bool_t,
@@ -160,43 +138,39 @@ let () =
           Some (Lang.string "/dev/dsp"),
           Some "OSS device to use." );
         ("", Lang.source_t k, None, None);
-      ] )
-    ~kind:(Lang.Unconstrained k) ~category:Lang.Output
+      ])
+    ~return_t:k ~category:`Output ~meth:Output.meth
     ~descr:"Output the source's stream to an OSS output device."
-    (fun p kind ->
+    (fun p ->
       let e f v = f (List.assoc v p) in
       let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
       let start = Lang.to_bool (List.assoc "start" p) in
       let on_start =
         let f = List.assoc "on_start" p in
-        fun () -> ignore (Lang.apply ~t:Lang.unit_t f [])
+        fun () -> ignore (Lang.apply f [])
       in
       let on_stop =
         let f = List.assoc "on_stop" p in
-        fun () -> ignore (Lang.apply ~t:Lang.unit_t f [])
+        fun () -> ignore (Lang.apply f [])
       in
       let clock_safe = e Lang.to_bool "clock_safe" in
       let device = e Lang.to_string "device" in
       let source = List.assoc "" p in
-      ( new output
-          ~start ~on_start ~on_stop ~infallible ~kind ~clock_safe device source
-        :> Source.source ));
-  let k = Lang.kind_type_of_kind_format ~fresh:1 Lang.audio_any in
-  Lang.add_operator "input.oss" ~active:true
-    ( Start_stop.input_proto
+      (new output
+         ~start ~on_start ~on_stop ~infallible ~kind ~clock_safe device source
+        :> Output.output));
+  let k = Lang.kind_type_of_kind_format Lang.audio_pcm in
+  Lang.add_operator "input.oss"
+    (Start_stop.active_source_proto ~clock_safe:true ~fallible_opt:(`Yep false)
     @ [
-        ( "clock_safe",
-          Lang.bool_t,
-          Some (Lang.bool true),
-          Some "Force the use of the dedicated OSS clock." );
         ( "device",
           Lang.string_t,
           Some (Lang.string "/dev/dsp"),
           Some "OSS device to use." );
-      ] )
-    ~kind:(Lang.Unconstrained k) ~category:Lang.Input
+      ])
+    ~meth:(Start_stop.meth ()) ~return_t:k ~category:`Input
     ~descr:"Stream from an OSS input device."
-    (fun p kind ->
+    (fun p ->
       let e f v = f (List.assoc v p) in
       let clock_safe = e Lang.to_bool "clock_safe" in
       let device = e Lang.to_string "device" in
@@ -204,11 +178,10 @@ let () =
       let fallible = Lang.to_bool (List.assoc "fallible" p) in
       let on_start =
         let f = List.assoc "on_start" p in
-        fun () -> ignore (Lang.apply ~t:Lang.unit_t f [])
+        fun () -> ignore (Lang.apply f [])
       in
       let on_stop =
         let f = List.assoc "on_stop" p in
-        fun () -> ignore (Lang.apply ~t:Lang.unit_t f [])
+        fun () -> ignore (Lang.apply f [])
       in
-      ( new input ~kind ~start ~on_start ~on_stop ~fallible ~clock_safe device
-        :> Source.source ))
+      new input ~kind ~start ~on_start ~on_stop ~fallible ~clock_safe device)
