@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,10 +20,11 @@
 
  *****************************************************************************)
 
+open Mm
 open Pulseaudio
 
 (** Dedicated clock. *)
-let get_clock = Tutils.lazy_cell (fun () -> new Clock.self_sync "pulse")
+let get_clock = Tutils.lazy_cell (fun () -> new Clock.clock "pulseaudio")
 
 (** Error translator *)
 let error_translator e =
@@ -39,10 +40,9 @@ class virtual base ~client ~device =
   let device = if device = "" then None else Some device in
   object
     val client_name = client
-
     val dev = device
-
     method virtual log : Log.t
+    method self_sync : Source.self_sync = (`Dynamic, dev <> None)
   end
 
 class output ~infallible ~start ~on_start ~on_stop ~kind p =
@@ -50,7 +50,6 @@ class output ~infallible ~start ~on_start ~on_stop ~kind p =
   let device = Lang.to_string (List.assoc "device" p) in
   let name = Printf.sprintf "pulse_out(%s:%s)" client device in
   let val_source = List.assoc "" p in
-  let channels = (Frame.type_of_kind kind).Frame.audio in
   let samples_per_second = Lazy.force Frame.audio_rate in
   let clock_safe = Lang.to_bool (List.assoc "clock_safe" p) in
   object (self)
@@ -74,7 +73,7 @@ class output ~infallible ~start ~on_start ~on_stop ~kind p =
         {
           sample_format = Sample_format_float32le;
           sample_rate = samples_per_second;
-          sample_chans = channels;
+          sample_chans = self#audio_channels;
         }
       in
       stream <-
@@ -89,22 +88,30 @@ class output ~infallible ~start ~on_start ~on_stop ~kind p =
             Pulseaudio.Simple.free s;
             stream <- None
 
-    method output_start =
-      if clock_safe then (get_clock ())#register_blocking_source;
-      self#open_device
+    method start = self#open_device
+    method stop = self#close_device
 
-    method output_stop =
-      if clock_safe then (get_clock ())#unregister_blocking_source;
-      self#close_device
-
-    method output_reset =
+    method reset =
       self#close_device;
       self#open_device
 
-    method output_send memo =
-      let stream = Utils.get_some stream in
-      let buf = AFrame.content memo 0 in
-      Simple.write stream buf 0 (Array.length buf.(0))
+    method send_frame memo =
+      let stream = Option.get stream in
+      let buf = AFrame.pcm memo in
+      let chans = Array.length buf in
+      let len = Audio.length buf in
+      let buf' =
+        Bigarray.Array1.create Bigarray.float32 Bigarray.c_layout (chans * len)
+      in
+      for c = 0 to chans - 1 do
+        let bufc = buf.(c) in
+        for i = 0 to len - 1 do
+          Bigarray.Array1.unsafe_set buf'
+            ((i * chans) + c)
+            (Bigarray.Array1.unsafe_get bufc i)
+        done
+      done;
+      Simple.write_ba stream buf'
   end
 
 class input ~kind p =
@@ -115,49 +122,32 @@ class input ~kind p =
   let fallible = Lang.to_bool (List.assoc "fallible" p) in
   let on_start =
     let f = List.assoc "on_start" p in
-    fun () -> ignore (Lang.apply ~t:Lang.unit_t f [])
+    fun () -> ignore (Lang.apply f [])
   in
   let on_stop =
     let f = List.assoc "on_stop" p in
-    fun () -> ignore (Lang.apply ~t:Lang.unit_t f [])
+    fun () -> ignore (Lang.apply f [])
   in
-  let channels = (Frame.type_of_kind kind).Frame.audio in
   let samples_per_second = Lazy.force Frame.audio_rate in
   object (self)
     inherit
-      Start_stop.input
-        ~content_kind:kind ~source_kind:"pulse"
-        ~name:(Printf.sprintf "pulse_in(%s)" device)
-        ~on_start ~on_stop ~autostart:start ~fallible as super
+      Start_stop.active_source
+        ~get_clock ~name:"input.pulseaudio" ~content_kind:kind ~clock_safe
+          ~on_start ~on_stop ~autostart:start ~fallible ()
 
     inherit base ~client ~device
-
-    method private set_clock =
-      super#set_clock;
-      if clock_safe then
-        Clock.unify self#clock
-          (Clock.create_known (get_clock () :> Clock.clock))
-
-    method private start =
-      if clock_safe then (get_clock ())#register_blocking_source;
-      self#open_device
-
-    method private stop =
-      if clock_safe then (get_clock ())#register_blocking_source;
-      self#close_device
-
-    method output_reset =
-      self#close_device;
-      self#open_device
-
+    method private start = self#open_device
+    method private stop = self#close_device
     val mutable stream = None
+    method remaining = -1
+    method abort_track = ()
 
     method private open_device =
       let ss =
         {
           sample_format = Sample_format_float32le;
           sample_rate = samples_per_second;
-          sample_chans = channels;
+          sample_chans = self#audio_channels;
         }
       in
       stream <-
@@ -166,21 +156,32 @@ class input ~kind p =
              ~dir:Dir_record ?dev ~sample:ss ())
 
     method private close_device =
-      Pulseaudio.Simple.free (Utils.get_some stream);
+      Pulseaudio.Simple.free (Option.get stream);
       stream <- None
 
-    method input frame =
+    method get_frame frame =
       assert (0 = AFrame.position frame);
-      let stream = Utils.get_some stream in
-      let buf = AFrame.content_of_type ~channels frame 0 in
-      Simple.read stream buf 0 (Array.length buf.(0));
+      let stream = Option.get stream in
+      let len = AFrame.size () in
+      let ibuf =
+        Bigarray.Array1.create Bigarray.float32 Bigarray.c_layout
+          (self#audio_channels * len)
+      in
+      let buf = AFrame.pcm frame in
+      Simple.read_ba stream ibuf;
+      for c = 0 to self#audio_channels - 1 do
+        let bufc = buf.(c) in
+        for i = 0 to len - 1 do
+          bufc.{i} <-
+            Bigarray.Array1.unsafe_get ibuf ((i * self#audio_channels) + c)
+        done
+      done;
       AFrame.add_break frame (AFrame.size ())
   end
 
 let () =
-  let k =
-    Lang.kind_type_of_kind_format ~fresh:1 (Lang.any_fixed_with ~audio:1 ())
-  in
+  let kind = Lang.audio_pcm in
+  let k = Lang.kind_type_of_kind_format kind in
   let proto =
     [
       ("client", Lang.string_t, Some (Lang.string "liquidsoap"), None);
@@ -194,24 +195,29 @@ let () =
         Some "Force the use of the dedicated Pulseaudio clock." );
     ]
   in
-  Lang.add_operator "output.pulseaudio" ~active:true
+  Lang.add_operator "output.pulseaudio"
     (Output.proto @ proto @ [("", Lang.source_t k, None, None)])
-    ~kind:(Lang.Unconstrained k) ~category:Lang.Output
+    ~return_t:k ~category:`Output ~meth:Output.meth
     ~descr:"Output the source's stream to a portaudio output device."
-    (fun p kind ->
+    (fun p ->
       let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
       let start = Lang.to_bool (List.assoc "start" p) in
       let on_start =
         let f = List.assoc "on_start" p in
-        fun () -> ignore (Lang.apply ~t:Lang.unit_t f [])
+        fun () -> ignore (Lang.apply f [])
       in
       let on_stop =
         let f = List.assoc "on_stop" p in
-        fun () -> ignore (Lang.apply ~t:Lang.unit_t f [])
+        fun () -> ignore (Lang.apply f [])
       in
-      (new output ~infallible ~on_start ~on_stop ~start ~kind p :> Source.source));
-  Lang.add_operator "input.pulseaudio" ~active:true
-    (Start_stop.input_proto @ proto)
-    ~kind:(Lang.Unconstrained k) ~category:Lang.Input
-    ~descr:"Stream from a portaudio input device." (fun p kind ->
-      (new input ~kind p :> Source.source))
+      let kind = Source.Kind.of_kind kind in
+      (new output ~infallible ~on_start ~on_stop ~start ~kind p
+        :> Output.output));
+  Lang.add_operator "input.pulseaudio"
+    (Start_stop.active_source_proto ~clock_safe:true ~fallible_opt:(`Yep false)
+    @ proto)
+    ~return_t:k ~category:`Input ~meth:(Start_stop.meth ())
+    ~descr:"Stream from a portaudio input device."
+    (fun p ->
+      let kind = Source.Kind.of_kind kind in
+      new input ~kind p)
