@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,15 +20,14 @@
 
  *****************************************************************************)
 
-open Lang_builtins
-
+let () = Lang.add_module "icy"
 let log = Log.make ["icy"; "update_metadata"]
 
 let () =
   let user_agent =
     Lang.product (Lang.string "User-Agent") (Lang.string Http.user_agent)
   in
-  add_builtin "icy.update_metadata" ~cat:Interaction
+  Lang.add_builtin "icy.update_metadata" ~category:`Interaction
     ~descr:"Update metata on an icecast mountpoint using the ICY protocol."
     [
       ("host", Lang.string_t, Some (Lang.string "localhost"), None);
@@ -48,12 +47,12 @@ let () =
         Some (Lang.string "http"),
         Some "Protocol to use. One of: \"icy\", \"http\" or \"https\"" );
       ( "encoding",
-        Lang.string_t,
-        Some (Lang.string ""),
-        Some "Encoding used to send metadata, default (UTF-8) if empty." );
+        Lang.nullable_t Lang.string_t,
+        Some Lang.null,
+        Some "Encoding used to send metadata, default (UTF-8) if null." );
       ( "headers",
         Lang.metadata_t,
-        Some (Lang.list ~t:Lang.metadata_t [user_agent]),
+        Some (Lang.list [user_agent]),
         Some "Additional headers." );
       ("", Lang.metadata_t, None, None);
     ]
@@ -65,9 +64,7 @@ let () =
       let icy_id = Lang.to_int (List.assoc "icy_id" p) in
       let metas = Lang.to_metadata (Lang.assoc "" 1 p) in
       let out_enc =
-        match Lang.to_string (List.assoc "encoding" p) with
-          | "" -> None
-          | s -> Some s
+        Lang.to_valued_option Lang.to_string (List.assoc "encoding" p)
       in
       let metas =
         let ret = Hashtbl.create (Hashtbl.length metas) in
@@ -98,7 +95,7 @@ let () =
           | "https" -> Cry.Https Cry.Source
           | _ ->
               raise
-                (Lang_errors.Invalid_value
+                (Error.Invalid_value
                    (v, "protocol should be one of: 'icy', 'http' or 'https'."))
       in
       let mount =
