@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,8 +20,7 @@
 
  *****************************************************************************)
 
-open Lang_builtins
-
+let () = Lang.add_module "audioscrobbler"
 let log = Log.make ["lastfm"; "submit"]
 
 let () =
@@ -55,15 +54,15 @@ let () =
           Some (Lang.string "broadcast"),
           Some
             "Source for tracks. Should be one of: \"broadcast\", \"user\", \
-             \"recommendation\" or \"unknown\". Since liquidsoap is intented \
+             \"recommendation\" or \"unknown\". Since liquidsoap is intended \
              for radio broadcasting, this is the default. Sources other than \
              user don't need duration to be set." )
         :: proto
       else proto
     in
     let tasks = Hashtbl.create 1 in
-    add_builtin name ~cat:Interaction (* TODO better cat *) ~descr proto
-      Lang.unit_t (fun p ->
+    Lang.add_builtin name ~category:`Interaction (* TODO better cat *) ~descr
+      proto Lang.unit_t (fun p ->
         let user = Lang.to_string (List.assoc "user" p) in
         let password = Lang.to_string (List.assoc "password" p) in
         let metas = Lang.to_metadata (Lang.assoc "" 1 p) in
@@ -79,19 +78,17 @@ let () =
               | "unknown" -> Liqfm.Unknown
               | _ ->
                   raise
-                    (Lang_errors.Invalid_value
-                       (List.assoc "source" p, "unknown lastfm submission mode"))
-            )
+                    (Error.Invalid_value
+                       (List.assoc "source" p, "unknown lastfm submission mode")))
           else Liqfm.Unknown
         in
         let length = Lang.to_bool (List.assoc "length" p) in
         let length =
-          if length = false && mode = Liqfm.User then begin
+          if length = false && mode = Liqfm.User then (
             log#severe
               "length information is required for \"user\" sources, setting to \
                true.";
-            true
-          end
+            true)
           else length
         in
         let task =
