@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,9 @@
  *****************************************************************************)
 
 open Extralib
-module S = LO.Server
+module S = Lo.Server
+
+let () = Lang.add_module "osc"
 
 let conf_osc =
   Dtools.Conf.void
@@ -35,7 +37,7 @@ let conf_port =
 let handlers = ref []
 let add_handler path t f = handlers := ((path, t), f) :: !handlers
 
-let handler path (data : LO.Message.data array) =
+let handler path (data : Lo.Message.data array) =
   let typ = function
     | `Float _ | `Double _ -> `Float
     | `Int32 _ | `Int64 _ -> `Int
@@ -57,36 +59,60 @@ let handler path (data : LO.Message.data array) =
     List.iter (fun f -> f v) h
   with _ -> ()
 
+(* We have to start the server _after_
+   daemonizing. See: savonet/liquidsoap#1365.
+
+   There are two cases:
+   - Server is requested before we daemonize (i.e.
+     in a top-level call), start it after daemonization.
+   - Server is requested after we daemonize (i.e.
+     in a callback), start it immediately. *)
 let server = ref None
+let should_start = ref false
+let started = ref false
+let started_m = Mutex.create ()
 let log = Log.make ["lo"]
 
 let start_server () =
-  if !server = None then (
-    let port = conf_port#get in
-    let s = S.create port handler in
-    server := Some s;
-    ignore
-      (Thread.create
-         (fun () ->
-           try
-             while true do
-               S.recv s
-             done
-           with
-             | LO.Server.Stopped -> ()
-             | exn ->
-                 let backtrace = Printexc.get_backtrace () in
-                 log#important "LO server thread exited with exception: %s\n%s"
-                   (Printexc.to_string exn) backtrace)
-         ()))
+  log#info "Starting OSC server";
+  let port = conf_port#get in
+  let s = S.create port handler in
+  server := Some s;
+  ignore
+    (Thread.create
+       (fun () ->
+         try
+           while true do
+             S.recv s
+           done
+         with
+           | Lo.Server.Stopped -> ()
+           | exn ->
+               let backtrace = Printexc.get_backtrace () in
+               log#important "OSC server thread exited with exception: %s\n%s"
+                 (Printexc.to_string exn) backtrace)
+       ())
 
-let _ =
-  Dtools.Init.make ~before:[Tutils.scheduler_shutdown_atom] (fun () ->
-      match !server with
-        | Some s ->
-            S.stop s;
-            server := None
-        | None -> ())
+let () =
+  Lifecycle.on_start
+    (Tutils.mutexify started_m (fun () ->
+         if !should_start && !server = None then start_server ()
+         else started := true))
+
+let () =
+  Lifecycle.on_core_shutdown
+    (Tutils.mutexify started_m (fun () ->
+         match !server with
+           | Some s ->
+               log#info "Stopping OSC server";
+               S.stop s;
+               server := None
+           | None -> ()))
+
+let start_server =
+  Tutils.mutexify started_m (fun () ->
+      if !started && !server = None then start_server ()
+      else should_start := true)
 
 let register name osc_t liq_t =
   let val_array vv =
@@ -95,19 +121,19 @@ let register name osc_t liq_t =
       | 2 -> Lang.product vv.(0) vv.(1)
       | _ -> assert false
   in
-  Lang.add_builtin ("osc." ^ name) ~category:"Interaction"
+  Lang.add_builtin ("osc." ^ name) ~category:`Interaction
     [
       ("", Lang.string_t, None, Some "OSC path.");
       ("", liq_t, None, Some "Initial value.");
-    ] (Lang.fun_t [] liq_t) ~descr:"Read from an OSC path." (fun p _ ->
+    ] (Lang.fun_t [] liq_t) ~descr:"Read from an OSC path." (fun p ->
       let path = Lang.to_string (Lang.assoc "" 1 p) in
       let v = Lang.assoc "" 2 p in
       let v = ref v in
       let handle vv = v := val_array vv in
       add_handler path osc_t handle;
       start_server ();
-      Lang.val_fun [] ~ret_t:liq_t (fun _ _ -> !v));
-  Lang.add_builtin ("osc.on_" ^ name) ~category:"Interaction"
+      Lang.val_fun [] (fun _ -> !v));
+  Lang.add_builtin ("osc.on_" ^ name) ~category:`Interaction
     [
       ("", Lang.string_t, None, Some "OSC path.");
       ( "",
@@ -116,38 +142,38 @@ let register name osc_t liq_t =
         Some "Callback function." );
     ]
     Lang.unit_t ~descr:"Register a callback on OSC messages."
-    (fun p _ ->
+    (fun p ->
       let path = Lang.to_string (Lang.assoc "" 1 p) in
       let f = Lang.assoc "" 2 p in
       let handle v =
         let v = val_array v in
-        ignore (Lang.apply ~t:Lang.unit_t f [("", v)])
+        ignore (Lang.apply f [("", v)])
       in
       add_handler path osc_t handle;
       start_server ();
       Lang.unit);
-  Lang.add_builtin ("osc.send_" ^ name) ~category:"Interaction"
+  Lang.add_builtin ("osc.send_" ^ name) ~category:`Interaction
     [
       ("host", Lang.string_t, None, Some "OSC client address.");
       ("port", Lang.int_t, None, Some "OSC client port.");
       ("", Lang.string_t, None, Some "OSC path.");
       ("", liq_t, None, Some "Value to send.");
-    ] Lang.unit_t ~descr:"Send a value to an OSC client." (fun p _ ->
+    ] Lang.unit_t ~descr:"Send a value to an OSC client." (fun p ->
       let host = Lang.to_string (List.assoc "host" p) in
       let port = Lang.to_int (List.assoc "port" p) in
       let path = Lang.to_string (Lang.assoc "" 1 p) in
       let v = Lang.assoc "" 2 p in
-      let address = LO.Address.create host port in
+      let address = Lo.Address.create host port in
       let osc_val v =
         match v.Lang.value with
-          | Lang.Bool b -> if b then [`True] else [`False]
-          | Lang.String s -> [`String s]
-          | Lang.Float x -> [`Float x]
+          | Lang.(Ground (Ground.Bool b)) -> if b then [`True] else [`False]
+          | Lang.(Ground (Ground.String s)) -> [`String s]
+          | Lang.(Ground (Ground.Float x)) -> [`Float x]
           | _ -> failwith "Unhandled value."
       in
       (* There was a bug in early versions of lo bindings and anyway we don't
          really want errors to show up here... *)
-      (try LO.send address path (osc_val v) with _ -> ());
+      (try Lo.send address path (osc_val v) with _ -> ());
       Lang.unit)
 
 let () =
