@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,19 +20,15 @@
 
  *****************************************************************************)
 
-open Lang_values
-
 (** Errors *)
 
-exception Error of (term * string)
+exception Error of (Term.t * string)
 
-let invalid t =
-  match t.term with Int _ | Bool _ | Float _ | String _ -> false | _ -> true
+let invalid t = match t.Term.term with Term.Ground _ -> false | _ -> true
 
 let generic_error t : exn =
   if invalid t then (
-    match t.term with
-      | Var _ -> Error (t, "variables are forbidden in encoding formats")
-      | _ -> Error (t, "complex expressions are forbidden in encoding formats")
-    )
+    match t.Term.term with
+      | Term.Var _ -> Error (t, "variables are forbidden in encoding formats")
+      | _ -> Error (t, "complex expressions are forbidden in encoding formats"))
   else Error (t, "unknown parameter name or invalid parameter value")
