@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,23 +20,25 @@
 
  *****************************************************************************)
 
-open Lang_values
+open Term
+open Term.Ground
 open Lang_encoders
 
 let make params =
   let defaults =
     {
-      Avi_format.channels
       (* We use a hardcoded value in order not to force the evaluation of the
-         number of channels too early, see #933. *) = 2;
+         number of channels too early, see #933. *)
+      Avi_format.channels = 2;
       samplerate = Frame.audio_rate;
     }
   in
   let avi =
     List.fold_left
       (fun f -> function
-        | "channels", { term = Int c; _ } -> { f with Avi_format.channels = c }
-        | "samplerate", { term = Int i; _ } ->
+        | "channels", { term = Ground (Int c); _ } ->
+            { f with Avi_format.channels = c }
+        | "samplerate", { term = Ground (Int i); _ } ->
             { f with Avi_format.samplerate = Lazy.from_val i }
         | _, t -> raise (generic_error t))
       defaults params
