@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,8 @@
 
  *****************************************************************************)
 
-open Lang_values
+open Term
+open Term.Ground
 open Lang_encoders
 
 let make params =
@@ -43,13 +44,12 @@ let make params =
   let check_samplerate ?t i =
     lazy
       (let i = Lazy.force i in
-       if not (List.mem i valid_samplerates) then begin
+       if not (List.mem i valid_samplerates) then (
          let err =
            Printf.sprintf "invalid samplerate value. Possible values: %s"
              (String.concat ", " (List.map string_of_int valid_samplerates))
          in
-         match t with Some t -> raise (Error (t, err)) | None -> failwith err
-       end;
+         match t with Some t -> raise (Error (t, err)) | None -> failwith err);
        i)
   in
   let defaults =
@@ -60,7 +60,7 @@ let make params =
       bitrate = 64;
       bitrate_mode = `Constant;
       (* We use a hardcoded value in order not to force the evaluation of the
-         number of channels too early, see #933. *)
+           number of channels too early, see #933. *)
       channels = 2;
       samplerate = check_samplerate Frame.audio_rate;
       sbr_mode = false;
@@ -71,39 +71,39 @@ let make params =
   let fdkaac =
     List.fold_left
       (fun f -> function
-        | "afterburner", { term = Bool b; _ } ->
+        | "afterburner", { term = Ground (Bool b); _ } ->
             { f with Fdkaac_format.afterburner = b }
-        | "aot", ({ term = String s; _ } as t) ->
+        | "aot", ({ term = Ground (String s); _ } as t) ->
             let aot =
               try Fdkaac_format.aot_of_string s
               with Not_found -> raise (Error (t, "invalid aot value"))
             in
             { f with Fdkaac_format.aot }
-        | "vbr", ({ term = Int i; _ } as t) ->
-            if not (List.mem i valid_vbr) then begin
+        | "vbr", ({ term = Ground (Int i); _ } as t) ->
+            if not (List.mem i valid_vbr) then (
               let err =
                 Printf.sprintf "invalid vbr mode. Possible values: %s"
                   (String.concat ", " (List.map string_of_int valid_vbr))
               in
-              raise (Error (t, err))
-            end;
+              raise (Error (t, err)));
             { f with Fdkaac_format.bitrate_mode = `Variable i }
-        | "bandwidth", { term = Int i; _ } ->
+        | "bandwidth", { term = Ground (Int i); _ } ->
             { f with Fdkaac_format.bandwidth = `Fixed i }
-        | "bandwidth", { term = String s; _ }
+        | "bandwidth", { term = Ground (String s); _ }
           when String.lowercase_ascii s = "auto" ->
             { f with Fdkaac_format.bandwidth = `Auto }
-        | "bitrate", { term = Int i; _ } -> { f with Fdkaac_format.bitrate = i }
-        | "channels", { term = Int i; _ } ->
+        | "bitrate", { term = Ground (Int i); _ } ->
+            { f with Fdkaac_format.bitrate = i }
+        | "channels", { term = Ground (Int i); _ } ->
             { f with Fdkaac_format.channels = i }
-        | "samplerate", ({ term = Int i; _ } as t) ->
+        | "samplerate", ({ term = Ground (Int i); _ } as t) ->
             {
               f with
               Fdkaac_format.samplerate = check_samplerate ~t (Lazy.from_val i);
             }
-        | "sbr_mode", { term = Bool b; _ } ->
+        | "sbr_mode", { term = Ground (Bool b); _ } ->
             { f with Fdkaac_format.sbr_mode = b }
-        | "transmux", ({ term = String s; _ } as t) ->
+        | "transmux", ({ term = Ground (String s); _ } as t) ->
             let transmux =
               try Fdkaac_format.transmux_of_string s
               with Not_found -> raise (Error (t, "invalid transmux value"))
