@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,8 @@
 
  *****************************************************************************)
 
-open Lang_values
+open Term
+open Term.Ground
 open Lang_encoders
 
 let make ?pos params =
@@ -40,27 +41,32 @@ let make ?pos params =
     let perhaps = function "" -> None | s -> Some s in
     List.fold_left
       (fun f -> function
-        | "channels", { term = Int i; _ } ->
+        | "channels", { term = Ground (Int i); _ } ->
             { f with Gstreamer_format.channels = i }
-        | "audio", { term = String s; _ } ->
+        | "audio", { term = Ground (String s); _ } ->
             { f with Gstreamer_format.audio = perhaps s }
-        | "has_video", { term = Bool b; _ } ->
+        | "has_video", { term = Ground (Bool b); _ } ->
             { f with Gstreamer_format.has_video = b }
-        | "video", { term = String s; _ } ->
-            { f with Gstreamer_format.video = perhaps s }
-        | "muxer", { term = String s; _ } ->
+        | "video", { term = Ground (String s); _ } ->
+            let video = perhaps s in
+            let has_video =
+              if video = None then false else f.Gstreamer_format.has_video
+            in
+            { f with Gstreamer_format.has_video; video }
+        | "muxer", { term = Ground (String s); _ } ->
             { f with Gstreamer_format.muxer = perhaps s }
-        | "metadata", { term = String s; _ } ->
+        | "metadata", { term = Ground (String s); _ } ->
             { f with Gstreamer_format.metadata = s }
-        | "log", { term = Int i; _ } -> { f with Gstreamer_format.log = i }
-        | "pipeline", { term = String s; _ } ->
+        | "log", { term = Ground (Int i); _ } ->
+            { f with Gstreamer_format.log = i }
+        | "pipeline", { term = Ground (String s); _ } ->
             { f with Gstreamer_format.pipeline = perhaps s }
         | _, t -> raise (generic_error t))
       defaults params
   in
   let dummy =
     {
-      Lang_values.t = T.fresh_evar ~level:(-1) ~pos;
+      Term.t = Type.fresh_evar ~level:(-1) ~pos;
       term = Encoder (Encoder.GStreamer gstreamer);
     }
   in
