@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -47,7 +47,7 @@ let create_encoder ~opus ~comments () =
           end;
           let maybe name value =
             ignore
-              (Utils.maybe
+              (Option.map
                  (fun value -> Opus.Encoder.apply_control (name value) x)
                  value)
           in
@@ -136,4 +136,4 @@ let create_opus = function
       { Ogg_encoder.encode; reset; id = None }
   | _ -> assert false
 
-let () = Hashtbl.add Ogg_encoder.encoders "opus" create_opus
+let () = Hashtbl.add Ogg_encoder.audio_encoders "opus" create_opus
