@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -97,6 +97,7 @@ let create speex ~metadata () =
       let n = !status in
       if (frame_size * n) + frame_size < len then (
         status := n + 1;
+
         (* Speex float API are values in - 32768. <= x <= 32767. ..
            I don't really trust this, it must be a bug,
            so using the int API. *)
@@ -110,7 +111,7 @@ let create speex ~metadata () =
             f (Array.sub buf.(0) (frame_size * n) frame_size);
             f (Array.sub buf.(1) (frame_size * n) frame_size);
           |]
-        else [| f (Array.sub buf.(0) (frame_size * n) frame_size) |] )
+        else [| f (Array.sub buf.(0) (frame_size * n) frame_size) |])
       else raise Internal
     in
     try
@@ -122,7 +123,7 @@ let create speex ~metadata () =
               let x = feed () in
               x.(0)
             in
-            Speex.Encoder.encode_page_int enc os feed )
+            Speex.Encoder.encode_page_int enc os feed)
         in
         add_page page
       done
@@ -165,7 +166,7 @@ let create_speex = function
               let title = Filename.basename s in
               try String.sub title 0 (String.rindex title '.')
               with Not_found -> title
-            with Not_found -> "Unknown" )
+            with Not_found -> "Unknown")
         in
         let metadata = [("title", title)] @ List.remove_assoc "title" m in
         let enc = create speex ~metadata () in
@@ -178,4 +179,4 @@ let create_speex = function
       { Ogg_encoder.reset; encode; id = None }
   | _ -> assert false
 
-let () = Hashtbl.add Ogg_encoder.encoders "speex" create_speex
+let () = Hashtbl.add Ogg_encoder.audio_encoders "speex" create_speex
