@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,8 +20,8 @@
 
  *****************************************************************************)
 
+open Mm
 module Img = Image.Generic
-module P = Img.Pixel
 
 let create_encoder ~theora ~metadata () =
   let quality, bitrate =
@@ -77,7 +77,7 @@ let create_encoder ~theora ~metadata () =
   in
   let enc =
     if width mod 16 <> 0 || height mod 16 <> 0 then
-      failwith "Invalide theora width/height (should be a multiple of 16).";
+      failwith "Invalid theora width/height (should be a multiple of 16).";
     Theora.Encoder.create info params metadata
   in
   let started = ref false in
@@ -91,38 +91,29 @@ let create_encoder ~theora ~metadata () =
   in
   let stream_start os = Ogg_muxer.flush_pages os in
   let yuv = Image.YUV420.create width height in
-  let y = Image.YUV420.y yuv in
-  let u = Image.YUV420.u yuv in
-  let v = Image.YUV420.v yuv in
-  let y_stride = Image.YUV420.y_stride yuv in
-  let uv_stride = Image.YUV420.uv_stride yuv in
-  let theora_yuv =
-    {
-      Theora.y_width = width;
-      Theora.y_height = height;
-      Theora.y_stride;
-      Theora.u_width = width / 2;
-      Theora.u_height = height / 2;
-      Theora.u_stride = uv_stride;
-      Theora.v_width = width / 2;
-      Theora.v_height = height / 2;
-      Theora.v_stride = uv_stride;
-      Theora.y;
-      Theora.u;
-      Theora.v;
-    }
-  in
-  let convert =
-    Video_converter.find_converter (P.RGB P.RGBA32) (P.YUV P.YUVJ420)
-  in
   let data_encoder data os _ =
     if not !started then started := true;
     let b, ofs, len =
       (data.Ogg_muxer.data, data.Ogg_muxer.offset, data.Ogg_muxer.length)
     in
     for i = ofs to ofs + len - 1 do
-      let frame = Img.of_RGBA32 b.(i) in
-      convert frame (Img.of_YUV420 yuv);
+      let img = Video.get b i in
+      let theora_yuv =
+        {
+          Theora.y_width = width;
+          Theora.y_height = height;
+          Theora.y_stride = Image.YUV420.y_stride img;
+          Theora.u_width = width / 2;
+          Theora.u_height = height / 2;
+          Theora.u_stride = Image.YUV420.uv_stride img;
+          Theora.v_width = width / 2;
+          Theora.v_height = height / 2;
+          Theora.v_stride = Image.YUV420.uv_stride img;
+          Theora.y = Image.YUV420.y img;
+          Theora.u = Image.YUV420.u img;
+          Theora.v = Image.YUV420.v img;
+        }
+      in
       Theora.Encoder.encode_buffer enc os theora_yuv
     done
   in
@@ -133,15 +124,30 @@ let create_encoder ~theora ~metadata () =
       let index =
         Int64.succ (Theora.Encoder.frames_of_granulepos enc granulepos)
       in
-      Ogg_muxer.Time (Int64.to_float index /. float fps) )
+      Ogg_muxer.Time (Int64.to_float index /. float fps))
     else Ogg_muxer.Time 0.
   in
   let end_of_stream os =
     (* Encode at least some data.. *)
-    if not !started then begin
+    if not !started then (
+      let theora_yuv =
+        {
+          Theora.y_width = width;
+          Theora.y_height = height;
+          Theora.y_stride = width;
+          Theora.u_width = width / 2;
+          Theora.u_height = height / 2;
+          Theora.u_stride = width / 2;
+          Theora.v_width = width / 2;
+          Theora.v_height = height / 2;
+          Theora.v_stride = width / 2;
+          Theora.y = Image.Data.alloc width;
+          Theora.u = Image.Data.alloc (width / 2);
+          Theora.v = Image.Data.alloc (width / 2);
+        }
+      in
       Image.YUV420.blank_all yuv;
-      Theora.Encoder.encode_buffer enc os theora_yuv
-    end;
+      Theora.Encoder.encode_buffer enc os theora_yuv);
     Theora.Encoder.eos enc os
   in
   {
@@ -153,14 +159,12 @@ let create_encoder ~theora ~metadata () =
     end_of_stream;
   }
 
-let create_theora = function
-  | Ogg_format.Theora theora ->
-      let reset ogg_enc m =
-        let metadata = Utils.list_of_metadata (Meta_format.to_metadata m) in
-        let enc = create_encoder ~theora ~metadata () in
-        Ogg_muxer.register_track ?fill:theora.Theora_format.fill ogg_enc enc
-      in
-      { Ogg_encoder.encode = Ogg_encoder.encode_video; reset; id = None }
-  | _ -> assert false
+let create_theora theora =
+  let reset ogg_enc m =
+    let metadata = Utils.list_of_metadata (Meta_format.to_metadata m) in
+    let enc = create_encoder ~theora ~metadata () in
+    Ogg_muxer.register_track ?fill:theora.Theora_format.fill ogg_enc enc
+  in
+  { Ogg_encoder.encode = Ogg_encoder.encode_video; reset; id = None }
 
-let () = Hashtbl.add Ogg_encoder.encoders "theora" create_theora
+let () = Ogg_encoder.theora_encoder := Some create_theora
