@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,23 +20,19 @@
 
  *****************************************************************************)
 
+open Mm
 open Source
 
 class amplify ~kind (source : source) override_field coeff =
   object (self)
     inherit operator ~name:"amplify" kind [source]
-
     val mutable override = None
-
     method stype = source#stype
-
     method is_ready = source#is_ready
-
     method remaining = source#remaining
-
     method abort_track = source#abort_track
-
     method seek = source#seek
+    method self_sync = source#self_sync
 
     method private get_frame buf =
       let offset = AFrame.position buf in
@@ -55,29 +51,27 @@ class amplify ~kind (source : source) override_field coeff =
                     in
                     self#log#info "Overriding amplification: %f." k;
                     override <- Some k
-                  with _ -> () ))
+                  with _ -> ()))
               (AFrame.get_all_metadata buf)
         | None -> ()
       end;
       let k = match override with Some o -> o | None -> coeff () in
       if k <> 1. then
         Audio.amplify k
-          (AFrame.content buf offset)
-          offset
-          (AFrame.position buf - offset);
-      if AFrame.is_partial buf && override <> None then begin
+          (Audio.sub (AFrame.pcm buf) offset (AFrame.position buf - offset));
+      if AFrame.is_partial buf && override <> None then (
         self#log#info "End of the current overriding.";
-        override <- None
-      end
+        override <- None)
   end
 
 let () =
-  let k = Lang.kind_type_of_kind_format ~fresh:2 Lang.any_fixed in
+  let kind = Lang.audio_pcm in
+  let k = Lang.kind_type_of_kind_format kind in
   Lang.add_operator "amplify"
     [
-      ("", Lang.float_getter_t 1, None, Some "Multiplicative factor.");
+      ("", Lang.getter_t Lang.float_t, None, Some "Multiplicative factor.");
       ( "override",
-        Lang.string_t,
+        Lang.nullable_t Lang.string_t,
         Some (Lang.string "liq_amplify"),
         Some
           "Specify the name of a metadata field that, when present and \
@@ -85,15 +79,14 @@ let () =
            track. Well-formed values are floats in decimal notation (e.g. \
            `0.7`) which are taken as normal/linear multiplicative factors; \
            values can be passed in decibels with the suffix `dB` (e.g. `-8.2 \
-           dB`, but the spaces do not matter). Set to empty string `\"\"` to \
-           disable." );
+           dB`, but the spaces do not matter)." );
       ("", Lang.source_t k, None, None);
     ]
-    ~kind:(Lang.Unconstrained k) ~category:Lang.SoundProcessing
-    ~descr:"Multiply the amplitude of the signal."
-    (fun p kind ->
+    ~return_t:k ~category:`Audio ~descr:"Multiply the amplitude of the signal."
+    (fun p ->
       let c = Lang.to_float_getter (Lang.assoc "" 1 p) in
       let s = Lang.to_source (Lang.assoc "" 2 p) in
-      let o = Lang.to_string (Lang.assoc "override" 1 p) in
-      let o = if o = "" then None else Some o in
+      let o = Lang.to_option (Lang.assoc "override" 1 p) in
+      let o = Option.map Lang.to_string o in
+      let kind = Source.Kind.of_kind kind in
       new amplify ~kind s o c)
