@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,9 +23,19 @@
 open Source
 
 class append ~kind ~insert_missing ~merge source f =
+  let sources = ref [] in
+  let failed = ref false in
+  let () =
+    Lang.iter_sources
+      ~on_reference:(fun () -> failed := true)
+      (fun s -> sources := s :: !sources)
+      f
+  in
+  let self_sync_type =
+    if !failed then lazy `Dynamic else Utils.self_sync_type !sources
+  in
   object (self)
     inherit operator ~name:"append" kind [source]
-
     val mutable state = `Idle
 
     method private get_frame buf =
@@ -39,58 +49,52 @@ class append ~kind ~insert_missing ~merge source f =
               if insert_missing && m = None then Some (Hashtbl.create 10) else m
             with
               | Some m when Utils.hashtbl_get m "liq_append" <> Some "false" ->
-                  let t = Lang.source_t (Lang.kind_type_of_frame_kind kind) in
                   let append =
-                    Lang.to_source (Lang.apply ~t f [("", Lang.metadata m)])
+                    Lang.to_source (Lang.apply f [("", Lang.metadata m)])
                   in
                   self#register append;
                   if finished then
-                    if append#is_ready then begin
+                    if append#is_ready then (
                       state <- `Append append;
                       if merge then (
                         let pos = Frame.position buf in
                         self#get_frame buf;
                         Frame.set_breaks buf
-                          (Utils.remove_one (( = ) pos) (Frame.breaks buf)) )
-                    end
-                    else begin
+                          (Utils.remove_one (( = ) pos) (Frame.breaks buf))))
+                    else (
                       self#log#important
                         "Track ends and append source is not ready: won't \
                          append.";
                       self#unregister append;
-                      state <- `Idle
-                    end
+                      state <- `Idle)
                   else state <- `Replay (Some append)
               | _ ->
                   self#log#important
                     "No metadata at beginning of track: won't append.";
-                  state <- (if finished then `Idle else `Replay None) )
+                  state <- (if finished then `Idle else `Replay None))
         | `Replay None ->
             source#get buf;
             if Frame.is_partial buf then state <- `Idle
         | `Replay (Some a) ->
             source#get buf;
             if Frame.is_partial buf then
-              if a#is_ready then begin
+              if a#is_ready then (
                 state <- `Append a;
                 if merge then (
                   let pos = Frame.position buf in
                   self#get_frame buf;
                   Frame.set_breaks buf
-                    (Utils.remove_one (( = ) pos) (Frame.breaks buf)) )
-              end
-              else begin
+                    (Utils.remove_one (( = ) pos) (Frame.breaks buf))))
+              else (
                 self#log#important
                   "Track ends and append source is not ready: won't append.";
                 state <- `Idle;
-                self#unregister a
-              end
+                self#unregister a)
         | `Append a ->
             a#get buf;
-            if Frame.is_partial buf then begin
+            if Frame.is_partial buf then (
               state <- `Idle;
-              self#unregister a
-            end
+              self#unregister a)
 
     method stype = source#stype
 
@@ -115,6 +119,12 @@ class append ~kind ~insert_missing ~merge source f =
         | `Replay (Some _) -> source#seek n
         | `Append s -> s#seek n
 
+    method self_sync =
+      ( Lazy.force self_sync_type,
+        match state with
+          | `Append s -> snd s#self_sync
+          | _ -> snd source#self_sync )
+
     (* Other behaviours could be wanted, but for now #abort_track won't cancel
      * any to-be-appended track. *)
     method abort_track = source#abort_track
@@ -139,12 +149,12 @@ class append ~kind ~insert_missing ~merge source f =
       state <- `Idle
 
     method private register a = a#get_ready activation
-
     method private unregister a = a#leave (self :> source)
   end
 
 let register =
-  let k = Lang.univ_t 1 in
+  let kind = Lang.any in
+  let k = Lang.kind_type_of_kind_format kind in
   Lang.add_operator "append"
     [
       ( "merge",
@@ -164,13 +174,14 @@ let register =
            This source is allowed to fail (produce nothing) if no relevant \
            track is to be appended." );
     ]
-    ~kind:(Lang.Unconstrained k) ~category:Lang.TrackProcessing
+    ~return_t:k ~category:`Track
     ~descr:
       "Append an extra track to every track. Set the metadata 'liq_append' to \
        'false' to inhibit appending on one track."
-    (fun p kind ->
+    (fun p ->
       let merge = Lang.to_bool (Lang.assoc "merge" 1 p) in
       let insert_missing = Lang.to_bool (Lang.assoc "insert_missing" 1 p) in
       let source = Lang.to_source (Lang.assoc "" 1 p) in
       let f = Lang.assoc "" 2 p in
+      let kind = Source.Kind.of_kind kind in
       new append ~kind ~insert_missing ~merge source f)
