@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 
  *****************************************************************************)
 
+open Mm
 open Source
 
 let chan = 0
@@ -45,26 +46,19 @@ let note_of_string s = note_of_string s - 12
 class chord ~kind metadata_name (source : source) =
   object (self)
     inherit operator ~name:"chord" kind [source]
-
     method stype = source#stype
-
     method remaining = source#remaining
-
     method is_ready = source#is_ready
-
     method abort_track = source#abort_track
-
     method seek = source#seek
-
+    method self_sync = source#self_sync
     val mutable notes_on = []
 
     method private get_frame buf =
       let offset = MFrame.position buf in
-      let toffset = Frame.position buf in
       source#get buf;
-      let m = Frame.content_of_type buf toffset (Frame.type_of_kind kind) in
+      let m = MFrame.midi buf in
       let pos = MFrame.position buf in
-      let m = m.Frame.midi in
       let meta = MFrame.get_all_metadata buf in
       let meta = List.filter (fun (p, _) -> offset <= p && p < pos) meta in
       let chords =
@@ -101,6 +95,7 @@ class chord ~kind metadata_name (source : source) =
         (fun (t, c, m) ->
           (* time, base, mode *)
           mute t;
+
           (* Negative base note means mute. *)
           if c >= 0 then (
             match m with
@@ -114,15 +109,16 @@ class chord ~kind metadata_name (source : source) =
               | "M7" -> play t [c; c + 4; c + 7; c + 11]
               | "m7" -> play t [c; c + 3; c + 7; c + 10]
               | "dim" -> play t [c; c + 3; c + 6]
-              | m -> self#log#debug "Unknown mode: %s\n%!" m ))
+              | m -> self#log#debug "Unknown mode: %s\n%!" m))
         chords
   end
 
 let () =
   (* TODO: is this really the type we want to give to it? *)
-  let in_k = Lang.kind_type_of_kind_format ~fresh:1 Lang.any_fixed in
+  let in_k = Lang.kind_type_of_kind_format Lang.any in
   let out_k =
-    Lang.kind_type_of_kind_format ~fresh:1 (Lang.any_fixed_with ~midi:1 ())
+    Lang.kind_type_of_kind_format
+      { Frame.audio = `Any; video = `Any; midi = Frame.midi_n 1 }
   in
   Lang.add_operator "midi.chord"
     [
@@ -132,10 +128,10 @@ let () =
         Some "Name of the metadata containing the chords." );
       ("", Lang.source_t in_k, None, None);
     ]
-    ~kind:(Lang.Unconstrained out_k) ~category:Lang.MIDIProcessing
-    ~descr:"Generate a chord."
-    (fun p kind ->
+    ~return_t:out_k ~category:`MIDI ~descr:"Generate a chord."
+    (fun p ->
       let f v = List.assoc v p in
       let src = Lang.to_source (f "") in
       let metadata = Lang.to_string (f "metadata") in
-      new chord ~kind metadata src)
+      (new chord ~kind:(Source.Kind.of_kind Lang.any) metadata src
+        :> Source.source))
