@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,64 +20,64 @@
 
  *****************************************************************************)
 
+open Mm
 open Source
 
 (* See http://en.wikipedia.org/wiki/Comb_filter *)
 
 class comb ~kind (source : source) delay feedback =
   let past_len = Frame.audio_of_seconds delay in
-  let channels = (Frame.type_of_kind kind).Frame.audio in
-  object
-    inherit operator ~name:"comb" kind [source]
-
+  object (self)
+    inherit operator ~name:"comb" kind [source] as super
     method stype = source#stype
-
     method remaining = source#remaining
-
     method seek = source#seek
-
+    method self_sync = source#self_sync
     method is_ready = source#is_ready
-
     method abort_track = source#abort_track
+    val mutable past = Audio.make 0 0 0.
 
-    val past = Array.init channels (fun _ -> Array.make past_len 0.)
+    method private wake_up s =
+      super#wake_up s;
+      past <- Audio.make self#audio_channels past_len 0.
 
     val mutable past_pos = 0
 
     method private get_frame buf =
       let offset = AFrame.position buf in
       source#get buf;
-      let b = AFrame.content buf offset in
+      let b = AFrame.pcm buf in
       let position = AFrame.position buf in
       let feedback = feedback () in
       for i = offset to position - 1 do
         for c = 0 to Array.length b - 1 do
-          let oldin = b.(c).(i) in
-          b.(c).(i) <- b.(c).(i) +. (past.(c).(past_pos) *. feedback);
-          past.(c).(past_pos) <- oldin
+          let oldin = b.(c).{i} in
+          b.(c).{i} <- b.(c).{i} +. (past.(c).{past_pos} *. feedback);
+          past.(c).{past_pos} <- oldin
         done;
         past_pos <- (past_pos + 1) mod past_len
       done
   end
 
 let () =
-  let k = Lang.kind_type_of_kind_format ~fresh:2 Lang.any_fixed in
+  let kind = Lang.audio_pcm in
+  let k = Lang.kind_type_of_kind_format kind in
   Lang.add_operator "comb"
     [
       ("delay", Lang.float_t, Some (Lang.float 0.001), Some "Delay in seconds.");
       ( "feedback",
-        Lang.float_getter_t 1,
+        Lang.getter_t Lang.float_t,
         Some (Lang.float (-6.)),
         Some "Feedback coefficient in dB." );
       ("", Lang.source_t k, None, None);
     ]
-    ~kind:(Lang.Unconstrained k) ~category:Lang.SoundProcessing
-    ~descr:"Comb filter."
-    (fun p kind ->
+    ~return_t:k ~category:`Audio ~descr:"Comb filter."
+    (fun p ->
       let f v = List.assoc v p in
       let duration, feedback, src =
         ( Lang.to_float (f "delay"),
           Lang.to_float_getter (f "feedback"),
           Lang.to_source (f "") )
       in
+      let kind = Source.Kind.of_kind kind in
       new comb ~kind src duration (fun () -> Audio.lin_of_dB (feedback ())))
