@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,25 +20,34 @@
 
  *****************************************************************************)
 
+open Mm
 open Source
 module Generator = Generator.From_frames
 module Generated = Generated.Make (Generator)
 
-let finalise_slave_clock slave_clock source =
-  Clock.forget source#clock slave_clock
+let finalise_child_clock child_clock source =
+  Clock.forget source#clock child_clock
 
 (** [rms_width] and [minimum_length] are all in samples.
   * [cross_length] is in ticks (like #remaining estimations).
   * We are assuming a fixed audio kind -- at least for now. *)
-class cross ~kind (s : source) ~cross_length ~override_duration ~rms_width
+class cross ~kind val_source ~cross_length ~override_duration ~rms_width
   ~minimum_length ~conservative ~active transition =
-  let channels = float (Frame.type_of_kind kind).Frame.audio in
+  let s = Lang.to_source val_source in
   object (self)
     inherit source ~name:"cross" kind as super
 
+    inherit
+      Child_support.base ~check_self_sync:true [val_source] as child_support
+
+    initializer Source.Kind.unify s#kind kind
     method stype = Source.Fallible
 
-    val mutable cross_length = cross_length
+    (* This is complicated. crossfade should never be used with [self_sync]
+     * sources but we do not have a static way of knowing it at the moment.
+     * Going with the same choice as above for now. *)
+    method self_sync = s#self_sync
+    val mutable cross_length = cross_length ()
 
     (* We need to store the end of a track, and compute the power of the signal
      * before the end of track. For doing so we need to remember a sliding window
@@ -46,31 +55,22 @@ class cross ~kind (s : source) ~cross_length ~override_duration ~rms_width
      * sum. The sliding window is necessary because of possibly inaccurate
      * remaining time estimaton. *)
     val mutable gen_before = Generator.create ()
-
     val mutable rms_before = 0.
-
     val mutable rmsi_before = 0
-
     val mutable mem_before = Array.make rms_width 0.
-
     val mutable mem_i = 0
-
     val mutable before_metadata = None
 
     (* Same for the new track. No need for a sliding window here. *)
     val mutable gen_after = Generator.create ()
-
     val mutable rms_after = 0.
-
     val mutable rmsi_after = 0
-
     val mutable after_metadata = None
 
-    (* An audio frame for intermediate computations.
-     * It is used to buffer the end and beginnings of tracks.
-     * Its past metadata should mimick that of the main stream in order
-     * to avoid metadata duplication. *)
-    val buf_frame = Frame.create kind
+    (* An audio frame for intermediate computations. It is used to buffer the
+       end and beginnings of tracks. Its past metadata should mimic that of the
+       main stream in order to avoid metadata duplication. *)
+    val mutable buf_frame = Frame.dummy
 
     method private reset_analysis =
       gen_before <- Generator.create ();
@@ -91,7 +91,6 @@ class cross ~kind (s : source) ~cross_length ~override_duration ~rms_width
 
     (* Give a default value for the transition source. *)
     val mutable transition_source = None
-
     val mutable pending_after = Generator.create ()
 
     method private prepare_transition_source s =
@@ -107,7 +106,9 @@ class cross ~kind (s : source) ~cross_length ~override_duration ~rms_width
             s#leave ~dynamic:true (self :> source);
             transition_source <- None
 
-    method private wake_up _ =
+    method private wake_up a =
+      super#wake_up a;
+      buf_frame <- Frame.create self#ctype;
       source#get_ready ~dynamic:true [(self :> source)];
       source#get_ready [(self :> source)];
       Lang.iter_sources
@@ -122,58 +123,38 @@ class cross ~kind (s : source) ~cross_length ~override_duration ~rms_width
         transition;
       self#cleanup_transition_source
 
-    (** See cross.ml for the details of clock management. *)
-
     method private set_clock =
-      let slave_clock = Clock.create_known (new Clock.clock self#id) in
-      (* Our external clock should stricly contain the slave clock. *)
-      Clock.unify self#clock
-        (Clock.create_unknown ~sources:[] ~sub_clocks:[slave_clock]);
-      (* The source must belong to our clock, since we need occasional
-       * control on its flow (to fold an end of track over a beginning).
-       *
-       * To be safe, also require that the transition's sources belong
-       * to that same clock, since we (dynamically) activate them as well.
-       * The activation mechanism is designed for a single thread,
-       * which multiples clocks wouldn't ensure. *)
-      Clock.unify slave_clock s#clock;
-      Lang.iter_sources (fun s -> Clock.unify slave_clock s#clock) transition;
-      (* Make sure the slave clock can be garbage collected, cf. cue_cut(). *)
-      Gc.finalise (finalise_slave_clock slave_clock) self
+      child_support#set_clock;
+      Lang.iter_sources
+        (fun s -> Clock.unify self#child_clock s#clock)
+        transition
 
-    val mutable master_time = 0
-
-    val mutable last_slave_tick = 0
-
-    (* in master time *)
-
-    (* Indicate that the source should be managed by relaying master ticks,
-     * and it has been used during the current tick, so it should be ticked
-     * even if we're not in active mode. *)
-    val mutable needs_tick = false
+    val mutable main_time = 0
+    val mutable last_child_tick = 0
 
+    (* in main time *)
     val mutable status = `Idle
 
-    method private slave_tick =
-      (Clock.get source#clock)#end_tick;
-      source#after_output;
+    method private child_tick =
+      child_support#child_tick;
       Frame.advance buf_frame;
-      needs_tick <- false;
-      last_slave_tick <- (Clock.get self#clock)#get_tick
+      last_child_tick <- (Clock.get self#clock)#get_tick
+
+    method before_output =
+      super#before_output;
+      child_support#before_output
 
     method after_output =
       super#after_output;
-      if needs_tick then self#slave_tick;
-      let master_clock = Clock.get self#clock in
+      child_support#after_output;
+      let main_clock = Clock.get self#clock in
       (* Is it really a new tick? *)
-      if master_time <> master_clock#get_tick then begin
-        (* Did the slave clock tick during this instant? *)
-        if active && last_slave_tick <> master_time then begin
-          self#slave_tick;
-          last_slave_tick <- master_time
-        end;
-        master_time <- master_clock#get_tick
-      end
+      if main_time <> main_clock#get_tick then (
+        (* Did the child clock tick during this instant? *)
+        if active && last_child_tick <> main_time then (
+          self#child_tick;
+          last_child_tick <- main_time);
+        main_time <- main_clock#get_tick)
 
     method private save_last_metadata mode buf_frame =
       let compare x y = -compare (fst x) (fst y) in
@@ -193,34 +174,29 @@ class cross ~kind (s : source) ~cross_length ~override_duration ~rms_width
                   try
                     let l = float_of_string v in
                     self#log#info "Setting crossfade duration to %.2fs" l;
-                    cross_length <- Frame.master_of_seconds l
-                  with _ -> () ) ))
+                    cross_length <- Frame.main_of_seconds l
+                  with _ -> ())))
         (Frame.get_all_metadata frame)
 
     method private get_frame frame =
       match status with
         | `Idle ->
             let rem = if conservative then 0 else source#remaining in
-            if rem < 0 || rem > cross_length then begin
+            if rem < 0 || rem > cross_length then (
               let p = Frame.position frame in
               source#get frame;
               self#save_last_metadata `Before frame;
               self#update_cross_length frame p;
-              needs_tick <- true
-            end
-            else begin
+              needs_tick <- true)
+            else (
               self#log#info "Buffering end of track...";
               status <- `Before;
               Frame.set_breaks buf_frame [Frame.position frame];
-              Frame.set_all_metadata buf_frame
-                ( match Frame.get_past_metadata frame with
-                  | Some x -> [(-1, x)]
-                  | None -> [] );
+              Frame.set_all_metadata buf_frame [];
               self#buffering cross_length;
               if status <> `Limit then
                 self#log#info "More buffering will be needed.";
-              self#get_frame frame
-            end
+              self#get_frame frame)
         | `Before ->
             (* We started buffering but the track didn't end.
              * Play the beginning of the buffer while filling it more. *)
@@ -231,60 +207,67 @@ class cross ~kind (s : source) ~cross_length ~override_duration ~rms_width
             (* The track finished.
              * We compute rms_after and launch the transition. *)
             if source#is_ready then self#analyze_after;
-            self#slave_tick;
+            self#child_tick;
             self#create_transition;
+
             (* Check if the new source is ready *)
-            if source#is_ready then self#get_frame frame
+            if (Option.get transition_source)#is_ready then self#get_frame frame
             else
               (* If not, finish this track, which requires our callers
                * to wait that we become ready again. *)
               Frame.add_break frame (Frame.position frame)
-        | `After ->
-            (Utils.get_some transition_source)#get frame;
+        | `After when (Option.get transition_source)#is_ready ->
+            (Option.get transition_source)#get frame;
             needs_tick <- true;
-            if Generator.length pending_after = 0 && Frame.is_partial frame then begin
+            if Generator.length pending_after = 0 && Frame.is_partial frame then (
               status <- `Idle;
               self#cleanup_transition_source;
+
               (* If underlying source if ready, try to continue filling up the frame
                * using it. Each call to [get_frame] must add exactly one break so
                * call it again and then remove the intermediate break that was just
                * just added. *)
-              if source#is_ready then begin
+              if source#is_ready then (
                 self#get_frame frame;
                 Frame.set_breaks frame
-                  ( match Frame.breaks frame with
+                  (match Frame.breaks frame with
                     | b :: _ :: l -> b :: l
-                    | _ -> assert false )
-              end
-            end
+                    | _ -> assert false)))
+        | `After ->
+            (* Here, transition source went down so we switch back to main source.
+               Our [is_ready] check ensures that we only get here when the main source
+               is ready. *)
+            status <- `Idle;
+            self#cleanup_transition_source;
+            self#get_frame frame
 
     (* [bufferize n] stores at most [n+d] samples from [s] in [gen_before],
      * where [d=AFrame.size-1]. *)
     method private buffering n =
-      (* For the first call, the position is the old position in the master
+      (* For the first call, the position is the old position in the main
        * frame. After that it'll always be 0. *)
-      if not (Frame.is_partial buf_frame) then self#slave_tick;
-      let start = Frame.position buf_frame in
+      if not (Frame.is_partial buf_frame) then self#child_tick;
+      let start = AFrame.position buf_frame in
       let stop =
         source#get buf_frame;
-        Frame.position buf_frame
+        AFrame.position buf_frame
       in
       self#save_last_metadata `Before buf_frame;
       self#update_cross_length buf_frame start;
-      let content =
-        let end_pos, c = Frame.content buf_frame start in
-        assert (end_pos >= stop);
-        c
-      in
       Generator.feed gen_before
         ~metadata:(Frame.get_all_metadata buf_frame)
-        content start (stop - start);
+        buf_frame.Frame.content
+        (Frame.main_of_audio start)
+        (Frame.main_of_audio (stop - start));
+
       (* Analyze them *)
-      let pcm = content.Frame.audio in
+      let pcm = AFrame.pcm buf_frame in
       for i = start to stop - 1 do
         let squares =
           Array.fold_left
-            (fun squares track -> squares +. (track.(i) *. track.(i)))
+            (fun squares track ->
+              let x = track.{i} in
+              squares +. (x *. x))
             0. pcm
         in
         rms_before <- rms_before -. mem_before.(mem_i) +. squares;
@@ -292,65 +275,60 @@ class cross ~kind (s : source) ~cross_length ~override_duration ~rms_width
         mem_i <- (mem_i + 1) mod rms_width;
         rmsi_before <- min rms_width (rmsi_before + 1)
       done;
+
       (* Should we buffer more or are we done ? *)
-      if AFrame.is_partial buf_frame then begin
+      if AFrame.is_partial buf_frame then (
         Generator.add_break gen_before;
-        status <- `Limit
-      end
+        status <- `Limit)
       else if n > 0 then self#buffering (n - stop + start)
 
     (* Analyze the beginning of a new track. *)
     method private analyze_after =
       let before_len = Generator.length gen_before in
       let rec f () =
-        let start = Frame.position buf_frame in
+        let start = AFrame.position buf_frame in
         let stop =
           source#get buf_frame;
-          Frame.position buf_frame
+          AFrame.position buf_frame
         in
-        let content =
-          let end_pos, c = Frame.content buf_frame start in
-          assert (end_pos >= stop);
-          c
-        in
-
         Generator.feed gen_after
           ~metadata:(Frame.get_all_metadata buf_frame)
-          content start (stop - start);
-
+          buf_frame.Frame.content
+          (Frame.main_of_audio start)
+          (Frame.main_of_audio (stop - start));
         let after_len = Generator.length gen_after in
-
-        if after_len <= rms_width then begin
-          let pcm = content.Frame.audio in
+        if after_len <= rms_width then (
+          let pcm = AFrame.pcm buf_frame in
           for i = start to stop - 1 do
             let squares =
               Array.fold_left
-                (fun squares track -> squares +. (track.(i) *. track.(i)))
+                (fun squares track ->
+                  let x = track.{i} in
+                  squares +. (x *. x))
                 0. pcm
             in
             rms_after <- rms_after +. squares;
             rmsi_after <- rmsi_after + 1
-          done
-        end;
-
+          done);
         self#save_last_metadata `After buf_frame;
         self#update_cross_length buf_frame start;
-
-        if AFrame.is_partial buf_frame then Generator.add_break gen_after
-        else begin
-          self#slave_tick;
-          if after_len < before_len then f ()
-        end
+        if AFrame.is_partial buf_frame && not source#is_ready then
+          Generator.add_break gen_after
+        else (
+          self#child_tick;
+          if after_len < before_len then f ())
       in
       f ()
 
     (* Sum up analysis and build the transition *)
     method private create_transition =
       let db_after =
-        Audio.dB_of_lin (sqrt (rms_after /. float rmsi_after /. channels))
+        Audio.dB_of_lin
+          (sqrt (rms_after /. float rmsi_after /. float self#audio_channels))
       in
       let db_before =
-        Audio.dB_of_lin (sqrt (rms_before /. float rmsi_before /. channels))
+        Audio.dB_of_lin
+          (sqrt (rms_before /. float rmsi_before /. float self#audio_channels))
       in
       let compound =
         Clock.collect_after (fun () ->
@@ -374,35 +352,38 @@ class cross ~kind (s : source) ~cross_length ~override_duration ~rms_width
             let f a b =
               let params =
                 [
-                  ("", Lang.float db_before);
-                  ("", Lang.float db_after);
-                  ("", Lang.metadata before_metadata);
-                  ("", Lang.metadata after_metadata);
-                  ("", Lang.source a);
-                  ("", Lang.source b);
+                  ( "",
+                    Lang.meth Lang.unit
+                      [
+                        ("source", Lang.source a);
+                        ("db_level", Lang.float db_before);
+                        ("metadata", Lang.metadata before_metadata);
+                      ] );
+                  ( "",
+                    Lang.meth Lang.unit
+                      [
+                        ("source", Lang.source b);
+                        ("db_level", Lang.float db_after);
+                        ("metadata", Lang.metadata after_metadata);
+                      ] );
                 ]
               in
-              let t = Lang.source_t (Lang.kind_type_of_frame_kind kind) in
-              Lang.to_source (Lang.apply ~t transition params)
+              Lang.to_source (Lang.apply transition params)
             in
             let compound =
               self#log#important "Analysis: %fdB / %fdB (%.2fs / %.2fs)"
                 db_before db_after
-                (Frame.seconds_of_master (Generator.length gen_before))
-                (Frame.seconds_of_master (Generator.length gen_after));
-              if
-                Frame.master_of_audio minimum_length
-                < Generator.length gen_after
+                (Frame.seconds_of_main (Generator.length gen_before))
+                (Frame.seconds_of_main (Generator.length gen_after));
+              if Frame.main_of_audio minimum_length < Generator.length gen_after
               then f before after
-              else begin
+              else (
                 self#log#important "Not enough data for crossing.";
-                (new Sequence.sequence ~kind [before; after] :> source)
-              end
+                (new Sequence.sequence ~kind [before; after] :> source))
             in
             Clock.unify compound#clock s#clock;
             compound)
       in
-      compound#get_ready ~dynamic:true [(self :> source)];
       self#cleanup_transition_source;
       self#prepare_transition_source compound;
       pending_after <- gen_after;
@@ -410,39 +391,42 @@ class cross ~kind (s : source) ~cross_length ~override_duration ~rms_width
       self#reset_analysis
 
     method remaining =
-      let with_buf buf =
-        let rem = source#remaining in
-        if rem < 0 then -1 else source#remaining + Generator.length buf
-      in
       match status with
-        | `Idle -> source#remaining
+        | `Before | `Idle -> source#remaining
         | `Limit -> -1
-        | `After -> with_buf gen_after
-        | `Before -> with_buf gen_before
+        | `After -> (Option.get transition_source)#remaining
 
-    (** Contrary to cross.ml, the transition is only created (and stored in
-    * the source instance variable) after that status has moved from `Limit to
-    * `After. If is_ready becomes false at this point, source.ml will end the
-    * track before the transition (or bare end of track) gets a chance
-    * to be played. *)
-    method is_ready = source#is_ready || List.mem status [`Limit; `After]
+    method is_ready =
+      match status with
+        | `Idle | `Before -> source#is_ready
+        | `Limit -> true
+        | `After -> (Option.get transition_source)#is_ready || source#is_ready
 
     method abort_track =
-      if status = `After && (Utils.get_some transition_source)#is_ready then
-        (Utils.get_some transition_source)#abort_track
+      if status = `After && (Option.get transition_source)#is_ready then
+        (Option.get transition_source)#abort_track
       else source#abort_track
   end
 
 let () =
-  let k = Lang.kind_type_of_kind_format ~fresh:1 Lang.audio_any in
+  let kind = Lang.audio_pcm in
+  let k = Lang.kind_type_of_kind_format kind in
+  let transition_arg =
+    Lang.method_t Lang.unit_t
+      [
+        ("source", ([], Lang.source_t k), "Source");
+        ("db_level", ([], Lang.float_t), "dB level of the source.");
+        ("metadata", ([], Lang.metadata_t), "Metadata of the source.");
+      ]
+  in
   Lang.add_operator "cross"
     [
       ( "duration",
-        Lang.float_t,
+        Lang.getter_t Lang.float_t,
         Some (Lang.float 5.),
         Some
           "Duration (in seconds) of buffered data from each track that is used \
-           to compute the transision between tracks." );
+           to compute the transition between tracks." );
       ( "override_duration",
         Lang.string_t,
         Some (Lang.string "liq_cross_duration"),
@@ -468,7 +452,7 @@ let () =
         Some
           "Do not trust remaining time estimations, always buffering data in \
            advance. This avoids being tricked by skips, either manual or \
-           caused by skip_blank()." );
+           caused by blank.skip()." );
       ( "active",
         Lang.bool_t,
         Some (Lang.bool false),
@@ -480,47 +464,37 @@ let () =
            stopped." );
       ( "",
         Lang.fun_t
-          [
-            (false, "", Lang.float_t);
-            (false, "", Lang.float_t);
-            (false, "", Lang.metadata_t);
-            (false, "", Lang.metadata_t);
-            (false, "", Lang.source_t k);
-            (false, "", Lang.source_t k);
-          ]
+          [(false, "", transition_arg); (false, "", transition_arg)]
           (Lang.source_t k),
         None,
         Some
           "Transition function, composing from the end of a track and the next \
-           track. It also takes the power of the signal before and after the \
-           transition, and the metadata." );
+           track. The sources corresponding to the two tracks are decorated \
+           with fields indicating the power of the signal before and after the \
+           transition (`power`), and the metadata (`metadata`)." );
       ("", Lang.source_t k, None, None);
     ]
-    ~kind:(Lang.Unconstrained k) ~category:Lang.SoundProcessing
+    ~return_t:k ~category:`Audio
     ~descr:
-      "Cross operator, allowing the composition of the N last seconds of a \
+      "Cross operator, allowing the composition of the _n_ last seconds of a \
        track with the beginning of the next track, using a transition function \
        depending on the relative power of the signal before and after the end \
        of track."
-    (fun p kind ->
-      let duration = Lang.to_float (List.assoc "duration" p) in
+    (fun p ->
+      let duration = Lang.to_float_getter (List.assoc "duration" p) in
       let override_duration =
         Lang.to_string (List.assoc "override_duration" p)
       in
-      let cross_length = Frame.master_of_seconds duration in
-
+      let cross_length () = Frame.main_of_seconds (duration ()) in
       let minimum = Lang.to_float (List.assoc "minimum" p) in
       let minimum_length = Frame.audio_of_seconds minimum in
-
       let rms_width = Lang.to_float (List.assoc "width" p) in
       let rms_width = Frame.audio_of_seconds rms_width in
-
       let transition = Lang.assoc "" 1 p in
-
       let conservative = Lang.to_bool (List.assoc "conservative" p) in
       let active = Lang.to_bool (List.assoc "active" p) in
-
-      let source = Lang.to_source (Lang.assoc "" 2 p) in
+      let source = Lang.assoc "" 2 p in
+      let kind = Source.Kind.of_kind kind in
       let c =
         new cross
           ~kind source transition ~conservative ~active ~cross_length ~rms_width
