@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -28,35 +28,38 @@ open Source
 class delay ~kind ~initial (source : source) delay =
   object (self)
     inherit operator ~name:"delay" kind [source]
-
     method stype = Fallible
-
     method remaining = source#remaining
 
-    method abort_track = source#abort_track
+    method abort_track =
+      self#end_track;
+      source#abort_track
 
     method seek = source#seek
-
+    method self_sync = source#self_sync
     val mutable last = if initial then Unix.time () else 0.
-
     val mutable in_track = false
+    method private delay_ok = Unix.time () -. last >= delay ()
 
-    method private delay_ok = Unix.time () -. last >= delay
+    method private end_track =
+      in_track <- false;
+      last <- Unix.time ()
 
-    method is_ready = in_track || (self#delay_ok && source#is_ready)
+    method is_ready =
+      let is_ready = source#is_ready in
+      if in_track && not is_ready then self#end_track;
+      self#delay_ok && is_ready
 
     method private get_frame buf =
       source#get buf;
       in_track <- true;
       (* The current track ends. *)
-      if Frame.is_partial buf then begin
-        in_track <- false;
-        last <- Unix.time ()
-      end
+      if Frame.is_partial buf then self#end_track
   end
 
 let () =
-  let kind = Lang.univ_t 1 in
+  let kind = Lang.any in
+  let return_t = Lang.kind_type_of_kind_format kind in
   Lang.add_operator "delay"
     [
       ( "initial",
@@ -64,20 +67,20 @@ let () =
         Some (Lang.bool false),
         Some "Start in unavailable state, as if a track had just finished." );
       ( "",
-        Lang.float_t,
+        Lang.getter_t Lang.float_t,
         None,
         Some
           "The source won't be ready less than this amount of seconds after \
            any end of track" );
-      ("", Lang.source_t kind, None, None);
+      ("", Lang.source_t return_t, None, None);
     ]
-    ~category:Lang.TrackProcessing
-    ~descr:
-      "Prevents the child from being ready again too fast after a end of track"
-    ~kind:(Lang.Unconstrained kind)
-    (fun p kind ->
+    ~category:`Track
+    ~descr:"Make the source unavailable for a given time between tracks."
+    ~return_t
+    (fun p ->
       let f n = Lang.assoc "" n p in
-      let d = Lang.to_float (f 1) in
+      let d = Lang.to_float_getter (f 1) in
       let s = Lang.to_source (f 2) in
       let initial = Lang.to_bool (List.assoc "initial" p) in
+      let kind = Source.Kind.of_kind kind in
       new delay ~kind ~initial s d)
