@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,50 +20,52 @@
 
  *****************************************************************************)
 
+open Mm
 open Source
 
 class echo ~kind (source : source) delay feedback ping_pong =
-  let channels = (Frame.type_of_kind kind).Frame.audio in
-  object
-    inherit operator ~name:"echo" kind [source]
-
+  object (self)
+    inherit operator ~name:"echo" kind [source] as super
     method stype = source#stype
-
     method remaining = source#remaining
-
     method seek = source#seek
-
+    method self_sync = source#self_sync
     method is_ready = source#is_ready
-
     method abort_track = source#abort_track
+    val mutable effect = None
 
-    val effect =
-      Audio.Effect.delay channels
-        (Lazy.force Frame.audio_rate)
-        ~ping_pong (delay ()) (feedback ())
+    method private wake_up a =
+      super#wake_up a;
+      effect <-
+        Some
+          (Audio.Effect.delay self#audio_channels
+             (Lazy.force Frame.audio_rate)
+             ~ping_pong (delay ()) (feedback ()))
 
     val mutable past_pos = 0
 
     method private get_frame buf =
       let offset = AFrame.position buf in
       source#get buf;
-      let b = AFrame.content buf offset in
+      let b = AFrame.pcm buf in
       let position = AFrame.position buf in
+      let effect = Option.get effect in
       effect#set_delay (delay ());
       effect#set_feedback (feedback ());
-      effect#process b offset (position - offset)
+      effect#process (Audio.sub b offset (position - offset))
   end
 
 let () =
-  let k = Lang.kind_type_of_kind_format ~fresh:3 Lang.any_fixed in
+  let kind = Lang.audio_pcm in
+  let k = Lang.kind_type_of_kind_format kind in
   Lang.add_operator "echo"
     [
       ( "delay",
-        Lang.float_getter_t 1,
+        Lang.getter_t Lang.float_t,
         Some (Lang.float 0.5),
         Some "Delay in seconds." );
       ( "feedback",
-        Lang.float_getter_t 2,
+        Lang.getter_t Lang.float_t,
         Some (Lang.float (-6.)),
         Some "Feedback coefficient in dB (negative)." );
       ( "ping_pong",
@@ -72,9 +74,8 @@ let () =
         Some "Use ping-pong delay." );
       ("", Lang.source_t k, None, None);
     ]
-    ~kind:(Lang.Unconstrained k) ~category:Lang.SoundProcessing
-    ~descr:"Add echo."
-    (fun p kind ->
+    ~return_t:k ~category:`Audio ~descr:"Add echo."
+    (fun p ->
       let f v = List.assoc v p in
       let duration, feedback, pp, src =
         ( Lang.to_float_getter (f "delay"),
@@ -86,8 +87,8 @@ let () =
         (* Check the initial value, wrap the getter with a converter. *)
         if feedback () > 0. then
           raise
-            (Lang_errors.Invalid_value
-               (f "feedback", "feedback should be negative"));
+            (Error.Invalid_value (f "feedback", "feedback should be negative"));
         fun () -> Audio.lin_of_dB (feedback ())
       in
+      let kind = Source.Kind.of_kind kind in
       new echo ~kind src duration feedback pp)
