@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,29 +24,26 @@ open Source
 open Complex
 
 class fir ~kind (source : source) freq beta numcoeffs =
-  let channels = (Frame.type_of_kind kind).Frame.audio in
   object (self)
-    inherit operator ~name:"fir_filter" kind [source]
+    inherit operator ~name:"fir_filter" kind [source] as super
 
     (* Needed to compute RC *)
     val f1 = (1. -. beta) *. (freq /. float_of_int (Frame.audio_of_seconds 1.))
-
     val f2 = (1. +. beta) *. (freq /. float_of_int (Frame.audio_of_seconds 1.))
-
     val tau = 0.5 /. (freq /. float_of_int (Frame.audio_of_seconds 1.))
 
     (* Misc *)
     val mutable nzeros = numcoeffs - 1
-
     val mutable gain = 0.
+    val mutable xv = [||]
 
-    val mutable xv = Array.make_matrix channels numcoeffs 0.
+    method wake_up a =
+      super#wake_up a;
+      xv <- Array.make_matrix self#audio_channels numcoeffs 0.
 
     (* Coefficients *)
     val mutable xcoeffs = Array.make numcoeffs 0.
-
     val mutable circle = [||]
-
     val mutable temp = Array.make 2048 { re = 0.; im = 0. }
 
     initializer
@@ -55,16 +52,16 @@ class fir ~kind (source : source) freq beta numcoeffs =
        zeros=%d."
       (freq /. float_of_int (Frame.audio_of_seconds 1.))
       beta f1 f2 tau nzeros;
+
     (* Init circle *)
     let circle =
       let rec mkcircle n =
         if n < 0 then [||]
-        else begin
-          let theta = Utils.pi *. float_of_int n /. 1024. in
+        else (
+          let theta = Float.pi *. float_of_int n /. 1024. in
           Array.append
             (mkcircle (n - 1))
-            [| { re = cos theta; im = sin theta } |]
-        end
+            [| { re = cos theta; im = sin theta } |])
       in
       mkcircle 1024
     in
@@ -75,7 +72,7 @@ class fir ~kind (source : source) freq beta numcoeffs =
       match (f <= f1, f <= f2) with
         | true, _ -> 1.
         | false, true ->
-            0.5 *. (1. +. cos (Utils.pi *. tau /. beta *. (f -. f1)))
+            0.5 *. (1. +. cos (Float.pi *. tau /. beta *. (f -. f1)))
         | false, false -> 0.
     in
     for i = 0 to 1024 do
@@ -84,6 +81,7 @@ class fir ~kind (source : source) freq beta numcoeffs =
     for i = 1 to 1024 do
       vec.(2048 - i) <- vec.(i)
     done;
+
     (* FFT *)
     let ( +~ ), ( -~ ), ( *~ ) = (Complex.add, Complex.sub, Complex.mul) in
     let rec fft t d s n =
@@ -91,6 +89,7 @@ class fir ~kind (source : source) freq beta numcoeffs =
         let h = n / 2 in
         for i = 0 to h - 1 do
           !t.(s + i) <- !d.(s + (2 * i));
+
           (* even *)
           !t.(s + h + i) <- !d.(s + (2 * i) + 1) (* odd  *)
         done;
@@ -101,9 +100,10 @@ class fir ~kind (source : source) freq beta numcoeffs =
           let wkt = circle.(i * a) *~ !t.(s + h + i) in
           !d.(s + i) <- !t.(s + i) +~ wkt;
           !d.(s + h + i) <- !t.(s + i) -~ wkt
-        done )
+        done)
     in
     fft (ref temp) (ref vec) 0 2048;
+
     (* inverse fft *)
     let h = (numcoeffs - 1) / 2 in
     xcoeffs <-
@@ -118,19 +118,16 @@ class fir ~kind (source : source) freq beta numcoeffs =
 
     (* Digital filter based on mkfilter/mkshape/gencode by A.J. Fisher *)
     method stype = source#stype
-
     method remaining = source#remaining
-
     method is_ready = source#is_ready
-
     method seek = source#seek
-
     method abort_track = source#abort_track
+    method self_sync = source#self_sync
 
     method private get_frame buf =
       let offset = AFrame.position buf in
       source#get buf;
-      let b = AFrame.content buf offset in
+      let b = AFrame.pcm buf in
       let shift a =
         for i = 0 to Array.length a - 2 do
           a.(i) <- a.(i + 1)
@@ -148,14 +145,15 @@ class fir ~kind (source : source) freq beta numcoeffs =
       for c = 0 to 1 do
         for i = offset to AFrame.position buf - 1 do
           shift xv.(c);
-          xv.(c).(nzeros) <- b.(c).(i) /. gain;
-          b.(c).(i) <- fold_left2 addtimes 0. xcoeffs xv.(c)
+          xv.(c).(nzeros) <- b.(c).{i} /. gain;
+          b.(c).{i} <- fold_left2 addtimes 0. xcoeffs xv.(c)
         done
       done
   end
 
 let () =
-  let k = Lang.kind_type_of_kind_format ~fresh:1 Lang.any_fixed in
+  let kind = Lang.audio_pcm in
+  let k = Lang.kind_type_of_kind_format kind in
   Lang.add_operator "filter.fir"
     [
       ( "frequency",
@@ -168,9 +166,8 @@ let () =
       ("coeffs", Lang.int_t, Some (Lang.int 255), Some "Number of coefficients");
       ("", Lang.source_t k, None, None);
     ]
-    ~kind:(Lang.Unconstrained k) ~category:Lang.SoundProcessing
-    ~descr:"Low-pass FIR filter."
-    (fun p kind ->
+    ~return_t:k ~category:`Audio ~descr:"Low-pass FIR filter."
+    (fun p ->
       let f v = List.assoc v p in
       let freq, beta, num, src =
         ( Lang.to_float (f "frequency"),
@@ -178,4 +175,5 @@ let () =
           Lang.to_int (f "coeffs"),
           Lang.to_source (f "") )
       in
-      new fir ~kind src freq beta num)
+      let kind = Source.Kind.of_kind kind in
+      (new fir ~kind src freq beta num :> Source.source))
