@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,9 +20,11 @@
 
  *****************************************************************************)
 
+open Mm
 open Source
 open Extralib
-module Img = Image.RGBA32
+
+let () = Lang.add_module "video.frei0r"
 
 type t = Float | Int | Bool
 
@@ -45,17 +47,12 @@ class frei0r_filter ~kind ~name bgra instance params (source : source) =
   let dt = 1. /. float fps in
   object
     inherit operator ~name:("frei0r." ^ name) kind [source]
-
     method stype = source#stype
-
     method remaining = source#remaining
-
     method seek = source#seek
-
     method is_ready = source#is_ready
-
+    method self_sync = source#self_sync
     method abort_track = source#abort_track
-
     val mutable t = 0.
 
     method private get_frame buf =
@@ -63,14 +60,18 @@ class frei0r_filter ~kind ~name bgra instance params (source : source) =
         | None -> ()
         | Some (rgb, offset, length) ->
             params ();
-            let rgb = rgb.(0) in
+            let rgb = Frame_content.Video.get_data rgb in
             for i = offset to offset + length - 1 do
-              let img = rgb.(i) in
-              if bgra then Img.swap_rb img;
-              let src = Img.data (Img.copy img) in
-              let dst = Img.data img in
+              (* TODO: we could try to be more efficient than converting to/from RGBA32 and swap colors... *)
+              let img = Video.get rgb i in
+              let img = Image.YUV420.to_RGBA32 img in
+              if bgra then Image.RGBA32.swap_rb img;
+              let src = Image.RGBA32.data (Image.RGBA32.copy img) in
+              let dst = Image.RGBA32.data img in
               Frei0r.update1 instance t src dst;
-              if bgra then Img.swap_rb img;
+              if bgra then Image.RGBA32.swap_rb img;
+              let img = Image.YUV420.of_RGBA32 img in
+              Video.set rgb i img;
               t <- t +. dt
             done
   end
@@ -78,8 +79,8 @@ class frei0r_filter ~kind ~name bgra instance params (source : source) =
 class frei0r_mixer ~kind ~name bgra instance params (source : source) source2 =
   let fps = Lazy.force Frame.video_rate in
   let dt = 1. /. float fps in
-  object
-    inherit operator ~name:("frei0r." ^ name) kind [source; source2]
+  object (self)
+    inherit operator ~name:("frei0r." ^ name) kind [source; source2] as super
 
     method stype =
       match (source#stype, source2#stype) with
@@ -93,25 +94,35 @@ class frei0r_mixer ~kind ~name bgra instance params (source : source) source2 =
 
     method is_ready = source#is_ready && source2#is_ready
 
+    method self_sync =
+      match (source#self_sync, source2#self_sync) with
+        | (`Static, v), (`Static, v') when v = v' -> (`Static, v || v')
+        | (_, v), (_, v') -> (`Dynamic, v || v')
+
     method abort_track =
       source#abort_track;
       source2#abort_track
 
     val mutable t = 0.
+    val mutable tmp = Frame.dummy
 
-    val tmp = Frame.create kind
+    method private wake_up a =
+      super#wake_up a;
+      tmp <- Frame.create self#ctype
 
     method private get_frame buf =
       (* Prepare buffer for the second source
        * at the same position as final buffer. *)
       Frame.clear tmp;
       Frame.set_breaks tmp [Frame.position buf];
+
       (* Get content in respective buffers *)
       let c = VFrame.get_content buf source in
       let c2 = VFrame.get_content tmp source2 in
       match (c, c2) with
         | Some (rgb, offset, length), Some (rgb', offset', length') ->
             params ();
+
             (* Mix content where the two streams are available.
              * We could cut one stream when the other is too short,
              * and/or attempt to get some more data in the buffers...
@@ -119,18 +130,23 @@ class frei0r_mixer ~kind ~name bgra instance params (source : source) source2 =
              * because there's usually only one image per video frame. *)
             assert (offset = offset');
             let length = min length length' in
-            let rgb = rgb.(0) in
-            let rgb' = rgb'.(0) in
+            let rgb = Frame_content.Video.get_data rgb in
+            let rgb' = Frame_content.Video.get_data rgb' in
             for i = offset to offset + length - 1 do
-              let img = rgb.(i) in
-              let img' = rgb'.(i) in
-              if bgra then Img.swap_rb img;
-              if bgra then Img.swap_rb img';
-              let src = Img.data (Img.copy img) in
-              let src' = Img.data img' in
-              let dst = Img.data img in
+              (* TODO: we could try to be more efficient than converting to/from RGBA32 and swap colors... *)
+              let img = Video.get rgb i in
+              let img = Image.YUV420.to_RGBA32 img in
+              let img' = Video.get rgb' i in
+              let img' = Image.YUV420.to_RGBA32 img' in
+              if bgra then Image.RGBA32.swap_rb img;
+              if bgra then Image.RGBA32.swap_rb img';
+              let src = Image.RGBA32.data (Image.RGBA32.copy img) in
+              let src' = Image.RGBA32.data img' in
+              let dst = Image.RGBA32.data img in
               Frei0r.update2 instance t src src' dst;
-              if bgra then Img.swap_rb img;
+              if bgra then Image.RGBA32.swap_rb img;
+              let img = Image.YUV420.of_RGBA32 img in
+              Video.set rgb i img;
               t <- t +. dt
             done
         | _ -> ()
@@ -141,83 +157,59 @@ class frei0r_source ~kind ~name bgra instance params =
   let dt = 1. /. float fps in
   object
     inherit source ~name:("frei0r." ^ name) kind
-
     method stype = Infallible
-
     method is_ready = true
-
+    method self_sync = (`Static, false)
     val mutable must_fail = false
-
     method abort_track = must_fail <- true
-
     method remaining = if must_fail then 0 else -1
-
     val mutable t = 0.
 
     method private get_frame frame =
-      if must_fail then begin
+      if must_fail then (
         must_fail <- false;
-        VFrame.add_break frame (VFrame.position frame)
-      end
-      else begin
+        VFrame.add_break frame (VFrame.position frame))
+      else (
         params ();
         let start = VFrame.position frame in
         let stop = VFrame.size () in
-        let buf = VFrame.content_of_type frame ~channels:1 in
-        let buf = buf.(0) in
+        let rgb = VFrame.yuva420p frame in
         for i = start to stop - 1 do
-          let img = buf.(i) in
-          let buf = Img.data img in
-          Frei0r.update0 instance t buf;
-          if bgra then Img.swap_rb img;
+          let img = Video.get rgb i in
+          let img = Image.YUV420.to_RGBA32 img in
+          let dst = Image.RGBA32.data img in
+          Frei0r.update0 instance t dst;
+          if bgra then Image.RGBA32.swap_rb img;
+          let img = Image.YUV420.of_RGBA32 img in
+          Video.set rgb i img;
           t <- t +. dt
         done;
-        VFrame.add_break frame stop
-      end
+        VFrame.add_break frame stop)
   end
 
 (** Make a list of parameters. *)
 let params plugin info =
-  (* This is only to get default parameters... *)
-  let instance = Frei0r.create plugin 8 8 in
-  let utv = ref 0 in
   let liq_params =
     List.init info.Frei0r.num_params (fun i ->
         try
           let info = Frei0r.param_info plugin i in
           let name = Utils.normalize_parameter_string info.Frei0r.param_name in
-          let t, d =
+          let t =
             match info.Frei0r.param_type with
-              | Frei0r.Bool ->
-                  ( Lang.bool_t,
-                    Some (Lang.bool (Frei0r.get_param_bool instance i)) )
-              | Frei0r.Double ->
-                  ( Lang.float_getter_t
-                      ( incr utv;
-                        !utv ),
-                    Some (Lang.float (Frei0r.get_param_float instance i)) )
-              | Frei0r.Color ->
-                  let r, g, b = Frei0r.get_param_color instance i in
-                  let r = int_of_float (r *. 255.) in
-                  let g = int_of_float (g *. 255.) in
-                  let b = int_of_float (b *. 255.) in
-                  let v = Lang.int ((r lsl 16) + (g lsl 8) + b) in
-                  (Lang.int_t, Some v)
-              | Frei0r.Position ->
-                  let t = Lang.product_t Lang.float_t Lang.float_t in
-                  let x, y = Frei0r.get_param_position instance i in
-                  let x = Lang.float x in
-                  let y = Lang.float y in
-                  let v = Lang.product x y in
-                  (t, Some v)
-              | Frei0r.String ->
-                  ( Lang.string_t,
-                    Some (Lang.string (Frei0r.get_param_string instance i)) )
+              | Frei0r.Bool -> Lang.bool_t
+              | Frei0r.Double -> Lang.getter_t Lang.float_t
+              | Frei0r.Color -> Lang.int_t
+              | Frei0r.Position -> Lang.product_t Lang.float_t Lang.float_t
+              | Frei0r.String -> Lang.string_t
           in
-          Some (name, t, d, Some (info.Frei0r.param_explanation ^ "."))
+          Some
+            ( name,
+              Lang.nullable_t t,
+              Some Lang.null,
+              Some (info.Frei0r.param_explanation ^ ".") )
         with Exit -> None)
   in
-  let liq_params = List.may_map id liq_params in
+  let liq_params = List.filter_map id liq_params in
   (* Initialize parameters and produce function to update float getters. *)
   let params instance p =
     let on_changed x0 =
@@ -234,19 +226,20 @@ let params plugin info =
             let name =
               Utils.normalize_parameter_string info.Frei0r.param_name
             in
-            let v = f name in
-            match info.Frei0r.param_type with
-              | Frei0r.Bool ->
+            let v = Lang.to_option (f name) in
+            match (v, info.Frei0r.param_type) with
+              | None, _ -> None
+              | Some v, Frei0r.Bool ->
                   Frei0r.set_param_bool instance i (Lang.to_bool v);
                   None
-              | Frei0r.Double ->
+              | Some v, Frei0r.Double ->
                   let x = Lang.to_float_getter v in
                   let x0 = x () in
                   let f x = Frei0r.set_param_float instance i x in
                   let oc = on_changed x0 in
                   f x0;
                   Some (fun () -> oc f (x ()))
-              | Frei0r.Color ->
+              | Some v, Frei0r.Color ->
                   let c = Lang.to_int v in
                   let r = (c lsr 16) land 0xff in
                   let g = (c lsr 8) land 0xff in
@@ -256,18 +249,18 @@ let params plugin info =
                   let b = float b /. 255. in
                   Frei0r.set_param_color instance i (r, g, b);
                   None
-              | Frei0r.Position ->
+              | Some v, Frei0r.Position ->
                   let x, y = Lang.to_product v in
                   let x = Lang.to_float x in
                   let y = Lang.to_float y in
                   Frei0r.set_param_position instance i (x, y);
                   None
-              | Frei0r.String ->
+              | Some v, Frei0r.String ->
                   Frei0r.set_param_string instance i (Lang.to_string v);
                   None
           with Not_found -> None)
     in
-    let act = List.may_map id act in
+    let act = List.filter_map id act in
     fun () -> List.iter (fun f -> f ()) act
   in
   (liq_params, params)
@@ -292,14 +285,12 @@ let register_plugin fname =
       | Frei0r.Mixer3 -> (3, 1)
   in
   if inputs > 2 then raise Unhandled_number_of_inputs;
-  let k =
-    if inputs = 0 then Lang.video_only else Lang.any_fixed_with ~video:1 ()
-  in
-  let k = Lang.kind_type_of_kind_format ~fresh:1 k in
+  let kind = Lang.video_yuva420p in
+  let return_t = Lang.kind_type_of_kind_format kind in
   let liq_params, params = params plugin info in
   let liq_params =
     let inputs =
-      List.init inputs (fun _ -> ("", Lang.source_t k, None, None))
+      List.init inputs (fun _ -> ("", Lang.source_t return_t, None, None))
     in
     liq_params @ inputs
   in
@@ -318,9 +309,8 @@ let register_plugin fname =
     a
   in
   let descr = Printf.sprintf "%s (by %s)." explanation author in
-  Lang.add_operator ("video.frei0r." ^ name) liq_params
-    ~kind:(Lang.Unconstrained k) ~category:Lang.VideoProcessing ~flags:[] ~descr
-    (fun p kind ->
+  Lang.add_operator ("video.frei0r." ^ name) liq_params ~return_t
+    ~category:`Video ~flags:[`Extra] ~descr (fun p ->
       let instance =
         let width = Lazy.force Frame.video_width in
         let height = Lazy.force Frame.video_height in
@@ -328,13 +318,14 @@ let register_plugin fname =
       in
       let f v = List.assoc v p in
       let params = params instance p in
+      let kind = Source.Kind.of_kind kind in
       if inputs = 1 then (
         let source = Lang.to_source (f "") in
-        new frei0r_filter ~kind ~name bgra instance params source )
+        new frei0r_filter ~kind ~name bgra instance params source)
       else if inputs = 2 then (
         let source = Lang.to_source (f "") in
         let source' = Lang.to_source (Lang.assoc "" 2 p) in
-        new frei0r_mixer ~kind ~name bgra instance params source source' )
+        new frei0r_mixer ~kind ~name bgra instance params source source')
       else if inputs = 0 then new frei0r_source ~kind ~name bgra instance params
       else assert false)
 
@@ -362,4 +353,5 @@ let register_plugins () =
   in
   List.iter add plugin_dirs
 
-let () = if frei0r_enable then register_plugins ()
+let () =
+  Lifecycle.before_init (fun () -> if frei0r_enable then register_plugins ())
