@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -28,59 +28,39 @@ type filter_family = Butterworth | Resonator
 
 class iir ~kind (source : source) filter_family filter_type order freq1 freq2
   qfactor =
-  let channels = (Frame.type_of_kind kind).Frame.audio in
   let rate = float (Frame.audio_of_seconds 1.) in
   object (self)
     inherit operator ~name:"iir_filter" kind [source]
 
     (* Params *)
     val raw_alpha1 = freq1 /. rate
-
     val raw_alpha2 = freq2 /. rate
-
-    val warped_alpha1 = tan (Utils.pi *. freq1 /. rate) /. Utils.pi
-
-    val warped_alpha2 = tan (Utils.pi *. freq2 /. rate) /. Utils.pi
-
+    val warped_alpha1 = tan (Float.pi *. freq1 /. rate) /. Float.pi
+    val warped_alpha2 = tan (Float.pi *. freq2 /. rate) /. Float.pi
     val mutable gain = 0.
 
     (* Used for computation *)
     val mutable polemask = 0
-
     val mutable splane_poles = [||]
-
     val mutable splane_numpoles = 0
-
     val mutable splane_zeros = [||]
-
     val mutable splane_numzeros = 0
-
     val mutable zplane_poles = [||]
-
     val mutable zplane_numpoles = 0
-
     val mutable zplane_zeros = [||]
-
     val mutable zplane_numzeros = 0
-
     val mutable topcoeffs = [||]
-
     val mutable botcoeffs = [||]
-
     val mutable dc_gain = { re = 0.; im = 0. }
-
     val mutable fc_gain = { re = 0.; im = 0. }
-
     val mutable hf_gain = { re = 0.; im = 0. }
 
     (* Coefficients *)
     val mutable xcoeffs = [||]
-
     val mutable ycoeffs = [||]
 
     (* I/O shift registries *)
     val mutable xv = [||]
-
     val mutable yv = [||]
 
     initializer
@@ -88,6 +68,7 @@ class iir ~kind (source : source) filter_family filter_type order freq1 freq2
     self#log#info "Alpha 1: %.013f (warped: %.013f)" raw_alpha1 warped_alpha1;
     self#log#info "Alpha 2: %.013f (warped: %.013f)" raw_alpha2 warped_alpha2;
     self#log#info "Q: %.013f" qfactor;
+    let channels = self#audio_channels in
     let cor a = { re = a; im = 0. }
     and ( +~ ), ( -~ ), ( *~ ), ( /~ ) =
       (Complex.add, Complex.sub, Complex.mul, Complex.div)
@@ -109,6 +90,7 @@ class iir ~kind (source : source) filter_family filter_type order freq1 freq2
       for i = 0 to n - 1 do
         multin z.(i) n c
       done;
+
       (* check that computed coeffs of z^k are all real *)
       for i = 0 to n do
         if abs_float c.(i).im > 1e-10 then
@@ -133,30 +115,29 @@ class iir ~kind (source : source) filter_family filter_type order freq1 freq2
           (* Compute S-plane poles (Butterworth) *)
           self#log#info "This is a Butterworth filter.";
           let choosepole z =
-            if z.re < 0. then begin
+            if z.re < 0. then (
               self#log#info "z = %+.013f %+.013f i." z.re z.im;
-              if polemask mod 2 == 0 then begin
+              if polemask mod 2 == 0 then (
                 splane_poles <- Array.append splane_poles [| z |];
-                splane_numpoles <- splane_numpoles + 1
-              end;
-              polemask <- polemask lsl 1
-            end
+                splane_numpoles <- splane_numpoles + 1);
+              polemask <- polemask lsl 1)
           in
           self#log#info "Theta:";
           for i = 0 to (2 * order) - 1 do
             let theta =
               match order mod 2 with
-                | 1 -> float_of_int i *. Utils.pi /. float_of_int order
-                | 0 -> (float_of_int i +. 0.5) *. Utils.pi /. float_of_int order
+                | 1 -> float_of_int i *. Float.pi /. float_of_int order
+                | 0 -> (float_of_int i +. 0.5) *. Float.pi /. float_of_int order
                 | _ -> assert false
             in
             self#log#info "%d: %+.013f --> %+.013f %+.013f i." i theta
               (cos theta) (sin theta);
             choosepole { re = cos theta; im = sin theta }
           done;
+
           (* Normalize *)
-          let w1 = cor (2. *. Utils.pi *. warped_alpha1) in
-          let w2 = cor (2. *. Utils.pi *. warped_alpha2) in
+          let w1 = cor (2. *. Float.pi *. warped_alpha1) in
+          let w2 = cor (2. *. Float.pi *. warped_alpha2) in
           begin
             match filter_type with
             | Band_stop ->
@@ -223,6 +204,7 @@ class iir ~kind (source : source) filter_family filter_type order freq1 freq2
                 yv <- Array.make_matrix channels (order + 1) 0.
             | _ -> assert false
           end;
+
           (* Compute Z-plane zeros & poles using bilinear transform *)
           self#log#info "S-Plane zeros:";
           self#log#info "%s"
@@ -282,21 +264,22 @@ class iir ~kind (source : source) filter_family filter_type order freq1 freq2
           zplane_numpoles <- 2;
           zplane_numzeros <- 2;
           zplane_zeros <- [| cor 1.; cor (-1.) |];
+
           (* where we want the peak to be *)
-          let theta = 2. *. Utils.pi *. raw_alpha1 in
-          if qfactor == infinity then begin
+          let theta = 2. *. Float.pi *. raw_alpha1 in
+          if qfactor == infinity then (
             self#log#info "Infinite Q factor!";
+
             (* oscillator *)
             let zp = { re = cos theta; im = sin theta } in
-            zplane_poles <- [| zp; Complex.conj zp |]
-          end
-          else begin
+            zplane_poles <- [| zp; Complex.conj zp |])
+          else (
             (* must iterate to find exact pole positions *)
             topcoeffs <- expand zplane_zeros zplane_numzeros;
             let r = exp (cor (-.theta /. (2. *. qfactor)))
             and thm = ref theta
             and th1 = ref 0.
-            and th2 = ref Utils.pi
+            and th2 = ref Float.pi
             and cvg = ref false in
             for i = 0 to 50 do
               let zp = r *~ { re = cos !thm; im = sin !thm } in
@@ -311,33 +294,37 @@ class iir ~kind (source : source) filter_family filter_type order freq1 freq2
               if abs_float phi < 1e-10 then cvg := true;
               thm := 0.5 *. (!th1 +. !th2)
             done;
+
             (* if we failed to converge ... *)
-            assert !cvg
-          end;
+            assert !cvg);
           xv <- Array.make_matrix channels zplane_numzeros 0.;
           yv <- Array.make_matrix channels zplane_numpoles 0.;
+
           (* Do we need to transform to Bandstop or Allpass? *)
           match filter_type with
             | Band_stop ->
                 (* Band-stop filter *)
                 self#log#info "This is a band-stop filter.";
+
                 (* compute Z-plane pole & zero positions for bandstop
-                   resonator (notch filter) *)
+                      resonator (notch filter) *)
                 (* place zeros exactly *)
                 let zp = { re = cos theta; im = sin theta } in
                 zplane_poles <- [| zp; Complex.conj zp |]
             | All_pass ->
                 (* All-pass filter *)
                 self#log#info "This is an all-pass filter.";
+
                 (* compute Z-plane pole & zero positions for allpass
-                   resonator *)
+                      resonator *)
                 zplane_zeros.(0) <-
                   zplane_poles.(0) /~ sqrt (cor (Complex.norm zplane_poles.(0)));
                 zplane_zeros.(1) <-
                   zplane_poles.(1) /~ sqrt (cor (Complex.norm zplane_poles.(1)))
             | Band_pass -> ()
-            | _ -> assert false )
+            | _ -> assert false)
     end;
+
     (* Now expand the polynomials *)
     self#log#info "Expanding polynomials...";
     topcoeffs <- expand zplane_zeros zplane_numzeros;
@@ -356,10 +343,11 @@ class iir ~kind (source : source) filter_family filter_type order freq1 freq2
             (Array.mapi
                (fun i a -> Printf.sprintf "%d: %+.013f %+.013f i." i a.re a.im)
                botcoeffs)));
+
     (* Gain *)
     dc_gain <- evaluate topcoeffs botcoeffs { re = 1.; im = 0. };
     let theta =
-      2. *. Utils.pi *. 0.5 *. (raw_alpha1 +. raw_alpha2)
+      2. *. Float.pi *. 0.5 *. (raw_alpha1 +. raw_alpha2)
       (* jwt for centre freq. *)
     in
     fc_gain <-
@@ -376,6 +364,7 @@ class iir ~kind (source : source) filter_family filter_type order freq1 freq2
     self#log#info "Gains:";
     self#log#info "DC=%+.013f Centre=%+.013f HF=%+.013f Final=%+.013f."
       (Complex.norm dc_gain) (Complex.norm fc_gain) (Complex.norm hf_gain) gain;
+
     (* X-coeffs *)
     for i = 0 to zplane_numzeros do
       xcoeffs <-
@@ -387,6 +376,7 @@ class iir ~kind (source : source) filter_family filter_type order freq1 freq2
       (String.concat "\n"
          (Array.to_list
             (Array.mapi (fun i a -> Printf.sprintf "%d: %+.013f." i a) xcoeffs)));
+
     (* Y-coeffs *)
     for i = 0 to zplane_numpoles do
       ycoeffs <-
@@ -402,21 +392,17 @@ class iir ~kind (source : source) filter_family filter_type order freq1 freq2
 
     (* Digital filter based on mkfilter/mkshape/gencode by A.J. Fisher *)
     method stype = source#stype
-
     method remaining = source#remaining
-
     method seek = source#seek
-
+    method self_sync = source#self_sync
     method is_ready = source#is_ready
-
     method abort_track = source#abort_track
-
     val mutable v_offs = 0
 
     method private get_frame buf =
       let offset = AFrame.position buf in
       source#get buf;
-      let b = AFrame.content buf offset in
+      let b = AFrame.pcm buf in
       let v_len = Array.length xv.(0) in
       let coeffs_len = Array.length xcoeffs in
       let fold_left2 init v coeffs l =
@@ -427,59 +413,61 @@ class iir ~kind (source : source) filter_family filter_type order freq1 freq2
         done;
         !result
       in
-      for c = 0 to channels - 1 do
+      for c = 0 to self#audio_channels - 1 do
         let xvc = xv.(c) in
         let yvc = yv.(c) in
         let bc = b.(c) in
         for i = offset to AFrame.position buf - 1 do
           v_offs <- (v_offs + 1) mod v_len;
-          xvc.((coeffs_len - 1 + v_offs) mod v_len) <- bc.(i) /. gain;
+          xvc.((coeffs_len - 1 + v_offs) mod v_len) <- bc.{i} /. gain;
           let insert =
             fold_left2 0. xvc xcoeffs 110 (* TODO: why 110? *)
             +. fold_left2 0. yvc ycoeffs (coeffs_len - 1)
           in
           yvc.((coeffs_len - 1 + v_offs) mod v_len) <- insert;
-          bc.(i) <- insert
+          bc.{i} <- insert
         done
       done
   end
 
 let () =
-  let k = Lang.kind_type_of_kind_format ~fresh:1 Lang.any_fixed in
+  Lang.add_module "filter.iir";
+  Lang.add_module "filter.iir.butterworth";
+  Lang.add_module "filter.iir.resonator";
+  let kind = Lang.audio_pcm in
+  let k = Lang.kind_type_of_kind_format kind in
   Lang.add_operator "filter.iir.butterworth.high"
     [
       ("frequency", Lang.float_t, None, Some "Corner frequency");
       ("order", Lang.int_t, Some (Lang.int 4), Some "Filter order");
       ("", Lang.source_t k, None, None);
     ]
-    ~kind:(Lang.Unconstrained k) ~category:Lang.SoundProcessing
-    ~descr:"IIR filter"
-    (fun p kind ->
+    ~return_t:k ~category:`Audio ~descr:"IIR filter"
+    (fun p ->
       let f v = List.assoc v p in
       let freq, order, src =
         ( Lang.to_float (f "frequency"),
           Lang.to_int (f "order"),
           Lang.to_source (f "") )
       in
+      let kind = Source.Kind.of_kind kind in
       new iir ~kind src Butterworth High_pass order freq 0. 0.);
-
   Lang.add_operator "filter.iir.butterworth.low"
     [
       ("frequency", Lang.float_t, None, Some "Corner frequency");
       ("order", Lang.int_t, Some (Lang.int 4), Some "Filter order");
       ("", Lang.source_t k, None, None);
     ]
-    ~kind:(Lang.Unconstrained k) ~category:Lang.SoundProcessing
-    ~descr:"IIR filter"
-    (fun p kind ->
+    ~return_t:k ~category:`Audio ~descr:"IIR filter"
+    (fun p ->
       let f v = List.assoc v p in
       let freq, order, src =
         ( Lang.to_float (f "frequency"),
           Lang.to_int (f "order"),
           Lang.to_source (f "") )
       in
+      let kind = Source.Kind.of_kind kind in
       new iir ~kind src Butterworth Low_pass order freq 0. 0.);
-
   Lang.add_operator "filter.iir.butterworth.bandpass"
     [
       ("frequency1", Lang.float_t, None, Some "First corner frequency");
@@ -487,9 +475,8 @@ let () =
       ("order", Lang.int_t, Some (Lang.int 4), Some "Filter order");
       ("", Lang.source_t k, None, None);
     ]
-    ~kind:(Lang.Unconstrained k) ~category:Lang.SoundProcessing
-    ~descr:"IIR filter"
-    (fun p kind ->
+    ~return_t:k ~category:`Audio ~descr:"IIR filter"
+    (fun p ->
       let f v = List.assoc v p in
       let freq1, freq2, order, src =
         ( Lang.to_float (f "frequency1"),
@@ -497,8 +484,8 @@ let () =
           Lang.to_int (f "order"),
           Lang.to_source (f "") )
       in
+      let kind = Source.Kind.of_kind kind in
       new iir ~kind src Butterworth Band_pass order freq1 freq2 0.);
-
   Lang.add_operator "filter.iir.butterworth.bandstop"
     [
       ("frequency1", Lang.float_t, None, Some "First corner frequency");
@@ -506,9 +493,8 @@ let () =
       ("order", Lang.int_t, Some (Lang.int 4), Some "Filter order");
       ("", Lang.source_t k, None, None);
     ]
-    ~kind:(Lang.Unconstrained k) ~category:Lang.SoundProcessing
-    ~descr:"IIR filter"
-    (fun p kind ->
+    ~return_t:k ~category:`Audio ~descr:"IIR filter"
+    (fun p ->
       let f v = List.assoc v p in
       let freq1, freq2, order, src =
         ( Lang.to_float (f "frequency1"),
@@ -516,55 +502,53 @@ let () =
           Lang.to_int (f "order"),
           Lang.to_source (f "") )
       in
+      let kind = Source.Kind.of_kind kind in
       new iir ~kind src Butterworth Band_stop order freq1 freq2 0.);
-
   Lang.add_operator "filter.iir.resonator.bandpass"
     [
       ("frequency", Lang.float_t, None, Some "Corner frequency");
       ("q", Lang.float_t, Some (Lang.float 60.), Some "Quality factor");
       ("", Lang.source_t k, None, None);
     ]
-    ~kind:(Lang.Unconstrained k) ~category:Lang.SoundProcessing
-    ~descr:"IIR filter"
-    (fun p kind ->
+    ~return_t:k ~category:`Audio ~descr:"IIR filter"
+    (fun p ->
       let f v = List.assoc v p in
       let freq, q, src =
         ( Lang.to_float (f "frequency"),
           Lang.to_float (f "q"),
           Lang.to_source (f "") )
       in
+      let kind = Source.Kind.of_kind kind in
       new iir ~kind src Resonator Band_pass 0 freq 0. q);
-
   Lang.add_operator "filter.iir.resonator.bandstop"
     [
       ("frequency", Lang.float_t, None, Some "Corner frequency");
       ("q", Lang.float_t, Some (Lang.float 60.), Some "Quality factor");
       ("", Lang.source_t k, None, None);
     ]
-    ~kind:(Lang.Unconstrained k) ~category:Lang.SoundProcessing
-    ~descr:"IIR filter"
-    (fun p kind ->
+    ~return_t:k ~category:`Audio ~descr:"IIR filter"
+    (fun p ->
       let f v = List.assoc v p in
       let freq, q, src =
         ( Lang.to_float (f "frequency"),
           Lang.to_float (f "q"),
           Lang.to_source (f "") )
       in
+      let kind = Source.Kind.of_kind kind in
       new iir ~kind src Resonator Band_pass 0 freq 0. q);
-
   Lang.add_operator "filter.iir.resonator.allpass"
     [
       ("frequency", Lang.float_t, None, Some "Corner frequency");
       ("q", Lang.float_t, Some (Lang.float 60.), Some "Quality factor");
       ("", Lang.source_t k, None, None);
     ]
-    ~kind:(Lang.Unconstrained k) ~category:Lang.SoundProcessing
-    ~descr:"IIR filter"
-    (fun p kind ->
+    ~return_t:k ~category:`Audio ~descr:"IIR filter"
+    (fun p ->
       let f v = List.assoc v p in
       let freq, q, src =
         ( Lang.to_float (f "frequency"),
           Lang.to_float (f "q"),
           Lang.to_source (f "") )
       in
+      let kind = Source.Kind.of_kind kind in
       new iir ~kind src Resonator Band_pass 0 freq 0. q)
