@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -27,11 +27,9 @@
 class max_duration ~kind ~override_meta ~duration source =
   object (self)
     inherit Source.operator ~name:"max_duration" kind []
-
     val mutable remaining = duration
-
     val mutable s : Source.source = source
-
+    method self_sync = source#self_sync
     method private set_clock = Clock.unify self#clock s#clock
 
     method private wake_up activation =
@@ -39,11 +37,8 @@ class max_duration ~kind ~override_meta ~duration source =
       s#get_ready activation
 
     method private sleep = s#leave (self :> Source.operator)
-
     method stype = Source.Fallible
-
     method is_ready = remaining > 0 && s#is_ready
-
     method abort_track = s#abort_track
 
     method remaining =
@@ -61,11 +56,10 @@ class max_duration ~kind ~override_meta ~duration source =
                 if lbl = override_meta then (
                   try
                     let v = float_of_string v in
-                    remaining <- Frame.master_of_seconds v;
+                    remaining <- Frame.main_of_seconds v;
                     self#log#info "Overriding remaining value: %.02f." v
                   with _ ->
-                    self#log#important "Invalid remaining override value: %s." v
-                  ))
+                    self#log#important "Invalid remaining override value: %s." v))
               m)
         (Frame.get_all_metadata buf)
 
@@ -74,15 +68,15 @@ class max_duration ~kind ~override_meta ~duration source =
       s#get buf;
       self#check_for_override ~offset buf;
       remaining <- remaining - Frame.position buf + offset;
-      if remaining <= 0 then begin
+      if remaining <= 0 then (
         s#leave (self :> Source.source);
-        s <- (new Blank.empty ~kind :> Source.source);
-        s#get_ready [(self :> Source.source)]
-      end
+        s <- Debug_sources.empty kind;
+        s#get_ready [(self :> Source.source)])
   end
 
 let () =
-  let k = Lang.univ_t 1 in
+  let kind = Lang.any in
+  let return_t = Lang.kind_type_of_kind_format kind in
   Lang.add_operator "max_duration"
     [
       ( "override",
@@ -92,14 +86,14 @@ let () =
           "Metadata field which, if present and containing a float, overrides \
            the remaining play time." );
       ("", Lang.float_t, None, Some "Maximum duration");
-      ("", Lang.source_t k, None, None);
+      ("", Lang.source_t return_t, None, None);
     ]
-    ~category:Lang.TrackProcessing ~descr:"Limit source duration"
-    ~kind:(Lang.Unconstrained k)
-    (fun p kind ->
+    ~category:`Track ~descr:"Limit source duration" ~return_t
+    (fun p ->
       let override_meta = Lang.to_string (List.assoc "override" p) in
       let duration =
-        Frame.master_of_seconds (Lang.to_float (Lang.assoc "" 1 p))
+        Frame.main_of_seconds (Lang.to_float (Lang.assoc "" 1 p))
       in
       let s = Lang.to_source (Lang.assoc "" 2 p) in
+      let kind = Source.Kind.of_kind kind in
       new max_duration ~kind ~override_meta ~duration s)
