@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,20 +20,17 @@
 
  *****************************************************************************)
 
+open Mm
 open Source
 
 class virtual base ~kind ~name (source : source) =
   object
     inherit operator ~name kind [source]
-
     method stype = source#stype
-
     method remaining = source#remaining
-
     method seek = source#seek
-
+    method self_sync = source#self_sync
     method is_ready = source#is_ready
-
     method abort_track = source#abort_track
   end
 
@@ -42,9 +39,8 @@ class merge ~kind (source : source) out =
     inherit base ~kind source ~name:"midi.merge_all"
 
     method private get_frame buf =
-      let offset = MFrame.position buf in
       source#get buf;
-      let m = MFrame.content buf offset in
+      let m = MFrame.midi buf in
       for c = 0 to Array.length m - 1 do
         MIDI.merge m.(out) m.(c);
         if c <> out then MIDI.clear_all m.(c)
@@ -56,42 +52,39 @@ class remove ~kind (source : source) t =
     inherit base ~kind source ~name:"midi.remove"
 
     method private get_frame buf =
-      let offset = MFrame.position buf in
       source#get buf;
-      let m = MFrame.content buf offset in
+      let m = MFrame.midi buf in
       List.iter (fun c -> if c < Array.length m then MIDI.clear_all m.(c)) t
   end
 
 let () =
-  let k =
-    Lang.kind_type_of_kind_format ~fresh:1 (Lang.any_fixed_with ~midi:1 ())
-  in
+  let kind = { Frame.audio = `Any; video = `Any; midi = Frame.midi_native } in
+  let k = Lang.kind_type_of_kind_format kind in
   Lang.add_operator "midi.merge_all"
     [
       ("track_out", Lang.int_t, Some (Lang.int 0), Some "Destination track.");
       ("", Lang.source_t k, None, None);
     ]
-    ~kind:(Lang.Unconstrained k) ~category:Lang.MIDIProcessing
-    ~descr:"Merge all MIDI tracks in one."
-    (fun p kind ->
+    ~return_t:k ~category:`MIDI ~descr:"Merge all MIDI tracks in one."
+    (fun p ->
       let f v = List.assoc v p in
       let out = Lang.to_int (f "track_out") in
       let src = Lang.to_source (f "") in
+      let kind = Source.Kind.of_kind kind in
       new merge ~kind src out)
 
 let () =
-  let k =
-    Lang.kind_type_of_kind_format ~fresh:1 (Lang.any_fixed_with ~midi:1 ())
-  in
+  let kind = { Frame.audio = `Any; video = `Any; midi = Frame.midi_native } in
+  let k = Lang.kind_type_of_kind_format kind in
   Lang.add_operator "midi.remove"
     [
       ("", Lang.list_t Lang.int_t, None, Some "Tracks to remove.");
       ("", Lang.source_t k, None, None);
     ]
-    ~kind:(Lang.Unconstrained k) ~category:Lang.MIDIProcessing
-    ~descr:"Remove MIDI tracks."
-    (fun p kind ->
+    ~return_t:k ~category:`MIDI ~descr:"Remove MIDI tracks."
+    (fun p ->
       (* let f v = List.assoc v p in *)
       let t = List.map Lang.to_int (Lang.to_list (Lang.assoc "" 1 p)) in
       let src = Lang.to_source (Lang.assoc "" 2 p) in
+      let kind = Source.Kind.of_kind kind in
       new remove ~kind src t)
