@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -26,37 +26,27 @@ let ( -- ) = Int64.sub
 let ( ++ ) = Int64.add
 
 let ticks_of_offset offset =
-  Int64.of_float (offset *. float (Lazy.force Frame.master_rate))
+  Int64.of_float (offset *. float (Lazy.force Frame.main_rate))
 
 class on_offset ~kind ~force ~offset ~override f s =
   object (self)
     inherit Source.operator ~name:"on_offset" kind [s]
-
     inherit Latest_metadata.source
-
     method stype = s#stype
-
     method is_ready = s#is_ready
-
     method remaining = s#remaining
-
     method abort_track = s#abort_track
-
     method seek n = s#seek n
-
+    method self_sync = s#self_sync
     val mutable elapsed = 0L
-
     val mutable offset = ticks_of_offset offset
-
     val mutable executed = false
 
     method private execute =
       self#log#info "Executing on_offset callback.";
-      let pos =
-        Int64.to_float elapsed /. float (Lazy.force Frame.master_rate)
-      in
+      let pos = Int64.to_float elapsed /. float (Lazy.force Frame.main_rate) in
       ignore
-        (Lang.apply ~t:Lang.unit_t f
+        (Lang.apply f
            [("", Lang.float pos); ("", Lang.metadata latest_metadata)]);
       executed <- true
 
@@ -82,16 +72,16 @@ class on_offset ~kind ~force ~offset ~override f s =
       let new_pos = Int64.of_int (Frame.position ab) in
       elapsed <- elapsed ++ new_pos -- pos;
       if (not executed) && offset <= elapsed then self#execute;
-      if Frame.is_partial ab then begin
+      if Frame.is_partial ab then (
         if force && not executed then self#execute;
         executed <- false;
         self#clear_latest_metadata;
-        elapsed <- 0L
-      end
+        elapsed <- 0L)
   end
 
 let () =
-  let kind = Lang.univ_t 1 in
+  let kind = Lang.any in
+  let return_t = Lang.kind_type_of_kind_format kind in
   Lang.add_operator "on_offset"
     [
       ( "offset",
@@ -122,17 +112,18 @@ let () =
            the current track, second is the latest metadata. That function \
            should be fast because it is executed in the main streaming thread."
       );
-      ("", Lang.source_t kind, None, None);
+      ("", Lang.source_t return_t, None, None);
     ]
-    ~category:Lang.TrackProcessing
+    ~category:`Track
     ~descr:
       "Call a given handler when position in track is equal or more than a \
        given amount of time."
-    ~kind:(Lang.Unconstrained kind)
-    (fun p kind ->
+    ~return_t
+    (fun p ->
       let offset = Lang.to_float (List.assoc "offset" p) in
       let force = Lang.to_bool (List.assoc "force" p) in
       let override = Lang.to_string (List.assoc "override" p) in
       let f = Lang.assoc "" 1 p in
       let s = Lang.to_source (Lang.assoc "" 2 p) in
+      let kind = Source.Kind.of_kind kind in
       new on_offset ~kind ~offset ~force ~override f s)
