@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -28,11 +28,15 @@ open Source
   * while looping on the last source -- this behavior would not be suited
   * to the current use of [sequence] in transitions. *)
 class sequence ~kind ?(merge = false) sources =
+  let self_sync_type = Utils.self_sync_type sources in
   object (self)
     inherit operator ~name:"sequence" kind sources
-
     val mutable seq_sources = sources
 
+    method self_sync =
+      ( Lazy.force self_sync_type,
+        match sources with hd :: _ -> snd hd#self_sync | [] -> false )
+
     method stype =
       match List.rev sources with hd :: _ -> hd#stype | [] -> Source.Fallible
 
@@ -61,73 +65,66 @@ class sequence ~kind ?(merge = false) sources =
     method remaining =
       if merge then (
         let ( + ) a b = if a < 0 || b < 0 then -1 else a + b in
-        List.fold_left ( + ) 0 (List.map (fun s -> s#remaining) seq_sources) )
+        List.fold_left ( + ) 0 (List.map (fun s -> s#remaining) seq_sources))
       else (List.hd seq_sources)#remaining
 
     method abort_track =
-      if merge then begin
+      if merge then (
         match List.rev seq_sources with
           | [] -> assert false
-          | hd :: _ -> seq_sources <- [hd]
-      end;
+          | hd :: _ -> seq_sources <- [hd]);
       match seq_sources with hd :: _ -> hd#abort_track | _ -> ()
 
     method private get_frame buf =
-      if head_ready then begin
+      if head_ready then (
         let hd = List.hd seq_sources in
         hd#get buf;
-        if Frame.is_partial buf then begin
+        if Frame.is_partial buf then (
           head_ready <- false;
-          if List.length seq_sources > 1 then begin
+          if List.length seq_sources > 1 then (
             seq_sources <- List.tl seq_sources;
             if merge && self#is_ready then (
               let pos = Frame.position buf in
               self#get_frame buf;
               Frame.set_breaks buf
-                (Utils.remove_one (( = ) pos) (Frame.breaks buf)) )
-          end
-        end
-      end
-      else begin
+                (Utils.remove_one (( = ) pos) (Frame.breaks buf))))))
+      else (
         match seq_sources with
           | a :: (_ :: _ as tl) ->
               if a#is_ready then head_ready <- true else seq_sources <- tl;
               self#get_frame buf
           | [a] ->
               assert a#is_ready;
+
               (* Our #is_ready ensures that. *)
               head_ready <- true;
               self#get_frame buf
-          | [] -> assert false
-      end
+          | [] -> assert false)
   end
 
 class merge_tracks ~kind source =
   object (self)
     inherit operator ~name:"sequence" kind [source]
-
     method stype = source#stype
-
     method is_ready = source#is_ready
-
     method abort_track = source#abort_track
-
     method remaining = -1
+    method self_sync = source#self_sync
 
     method private get_frame buf =
       source#get buf;
-      if Frame.is_partial buf && source#is_ready then begin
+      if Frame.is_partial buf && source#is_ready then (
         self#log#info "End of track: merging.";
         self#get_frame buf;
         Frame.set_breaks buf
-          ( match Frame.breaks buf with
+          (match Frame.breaks buf with
             | b :: _ :: l -> b :: l
-            | _ -> assert false )
-      end
+            | _ -> assert false))
   end
 
 let () =
-  let k = Lang.univ_t 1 in
+  let kind = Lang.any in
+  let k = Lang.kind_type_of_kind_format kind in
   Lang.add_operator "sequence"
     [
       ( "merge",
@@ -139,24 +136,28 @@ let () =
            merge_tracks() if you need that too." );
       ("", Lang.list_t (Lang.source_t k), None, None);
     ]
-    ~category:Lang.TrackProcessing
+    ~category:`Track
     ~descr:
       "Play only one track of every successive source, except for the last one \
        which is played as much as available."
-    ~kind:(Lang.Unconstrained k)
-    (fun p kind ->
+    ~return_t:k
+    (fun p ->
+      let kind = Source.Kind.of_kind kind in
       new sequence
         ~kind
         ~merge:(Lang.to_bool (List.assoc "merge" p))
         (Lang.to_source_list (List.assoc "" p)))
 
 let () =
-  let k = Lang.univ_t 1 in
+  let kind = Lang.any in
+  let k = Lang.kind_type_of_kind_format kind in
   Lang.add_operator "merge_tracks"
     [("", Lang.source_t k, None, None)]
-    ~category:Lang.TrackProcessing
+    ~category:`Track
     ~descr:
       "Merge consecutive tracks from the input source. They will be considered \
        as one big track, so `on_track()` will not trigger for example."
-    ~kind:(Lang.Unconstrained k)
-    (fun p kind -> new merge_tracks ~kind (Lang.to_source (List.assoc "" p)))
+    ~return_t:k
+    (fun p ->
+      let kind = Source.Kind.of_kind kind in
+      new merge_tracks ~kind (Lang.to_source (List.assoc "" p)))
