@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,60 +20,58 @@
 
  *****************************************************************************)
 
+open Mm
 open Source
 
-class bpm ~kind (source : source) cb every =
-  let every = Frame.audio_of_seconds every in
-  object
-    inherit operator ~name:"bpm" kind [source]
-
+class bpm ~kind (source : source) =
+  object (self)
+    inherit operator ~name:"bpm" kind [source] as super
     method stype = source#stype
-
     method is_ready = source#is_ready
-
+    method self_sync = source#self_sync
     method remaining = source#remaining
-
     method seek = source#seek
-
     method abort_track = source#abort_track
+    val mutable bpm = None
 
-    val bpm =
-      Soundtouch.BPM.make (Frame.type_of_kind kind).Frame.audio
-        (Lazy.force Frame.audio_rate)
-
-    val mutable n = 0
+    method wake_up a =
+      super#wake_up a;
+      bpm <-
+        Some
+          (Soundtouch.BPM.make
+             (Frame_content.Audio.channels_of_format self#ctype.Frame.audio)
+             (Lazy.force Frame.audio_rate))
 
     method private get_frame buf =
+      let bpm = Option.get bpm in
       let offset = AFrame.position buf in
       source#get buf;
       let len = AFrame.position buf - offset in
-      Soundtouch.BPM.put_samples_ni bpm (AFrame.content buf offset) offset len;
-      n <- n + len;
-      if n >= every then (
-        n <- 0;
-        let bpm = Soundtouch.BPM.get_bpm bpm in
-        ignore (cb [("", Lang.float bpm)]) )
+      let buf = AFrame.pcm buf in
+      let ibuf = Audio.interleave (Audio.sub buf offset len) in
+      Soundtouch.BPM.put_samples_ba bpm ibuf
+
+    method bpm =
+      match bpm with Some bpm -> Soundtouch.BPM.get_bpm bpm | None -> 0.
   end
 
 let () =
-  let k = Lang.kind_type_of_kind_format ~fresh:1 Lang.any_fixed in
+  let kind = Lang.audio_pcm in
+  let k = Lang.kind_type_of_kind_format kind in
   Lang.add_operator "bpm"
-    [
-      ( "every",
-        Lang.float_t,
-        Some (Lang.float 1.),
-        Some "Interval at which BPM is computed (in second)." );
-      ( "",
-        Lang.fun_t [(false, "", Lang.float_t)] Lang.unit_t,
-        None,
-        Some "Callback function." );
-      ("", Lang.source_t k, None, None);
-    ]
-    ~kind:(Lang.Unconstrained k) ~category:Lang.SoundProcessing
-    ~descr:"Detect the BPM." ~flags:[]
-    (fun p kind ->
-      let f v = List.assoc v p in
-      let every = Lang.to_float (f "every") in
-      let cb = Lang.to_fun (Lang.assoc "" 1 p) ~t:Lang.unit_t in
-      let s = Lang.to_source (Lang.assoc "" 2 p) in
-      new bpm ~kind s cb every)
+    [("", Lang.source_t k, None, None)]
+    ~return_t:k ~category:`Visualization
+    ~descr:
+      "Detect the BPM (number of beats per minute). The returned source has a \
+       method `bpm`, which can be called to compute it."
+    ~meth:
+      [
+        ( "bpm",
+          ([], Lang.fun_t [] Lang.float_t),
+          "Compute the current BPM.",
+          fun s -> Lang.val_fun [] (fun _ -> Lang.float s#bpm) );
+      ]
+    (fun p ->
+      let s = Lang.to_source (List.assoc "" p) in
+      let kind = Source.Kind.of_kind kind in
+      new bpm ~kind s)
