@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -29,16 +29,7 @@ open Source
 
 (* A transition is a value of type (source,source) -> source *)
 type transition = Lang.value
-
-type child = {
-  source : source;
-  transition : transition;
-  (* Also remind the current metadata in order to be able to restore it
-   * when switching back in the middle of a track. This is an expected
-   * behaviour, but notice that it makes extra assumptions about the nature of
-   * metadata, tying them to the notion of track. *)
-  mutable cur_meta : Request.metadata option;
-}
+type child = { source : source; transition : transition }
 
 (** The switch can either happen at any time in the stream (insensitive)
   * or only at track limits (sensitive). *)
@@ -46,20 +37,30 @@ type track_mode = Sensitive | Insensitive
 
 class virtual switch ~kind ~name ~override_meta ~transition_length
   ?(mode = fun () -> true) ?(replay_meta = true) (cases : child list) =
+  let sources = ref (List.map (fun c -> c.source) cases) in
+  let failed = ref false in
+  let () =
+    List.iter
+      (Lang.iter_sources
+         ~on_reference:(fun () -> failed := true)
+         (fun s -> sources := s :: !sources))
+      (List.map (fun c -> c.transition) cases)
+  in
+  let self_sync_type =
+    if !failed then lazy `Dynamic else Utils.self_sync_type !sources
+  in
   object (self)
     inherit operator ~name kind (List.map (fun x -> x.source) cases)
-
     val mutable transition_length = transition_length
-
     val mutable selected : (child * source) option = None
 
-    (** We have to explictly manage our children as they are dynamically created
+    (** We have to explicitly manage our children as they are dynamically created
     * by application of the transition functions. In particular we need a list
     * of all children that have output data in the current round. *)
     val mutable to_finish = []
 
     (** Indicates that the former child was left without having finished its
-    * track, in which case the switch will artifically produce an EOT. *)
+    * track, in which case the switch will artificially produce an EOT. *)
     val mutable need_eot = false
 
     (** The selection method should return None or Some c,
@@ -83,6 +84,7 @@ class virtual switch ~kind ~name ~override_meta ~transition_length
     method after_output =
       (* Advance the memo frame. *)
       self#advance;
+
       (* Propagate to all sub-sources, i.e. our children and the transitions
        * wrapping them:
        *  - current transition is [selected],
@@ -95,6 +97,7 @@ class virtual switch ~kind ~name ~override_meta ~transition_length
       end;
       List.iter (fun s -> s#after_output) to_finish;
       to_finish <- [];
+
       (* Selection may have been triggered by a call to #is_ready, without
        * any call to #get_ready (in particular if #select returned None).
        * It is cleared here in order to get a chance to be re-computed later. *)
@@ -124,6 +127,19 @@ class virtual switch ~kind ~name ~override_meta ~transition_length
 
     method is_ready = need_eot || selected <> None || self#cached_select <> None
 
+    (* This one is tricky. We do not want to call #cached_select as
+       this requires some run-time info from underlying sources
+       (mostly ctype to be set). The only case that matters if no
+       sources are selected is to know if we are [`Static, false] for
+       any caller such as [cross]. Since the source is not ready, we
+       can return anything so we check if any source might be not
+       self_sync in this case *)
+    method self_sync =
+      ( Lazy.force self_sync_type,
+        match selected with
+          | Some (_, source) -> snd source#self_sync
+          | None -> List.exists (fun c -> snd c.source#self_sync) cases )
+
     method private get_frame ab =
       (* Choose the next child to be played.
        * [forget] tells that the current child has finished its track,
@@ -140,6 +156,7 @@ class virtual switch ~kind ~name ~override_meta ~transition_length
               match selected with
                 | None ->
                     self#log#important "Switch to %s." c.source#id;
+
                     (* The source is already ready, this call is only there for
                      * allowing an uniform treatment of switches, triggering
                      * a #leave call. *)
@@ -153,7 +170,8 @@ class virtual switch ~kind ~name ~override_meta ~transition_length
                     to_finish <- old_s :: to_finish;
                     Clock.collect_after (fun () ->
                         let old_source =
-                          if forget then Blank.empty kind else old_c.source
+                          if forget then Debug_sources.empty kind
+                          else old_c.source
                         in
                         let new_source =
                           (* Force insertion of old metadata if relevant.
@@ -163,29 +181,30 @@ class virtual switch ~kind ~name ~override_meta ~transition_length
                            * else (this is thanks to Frame.get_chunk).
                            * A quicker hack might have been doable if there wasn't a
                            * transition in between. *)
-                          match c.cur_meta with
+                          match c.source#last_metadata with
                             | Some m when replay_meta ->
                                 new Insert_metadata.replay ~kind m c.source
                             | _ -> c.source
                         in
                         let s =
-                          let t =
-                            Lang.source_t (Lang.kind_type_of_frame_kind kind)
-                          in
                           Lang.to_source
-                            (Lang.apply ~t c.transition
+                            (Lang.apply c.transition
                                [
                                  ("", Lang.source old_source);
                                  ("", Lang.source new_source);
                                ])
                         in
                         let s =
-                          new Max_duration.max_duration
-                            ~kind ~override_meta ~duration:transition_length s
-                        in
-                        let s =
-                          new Sequence.sequence
-                            ~kind ~merge:true [s; new_source]
+                          match s#id with
+                            | id when id = new_source#id -> s
+                            | _ ->
+                                let s =
+                                  new Max_duration.max_duration
+                                    ~kind ~override_meta
+                                    ~duration:transition_length s
+                                in
+                                new Sequence.sequence
+                                  ~kind ~merge:true [s; new_source]
                         in
                         Clock.unify s#clock self#clock;
                         s#get_ready activation;
@@ -202,39 +221,22 @@ class virtual switch ~kind ~name ~override_meta ~transition_length
                     selected <- None
                 | None -> ())
       in
-      (* #select is called only when selected=None, and the cache is cleared
+      (* #select is called only when selected=None, and the cache is cleared	
        * as soon as the new selection is set. *)
       assert (selected = None || cached_selected = None);
-      if need_eot then begin
+      if need_eot then (
         need_eot <- false;
-        Frame.add_break ab (Frame.position ab)
-      end
+        Frame.add_break ab (Frame.position ab))
       else (
         match selected with
           | None ->
               reselect ~forget:true ();
+
               (* Our #is_ready, and caching, ensure the following. *)
               assert (selected <> None);
               self#get_frame ab
-          | Some (c, s) ->
+          | Some (_, s) ->
               s#get ab;
-              c.cur_meta <-
-                (if Frame.is_partial ab then None
-                else (
-                  match
-                    List.fold_left
-                      (function
-                        | None -> fun (p, m) -> Some (p, m)
-                        | Some (curp, curm) ->
-                            fun (p, m) ->
-                              Some (if p >= curp then (p, m) else (curp, curm)))
-                      (match c.cur_meta with
-                        | None -> None
-                        | Some m -> Some (-1, m))
-                      (Frame.get_all_metadata ab)
-                  with
-                    | None -> None
-                    | Some (_, m) -> Some (Hashtbl.copy m)));
               if Frame.is_partial ab then reselect ~forget:true ()
               else if not (mode ()) then reselect ())
 
@@ -245,75 +247,22 @@ class virtual switch ~kind ~name ~override_meta ~transition_length
       match selected with Some (_, s) -> s#abort_track | None -> ()
 
     method seek n = match selected with Some (_, s) -> s#seek n | None -> 0
+    method selected = Option.map (fun (_, s) -> s) selected
   end
 
 (** Common tools for Lang bindings of switch operators *)
 
-let common kind =
-  [
-    ( "track_sensitive",
-      Lang.bool_getter_t 0,
-      Some (Lang.bool true),
-      Some "Re-select only on end of tracks." );
-    ( "transition_length",
-      Lang.float_t,
-      Some (Lang.float 5.),
-      Some "Maximun transition duration." );
-    ( "override",
-      Lang.string_t,
-      Some (Lang.string "liq_transition_length"),
-      Some
-        "Metadata field which, if present and containing a float, overrides \
-         the `transition_length` parameter." );
-    ( "replay_metadata",
-      Lang.bool_t,
-      Some (Lang.bool true),
-      Some
-        "Replay the last metadata of a child when switching to it in the \
-         middle of a track." );
-    (let transition_t =
-       Lang.fun_t
-         [(false, "", Lang.source_t kind); (false, "", Lang.source_t kind)]
-         (Lang.source_t kind)
-     in
-     ( "transitions",
-       Lang.list_t transition_t,
-       Some (Lang.list ~t:transition_t []),
-       Some "Transition functions, padded with `fun (x,y) -> y` functions." ));
-  ]
-
-let default_transition k =
-  let t = Lang.source_t (Lang.kind_type_of_frame_kind k) in
-  Lang.val_fun [("", "x", t, None); ("", "y", t, None)] ~ret_t:t (fun e _ ->
-      List.assoc "y" e)
-
-let extract_common ~kind p l =
-  let ts = Lang.to_bool_getter (List.assoc "track_sensitive" p) in
-  let tr =
-    let tr = Lang.to_list (List.assoc "transitions" p) in
-    let ltr = List.length tr in
-    if ltr > l then
-      raise
-        (Lang_errors.Invalid_value
-           (List.assoc "transitions" p, "Too many transitions"));
-    if ltr < l then tr @ Utils.make_list (l - ltr) (default_transition kind)
-    else tr
-  in
-  let replay = Lang.to_bool (List.assoc "replay_metadata" p) in
-  let tl =
-    Frame.master_of_seconds (Lang.to_float (List.assoc "transition_length" p))
-  in
-  let override_meta = Lang.to_string (List.assoc "override" p) in
-  (replay, ts, tr, tl, override_meta)
+let default_transition =
+  Lang.val_fun [("", "x", None); ("", "y", None)] (fun e -> List.assoc "y" e)
 
 (** Switch: switch according to user-defined predicates. *)
 
-let satisfied f = Lang.to_bool (Lang.apply ~t:Lang.bool_t f [])
+let satisfied f = Lang.to_bool (Lang.apply f [])
 
 let trivially_true = function
   | {
       Lang.value =
-        Lang.Fun (_, _, _, { Lang_values.term = Lang_values.Bool true; _ });
+        Lang.Fun (_, _, _, { Term.term = Term.(Ground (Ground.Bool true)); _ });
       _;
     } ->
       true
@@ -321,8 +270,21 @@ let trivially_true = function
 
 let third (_, _, s) = s
 
-class lang_switch ~kind ~override_meta ~transition_length mode ?replay_meta
-  (children : (Lang.value * bool * child) list) =
+(** Like [List.find] but evaluates [f] on every element when [strict] is
+    [true]. *)
+let find ?(strict = false) f l =
+  let rec aux = function
+    | x :: l ->
+        if f x then (
+          if strict then List.iter (fun x -> ignore (f x)) l;
+          x)
+        else aux l
+    | [] -> raise Not_found
+  in
+  aux l
+
+class lang_switch ~kind ~override_meta ~all_predicates ~transition_length mode
+  ?replay_meta (children : (Lang.value * bool * child) list) =
   object
     inherit
       switch
@@ -338,7 +300,7 @@ class lang_switch ~kind ~override_meta ~transition_length mode ?replay_meta
       try
         Some
           (third
-             (List.find
+             (find ~strict:all_predicates
                 (fun (d, single, s) ->
                   (* Check single constraints *)
                   (if selected s then not single else true)
@@ -357,29 +319,77 @@ class lang_switch ~kind ~override_meta ~transition_length mode ?replay_meta
   end
 
 let () =
-  let kind = Lang.univ_t 1 in
+  let kind = Lang.any in
+  let return_t = Lang.kind_type_of_kind_format kind in
   let pred_t = Lang.fun_t [] Lang.bool_t in
-  let proto =
+  Lang.add_operator "switch" ~category:`Track
+    ~descr:
+      "At the beginning of a track, select the first source whose predicate is \
+       true."
+    ~meth:
+      [
+        ( "selected",
+          ( [],
+            Lang.fun_t []
+              (Lang.nullable_t Lang.(source_t (kind_type_of_kind_format kind)))
+          ),
+          "Currently selected source.",
+          fun s ->
+            Lang.val_fun [] (fun _ ->
+                match s#selected with
+                  | Some s -> Lang.source s
+                  | None -> Lang.null) );
+      ]
     [
+      ( "track_sensitive",
+        Lang.getter_t Lang.bool_t,
+        Some (Lang.bool true),
+        Some "Re-select only on end of tracks." );
+      ( "transition_length",
+        Lang.float_t,
+        Some (Lang.float 5.),
+        Some "Maximum transition duration." );
+      ( "override",
+        Lang.string_t,
+        Some (Lang.string "liq_transition_length"),
+        Some
+          "Metadata field which, if present and containing a float, overrides \
+           the `transition_length` parameter." );
+      ( "replay_metadata",
+        Lang.bool_t,
+        Some (Lang.bool true),
+        Some
+          "Replay the last metadata of a child when switching to it in the \
+           middle of a track." );
+      ( "all_predicates",
+        Lang.bool_t,
+        Some (Lang.bool false),
+        Some "Always evaluate all predicates when re-selecting." );
+      (let transition_t =
+         Lang.fun_t
+           [
+             (false, "", Lang.source_t return_t);
+             (false, "", Lang.source_t return_t);
+           ]
+           (Lang.source_t return_t)
+       in
+       ( "transitions",
+         Lang.list_t transition_t,
+         Some (Lang.list []),
+         Some "Transition functions, padded with `fun (x,y) -> y` functions." ));
       ( "single",
         Lang.list_t Lang.bool_t,
-        Some (Lang.list ~t:Lang.bool_t []),
+        Some (Lang.list []),
         Some
           "Forbid the selection of a branch for two tracks in a row. The empty \
            list stands for `[false,...,false]`." );
       ( "",
-        Lang.list_t (Lang.product_t pred_t (Lang.source_t kind)),
+        Lang.list_t (Lang.product_t pred_t (Lang.source_t return_t)),
         None,
         Some "Sources with the predicate telling when they can be played." );
     ]
-  in
-  Lang.add_operator "switch" ~category:Lang.TrackProcessing
-    ~descr:
-      "At the beginning of a track, select the first source whose predicate is \
-       true."
-    (common kind @ proto)
-    ~kind:(Lang.Unconstrained kind)
-    (fun p kind ->
+    ~return_t
+    (fun p ->
       let children =
         List.map
           (fun p ->
@@ -387,191 +397,45 @@ let () =
             (pred, Lang.to_source s))
           (Lang.to_list (List.assoc "" p))
       in
-      let replay_meta, ts, tr, tl, override_meta =
-        extract_common ~kind p (List.length children)
+      let ts = Lang.to_bool_getter (List.assoc "track_sensitive" p) in
+      let tr =
+        let l = List.length children in
+        let tr = Lang.to_list (List.assoc "transitions" p) in
+        let ltr = List.length tr in
+        if ltr > l then
+          raise
+            (Error.Invalid_value
+               (List.assoc "transitions" p, "Too many transitions"));
+        if ltr < l then tr @ List.init (l - ltr) (fun _ -> default_transition)
+        else tr
       in
+      let replay_meta = Lang.to_bool (List.assoc "replay_metadata" p) in
+      let tl =
+        Frame.main_of_seconds (Lang.to_float (List.assoc "transition_length" p))
+      in
+      let override_meta = Lang.to_string (List.assoc "override" p) in
+      let all_predicates = Lang.to_bool (List.assoc "all_predicates" p) in
       let singles =
         List.map Lang.to_bool (Lang.to_list (List.assoc "single" p))
       in
       let singles =
-        if singles = [] then Utils.make_list (List.length children) false
+        if singles = [] then List.init (List.length children) (fun _ -> false)
         else singles
       in
       let children =
         List.map2
-          (fun t (f, s) -> (f, { source = s; cur_meta = None; transition = t }))
+          (fun t (f, s) -> (f, { source = s; transition = t }))
           tr children
       in
       let children =
         try List.map2 (fun (d, s) single -> (d, single, s)) children singles
         with Invalid_argument s when s = "List.map2" ->
           raise
-            (Lang_errors.Invalid_value
+            (Error.Invalid_value
                ( List.assoc "single" p,
                  "there should be exactly one flag per children" ))
       in
+      let kind = Source.Kind.of_kind kind in
       new lang_switch
-        ~kind ~replay_meta ~override_meta ~transition_length:tl ts children)
-
-(** Fallback selector: switch to the first ready source. *)
-class fallback ~kind ~override_meta ~transition_length ?replay_meta mode
-  children =
-  object
-    inherit
-      switch
-        ~name:"fallback" ~kind ~mode ~override_meta ~transition_length
-          ?replay_meta children
-
-    method private select =
-      try Some (List.find (fun s -> s.source#is_ready) children)
-      with Not_found -> None
-
-    method stype =
-      if List.exists (fun s -> s.source#stype = Infallible) children then
-        Infallible
-      else Fallible
-  end
-
-let () =
-  let kind = Lang.univ_t 1 in
-  let proto =
-    [
-      ( "",
-        Lang.list_t (Lang.source_t kind),
-        None,
-        Some "Select the first ready source in this list." );
-    ]
-  in
-  Lang.add_operator "fallback" ~category:Lang.TrackProcessing
-    ~descr:"At the beginning of each track, select the first ready child."
-    (common kind @ proto)
-    ~kind:(Lang.Unconstrained kind)
-    (fun p kind ->
-      let children = Lang.to_source_list (List.assoc "" p) in
-      let replay_meta, ts, tr, tl, override_meta =
-        extract_common ~kind p (List.length children)
-      in
-      let children =
-        List.map2
-          (fun t s -> { transition = t; cur_meta = None; source = s })
-          tr children
-      in
-      new fallback
-        ~kind ~replay_meta ~override_meta ~transition_length:tl ts children)
-
-(** Random switch *)
-exception Found of child option
-
-class random ~kind ~override_meta ~transition_length ?replay_meta strict mode
-  children =
-  let name = if strict then "rotate" else "random" in
-  object
-    inherit
-      switch
-        ~name ~kind ~override_meta ~transition_length ?replay_meta ~mode
-          (List.map snd children) as super
-
-    val mutable position = -1
-
-    val mutable track_pending = false
-
-    val mutable tracks_played = 0
-
-    method private select =
-      let children_count = List.length children in
-      try
-        if children_count = 0 then raise (Found None);
-
-        (* First try to select another track from the source
-           currently playing. *)
-        if strict && position <> -1 then (
-          let weight, s = List.nth children position in
-          if s.source#is_ready && tracks_played < weight () then (
-            track_pending <- true;
-            raise (Found (Some s))));
-
-        (* Otherwise, select the next source to be played. Respect original
-           list order but skip over unavailale sources. *)
-        tracks_played <- 0;
-        track_pending <- true;
-        let ready_list, ready_count =
-          List.fold_left
-            (fun (l, n) (_, s) ->
-              if s.source#is_ready then (Some s :: l, n + 1) else (None :: l, n))
-            ([], 0) children
-        in
-        let ready_list = List.rev ready_list in
-
-        if ready_count = 0 then raise (Found None);
-
-        let rec f () =
-          if strict then position <- (position + 1) mod children_count
-          else position <- Random.int children_count;
-          match List.nth ready_list position with
-            | Some s -> raise (Found (Some s))
-            | None -> f ()
-        in
-        f ()
-      with Found s -> s
-
-    method stype =
-      if List.exists (fun (_, s) -> s.source#stype = Infallible) children then
-        Infallible
-      else Fallible
-
-    method private get_frame buf =
-      if track_pending then (
-        track_pending <- false;
-        tracks_played <- tracks_played + 1);
-      super#get_frame buf
-  end
-
-let () =
-  let add name strict descr weight_descr =
-    let kind = Lang.univ_t 1 in
-    Lang.add_operator name ~descr ~category:Lang.TrackProcessing
-      (common kind
-      @ [
-          ( "weights",
-            Lang.list_t (Lang.int_getter_t 2),
-            Some (Lang.list ~t:(Lang.int_getter_t 2) []),
-            Some weight_descr );
-          ("", Lang.list_t (Lang.source_t kind), None, None);
-        ])
-      ~kind:(Lang.Unconstrained kind)
-      (fun p kind ->
-        let children = Lang.to_source_list (List.assoc "" p) in
-        let replay_meta, ts, tr, tl, override_meta =
-          extract_common ~kind p (List.length children)
-        in
-        let weights =
-          List.map Lang.to_int_getter (Lang.to_list (List.assoc "weights" p))
-        in
-        let weights =
-          if weights <> [] then weights
-          else Utils.make_list (List.length children) (fun () -> 1)
-        in
-        let children =
-          if List.length weights <> List.length children then
-            raise
-              (Lang_errors.Invalid_value
-                 ( List.assoc "weights" p,
-                   "there should be as many weights as sources" ));
-          List.map2
-            (fun w c -> (w, c))
-            weights
-            (List.map2
-               (fun tr s -> { transition = tr; source = s; cur_meta = None })
-               tr children)
-        in
-        new random
-          ~kind ~replay_meta ~override_meta ~transition_length:tl strict ts
-          children)
-  in
-  add "random" false
-    "At the beginning of every track, select a random ready child."
-    "Weights of the children (padded with 1), defining for each child the \
-     probability that it is selected.";
-  add "rotate" true "Rotate between the sources."
-    "Weights of the children (padded with 1), defining for each child how many \
-     tracks are played from it per round, if that many are actually available."
+        ~kind ~replay_meta ~override_meta ~all_predicates ~transition_length:tl
+        ts children)
