@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,8 +20,8 @@
 
  *****************************************************************************)
 
+open Mm
 open Source
-module Img = Image.RGBA32
 
 class text ~kind init render_text ttf ttf_size color tx ty speed cycle meta text
   (source : source) =
@@ -30,35 +30,26 @@ class text ~kind init render_text ttf ttf_size color tx ty speed cycle meta text
   let video_width = Lazy.force Frame.video_width in
   object (self)
     inherit operator ~name:"video.add_text" kind [source]
-
     method stype = source#stype
-
     method remaining = source#remaining
-
     method seek = source#seek
-
     method is_ready = source#is_ready
-
     method abort_track = source#abort_track
-
+    method self_sync = source#self_sync
     val mutable text_frame = None
-
     val mutable pos_x = tx ()
-
     val mutable pos_y = ty ()
-
     val mutable font = None
-
     val mutable cur_text = text ()
 
     method private render_text text =
-      let w, h, get_pixel = render_text ~font:ttf ~size:ttf_size text in
-      let tf = Img.create w h in
+      let w, h, get_pixel_rgba = render_text ~font:ttf ~size:ttf_size text in
+      let tf = Video.Image.create w h in
       let tr, tg, tb = Image.RGB8.Color.of_int color in
       for y = 0 to h - 1 do
         for x = 0 to w - 1 do
-          let a = get_pixel x y in
-          Img.set_pixel tf x y (tr, tg, tb, a)
+          let a = get_pixel_rgba x y in
+          Video.Image.set_pixel_rgba tf x y (tr, tg, tb, a)
         done
       done;
       text_frame <- Some tf
@@ -68,15 +59,14 @@ class text ~kind init render_text ttf ttf_size color tx ty speed cycle meta text
         | Some tf -> tf
         | None ->
             self#render_text cur_text;
-            Utils.get_some text_frame
+            Option.get text_frame
 
     method private get_frame ab =
       match VFrame.get_content ab source with
-        | None -> ()
         | Some (rgb, off, len) ->
-            let rgb = rgb.(0) in
+            let rgb = Frame_content.Video.get_data rgb in
             let tf = self#get_text_frame in
-            let tfw = Img.width tf in
+            let tfw = Video.Image.width tf in
             let text =
               match meta with
                 | None -> text ()
@@ -90,25 +80,28 @@ class text ~kind init render_text ttf ttf_size color tx ty speed cycle meta text
             in
             if cur_text <> text then (
               cur_text <- text;
-              self#render_text cur_text );
+              self#render_text cur_text);
             for i = off to off + len - 1 do
               if speed = 0 then (
-                Img.add tf rgb.(i) ~x:pos_x ~y:pos_y;
+                Video.Image.add tf (Video.get rgb i) ~x:pos_x ~y:pos_y;
                 pos_x <- tx ();
-                pos_y <- ty () )
+                pos_y <- ty ())
               else (
-                if pos_x <> -tfw then Img.add tf rgb.(i) ~x:pos_x ~y:pos_y;
+                if pos_x <> -tfw then
+                  Video.Image.add tf (Video.get rgb i) ~x:pos_x ~y:pos_y;
                 pos_x <- pos_x - speed;
                 if pos_x < -tfw then
                   if cycle then pos_x <- video_width
-                  else pos_x <- -tfw (* avoid overflows *) )
+                  else pos_x <- -tfw (* avoid overflows *))
             done
+        | _ -> ()
   end
 
 let register name init render_text =
-  let k =
-    Lang.kind_type_of_kind_format ~fresh:2 (Lang.any_fixed_with ~video:1 ())
+  let kind =
+    { Frame.audio = `Any; video = Frame.video_yuva420p; midi = `Any }
   in
+  let k = Lang.kind_type_of_kind_format kind in
   let add_operator op =
     Lang.add_operator op
       [
@@ -122,8 +115,8 @@ let register name init render_text =
           Lang.int_t,
           Some (Lang.int 0xffffff),
           Some "Text color (in 0xRRGGBB format)." );
-        ("x", Lang.int_getter_t 1, Some (Lang.int 10), Some "x offset.");
-        ("y", Lang.int_getter_t 2, Some (Lang.int 10), Some "y offset.");
+        ("x", Lang.getter_t Lang.int_t, Some (Lang.int 10), Some "x offset.");
+        ("y", Lang.getter_t Lang.int_t, Some (Lang.int 10), Some "y offset.");
         ( "speed",
           Lang.int_t,
           Some (Lang.int 70),
@@ -135,19 +128,16 @@ let register name init render_text =
           Some (Lang.bool true),
           Some "Cycle text when it reaches left boundary." );
         ( "metadata",
-          Lang.string_t,
-          Some (Lang.string ""),
-          Some
-            "Change text on a particular metadata (empty string means \
-             disabled)." );
-        ("", Lang.string_getter_t 1, None, Some "Text to display.");
+          Lang.nullable_t Lang.string_t,
+          Some Lang.null,
+          Some "Change text on a particular metadata." );
+        ("", Lang.getter_t Lang.string_t, None, Some "Text to display.");
         ("", Lang.source_t k, None, None);
       ]
-      ~kind:(Lang.Unconstrained k) ~category:Lang.VideoProcessing
-      ~descr:"Display a text."
-      (fun p kind ->
+      ~return_t:k ~category:`Video ~descr:"Display a text."
+      (fun p ->
         let f v = List.assoc v p in
-        let ttf, ttf_size, color, x, y, speed, cycle, meta, txt, source =
+        let ttf, ttf_size, color, x, y, speed, cycle, txt, source =
           ( Lang.to_string (f "font"),
             Lang.to_int (f "size"),
             Lang.to_int (f "color"),
@@ -155,15 +145,17 @@ let register name init render_text =
             Lang.to_int_getter (f "y"),
             Lang.to_int (f "speed"),
             Lang.to_bool (f "cycle"),
-            Lang.to_string (f "metadata"),
             Lang.to_string_getter (Lang.assoc "" 1 p),
             Lang.to_source (Lang.assoc "" 2 p) )
         in
         let speed = speed / Lazy.force Frame.video_rate in
-        let meta = if meta = "" then None else Some meta in
-        ( new text
-            ~kind init render_text ttf ttf_size color x y speed cycle meta txt
-            source
-          :> Source.source ))
+        let meta =
+          Lang.to_valued_option Lang.to_string (List.assoc "metadata" p)
+        in
+        let kind = Source.Kind.of_kind kind in
+        (new text
+           ~kind init render_text ttf ttf_size color x y speed cycle meta txt
+           source
+          :> Source.source))
   in
   add_operator ("video.add_text." ^ name)
