@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,30 +20,40 @@
 
  *****************************************************************************)
 
+open Mm
+open Tsdl
+open Tsdl_ttf
+
 let init () =
   Sdl_utils.init [];
-  Sdl_utils.ttf_init ()
+  Sdl_utils.check Tsdl_ttf.Ttf.init ()
 
 let get_font font size =
-  try Sdlttf.open_font font size with
-    | Sdlttf.SDLttf_exception s ->
-        raise (Lang_errors.Invalid_value (Lang.string font, s))
-    | e ->
-        raise
-          (Lang_errors.Invalid_value (Lang.string font, Printexc.to_string e))
+  let font = if font = "" then Configure.default_font else font in
+  try Sdl_utils.check (Ttf.open_font font) size
+  with e ->
+    raise
+      (Error.Invalid_value
+         (Lang.string font, Printexc.to_string e ^ "(font: " ^ font ^ ")"))
 
 let render_text ~font ~size text =
   let text = if text = "" then " " else text in
   let font = get_font font size in
+  let white = Sdl.Color.create ~r:0xff ~g:0xff ~b:0xff ~a:0xff in
+  let black = Sdl.Color.create ~r:0x00 ~g:0x00 ~b:0x00 ~a:0xff in
   let ts =
-    Sdlttf.render_utf8_shaded font text ~bg:Sdlvideo.black ~fg:Sdlvideo.white
-  in
-  let w, h =
-    let si = Sdlvideo.surface_info ts in
-    (si.Sdlvideo.w, si.Sdlvideo.h)
+    Sdl_utils.check (fun () -> Ttf.render_utf8_shaded font text white black) ()
   in
+  let img = Sdl_utils.Surface.to_img ts in
+  let w = Video.Image.width img in
+  let h = Video.Image.height img in
+  Sdl.free_surface ts;
+
+  (* TODO: improve performance *)
   let get_pixel x y =
-    let r, _, _ = Sdlvideo.get_pixel_color ts ~x ~y in
+    assert (0 <= x && x < w);
+    assert (0 <= y && y < h);
+    let r, _, _, _ = Video.Image.get_pixel_rgba img x y in
     r
   in
   (w, h, get_pixel)
