@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,34 +25,35 @@ open Source
 type mode = RMS | Peak
 
 class window ~kind mode duration source =
-  let channels = (Frame.type_of_kind kind).Frame.audio in
-  object
+  object (self)
     inherit
       operator
         kind [source]
-        ~name:(match mode with RMS -> "rms" | Peak -> "peak")
+        ~name:(match mode with RMS -> "rms" | Peak -> "peak") as super
 
     method stype = source#stype
-
     method is_ready = source#is_ready
-
     method remaining = source#remaining
-
     method seek = source#seek
-
     method abort_track = source#abort_track
+    method self_sync = source#self_sync
 
     (** Accumulator (e.g. sum of squares). *)
-    val acc = Array.make channels 0.
+    val mutable acc = [||]
 
-    (** Duration of the accumlated data. *)
+    (** Duration of the accumulated data (in samples). *)
     val mutable acc_dur = 0
 
     (** Last computed value (rms or peak). *)
-    val mutable value = Array.make channels 0.
+    val mutable value = [||]
+
+    method wake_up a =
+      super#wake_up a;
+      let channels = self#audio_channels in
+      acc <- Array.make channels 0.;
+      value <- Array.make channels 0.
 
     val m = Mutex.create ()
-
     method value = Tutils.mutexify m (fun () -> value) ()
 
     method private get_frame buf =
@@ -62,10 +63,10 @@ class window ~kind mode duration source =
       if duration > 0. then (
         let duration = Frame.audio_of_seconds duration in
         let position = AFrame.position buf in
-        let buf = AFrame.content buf offset in
+        let buf = AFrame.pcm buf in
         for i = offset to position - 1 do
-          for c = 0 to channels - 1 do
-            let x = buf.(c).(i) in
+          for c = 0 to self#audio_channels - 1 do
+            let x = buf.(c).{i} in
             match mode with
               | RMS -> acc.(c) <- acc.(c) +. (x *. x)
               | Peak -> acc.(c) <- max acc.(c) (abs_float x)
@@ -74,7 +75,7 @@ class window ~kind mode duration source =
           if acc_dur >= duration then (
             let dur = float acc_dur in
             let value' =
-              Array.init channels (fun i ->
+              Array.init self#audio_channels (fun i ->
                   match mode with
                     | RMS ->
                         let v = sqrt (acc.(i) /. dur) in
@@ -86,50 +87,43 @@ class window ~kind mode duration source =
                         v)
             in
             acc_dur <- 0;
-            Tutils.mutexify m (fun () -> value <- value') () )
-        done )
+            Tutils.mutexify m (fun () -> value <- value') ())
+        done)
   end
 
-let declare mode suffix format fun_ret_t f_ans =
-  let k = Lang.kind_type_of_kind_format ~fresh:3 format in
-  let return_t = Lang.product_t (Lang.fun_t [] fun_ret_t) (Lang.source_t k) in
+let declare mode suffix kind fun_ret_t f_ans =
   let name = match mode with RMS -> "rms" | Peak -> "peak" in
   let doc = match mode with RMS -> "RMS volume" | Peak -> "peak volume" in
-  Lang.add_builtin (name ^ suffix)
-    ~category:(Lang.string_of_category Lang.Visualization)
+  let return_t = Lang.kind_type_of_kind_format kind in
+  Lang.add_operator (name ^ suffix) ~category:`Audio
+    ~meth:
+      [
+        ( name,
+          ([], Lang.fun_t [] fun_ret_t),
+          "Current value for the " ^ doc ^ ".",
+          fun s -> Lang.val_fun [] (fun _ -> f_ans s#value) );
+      ]
+    ~return_t
     ~descr:
-      ( "Get current " ^ doc
-      ^ " of the source. Returns a pair `(f,s)` where s is a new source and \
-         `f` is a function of type `() -> float` and returns the current " ^ doc
-      ^ " of the source, with `0.0 <= " ^ doc
-      ^ " <= 1.0`. Some operators like `amplify` and `compress` can produce \
-         amplitudes greater than 1.0 if misconfigured, inducing clipping in \
-         the output." )
+      ("Get current " ^ doc
+     ^ " of the source. Returns the source with a method `" ^ name
+     ^ "` to compute the current " ^ doc ^ " of the source, with `0.0 <= " ^ doc
+     ^ " <= 1.0`.")
     [
-      ( "id",
-        Lang.string_t,
-        Some (Lang.string ""),
-        Some "Force the value of the source ID." );
       ( "duration",
-        Lang.float_getter_t 2,
+        Lang.getter_t Lang.float_t,
         Some (Lang.float 0.5),
         Some
           "Duration of the window (in seconds). A value <= 0, means that \
            computation should not be performed." );
-      ("", Lang.source_t k, None, None);
+      ("", Lang.source_t return_t, None, None);
     ]
-    return_t
-    (fun p t ->
+    (fun p ->
       let f v = List.assoc v p in
       let src = Lang.to_source (f "") in
-      let id = Lang.to_string (f "id") in
       let duration = Lang.to_float_getter (f "duration") in
-      let _, t = Lang.of_product_t t in
-      let kind = Lang.frame_kind_of_kind_type (Lang.of_source_t t) in
-      let s = new window ~kind mode duration src in
-      if id <> "" then s#set_id id;
-      let f = Lang.val_fun [] ~ret_t:fun_ret_t (fun _ _ -> f_ans s#value) in
-      Lang.product f (Lang.source (s :> Source.source)))
+      let kind = Source.Kind.of_kind kind in
+      new window ~kind mode duration src)
 
 let () =
   let mean value =
@@ -140,13 +134,8 @@ let () =
   let stereo value =
     Lang.product (Lang.float value.(0)) (Lang.float value.(1))
   in
-  declare RMS "" Lang.any_fixed Lang.float_t mean;
-  declare RMS ".stereo"
-    (Lang.any_fixed_with ~audio:2 ())
-    (Lang.product_t Lang.float_t Lang.float_t)
-    stereo;
-  declare Peak "" Lang.any_fixed Lang.float_t mean;
-  declare Peak ".stereo"
-    (Lang.any_fixed_with ~audio:2 ())
-    (Lang.product_t Lang.float_t Lang.float_t)
-    stereo
+  let kind = { Frame.audio = Frame.audio_stereo; video = `Any; midi = `Any } in
+  declare RMS "" Lang.audio_pcm Lang.float_t mean;
+  declare RMS ".stereo" kind (Lang.product_t Lang.float_t Lang.float_t) stereo;
+  declare Peak "" Lang.audio_pcm Lang.float_t mean;
+  declare Peak ".stereo" kind (Lang.product_t Lang.float_t Lang.float_t) stereo
