@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
 
  *****************************************************************************)
 
+open Mm
+
 (* Buffered ALSA output *)
 
 open Alsa
@@ -27,11 +29,7 @@ open Alsa
 class output ~kind ~clock_safe ~infallible ~on_stop ~on_start ~start dev source
   =
   let buffer_length = AFrame.size () in
-  let buffer_chans = (Frame.type_of_kind kind).Frame.audio in
   let alsa_buffer = Alsa_settings.alsa_buffer#get in
-  let blank () =
-    Array.init buffer_chans (fun _ -> Array.make buffer_length 0.)
-  in
   let nb_blocks = Alsa_settings.conf_buffer_length#get in
   let samples_per_second = Lazy.force Frame.audio_rate in
   let periods = Alsa_settings.periods#get in
@@ -39,10 +37,16 @@ class output ~kind ~clock_safe ~infallible ~on_stop ~on_start ~start dev source
   object (self)
     inherit
       Output.output
-        ~infallible ~on_stop ~on_start ~content_kind:kind ~name
-          ~output_kind:"output.alsa" source start as super
+        ~infallible ~on_stop ~on_start ~content_kind:(Source.Kind.of_kind kind)
+          ~name ~output_kind:"output.alsa" source start as super
 
-    inherit [float array array] IoRing.output ~nb_blocks ~blank as ioring
+    inherit [Frame_content.Audio.data] IoRing.output ~nb_blocks as ioring
+    val mutable initialized = false
+
+    method wake_up a =
+      super#wake_up a;
+      let blank () = Audio.make self#audio_channels buffer_length 0. in
+      ioring#init blank
 
     method private set_clock =
       super#set_clock;
@@ -50,22 +54,22 @@ class output ~kind ~clock_safe ~infallible ~on_stop ~on_start ~start dev source
         Clock.unify self#clock
           (Clock.create_known (Alsa_settings.get_clock () :> Clock.clock))
 
-    method output_start =
-      ioring#output_start;
-      if clock_safe then (Alsa_settings.get_clock ())#register_blocking_source
-
-    method output_stop =
-      ioring#output_stop;
-      if clock_safe then (Alsa_settings.get_clock ())#unregister_blocking_source
-
     val mutable device = None
-
+    method self_sync = (`Dynamic, device <> None)
     val mutable alsa_rate = samples_per_second
+    val mutable samplerate_converter = None
 
-    val samplerate_converter = Audio_converter.Samplerate.create buffer_chans
+    method samplerate_converter =
+      (* Defer the creation of the converter until we know the kind and thus the number of channels. *)
+      match samplerate_converter with
+        | Some samplerate_converter -> samplerate_converter
+        | None ->
+            let sc = Audio_converter.Samplerate.create self#audio_channels in
+            samplerate_converter <- Some sc;
+            sc
 
     val mutable alsa_write =
-      fun pcm buf ofs len -> Pcm.writen_float pcm buf ofs len
+      fun pcm buf ofs len -> Pcm.writen_float_ba pcm (Audio.sub buf ofs len)
 
     method get_device =
       match device with
@@ -74,23 +78,27 @@ class output ~kind ~clock_safe ~infallible ~on_stop ~on_start ~start dev source
             self#log#important "Using ALSA %s." (Alsa.get_version ());
             let dev = Pcm.open_pcm dev [Pcm.Playback] [] in
             let params = Pcm.get_params dev in
+            let channels = self#audio_channels in
             let bufsize, periods =
-              ( try
-                  Pcm.set_access dev params Pcm.Access_rw_noninterleaved;
-                  Pcm.set_format dev params Pcm.Format_float
-                with _ ->
-                  (* If we can't get floats we fallback on interleaved s16le *)
-                  self#log#severe "Falling back on interleaved S16LE";
-                  Pcm.set_access dev params Pcm.Access_rw_interleaved;
-                  Pcm.set_format dev params Pcm.Format_s16_le;
-                  alsa_write <-
-                    (fun pcm buf ofs len ->
-                      let sbuf = Bytes.create (2 * len * Array.length buf) in
-                      Audio.S16LE.of_audio buf ofs sbuf 0 len;
-                      Pcm.writei pcm (Bytes.unsafe_to_string sbuf) 0 len) );
-              Pcm.set_channels dev params buffer_chans;
+              (try
+                 Pcm.set_access dev params Pcm.Access_rw_noninterleaved;
+                 Pcm.set_format dev params Pcm.Format_float
+               with _ ->
+                 (* If we can't get floats we fallback on interleaved s16le *)
+                 self#log#severe "Falling back on interleaved S16LE";
+                 Pcm.set_access dev params Pcm.Access_rw_interleaved;
+                 Pcm.set_format dev params Pcm.Format_s16_le;
+                 alsa_write <-
+                   (fun pcm buf ofs len ->
+                     let sbuf =
+                       Bytes.create (Audio.S16LE.size (Array.length buf) len)
+                     in
+                     Audio.S16LE.of_audio (Audio.sub buf ofs len) sbuf 0;
+                     Pcm.writei pcm sbuf 0 len));
+              Pcm.set_channels dev params channels;
               alsa_rate <-
                 Pcm.set_rate_near dev params samples_per_second Dir_eq;
+
               (* Size in frames, must be set after the samplerate.
                * This setting is critical as a too small bufsize will easily result in
                * underruns when the thread isn't fast enough.
@@ -104,8 +112,9 @@ class output ~kind ~clock_safe ~infallible ~on_stop ~on_start ~start dev source
               (bufsize, fst (Pcm.get_periods_max params))
             in
             self#log#important
-              "Samplefreq=%dHz, Bufsize=%dB, Frame=%dB, Periods=%d" alsa_rate
-              bufsize
+              "channels=%d, samplerate=%dHz, buffer size=%dB, frame size=%dB, \
+               periods=%d"
+              channels alsa_rate bufsize
               (Pcm.get_frame_size params)
               periods;
             Pcm.set_params dev params;
@@ -122,36 +131,35 @@ class output ~kind ~clock_safe ~infallible ~on_stop ~on_start ~start dev source
     method push_block data =
       let dev = self#get_device in
       try
-        let len = Array.length data.(0) in
+        let len = Audio.length data in
         let rec f pos =
           if pos < len then (
             let ret = alsa_write dev data pos (len - pos) in
-            f (pos + ret) )
+            f (pos + ret))
         in
         f 0
       with e ->
+        let bt = Printexc.get_raw_backtrace () in
         begin
           match e with
           | Buffer_xrun -> self#log#severe "Underrun!"
           | _ -> self#log#severe "Alsa error: %s" (string_of_error e)
         end;
-        if e = Buffer_xrun || e = Suspended || e = Interrupted then begin
-          self#log#severe "Trying to recover..";
-          Pcm.recover dev e
-        end
-        else raise e
+        if e = Buffer_xrun || e = Suspended || e = Interrupted then (
+          self#log#severe "Trying to recover.";
+          Pcm.recover dev e)
+        else Printexc.raise_with_backtrace e bt
 
-    method output_send buf =
-      let buf = AFrame.content buf 0 in
+    method send_frame buf =
+      let buf = AFrame.pcm buf in
       let ratio = float alsa_rate /. float samples_per_second in
       let buf =
-        Audio_converter.Samplerate.resample samplerate_converter ratio buf 0
-          (Array.length buf.(0))
+        Audio_converter.Samplerate.resample self#samplerate_converter ratio buf
       in
-      let f data = Audio.blit buf 0 data 0 (Audio.duration buf) in
+      let f data = Audio.blit buf data in
       ioring#put_block f
 
-    method output_reset =
+    method reset =
       self#close;
       ignore self#get_device
   end
