@@ -1,6 +1,6 @@
 (*****************************************************************************
 
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -19,6 +19,8 @@
 
  *****************************************************************************)
 
+open Mm
+
 class output ~kind ~infallible ~autostart ~on_start ~on_stop source =
   let video_width = Lazy.force Frame.video_width in
   let video_height = Lazy.force Frame.video_height in
@@ -29,47 +31,42 @@ class output ~kind ~infallible ~autostart ~on_start ~on_stop source =
           ~on_stop ~content_kind:kind source autostart
 
     val mutable sleep = false
+    method stop = sleep <- true
 
-    method output_stop = sleep <- true
-
-    method output_start =
+    method start =
       Graphics.open_graph "";
       Graphics.set_window_title "Liquidsoap";
       Graphics.resize_window video_width video_height;
       sleep <- false
 
-    method output_send buf =
-      let rgb =
-        let stop, c = Frame.content buf 0 in
-        assert (stop = Lazy.force Frame.size);
-        c.Frame.video.(0)
-      in
-      for frame = 0 to 0 do
-        let img = Image.RGBA32.to_int_image rgb.(frame) in
-        let img = Graphics.make_image img in
-        Graphics.draw_image img 0 0
-      done
+    method send_frame buf =
+      let rgb = Video.get (VFrame.yuva420p buf) 0 in
+      let img = Video.Image.to_int_image rgb in
+      let img = Graphics.make_image img in
+      Graphics.draw_image img 0 0
 
-    method output_reset = ()
+    method reset = ()
   end
 
 let () =
-  let k = Lang.kind_type_of_kind_format ~fresh:1 Lang.video_only in
-  Lang.add_operator "output.graphics" ~active:true
+  let kind = Lang.video_yuva420p in
+  let k = Lang.kind_type_of_kind_format kind in
+  Lang.add_operator "output.graphics"
     (Output.proto @ [("", Lang.source_t k, None, None)])
-    ~kind:(Lang.Unconstrained k) ~category:Lang.Output
+    ~return_t:k ~category:`Output ~meth:Output.meth
     ~descr:"Display video stream using the Graphics library."
-    (fun p kind ->
+    (fun p ->
       let autostart = Lang.to_bool (List.assoc "start" p) in
       let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
       let on_start =
         let f = List.assoc "on_start" p in
-        fun () -> ignore (Lang.apply ~t:Lang.unit_t f [])
+        fun () -> ignore (Lang.apply f [])
       in
       let on_stop =
         let f = List.assoc "on_stop" p in
-        fun () -> ignore (Lang.apply ~t:Lang.unit_t f [])
+        fun () -> ignore (Lang.apply f [])
       in
       let source = List.assoc "" p in
-      ( new output ~kind ~infallible ~autostart ~on_start ~on_stop source
-        :> Source.source ))
+      let kind = Source.Kind.of_kind kind in
+      (new output ~kind ~infallible ~autostart ~on_start ~on_stop source
+        :> Output.output))
