@@ -1,7 +1,7 @@
 (*****************************************************************************
   
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -75,48 +75,59 @@ module Make (T : T) = struct
         ("mount", Lang.string_t, None, None);
         ("port", Lang.int_t, Some (Lang.int 8000), None);
         ( "user",
-          Lang.string_t,
-          Some (Lang.string ""),
-          Some "User for client connection, disabled if empty." );
-        ("password", Lang.string_t, Some (Lang.string "hackme"), None);
+          Lang.nullable_t Lang.string_t,
+          Some Lang.null,
+          Some
+            "User for client connection. You also need to setup a `password`."
+        );
+        ( "password",
+          Lang.nullable_t Lang.string_t,
+          Some Lang.null,
+          Some
+            "Password for client connection. A `user` must also be set. We \
+             check for this password is checked unless an `auth` function is \
+             defined, which is used in this case." );
         ( "timeout",
           Lang.float_t,
           Some (Lang.float 30.),
-          Some "Timeout for network operations." );
+          Some "Timeout for network operations (in seconds)." );
         ( "encoding",
           Lang.string_t,
           Some (Lang.string ""),
           Some
             "Encoding used to send metadata. If empty, defaults to \
              \"ISO-8859-1\" for non-ogg formats and \"UTF-8\" otherwise." );
-        ("url", Lang.string_t, Some (Lang.string ""), None);
+        ("url", Lang.nullable_t Lang.string_t, Some Lang.null, None);
         ( "metaint",
           Lang.int_t,
-          Some (Lang.int 16000),
+          Some (Lang.int 8192),
           Some "Interval used to send ICY metadata" );
         ( "auth",
-          Lang.fun_t
-            [(false, "", Lang.string_t); (false, "", Lang.string_t)]
-            Lang.bool_t,
+          Lang.nullable_t
+            (Lang.fun_t
+               [
+                 (false, "address", Lang.string_t);
+                 (false, "", Lang.string_t);
+                 (false, "", Lang.string_t);
+               ]
+               Lang.bool_t),
+          Some Lang.null,
           Some
-            (Lang.val_cst_fun
-               [("", Lang.string_t, None); ("", Lang.string_t, None)]
-               (Lang.bool false)),
-          Some
-            "Authentication function. `f(login,password)` returns `true` if \
-             the user should be granted access for this login. Override any \
-             other method if used." );
+            "Authentication function. `f(~address,login,password)` returns \
+             `true` if the user should be granted access for this login. When \
+             defined, `user` and `password` arguments are not taken in \
+             account." );
         ( "buffer",
           Lang.int_t,
           Some (Lang.int (5 * 65535)),
-          Some "Maximun buffer per-client." );
+          Some "Maximum buffer per-client." );
         ( "burst",
           Lang.int_t,
           Some (Lang.int 65534),
           Some "Initial burst of data sent to the client." );
         ( "chunk",
           Lang.int_t,
-          Some (Lang.int 1024),
+          Some (Lang.int Utils.pagesize),
           Some "Send data to clients using chunks of at least this length." );
         ( "on_connect",
           Lang.fun_t
@@ -130,25 +141,26 @@ module Make (T : T) = struct
           Some
             (Lang.val_cst_fun
                [
-                 ("headers", Lang.metadata_t, None);
-                 ("uri", Lang.string_t, None);
-                 ("protocol", Lang.string_t, None);
-                 ("", Lang.string_t, None);
+                 ("headers", None); ("uri", None); ("protocol", None); ("", None);
                ]
                Lang.unit),
-          Some "Callback executed when connection is established." );
+          Some
+            "Callback executed when connection is established (takes headers, \
+             connection uri, protocol and client's IP as arguments)." );
         ( "on_disconnect",
           Lang.fun_t [(false, "", Lang.string_t)] Lang.unit_t,
-          Some (Lang.val_cst_fun [("", Lang.string_t, None)] Lang.unit),
-          Some "Callback executed when connection stops." );
+          Some (Lang.val_cst_fun [("", None)] Lang.unit),
+          Some
+            "Callback executed when connection stops (takes client's IP as \
+             argument)." );
         ( "headers",
           Lang.metadata_t,
-          Some (Lang.list ~t:(Lang.product_t Lang.string_t Lang.string_t) []),
+          Some (Lang.list []),
           Some "Additional headers." );
         ( "dumpfile",
-          Lang.string_t,
-          Some (Lang.string ""),
-          Some "Dump stream to file, for debugging purpose. Disabled if empty."
+          Lang.nullable_t Lang.string_t,
+          Some Lang.null,
+          Some "Dump stream to file, for debugging purpose. Disabled if null."
         );
         ("", Lang.source_t kind, None, None);
       ]
@@ -161,7 +173,7 @@ module Make (T : T) = struct
   }
 
   type client = {
-    mutable buffer : Buffer.t;
+    buffer : Strings.Mutable.t;
     condition : Duppy_c.condition;
     condition_m : Duppy_m.mutex;
     mutex : Mutex.t;
@@ -178,10 +190,8 @@ module Make (T : T) = struct
   }
 
   let add_meta c data =
-    let get_meta meta =
-      let f x =
-        try Some (Hashtbl.find (Utils.get_some meta) x) with _ -> None
-      in
+    let mk_icy_meta meta =
+      let f x = try Some (Hashtbl.find meta x) with _ -> None in
       let meta_info =
         match (f "artist", f "title") with
           | Some a, Some t -> Some (Printf.sprintf "%s - %s" a t)
@@ -207,64 +217,65 @@ module Make (T : T) = struct
       (* Pad string to a multiple of 16 bytes. *)
       let len = String.length meta in
       let pad = (len / 16) + 1 in
-      let ret = Bytes.make ((pad * 16) + 1) ' ' in
+      let ret = Bytes.make ((pad * 16) + 1) '\000' in
       Bytes.set ret 0 (Char.chr pad);
       String.blit meta 0 ret 1 len;
       let ret = Bytes.unsafe_to_string ret in
       if ret <> c.latest_meta then (
         c.latest_meta <- ret;
-        ret )
+        ret)
       else "\000"
     in
-    let rec process meta rem data =
-      let pos = c.metaint - c.metapos in
-      let before = String.sub data 0 pos in
-      let after = String.sub data pos (String.length data - pos) in
-      if String.length after > c.metaint then (
-        let rem = Printf.sprintf "%s%s%s" rem before meta in
-        c.metapos <- 0;
-        process "\000" rem after )
-      else (
-        c.metapos <- String.length after;
-        Printf.sprintf "%s%s%s%s" rem before meta after )
+    let get_meta () =
+      let meta =
+        Tutils.mutexify c.meta.metadata_m
+          (fun () ->
+            let meta = c.meta.metadata in
+            c.meta.metadata <- None;
+            meta)
+          ()
+      in
+      match meta with Some meta -> mk_icy_meta meta | None -> "\000"
     in
-    if c.metaint > 0 then
-      if String.length data + c.metapos > c.metaint then (
-        let meta =
-          Tutils.mutexify c.meta.metadata_m (fun () -> c.meta.metadata) ()
-        in
-        process (get_meta meta) "" data )
+    let rec process cur data =
+      let len = Strings.length data in
+      if c.metaint <= c.metapos + len then (
+        let meta = get_meta () in
+        let next_meta_pos = c.metaint - c.metapos in
+        let before = Strings.sub data 0 next_meta_pos in
+        let after = Strings.sub data next_meta_pos (len - next_meta_pos) in
+        let cur = Strings.concat [cur; before; Strings.of_string meta] in
+        c.metapos <- 0;
+        process cur after)
       else (
-        c.metapos <- c.metapos + String.length data;
-        data )
-    else data
+        c.metapos <- c.metapos + len;
+        Strings.concat [cur; data])
+    in
+    if c.metaint > 0 then process Strings.empty data else data
 
   let rec client_task c =
     let __pa_duppy_0 =
       Duppy.Monad.Io.exec ~priority:Tutils.Maybe_blocking c.handler
         (Tutils.mutexify c.mutex
            (fun () ->
-             let buflen = Buffer.length c.buffer in
+             let buflen = Strings.Mutable.length c.buffer in
              let data =
-               if buflen > c.chunk then (
-                 let data = Some (add_meta c (Buffer.contents c.buffer)) in
-                 Buffer.reset c.buffer;
-                 data )
-               else None
+               if buflen > c.chunk then
+                 add_meta c (Strings.Mutable.flush c.buffer)
+               else Strings.empty
              in
              Duppy.Monad.return data)
            ())
     in
     Duppy.Monad.bind __pa_duppy_0 (fun data ->
         Duppy.Monad.bind
-          ( match data with
-            | None ->
-                Duppy.Monad.bind (Duppy_m.lock c.condition_m) (fun () ->
-                    Duppy.Monad.bind (Duppy_c.wait c.condition c.condition_m)
-                      (fun () -> Duppy_m.unlock c.condition_m))
-            | Some s ->
-                Duppy.Monad.Io.write ?timeout:(Some c.timeout)
-                  ~priority:Tutils.Non_blocking c.handler (Bytes.of_string s) )
+          (if Strings.is_empty data then
+           Duppy.Monad.bind (Duppy_m.lock c.condition_m) (fun () ->
+               Duppy.Monad.bind (Duppy_c.wait c.condition c.condition_m)
+                 (fun () -> Duppy_m.unlock c.condition_m))
+          else
+            Duppy.Monad.Io.write ?timeout:(Some c.timeout)
+              ~priority:Tutils.Non_blocking c.handler (Strings.to_bytes data))
           (fun () ->
             let __pa_duppy_0 =
               Duppy.Monad.Io.exec ~priority:Tutils.Maybe_blocking c.handler
@@ -291,7 +302,7 @@ module Make (T : T) = struct
     let on_disconnect = List.assoc "on_disconnect" p in
     let on_connect ~headers ~protocol ~uri s =
       ignore
-        (Lang.apply ~t:Lang.unit_t on_connect
+        (Lang.apply on_connect
            [
              ("headers", Lang.metadata headers);
              ("uri", Lang.string uri);
@@ -300,7 +311,7 @@ module Make (T : T) = struct
            ])
     in
     let on_disconnect s =
-      ignore (Lang.apply ~t:Lang.unit_t on_disconnect [("", Lang.string s)])
+      ignore (Lang.apply on_disconnect [("", Lang.string s)])
     in
     let metaint = Lang.to_int (List.assoc "metaint" p) in
     let data = encoder_data p in
@@ -323,13 +334,13 @@ module Make (T : T) = struct
     let () =
       if chunk > buflen then
         raise
-          (Lang_errors.Invalid_value
+          (Error.Invalid_value
              ( List.assoc "buffer" p,
                "Maximum buffering inferior to chunk length" ))
       else ();
       if burst > buflen then
         raise
-          (Lang_errors.Invalid_value
+          (Error.Invalid_value
              ( List.assoc "buffer" p,
                "Maximum buffering inferior to burst length" ))
       else ()
@@ -343,40 +354,53 @@ module Make (T : T) = struct
     let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
     let on_start =
       let f = List.assoc "on_start" p in
-      fun () -> ignore (Lang.apply ~t:Lang.unit_t f [])
+      fun () -> ignore (Lang.apply f [])
     in
     let on_stop =
       let f = List.assoc "on_stop" p in
-      fun () -> ignore (Lang.apply ~t:Lang.unit_t f [])
+      fun () -> ignore (Lang.apply f [])
+    in
+    let url =
+      List.assoc "url" p |> Lang.to_option |> Option.map Lang.to_string
     in
-    let url = match s "url" with "" -> None | x -> Some x in
     let port = e Lang.to_int "port" in
-    let default_user = s "user" in
-    let default_password = s "password" in
-    (* Cf sources/harbor_input.ml *)
-    let trivially_false = function
-      | {
-          Lang.value =
-            Lang.Fun (_, _, _, { Lang_values.term = Lang_values.Bool false; _ });
-          _;
-        } ->
-          true
-      | _ -> false
+    let default_user =
+      List.assoc "user" p |> Lang.to_option |> Option.map Lang.to_string
     in
-    let auth_function = List.assoc "auth" p in
-    let login user password =
+    let default_password =
+      List.assoc "password" p |> Lang.to_option |> Option.map Lang.to_string
+    in
+    let address_resolver s =
+      let s = Harbor.file_descr_of_socket s in
+      Utils.name_of_sockaddr ~rev_dns:Harbor_base.conf_revdns#get
+        (Unix.getpeername s)
+    in
+    let auth_function = List.assoc "auth" p |> Lang.to_option in
+    let login ~socket user password =
+      let address = address_resolver socket in
       let user, password =
         let f = Configure.recode_tag in
         (f user, f password)
       in
-      let default_login = user = default_user && password = default_password in
-      if not (trivially_false auth_function) then
-        Lang.to_bool
-          (Lang.apply ~t:Lang.bool_t auth_function
-             [("", Lang.string user); ("", Lang.string password)])
-      else default_login
+      match auth_function with
+        | Some f ->
+            Lang.to_bool
+              (Lang.apply f
+                 [
+                   ("address", Lang.string address);
+                   ("", Lang.string user);
+                   ("", Lang.string password);
+                 ])
+        | None -> (
+            match (default_user, default_password) with
+              | _, None -> false
+              | None, _ -> false
+              | Some default_user, Some default_password ->
+                  user = default_user && password = default_password)
+    in
+    let dumpfile =
+      Lang.to_valued_option Lang.to_string (List.assoc "dumpfile" p)
     in
-    let dumpfile = match s "dumpfile" with "" -> None | s -> Some s in
     let extra_headers =
       List.map
         (fun v ->
@@ -388,31 +412,21 @@ module Make (T : T) = struct
       (** File descriptor where to dump. *)
       inherit
         Output.encoded
-          ~content_kind:kind ~output_kind:T.source_name ~infallible ~autostart
-            ~on_start ~on_stop ~name:mount source
+          ~content_kind:(Source.Kind.of_kind kind) ~output_kind:T.source_name
+            ~infallible ~autostart ~on_start ~on_stop ~name:mount source
 
       val mutable dump = None
-
       val mutable encoder = None
-
       val mutable clients = Queue.create ()
-
       val clients_m = Mutex.create ()
-
       val duppy_c = Duppy_c.create ()
-
       val duppy_m = Duppy_m.create ()
-
       val mutable chunk_len = 0
-
-      val mutable burst_data = []
-
-      val mutable burst_pos = 0
-
+      val burst_data = Strings.Mutable.empty ()
       val metadata = { metadata = None; metadata_m = Mutex.create () }
 
       method encode frame ofs len =
-        (Utils.get_some encoder).Encoder.encode frame ofs len
+        (Option.get encoder).Encoder.encode frame ofs len
 
       method insert_metadata m =
         let m = Meta_format.to_metadata m in
@@ -420,13 +434,12 @@ module Make (T : T) = struct
         Tutils.mutexify metadata.metadata_m
           (fun () -> metadata.metadata <- Some m)
           ();
-        (Utils.get_some encoder).Encoder.insert_metadata
+        (Option.get encoder).Encoder.insert_metadata
           (Meta_format.export_metadata m)
 
       method add_client ~protocol ~headers ~uri ~args s =
         let ip =
-          (* Show port = true to catch different clients from same
-             * ip *)
+          (* Show port = true to catch different clients from same ip *)
           let fd = Harbor.file_descr_of_socket s in
           Utils.name_of_sockaddr ~show_port:true (Unix.getpeername fd)
         in
@@ -446,10 +459,9 @@ module Make (T : T) = struct
           Printf.sprintf "%s 200 OK\r\nContent-type: %s\r\n%s%s\r\n" protocol
             data.format icyheader extra_headers
         in
-        let buffer = Buffer.create buflen in
-        ( match (Utils.get_some encoder).Encoder.header with
-          | Some s -> Buffer.add_string buffer s
-          | None -> () );
+        let buffer =
+          Strings.Mutable.of_strings (Option.get encoder).Encoder.header
+        in
         let close () = try Harbor.close s with _ -> () in
         let rec client =
           {
@@ -475,41 +487,51 @@ module Make (T : T) = struct
             data = "";
             on_error =
               (fun e ->
-                ( match e with
-                  | Duppy.Io.Timeout -> self#log#f 5 "Timeout error"
-                  | Duppy.Io.Io_error -> self#log#f 5 "I/O error"
-                  | Duppy.Io.Unix (c, p, m) ->
-                      self#log#f 5 "%s"
-                        (Printexc.to_string (Unix.Unix_error (c, p, m)))
-                  | Duppy.Io.Unknown e ->
-                      self#log#f 5 "%s" (Printexc.to_string e) );
-                self#log#f 4 "Client %s disconnected" ip;
+                let bt = Printexc.get_backtrace () in
+                let msg =
+                  match e with
+                    | Duppy.Io.Timeout ->
+                        Printf.sprintf "Timeout error for %s" ip
+                    | Duppy.Io.Io_error -> Printf.sprintf "I/O error for %s" ip
+                    | Duppy.Io.Unix (c, p, m) ->
+                        Printf.sprintf "Unix error for %s: %s" ip
+                          (Printexc.to_string (Unix.Unix_error (c, p, m)))
+                    | Duppy.Io.Unknown e ->
+                        Printf.sprintf "%s" (Printexc.to_string e)
+                in
+                Utils.log_exception ~log:self#log ~bt msg;
+                self#log#info "Client %s disconnected" ip;
                 Tutils.mutexify client.mutex
                   (fun () ->
                     client.state <- Done;
-                    Buffer.reset buffer)
+                    ignore (Strings.Mutable.flush client.buffer))
                   ();
                 on_disconnect ip;
                 Harbor.Close (Harbor.mk_simple ""));
           }
         in
+        self#log#info "Serving client %s." ip;
         Duppy.Monad.bind
           (Duppy.Monad.catch
-             ( if default_user <> "" || not (trivially_false auth_function) then
-               Duppy.Monad.Io.exec ~priority:Tutils.Maybe_blocking handler
-                 (Harbor.http_auth_check ~args ~login:(default_user, login)
-                    headers)
-             else Duppy.Monad.return () )
+             (if
+              (default_user <> None && default_password <> None)
+              || auth_function <> None
+             then (
+              let default_user = Option.value default_user ~default:"" in
+              Duppy.Monad.Io.exec ~priority:Tutils.Maybe_blocking handler
+                (Harbor.http_auth_check ~args ~login:(default_user, login) s
+                   headers))
+             else Duppy.Monad.return ())
              (function
                | Harbor.Relay _ -> assert false
                | Harbor.Close s ->
-                   self#log#f 4 "Client %s failed to authenticate!" ip;
+                   self#log#info "Client %s failed to authenticate!" ip;
                    client.state <- Done;
                    Harbor.reply s))
           (fun () ->
             Duppy.Monad.Io.exec ~priority:Tutils.Maybe_blocking handler
               (Harbor.relayed reply (fun () ->
-                   self#log#f 4 "Client %s connected" ip;
+                   self#log#info "Client %s connected" ip;
                    Tutils.mutexify clients_m
                      (fun () -> Queue.push client clients)
                      ();
@@ -518,28 +540,21 @@ module Make (T : T) = struct
                    on_connect ~protocol ~uri ~headers:h_headers ip)))
 
       method send b =
-        let slen = String.length b in
+        let slen = Strings.length b in
         if slen > 0 then (
-          chunk_len <- chunk_len + String.length b;
+          chunk_len <- chunk_len + slen;
           let wake_up =
             if chunk_len >= chunk then (
               chunk_len <- 0;
-              true )
+              true)
             else false
           in
-          let rec f acc len l =
-            match l with
-              | x :: l' ->
-                  let len' = String.length x in
-                  if len + len' < burst then f (x :: acc) (len + len') l'
-                  else (x :: acc, len' - burst + len)
-              | [] -> (acc, 0)
-          in
-          let data, pos = f [] 0 (b :: List.rev burst_data) in
-          burst_data <- data;
-          burst_pos <- pos;
+          Strings.Mutable.append_strings burst_data b;
+          Strings.Mutable.keep burst_data burst;
           let new_clients = Queue.create () in
-          (match dump with Some s -> output_string s b | None -> ());
+          (match dump with
+            | Some s -> Strings.iter (output_substring s) b
+            | None -> ());
           Tutils.mutexify clients_m
             (fun () ->
               Queue.iter
@@ -549,20 +564,14 @@ module Make (T : T) = struct
                       (fun () ->
                         match c.state with
                           | Hello ->
-                              ( match burst_data with
-                                | x :: l ->
-                                    Buffer.add_substring c.buffer x burst_pos
-                                      (String.length x - burst_pos);
-                                    List.iter (Buffer.add_string c.buffer) l
-                                | _ -> () );
+                              Strings.Mutable.append c.buffer burst_data;
                               Queue.push c new_clients;
                               true
                           | Sending ->
-                              let buf = Buffer.length c.buffer in
+                              let buf = Strings.Mutable.length c.buffer in
                               if buf + slen > buflen then
-                                Utils.buffer_drop c.buffer (min buf slen)
-                              else ();
-                              Buffer.add_string c.buffer b;
+                                Strings.Mutable.drop c.buffer (min buf slen);
+                              Strings.Mutable.append_strings c.buffer b;
                               Queue.push c new_clients;
                               false
                           | Done -> false)
@@ -580,10 +589,10 @@ module Make (T : T) = struct
                   (Duppy_c.broadcast duppy_c)
               else ();
               clients <- new_clients)
-            () )
+            ())
         else ()
 
-      method output_start =
+      method start =
         assert (encoder = None);
         let enc = data.factory self#id in
         encoder <- Some (enc Meta_format.empty_metadata);
@@ -603,8 +612,8 @@ module Make (T : T) = struct
           | Some f -> dump <- Some (open_out_bin f)
           | None -> ()
 
-      method output_stop =
-        ignore ((Utils.get_some encoder).Encoder.stop ());
+      method stop =
+        ignore ((Option.get encoder).Encoder.stop ());
         encoder <- None;
         Harbor.remove_http_handler ~port ~verb:`Get ~uri ();
         let new_clients = Queue.create () in
@@ -625,17 +634,20 @@ module Make (T : T) = struct
           ();
         match dump with Some f -> close_out f | None -> ()
 
-      method output_reset =
-        self#output_stop;
-        self#output_start
+      method reset =
+        self#stop;
+        self#start
     end
 
   let () =
-    let k = Lang.univ_t 1 in
-    Lang.add_operator ~category:Lang.Output ~active:true
-      ~descr:T.source_description T.source_name (proto k)
-      ~kind:(Lang.Unconstrained k) (fun p kind ->
-        (new output ~kind p :> Source.source))
+    let kind = Lang.any in
+    let return_t = Lang.kind_type_of_kind_format kind in
+    Lang.add_operator ~category:`Output ~descr:T.source_description
+      ~meth:Output.meth T.source_name (proto return_t) ~return_t (fun p ->
+        let format_val = Lang.assoc "" 1 p in
+        let format = Lang.to_format format_val in
+        let kind = Encoder.kind_of_format format in
+        (new output ~kind p :> Output.output))
 end
 
 module Unix_output = struct
