@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -55,8 +55,7 @@ module Icecast = struct
             | Mp3_format.CBR x -> (None, Some x)
             | Mp3_format.ABR x -> (None, x.Mp3_format.mean_bitrate)
             | Mp3_format.VBR x ->
-                ( Some (string_of_int (Utils.get_some x.Mp3_format.quality)),
-                  None )
+                (Some (string_of_int (Option.get x.Mp3_format.quality)), None)
         in
         {
           quality;
@@ -85,12 +84,12 @@ module Icecast = struct
           samplerate = Some (Lazy.force m.External_encoder_format.samplerate);
           channels = Some m.External_encoder_format.channels;
         }
-    | Encoder.GStreamer m ->
+    | Encoder.GStreamer gst ->
         {
           quality = None;
           bitrate = None;
           samplerate = None;
-          channels = Some (Gstreamer_format.audio_channels m);
+          channels = Some (Gstreamer_format.audio_channels gst);
         }
     | Encoder.Flac m ->
         {
@@ -120,47 +119,44 @@ module Icecast = struct
           samplerate = Some (Lazy.force m.Avi_format.samplerate);
           channels = Some m.Avi_format.channels;
         }
-    | Encoder.Ogg o -> (
-        match o with
-          | [
-           Ogg_format.Vorbis
-             {
-               Vorbis_format.channels = n;
-               mode = Vorbis_format.VBR q;
-               samplerate = s;
-               _;
-             };
-          ] ->
+    | Encoder.Ogg { Ogg_format.audio; _ } -> (
+        match audio with
+          | Some
+              (Ogg_format.Vorbis
+                {
+                  Vorbis_format.channels = n;
+                  mode = Vorbis_format.VBR q;
+                  samplerate = s;
+                  _;
+                }) ->
               {
                 quality = Some (string_of_float q);
                 bitrate = None;
                 samplerate = Some (Lazy.force s);
                 channels = Some n;
               }
-          | [
-           Ogg_format.Vorbis
-             {
-               Vorbis_format.channels = n;
-               mode = Vorbis_format.ABR (_, b, _);
-               samplerate = s;
-               _;
-             };
-          ] ->
+          | Some
+              (Ogg_format.Vorbis
+                {
+                  Vorbis_format.channels = n;
+                  mode = Vorbis_format.ABR (_, b, _);
+                  samplerate = s;
+                  _;
+                }) ->
               {
                 quality = None;
                 bitrate = b;
                 samplerate = Some (Lazy.force s);
                 channels = Some n;
               }
-          | [
-           Ogg_format.Vorbis
-             {
-               Vorbis_format.channels = n;
-               mode = Vorbis_format.CBR b;
-               samplerate = s;
-               _;
-             };
-          ] ->
+          | Some
+              (Ogg_format.Vorbis
+                {
+                  Vorbis_format.channels = n;
+                  mode = Vorbis_format.CBR b;
+                  samplerate = s;
+                  _;
+                }) ->
               {
                 quality = None;
                 bitrate = Some b;
@@ -173,7 +169,7 @@ module Icecast = struct
                 bitrate = None;
                 samplerate = None;
                 channels = None;
-              } )
+              })
 end
 
 module M = Icecast_utils.Icecast_v (Icecast)
@@ -211,45 +207,45 @@ let proto kind =
         Some (Lang.float 30.),
         Some "Timeout for network read and write." );
       ( "user",
-        Lang.string_t,
-        Some (Lang.string ""),
+        Lang.nullable_t Lang.string_t,
+        Some Lang.null,
         Some
           "User for shout source connection. Defaults to \"source\" for \
            icecast connections. Useful only in special cases, like with \
            per-mountpoint users." );
       ("password", Lang.string_t, Some (Lang.string "hackme"), None);
       ( "encoding",
-        Lang.string_t,
-        Some (Lang.string ""),
+        Lang.nullable_t Lang.string_t,
+        Some Lang.null,
         Some
           "Encoding used to send metadata and stream info (name, genre and \
-           description). If empty, defaults to \"UTF-8\" for \"http(s)\" \
+           description). If null, defaults to \"UTF-8\" for \"http(s)\" \
            protocol and \"ISO-8859-1\" for \"icy\" protocol." );
-      ("genre", Lang.string_t, Some (Lang.string ""), None);
+      ("genre", Lang.nullable_t Lang.string_t, Some Lang.null, None);
       ( "protocol",
         Lang.string_t,
         Some (Lang.string "http"),
         Some
           "Protocol of the streaming server: 'http' or 'https' for Icecast, \
            'icy' for shoutcast." );
-      ( "verb",
+      ( "method",
         Lang.string_t,
         Some (Lang.string "source"),
         Some
-          "Verb to use with the 'http(s)' protocol. One of: 'source', 'put' or \
-           'post'." );
+          "method to use with the 'http(s)' protocol. One of: 'source', 'put' \
+           or 'post'." );
       ( "chunked",
         Lang.bool_t,
         Some (Lang.bool false),
-        Some "Used cunked transfer with the 'http(s)' protocol." );
+        Some "Used chunked transfer with the 'http(s)' protocol." );
       ( "icy_metadata",
         Lang.string_t,
         Some (Lang.string "guess"),
         Some
           "Send new metadata using the ICY protocol. One of: \"guess\", \
            \"true\", \"false\"" );
-      ("url", Lang.string_t, Some (Lang.string ""), None);
-      ("description", Lang.string_t, Some (Lang.string ""), None);
+      ("url", Lang.nullable_t Lang.string_t, Some Lang.null, None);
+      ("description", Lang.nullable_t Lang.string_t, Some Lang.null, None);
       ( "on_connect",
         Lang.fun_t [] Lang.unit_t,
         Some (Lang.val_cst_fun [] Lang.unit),
@@ -260,24 +256,21 @@ let proto kind =
         Some "Callback executed when connection stops." );
       ( "on_error",
         Lang.fun_t [(false, "", Lang.string_t)] Lang.float_t,
-        Some (Lang.val_cst_fun [("", Lang.string_t, None)] (Lang.float 3.)),
+        Some (Lang.val_cst_fun [("", None)] (Lang.float 3.)),
         Some
           "Callback executed when an error happens. The callback receives a \
-           string representation of the error that occured and returns a \
+           string representation of the error that occurred and returns a \
            float. If returned value is positive, connection will be tried \
            again after this amount of time (in seconds)." );
       ("public", Lang.bool_t, Some (Lang.bool true), None);
       ( "headers",
         Lang.metadata_t,
-        Some
-          (Lang.list
-             ~t:(Lang.product_t Lang.string_t Lang.string_t)
-             [user_agent]),
+        Some (Lang.list [user_agent]),
         Some "Additional headers." );
       ( "dumpfile",
-        Lang.string_t,
-        Some (Lang.string ""),
-        Some "Dump stream to file, for debugging purpose. Disabled if empty." );
+        Lang.nullable_t Lang.string_t,
+        Some Lang.null,
+        Some "Dump stream to file, for debugging purpose. Disabled if null." );
       ("", Lang.source_t kind, None, None);
     ]
 
@@ -286,41 +279,38 @@ let proto kind =
 class output ~kind p =
   let e f v = f (List.assoc v p) in
   let s v = e Lang.to_string v in
-
+  let s_opt v = e (Lang.to_valued_option Lang.to_string) v in
   let on_connect = List.assoc "on_connect" p in
   let on_disconnect = List.assoc "on_disconnect" p in
   let on_error = List.assoc "on_error" p in
-  let on_connect () = ignore (Lang.apply ~t:Lang.unit_t on_connect []) in
-  let on_disconnect () = ignore (Lang.apply ~t:Lang.unit_t on_disconnect []) in
+  let on_connect () = ignore (Lang.apply on_connect []) in
+  let on_disconnect () = ignore (Lang.apply on_disconnect []) in
   let on_error error =
     let msg = Printexc.to_string error in
-    Lang.to_float (Lang.apply ~t:Lang.unit_t on_error [("", Lang.string msg)])
+    Lang.to_float (Lang.apply on_error [("", Lang.string msg)])
   in
-
   let data = encoder_data p in
-
   let chunked = Lang.to_bool (List.assoc "chunked" p) in
-
   let protocol =
-    let verb =
-      let v = List.assoc "verb" p in
-      match Lang.to_string v with
+    let m =
+      let m = List.assoc "method" p in
+      match Lang.to_string m with
         | "source" -> Cry.Source
         | "put" -> Cry.Put
         | "post" -> Cry.Post
         | _ ->
             raise
-              (Lang_errors.Invalid_value
-                 (v, "Valid values are: 'source' 'put' or 'post'."))
+              (Error.Invalid_value
+                 (m, "Valid values are: 'source' 'put' or 'post'."))
     in
     let v = List.assoc "protocol" p in
     match Lang.to_string v with
-      | "http" -> Cry.Http verb
-      | "https" -> Cry.Https verb
+      | "http" -> Cry.Http m
+      | "https" -> Cry.Https m
       | "icy" -> Cry.Icy
       | _ ->
           raise
-            (Lang_errors.Invalid_value
+            (Error.Invalid_value
                (v, "Valid values are 'http' (icecast) and 'icy' (shoutcast)"))
   in
   let icy_metadata =
@@ -332,7 +322,7 @@ class output ~kind p =
         | "false" -> `False
         | _ ->
             raise
-              (Lang_errors.Invalid_value
+              (Error.Invalid_value
                  (v, "Valid values are 'guess', 'true' or 'false'"))
     in
     match (data.format, icy) with
@@ -343,29 +333,25 @@ class output ~kind p =
           false
       | _, _ ->
           raise
-            (Lang_errors.Invalid_value
+            (Error.Invalid_value
                ( List.assoc "icy_metadata" p,
                  "Could not guess icy_metadata for this format, please specify \
                   either 'true' or 'false'." ))
   in
-
   let out_enc =
-    match Lang.to_string (List.assoc "encoding" p) with
-      | "" -> if protocol = Cry.Icy then "ISO-8859-1" else "UTF-8"
-      | s -> String.uppercase_ascii s
+    match s_opt "encoding" with
+      | None | Some "" -> if protocol = Cry.Icy then "ISO-8859-1" else "UTF-8"
+      | Some s -> String.uppercase_ascii s
   in
-
   let source = Lang.assoc "" 2 p in
-
   let icy_id = Lang.to_int (List.assoc "icy_id" p) in
-
   let mount = s "mount" in
   let name = Configure.recode_tag ~out_enc (s "name") in
   let mount, name =
     match (protocol, name, mount) with
       | Cry.Http _, name, mount when name = no_name && mount = no_mount ->
           raise
-            (Lang_errors.Invalid_value
+            (Error.Invalid_value
                ( List.assoc "mount" p,
                  "Either name or mount must be defined for icecast sources." ))
       | Cry.Icy, name, _ when name = no_name ->
@@ -374,35 +360,37 @@ class output ~kind p =
       | _, name, mount when name = no_name -> (Cry.Icecast_mount mount, mount)
       | _ -> (Cry.Icecast_mount mount, name)
   in
-
   let autostart = Lang.to_bool (List.assoc "start" p) in
   let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
   let on_start =
     let f = List.assoc "on_start" p in
-    fun () -> ignore (Lang.apply ~t:Lang.unit_t f [])
+    fun () -> ignore (Lang.apply f [])
   in
   let on_stop =
     let f = List.assoc "on_stop" p in
-    fun () -> ignore (Lang.apply ~t:Lang.unit_t f [])
+    fun () -> ignore (Lang.apply f [])
   in
-
   let host = s "host" in
   let port = e Lang.to_int "port" in
   let user =
-    match (protocol, s "user") with
-      | Cry.Http _, "" | Cry.Https _, "" -> "source"
-      | _, user -> user
+    match (protocol, s_opt "user") with
+      | Cry.Http _, None | Cry.Https _, None -> "source"
+      | _, user -> Option.value ~default:"" user
   in
   let password = s "password" in
-  let genre = Configure.recode_tag ~out_enc (s "genre") in
-  let url = s "url" in
+  let genre =
+    Option.map (fun s -> Configure.recode_tag ~out_enc s) (s_opt "genre")
+  in
+  let url = s_opt "url" in
   let timeout = e Lang.to_float "timeout" in
   let connection_timeout =
     let v = e Lang.to_float "connection_timeout" in
     if v > 0. then Some v else None
   in
-  let dumpfile = match s "dumpfile" with "" -> None | s -> Some s in
-  let description = Configure.recode_tag ~out_enc (s "description") in
+  let dumpfile = s_opt "dumpfile" in
+  let description =
+    Option.map (fun s -> Configure.recode_tag ~out_enc s) (s_opt "description")
+  in
   let public = e Lang.to_bool "public" in
   let headers =
     List.map
@@ -412,12 +400,11 @@ class output ~kind p =
       (Lang.to_list (List.assoc "headers" p))
   in
   let connection = Cry.create ~timeout ?connection_timeout () in
-
   object (self)
     inherit
       Output.encoded
-        ~content_kind:kind ~output_kind:"output.icecast" ~infallible ~autostart
-          ~on_start ~on_stop ~name source
+        ~content_kind:(Source.Kind.of_kind kind) ~output_kind:"output.icecast"
+          ~infallible ~autostart ~on_start ~on_stop ~name source
 
     (** In this operator, we don't exactly follow the start/stop
     * mechanism of Output.encoded because we want to control
@@ -444,11 +431,11 @@ class output ~kind p =
        * is connected. *)
       match (Cry.get_status connection, encoder) with
         | Cry.Connected _, Some enc -> enc.Encoder.encode frame ofs len
-        | _ -> ""
+        | _ -> Strings.empty
 
     method insert_metadata m =
       (* Update metadata using ICY if told to.. *)
-      if icy_metadata then begin
+      if icy_metadata then (
         let get h l k = try (k, Hashtbl.find h k) :: l with _ -> l in
         let getd h k d l =
           try (k, Hashtbl.find h k) :: l with _ -> (k, d) :: l
@@ -459,7 +446,7 @@ class output ~kind p =
             | (_, s) :: _ -> (
                 let title = Filename.basename s in
                 try String.sub title 0 (String.rindex title '.')
-                with Not_found -> title )
+                with Not_found -> title)
             | [] -> "Unknown"
         in
         let default_song =
@@ -496,15 +483,14 @@ class output ~kind p =
               with e ->
                 self#log#important
                   "Metadata update may have failed with error: %s"
-                  (Printexc.to_string e) )
+                  (Printexc.to_string e))
           | Cry.Disconnected -> ()
-        (* Do nothing if shout connection isn't available *)
-      end
+        (* Do nothing if shout connection isn't available *))
       else (
         (* Encoder is not always present.. *)
-          match encoder with
+        match encoder with
           | Some encoder -> encoder.Encoder.insert_metadata m
-          | None -> () )
+          | None -> ())
 
     method send b =
       match Cry.get_status connection with
@@ -512,30 +498,32 @@ class output ~kind p =
             if Unix.time () > restart_time then self#icecast_start
         | Cry.Connected _ -> (
             try
-              Cry.send connection b;
-              match dump with Some s -> output_string s b | None -> ()
+              Strings.iter
+                (fun s offset length -> Cry.send connection ~offset ~length s)
+                b;
+              match dump with
+                | Some s -> Strings.iter (output_substring s) b
+                | None -> ()
             with e ->
               self#log#severe "Error while sending data: %s!"
                 (Printexc.to_string e);
               let delay = on_error e in
-              if delay >= 0. then begin
+              if delay >= 0. then (
                 (* Ask for a restart after [restart_time]. *)
                 self#icecast_stop;
                 restart_time <- Unix.time () +. delay;
                 self#log#important "Will try to reconnect in %.02f seconds."
-                  delay
-              end
-              else raise e )
+                  delay)
+              else raise e)
 
     (** It there's too much latency, we'll stop trying to catchup.
     * Reconnect to cancel the latency on the server's side too. *)
-    method output_reset =
-      self#output_stop;
-      self#output_start
+    method reset =
+      self#stop;
+      self#start
 
-    method output_start = self#icecast_start
-
-    method output_stop = self#icecast_stop
+    method start = self#icecast_start
+    method stop = self#icecast_stop
 
     method icecast_start =
       assert (encoder = None);
@@ -567,7 +555,7 @@ class output ~kind p =
         with Not_found -> Printf.sprintf "liquidsoap %s" Configure.version
       in
       let source =
-        Cry.connection ~host ~port ~user ~password ~genre ~url ~description
+        Cry.connection ~host ~port ~user ~password ?genre ?url ?description
           ~name ~public ~protocol ~mount ~chunked ~audio_info ~user_agent
           ~content_type:data.format ()
       in
@@ -576,10 +564,10 @@ class output ~kind p =
           (* User-Agent has already been passed to Cry.. *)
           if x <> "User-Agent" then Hashtbl.replace source.Cry.headers x y)
         headers;
-
       try
         Cry.connect connection source;
         self#log#important "Connection setup was successful.";
+
         (* Execute on_connect hook. *)
         on_connect ()
       with
@@ -588,18 +576,17 @@ class output ~kind p =
       | e ->
         self#log#severe "Connection failed: %s" (Printexc.to_string e);
         let delay = on_error e in
-        if delay >= 0. then begin
+        if delay >= 0. then (
           self#log#important "Will try again in %.02f sec." delay;
           self#icecast_stop;
-          restart_time <- Unix.time () +. delay
-        end
+          restart_time <- Unix.time () +. delay)
         else raise e
 
     method icecast_stop =
       (* In some cases it might be possible to output the remaining data,
        * but it's not worth the trouble. *)
       begin
-        try ignore ((Utils.get_some encoder).Encoder.stop ()) with _ -> ()
+        try ignore ((Option.get encoder).Encoder.stop ()) with _ -> ()
       end;
       encoder <- None;
       begin
@@ -614,8 +601,11 @@ class output ~kind p =
   end
 
 let () =
-  let k = Lang.univ_t 1 in
-  Lang.add_operator "output.icecast" ~active:true ~category:Lang.Output
+  let return_t = Lang.univ_t () in
+  Lang.add_operator "output.icecast" ~category:`Output
     ~descr:"Encode and output the stream to an icecast2 or shoutcast server."
-    (proto k) ~kind:(Lang.Unconstrained k) (fun p kind ->
-      (new output ~kind p :> Source.source))
+    ~meth:Output.meth (proto return_t) ~return_t (fun p ->
+      let format_val = Lang.assoc "" 1 p in
+      let format = Lang.to_format format_val in
+      let kind = Encoder.kind_of_format format in
+      (new output ~kind p :> Output.output))
