@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,30 +25,17 @@
 open Source
 
 let proto =
-  [
-    ( "fallible",
-      Lang.bool_t,
-      Some (Lang.bool false),
-      Some
-        "Allow the child source to fail, in which case the output will be \
-         (temporarily) stopped." );
-    ( "on_start",
-      Lang.fun_t [] Lang.unit_t,
-      Some (Lang.val_cst_fun [] Lang.unit),
-      Some "Callback executed when outputting starts." );
-    ( "on_stop",
-      Lang.fun_t [] Lang.unit_t,
-      Some (Lang.val_cst_fun [] Lang.unit),
-      Some "Callback executed when outputting stops." );
-    ( "start",
-      Lang.bool_t,
-      Some (Lang.bool true),
-      Some
-        "Automatically start outputting whenever possible. If true, an \
-         infallible (normal) output will start outputting as soon as it is \
-         created, and a fallible output will (re)start as soon as its source \
-         becomes available for streaming." );
-  ]
+  Start_stop.output_proto
+  @ [
+      ( "fallible",
+        Lang.bool_t,
+        Some (Lang.bool false),
+        Some
+          "Allow the child source to fail, in which case the output will be \
+           stopped until the source is available again." );
+    ]
+
+let meth = Start_stop.meth ()
 
 (** Given abstract start stop and send methods, creates an output.
   * Takes care of pulling the data out of the source, type checkings,
@@ -62,73 +49,74 @@ class virtual output ~content_kind ~output_kind ?(name = "") ~infallible
     (* This should be done before the active_operator initializer
      * attaches us to a clock. *)
     if infallible && source#stype <> Infallible then
-      raise (Lang_errors.Invalid_value (val_source, "That source is fallible"))
+      raise (Error.Invalid_value (val_source, "That source is fallible"))
 
     inherit active_operator ~name:output_kind content_kind [source] as super
+    inherit Start_stop.base ~on_start ~on_stop as start_stop
+    method virtual private start : unit
+    method virtual private stop : unit
+    method virtual private send_frame : Frame.t -> unit
+    method self_sync = source#self_sync
+    method stype = if infallible then Source.Infallible else Source.Fallible
 
-    inherit
-      Start_stop.base
-        ~name ~source_kind:output_kind ~interactive:true ~on_start ~on_stop
-          ~autostart as start_stop
+    (* Registration of Telnet commands must be delayed because some operators
+       change their id at initialization time. *)
+    val mutable registered_telnet = false
 
-    (* Eventually we can simply rename them... *)
-    method private start = self#output_start
-
-    method private stop = self#output_stop
-
-    method virtual private output_start : unit
-
-    method virtual private output_stop : unit
-
-    method virtual private output_send : Frame.t -> unit
-
-    method stype = source#stype
-
-    initializer
-    (* Add a few more server controls *)
-    self#register_command "skip"
-      (fun _ ->
-        self#skip;
-        "Done")
-      ~descr:"Skip current song.";
-    self#register_command "metadata" ~descr:"Print current metadata." (fun _ ->
-        let q = self#metadata_queue in
-        fst
-          (Queue.fold
-             (fun (s, i) m ->
-               let s =
-                 s
-                 ^ (if s = "" then "--- " else "\n--- ")
-                 ^ string_of_int i ^ " ---\n"
-                 ^ Request.string_of_metadata m
-               in
-               (s, i - 1))
-             ("", Queue.length q)
-             q));
-    self#register_command "remaining" ~descr:"Display estimated remaining time."
-      (fun _ ->
-        let r = source#remaining in
-        if r < 0 then "(undef)"
-        else (
-          let t = Frame.seconds_of_master r in
-          Printf.sprintf "%.2f" t ))
+    method private register_telnet =
+      if not registered_telnet then (
+        registered_telnet <- true;
+        (* Add a few more server controls *)
+        let ns = [self#id] in
+        Server.add ~ns "skip"
+          (fun _ ->
+            self#skip;
+            "Done")
+          ~descr:"Skip current song.";
+        Server.add ~ns "metadata" ~descr:"Print current metadata." (fun _ ->
+            let q = self#metadata_queue in
+            fst
+              (Queue.fold
+                 (fun (s, i) m ->
+                   let s =
+                     s
+                     ^ (if s = "" then "--- " else "\n--- ")
+                     ^ string_of_int i ^ " ---\n"
+                     ^ Request.string_of_metadata m
+                   in
+                   (s, i - 1))
+                 ("", Queue.length q)
+                 q));
+        Server.add ~ns "remaining" ~descr:"Display estimated remaining time."
+          (fun _ ->
+            let r = source#remaining in
+            if r < 0 then "(undef)"
+            else (
+              let t = Frame.seconds_of_main r in
+              Printf.sprintf "%.2f" t)))
 
     method is_ready =
-      if infallible then begin
+      if infallible then (
         assert source#is_ready;
-        true
-      end
+        true)
       else source#is_ready
 
     method remaining = source#remaining
-
     method abort_track = source#abort_track
-
     method seek len = source#seek len
 
     (* Operator startup *)
     method private wake_up activation =
-      start_stop#wake_up activation;
+      (* We prefer [name] as an ID over the default,
+       * but do not overwrite user-defined ID.
+       * Our ID will be used for the server interface. *)
+      if name <> "" then self#set_id ~definitive:false name;
+
+      self#log#debug "Clock is %s."
+        (Source.Clock_variables.to_string self#clock);
+      self#log#info "Content type is %s."
+        (Frame.string_of_content_type self#ctype);
+
       (* Get our source ready.
        * This can take a while (preparing playlists, etc). *)
       source#get_ready ((self :> operator) :: activation);
@@ -136,19 +124,18 @@ class virtual output ~content_kind ~output_kind ?(name = "") ~infallible
         while not source#is_ready do
           self#log#important "Waiting for %S to be ready..." source#id;
           Thread.delay 1.
-        done
+        done;
 
-    method private may_start = if self#is_ready then start_stop#may_start
+      start_stop#transition_to (if autostart then `Started else `Stopped);
 
-    method output_get_ready = self#may_start
+      self#register_telnet
 
     method private sleep =
-      self#do_stop;
+      start_stop#transition_to `Stopped;
       source#leave (self :> operator)
 
     (* Metadata stuff: keep track of what was streamed. *)
     val q_length = 10
-
     val metadata_q = Queue.create ()
 
     method private add_metadata m =
@@ -159,46 +146,41 @@ class virtual output ~content_kind ~output_kind ?(name = "") ~infallible
 
     (* The output process *)
     val mutable skip = false
-
     method private skip = skip <- true
-
-    method private get_frame buf = source#get buf
+    method private get_frame buf = if Frame.is_partial buf then source#get buf
 
     method private output =
-      self#may_start;
-
-      if is_started then begin
+      if self#is_ready && state <> `Stopped then
+        start_stop#transition_to `Started;
+      if start_stop#state = `Started then (
         (* Complete filling of the frame *)
         let get_count = ref 0 in
-        while Frame.is_partial memo && self#is_ready do
+        while Frame.is_partial self#memo && self#is_ready do
           incr get_count;
           if !get_count > Lazy.force Frame.size then
             self#log#severe
               "Warning: there may be an infinite sequence of empty tracks!";
-          source#get memo
+          source#get self#memo
         done;
         List.iter
           (fun (_, m) -> self#add_metadata m)
-          (Frame.get_all_metadata memo);
-        (* Output that frame if it has some data *)
-        if Frame.position memo > 0 then self#output_send memo;
-        if Frame.is_partial memo then begin
-          self#log#important "Source failed (no more tracks) stopping output...";
-          request_stop <- true
-        end
-      end;
+          (Frame.get_all_metadata self#memo);
 
-      self#may_stop
+        (* Output that frame if it has some data *)
+        if Frame.position self#memo > 0 then self#send_frame self#memo;
+        if Frame.is_partial self#memo then (
+          self#log#important "Source failed (no more tracks) stopping output...";
+          self#transition_to `Idle))
 
     method after_output =
       (* Let [memo] be cleared and signal propagated *)
       super#after_output;
+
       (* Perform skip if needed *)
-      if skip then begin
+      if skip then (
         self#log#important "Performing user-requested skip";
         skip <- false;
-        self#abort_track
-      end
+        self#abort_track)
   end
 
 class dummy ~infallible ~on_start ~on_stop ~autostart ~kind source =
@@ -208,33 +190,31 @@ class dummy ~infallible ~on_start ~on_stop ~autostart ~kind source =
         source autostart ~name:"dummy" ~output_kind:"output.dummy" ~infallible
           ~on_start ~on_stop ~content_kind:kind
 
-    method private output_reset = ()
-
-    method private output_start = ()
-
-    method private output_stop = ()
-
-    method private output_send _ = ()
+    method private reset = ()
+    method private start = ()
+    method private stop = ()
+    method private send_frame _ = ()
   end
 
 let () =
-  let kind = Lang.univ_t 1 in
-  Lang.add_operator "output.dummy" ~active:true
-    (proto @ [("", Lang.source_t kind, None, None)])
-    ~category:Lang.Output ~descr:"Dummy output for debugging purposes."
-    ~kind:(Lang.Unconstrained kind)
-    (fun p kind ->
+  let kind = Lang.any in
+  let return_t = Lang.kind_type_of_kind_format kind in
+  Lang.add_operator "output.dummy"
+    (proto @ [("", Lang.source_t return_t, None, None)])
+    ~category:`Output ~descr:"Dummy output for debugging purposes." ~meth
+    ~return_t
+    (fun p ->
       let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
       let autostart = Lang.to_bool (List.assoc "start" p) in
       let on_start = List.assoc "on_start" p in
       let on_stop = List.assoc "on_stop" p in
-      let on_start () = ignore (Lang.apply ~t:Lang.unit_t on_start []) in
-      let on_stop () = ignore (Lang.apply ~t:Lang.unit_t on_stop []) in
-      ( new dummy
-          ~kind ~on_start ~on_stop ~infallible ~autostart (List.assoc "" p)
-        :> Source.source ))
+      let on_start () = ignore (Lang.apply on_start []) in
+      let on_stop () = ignore (Lang.apply on_stop []) in
+      let kind = Source.Kind.of_kind kind in
+      new dummy
+        ~kind ~on_start ~on_stop ~infallible ~autostart (List.assoc "" p))
 
-(** More concrete abstract-class, which takes care of the #output_send
+(** More concrete abstract-class, which takes care of the #send_frame
   * method for outputs based on encoders. *)
 class virtual encoded ~content_kind ~output_kind ~name ~infallible ~on_start
   ~on_stop ~autostart source =
@@ -245,12 +225,10 @@ class virtual encoded ~content_kind ~output_kind ~name ~infallible ~on_start
           autostart
 
     method virtual private insert_metadata : Meta_format.export_metadata -> unit
-
     method virtual private encode : Frame.t -> int -> int -> 'a
-
     method virtual private send : 'a -> unit
 
-    method private output_send frame =
+    method private send_frame frame =
       let rec output_chunks frame =
         let f start stop =
           begin
@@ -269,7 +247,7 @@ class virtual encoded ~content_kind ~output_kind ~name ~infallible ~on_start
             output_chunks frame (stop :: l)
       in
       output_chunks frame
-        ( 0
+        (0
         :: List.sort compare
              (List.map fst (Frame.get_all_metadata frame) @ Frame.breaks frame)
         )
