@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -55,5 +55,4 @@ let register mimetype =
   Playlist_parser.parsers#register mimetype
     { Playlist_parser.strict = true; Playlist_parser.parser = tracks }
 
-let () =
-  ignore (Dtools.Init.at_start (fun () -> List.iter register conf_xml#get))
+let () = Lifecycle.on_start (fun () -> List.iter register conf_xml#get)
