@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -51,11 +51,6 @@ let parse_uri uri =
       (String.sub uri 0 i, String.sub uri (i + 1) (String.length uri - (i + 1)))
   with _ -> None
 
-let cleanup =
-  let re1 = Str.regexp "^[\t ]*" in
-  let re2 = Str.regexp "[\t ]*$" in
-  fun s -> Str.global_replace re1 "" (Str.global_replace re2 "" s)
-
 (** Metadata *)
 
 type metadata = (string, string) Hashtbl.t
@@ -63,15 +58,14 @@ type metadata = (string, string) Hashtbl.t
 let string_of_metadata metadata =
   let b = Buffer.create 20 in
   let f = Format.formatter_of_buffer b in
-  let escape f s = Utils.escape_utf8 f s in
+  let escape f s = Utils.escape_utf8_formatter f s in
   let first = ref true in
   Hashtbl.iter
     (fun k v ->
-      if !first then begin
+      if !first then (
         first := false;
-        try Format.fprintf f "%s=%a" k escape v with _ -> ()
-      end
-      else (try Format.fprintf f "\n%s=%a" k escape v with _ -> ()))
+        try Format.fprintf f "%s=\"%a\"" k escape v with _ -> ())
+      else (try Format.fprintf f "\n%s=\"%a\"" k escape v with _ -> ()))
     metadata;
   Format.pp_print_flush f ();
   Buffer.contents b
@@ -134,7 +128,7 @@ type status = Idle | Resolving | Ready | Playing | Destroyed
 type t = {
   id : int;
   initial_uri : string;
-  kind : Frame.content_kind option;
+  mutable ctype : Frame.content_type option;
   (* No kind for raw requests *)
   persistent : bool;
   (* The status of a request gives partial information of what's being done
@@ -151,11 +145,12 @@ type t = {
   log : log;
   mutable root_metadata : metadata;
   mutable indicators : indicator list list;
-  mutable decoder : (unit -> Decoder.file_decoder) option;
+  mutable decoder : (unit -> Decoder.file_decoder_ops) option;
 }
 
-let kind x = x.kind
-let initial_uri x = x.initial_uri
+let ctype r = r.ctype
+let initial_uri r = r.initial_uri
+let status r = r.status
 
 let indicator ?(metadata = Hashtbl.create 10) ?temporary s =
   { string = home_unrelate s; temporary = temporary = Some true; metadata }
@@ -233,6 +228,13 @@ let () =
     | No_indicator -> Some "All options exhausted while processing request"
     | _ -> None)
 
+let string_of_indicators t =
+  let i = t.indicators in
+  let string_of_list l = "[" ^ String.concat ", " l ^ "]" in
+  let i = List.map (List.map (fun i -> i.string)) i in
+  let i = List.map string_of_list i in
+  string_of_list i
+
 let peek_indicator t =
   match t.indicators with
     | (h :: _) :: _ -> h
@@ -248,10 +250,9 @@ let rec pop_indicator t =
       | [] :: _ -> assert false
       | [] -> raise No_indicator
   in
-  if i.temporary then begin
+  if i.temporary then (
     try Unix.unlink i.string
-    with e -> log#severe "Unlink failed: %S" (Printexc.to_string e)
-  end;
+    with e -> log#severe "Unlink failed: %S" (Printexc.to_string e));
   t.decoder <- None;
   if repop then pop_indicator t
 
@@ -319,67 +320,75 @@ let file_is_readable name =
     true
   with Unix.Unix_error _ -> false
 
+let read_metadata t =
+  let indicator = peek_indicator t in
+  let name = indicator.string in
+  if not (file_exists name) then
+    log#important "File %s does not exist!" (Utils.quote_string name)
+  else if not (file_is_readable name) then
+    log#important "Read permission denied for %s!" (Utils.quote_string name)
+  else
+    List.iter
+      (fun (_, resolver) ->
+        try
+          let ans = resolver name in
+          List.iter
+            (fun (k, v) ->
+              let k = String.lowercase_ascii k in
+              if conf_override_metadata#get || get_metadata t k = None then
+                Hashtbl.replace indicator.metadata k v)
+            ans;
+          if conf_duration#get && get_metadata t "duration" = None then (
+            try
+              Hashtbl.replace indicator.metadata "duration"
+                (string_of_float (duration name))
+            with Not_found -> ())
+        with _ -> ())
+      (get_decoders conf_metadata_decoders mresolvers)
+
 let local_check t =
-  let check_decodable kind =
+  let check_decodable ctype =
     try
       while t.decoder = None && file_exists (peek_indicator t).string do
         let indicator = peek_indicator t in
         let name = indicator.string in
         let metadata = get_all_metadata t in
-        if not (file_is_readable name) then begin
-          log#important "Read permission denied for %S!" name;
+        if not (file_is_readable name) then (
+          log#important "Read permission denied for %s!"
+            (Utils.quote_string name);
           add_log t "Read permission denied!";
-          pop_indicator t
-        end
+          pop_indicator t)
         else (
-          match Decoder.get_file_decoder ~metadata name kind with
+          match Decoder.get_file_decoder ~metadata ~ctype name with
             | Some (decoder_name, f) ->
                 t.decoder <- Some f;
                 set_root_metadata t "decoder" decoder_name;
-                List.iter
-                  (fun (_, resolver) ->
-                    try
-                      let ans = resolver name in
-                      List.iter
-                        (fun (k, v) ->
-                          let k = String.lowercase_ascii k in
-                          if
-                            conf_override_metadata#get
-                            || get_metadata t k = None
-                          then Hashtbl.replace indicator.metadata k (cleanup v))
-                        ans;
-                      if conf_duration#get && get_metadata t "duration" = None
-                      then (
-                        try
-                          Hashtbl.replace indicator.metadata "duration"
-                            (string_of_float (duration name))
-                        with Not_found -> () )
-                    with _ -> ())
-                  (get_decoders conf_metadata_decoders mresolvers);
+                read_metadata t;
                 t.status <- Ready
-            | None -> pop_indicator t )
+            | None -> pop_indicator t)
       done
     with No_indicator -> ()
   in
-  match t.kind with None -> () | Some k -> check_decodable k
+  match t.ctype with None -> () | Some t -> check_decodable t
 
 let push_indicators t l =
   if l <> [] then (
     let hd = List.hd l in
-    add_log t (Printf.sprintf "Pushed [%S;...]." hd.string);
+    add_log t (Printf.sprintf "Pushed [%s;...]." (Utils.quote_string hd.string));
     t.indicators <- l :: t.indicators;
     t.decoder <- None;
+
     (* Performing a local check is quite fast and allows the request
      * to be instantly available if it is only made of valid local files,
      * without any need for a resolution process. *)
     (* TODO sometimes it's not that fast actually, and it'd be nice
      * to be able to disable this check in some cases, like playlist.safe. *)
-    local_check t )
+    local_check t)
 
 let is_ready t =
   t.indicators <> []
   && Sys.file_exists (peek_indicator t).string
-  && (t.decoder <> None || t.kind = None)
+  && (t.decoder <> None || t.ctype = None)
 
 (** [get_filename request] returns
   * [Some f] if the request successfully lead to a local file [f],
@@ -391,16 +400,18 @@ let update_metadata t =
   let replace = Hashtbl.replace t.root_metadata in
   replace "rid" (string_of_int t.id);
   replace "initial_uri" t.initial_uri;
+
   (* TOP INDICATOR *)
   replace "temporary"
-    ( match t.indicators with
+    (match t.indicators with
       | (h :: _) :: _ -> if h.temporary then "true" else "false"
-      | _ -> "false" );
+      | _ -> "false");
   begin
     match get_filename t with
     | Some f -> replace "filename" f
     | None -> ()
   end;
+
   (* STATUS *)
   begin
     match t.resolving with
@@ -409,21 +420,23 @@ let update_metadata t =
   end;
   begin
     match t.on_air with
-    | Some d -> replace "on_air" (pretty_date (Unix.localtime d))
+    | Some d ->
+        replace "on_air" (pretty_date (Unix.localtime d));
+        replace "on_air_timestamp" (Printf.sprintf "%.02f" d)
     | None -> ()
   end;
   begin
-    match t.kind with
+    match t.ctype with
     | None -> ()
-    | Some k -> replace "kind" (Frame.string_of_content_kind k)
+    | Some ct -> replace "kind" (Frame.string_of_content_type ct)
   end;
   replace "status"
-    ( match t.status with
+    (match t.status with
       | Idle -> "idle"
       | Resolving -> "resolving"
       | Ready -> "ready"
       | Playing -> "playing"
-      | Destroyed -> "destroyed" )
+      | Destroyed -> "destroyed")
 
 let get_metadata t k =
   update_metadata t;
@@ -467,7 +480,7 @@ let leak_warning =
   Dtools.Conf.int ~p:(conf#plug "leak_warning") ~d:100
     "Number of requests at which a leak warning should be issued."
 
-let create ~kind ?(metadata = []) ?(persistent = false) ?(indicators = []) u =
+let create ?(metadata = []) ?(persistent = false) ?(indicators = []) u =
   (* Find instantaneous request loops *)
   let () =
     let n = Pool.size () in
@@ -484,7 +497,8 @@ let create ~kind ?(metadata = []) ?(persistent = false) ?(indicators = []) u =
     {
       id = rid;
       initial_uri = u;
-      kind;
+      ctype = None;
+      (* This is fixed when resolving the request. *)
       persistent;
       on_air = None;
       resolving = None;
@@ -500,9 +514,6 @@ let create ~kind ?(metadata = []) ?(persistent = false) ?(indicators = []) u =
   push_indicators t (if indicators = [] then [indicator u] else indicators);
   t
 
-let create_raw = create ~kind:None
-let create ~kind = create ~kind:(Some kind)
-
 let on_air t =
   t.on_air <- Some (Unix.time ());
   t.status <- Playing;
@@ -512,18 +523,19 @@ let destroy ?force t =
   assert (t.status <> Destroyed);
   t.on_air <- None;
   if t.status = Playing then t.status <- Ready;
-  if force = Some true || not t.persistent then begin
+  if force = Some true || not t.persistent then (
     t.status <- Idle;
+
     (* Freeze the metadata *)
     t.root_metadata <- get_all_metadata t;
+
     (* Remove the URIs, unlink temporary files *)
     while t.indicators <> [] do
       pop_indicator t
     done;
     t.status <- Destroyed;
     add_log t "Request finished.";
-    Pool.kill t.id grace_time#get
-  end
+    Pool.kill t.id grace_time#get)
 
 let clean () =
   Pool.iter (fun _ r -> if r.status <> Destroyed then destroy ~force:true r)
@@ -551,8 +563,8 @@ let is_static s =
       | Some (proto, _) -> (
           match protocols#get proto with
             | Some handler -> handler.static
-            | None -> false )
-      | None -> false )
+            | None -> false)
+      | None -> false)
 
 (** Resolving engine. *)
 
@@ -560,12 +572,17 @@ type resolve_flag = Resolved | Failed | Timeout
 
 exception ExnTimeout
 
-let resolve t timeout =
+let resolve ~ctype t timeout =
+  assert (
+    t.ctype = None || Frame.compatible (Option.get t.ctype) (Option.get ctype));
+  log#debug "Resolving request %s." (string_of_indicators t);
+  t.ctype <- ctype;
   t.resolving <- Some (Unix.time ());
   t.status <- Resolving;
   let maxtime = Unix.time () +. timeout in
   let resolve_step () =
     let i = peek_indicator t in
+    log#f 6 "Resolve step %s in %s." i.string (string_of_indicators t);
     (* If the file is local we only need to check that it's valid,
      * we'll actually do that in a single local_check for all local indicators
      * on the top of the stack. *)
@@ -576,28 +593,31 @@ let resolve t timeout =
             match protocols#get proto with
               | Some handler ->
                   add_log t
-                    (Printf.sprintf "Resolving %S (timeout %.0fs)..." i.string
+                    (Printf.sprintf "Resolving %s (timeout %.0fs)..."
+                       (Utils.quote_string i.string)
                        timeout);
                   let production =
                     handler.resolve ~log:(add_log t) arg maxtime
                   in
-                  if production = [] then begin
+                  if production = [] then (
                     log#info
-                      "Failed to resolve %S! For more info, see server command \
-                       'trace %d'."
-                      i.string t.id;
-                    ignore (pop_indicator t)
-                  end
+                      "Failed to resolve %s! For more info, see server command \
+                       `request.trace %d`."
+                      (Utils.quote_string i.string)
+                      t.id;
+                    ignore (pop_indicator t))
                   else push_indicators t production
               | None ->
-                  log#important "Unknown protocol %S in URI %S!" proto i.string;
+                  log#important "Unknown protocol %S in URI %s!" proto
+                    (Utils.quote_string i.string);
                   add_log t "Unknown protocol!";
-                  pop_indicator t )
+                  pop_indicator t)
         | None ->
             let log_level = if i.string = "" then 4 else 3 in
-            log#f log_level "Nonexistent file or ill-formed URI %S!" i.string;
+            log#f log_level "Nonexistent file or ill-formed URI %s!"
+              (Utils.quote_string i.string);
             add_log t "Nonexistent file or ill-formed URI!";
-            pop_indicator t )
+            pop_indicator t)
   in
   let result =
     try
@@ -606,7 +626,7 @@ let resolve t timeout =
         if timeleft > 0. then resolve_step ()
         else (
           add_log t "Global timeout.";
-          raise ExnTimeout )
+          raise ExnTimeout)
       done;
       Resolved
     with
@@ -615,6 +635,7 @@ let resolve t timeout =
           add_log t "Every possibility failed!";
           Failed
   in
+  log#debug "Resolved to %s." (string_of_indicators t);
   let excess = Unix.time () -. maxtime in
   if excess > 0. then log#severe "Time limit exceeded by %.2f secs!" excess;
   t.resolving <- None;
