@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,8 +22,4 @@
 
 (** Default runner. *)
 
-module Runner = struct
-  let options = Main.options
-end
-
-module Main = Main.Make (Runner)
+let () = Main.start ()
