@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,8 +25,7 @@
   *
   * Sources can produce a stream, if something pulls it.
   * Sources can pull streams from other sources (such non-elementary sources
-  * are called operators).
-  * But who starts pulling?
+  * are called operators). But who starts pulling?
   *
   * Some sources have a noticeable effect, for example outputs.
   * Some are indirectly needed by outputs.
@@ -36,8 +35,8 @@
   *
   * Clocks are in charge of animating active sources.
   * Each clock "owns" a number of active sources, and indirectly some
-  * sources owned by those active sources,
-  * and controls access to their streams. *)
+  * sources owned by those active sources, and controls access to their
+  * streams. *)
 
 (** Fallibility type MUST be defined BEFORE clocks.
   * Otherwise the module cannot be well-typed since the list of all
@@ -58,7 +57,7 @@ type source_t = Fallible | Infallible
   *
   * Most clocks are passive, i.e. they don't run anything
   * directly, but may only tick when something happens to a source.
-  * The wallclock (formerly root.ml) is the default, active clock:
+  * The clock is the default, active clock:
   * when started, it launches a thread which keeps ticking regularly.
   *
   * A clock needs to know all the active sources under its control,
@@ -75,30 +74,28 @@ type source_t = Fallible | Infallible
   *
   * The idea is that when an output is created it assigns a clock to itself
   * according to its sources' clocks. Eventually, all remaining unknown clocks
-  * are forced to wallclock. *)
+  * are forced to clock. *)
+
+type sync = [ `Auto | `CPU | `None ]
+type self_sync = [ `Static | `Dynamic ] * bool
 
 class type ['a, 'b] proto_clock =
   object
     method id : string
+    method sync_mode : sync
 
     (** Attach an active source, detach active sources by filter. *)
 
     method attach : 'a -> unit
-
     method detach : ('a -> bool) -> unit
 
     (** Attach a sub_clock, get all subclocks, see below. *)
 
     method attach_clock : 'b -> unit
-
     method detach_clock : 'b -> unit
-
     method sub_clocks : 'b list
-
     method start_outputs : ('a -> bool) -> unit -> 'a list
-
     method get_tick : int
-
     method end_tick : unit
   end
 
@@ -142,7 +139,7 @@ type 'a var =
 and 'a link_t =
   | Unknown of 'a list * 'a var list
       (** the clock variable is unknown but depends on other variables *)
-  | Same_as of 'a var  (** the clock variable is subtituted by another *)
+  | Same_as of 'a var  (** the clock variable is substituted by another *)
 
 let debug = Utils.getenv_opt "LIQUIDSOAP_DEBUG" <> None
 let create_known c = Known c
@@ -204,12 +201,10 @@ let rec unify a b =
     | Link ra, Link rb when ra == rb -> ()
     | ( Link ({ contents = Unknown (sa, ca) } as ra),
         Link ({ contents = Unknown (sb, cb) } as rb) ) ->
-        (* TODO perhaps optimize ca@cb *)
         occurs_check a b;
         let merge =
           let s = List.sort_uniq Stdlib.compare (List.rev_append sa sb) in
           let sc = List.sort_uniq Stdlib.compare (List.rev_append ca cb) in
-          (* Using List.rev_append to remain tail-recursive, see #1108. *)
           Link (ref (Unknown (s, sc)))
         in
         ra := Same_as merge;
@@ -228,8 +223,115 @@ let rec forget var subclock =
     | Link ({ contents = Unknown (sources, clocks) } as r) ->
         r := Unknown (sources, List.filter (( <> ) subclock) clocks)
 
+(** Kind with variables in order to unify kinds. *)
+module Kind = struct
+  open Frame
+
+  exception Conflict of string * string
+
+  type kind = Frame.kind Unifier.t
+  type t = kind fields
+
+  let deref = Unifier.deref
+
+  let of_kind { audio; video; midi } =
+    let f = Unifier.make in
+    { audio = f audio; video = f video; midi = f midi }
+
+  let set_audio { video; midi } audio =
+    { Frame.audio = Unifier.make audio; video; midi }
+
+  let set_video { audio; midi } video =
+    { Frame.audio; video = Unifier.make video; midi }
+
+  let set_midi { audio; video } midi =
+    { Frame.audio; video; midi = Unifier.make midi }
+
+  let content_type { audio; video; midi } =
+    let get t v =
+      match deref v with
+        | `Internal | `Any -> (
+            match t with
+              | `Audio -> Frame_content.default_audio ()
+              | `Video -> Frame_content.default_video ()
+              | `Midi -> Frame_content.default_midi ())
+        | `Format f -> f
+        | `Kind k -> Frame_content.default_format k
+    in
+    {
+      Frame.audio = get `Audio audio;
+      video = get `Video video;
+      midi = get `Midi midi;
+    }
+
+  let to_string { audio; video; midi } =
+    Frame.string_of_content_kind
+      { Frame.audio = deref audio; video = deref video; midi = deref midi }
+
+  let unify_kind k k' =
+    let to_s k = Frame.string_of_kind (deref k) in
+    try
+      match (deref k, deref k') with
+        (* Formats *)
+        | `Kind ki, `Kind ki' when ki = ki' -> Unifier.(k <-- k')
+        (* Params *)
+        | `Format f, `Format f' -> Frame_content.merge f f'
+        (* Format/params *)
+        | `Kind ki, `Format f when Frame_content.kind f = ki ->
+            Unifier.(k <-- k')
+        | `Format f, `Kind ki when Frame_content.kind f = ki ->
+            Unifier.(k' <-- k)
+        (* `Internal/'a *)
+        | `Internal, `Internal -> Unifier.(k <-- k')
+        | `Internal, `Kind ki when Frame_content.is_internal ki ->
+            Unifier.(k <-- k')
+        | `Internal, `Format f when Frame_content.(is_internal (kind f)) ->
+            Unifier.(k <-- k')
+        | `Kind ki, `Internal when Frame_content.is_internal ki ->
+            Unifier.(k' <-- k)
+        | `Format f, `Internal when Frame_content.(is_internal (kind f)) ->
+            Unifier.(k' <-- k)
+        (* Any/'a *)
+        | `Any, _ -> Unifier.(k <-- k')
+        | _, `Any -> Unifier.(k' <-- k)
+        | _ -> assert false
+    with _ -> raise (Conflict (to_s k, to_s k'))
+
+  let unify k k' =
+    unify_kind k.audio k'.audio;
+    unify_kind k.video k'.video;
+    unify_kind k.midi k'.midi
+end
+
 (** {1 Sources} *)
 
+(** Instrumentation. *)
+
+type metadata = (int * (string, string) Hashtbl.t) list
+type clock_sync_mode = [ sync | `Unknown ]
+
+type watcher = {
+  get_ready :
+    stype:source_t ->
+    is_active:bool ->
+    id:string ->
+    ctype:Frame.content_type ->
+    clock_id:string ->
+    clock_sync_mode:clock_sync_mode ->
+    unit;
+  leave : unit -> unit;
+  get_frame :
+    start_time:float ->
+    end_time:float ->
+    start_position:int ->
+    end_position:int ->
+    is_partial:bool ->
+    metadata:metadata ->
+    unit;
+  before_output : unit -> unit;
+  after_output : unit -> unit;
+}
+
 let source_log = Log.make ["source"]
 
 (** Has any output been created? This is used by Main to decide if
@@ -245,32 +347,51 @@ let add_new_output, iterate_new_outputs =
         List.iter f !l;
         l := []) )
 
-class virtual operator ?(name = "src") content_kind sources =
+(** Generate an identifier from the name of the source. *)
+let generate_id =
+  let t = Hashtbl.create 10 in
+  fun name ->
+    if not (Hashtbl.mem t name) then Hashtbl.add t name (ref (-1));
+    let n = Hashtbl.find t name in
+    incr n;
+    name ^ "_" ^ string_of_int !n
+
+class virtual operator ?(name = "src") ?audio_in ?video_in ?midi_in out_kind
+  sources =
+  let f kind (fn, el) = match el with None -> kind | Some v -> fn kind v in
+  let in_kind =
+    List.fold_left f out_kind
+      [
+        (Kind.set_audio, audio_in);
+        (Kind.set_video, video_in);
+        (Kind.set_midi, midi_in);
+      ]
+  in
   object (self)
+    (** Monitoring *)
+    val mutable watchers = []
+
+    method add_watcher w = watchers <- w :: watchers
+    method private iter_watchers fn = List.iter fn watchers
 
     (** Logging and identification *)
 
     val mutable log = source_log
-
     method private create_log = log <- Log.make [self#id]
-
-    method private log = log
-
+    method log = log
     val mutable id = ""
-
     val mutable definitive_id = false
-
-    initializer id <- name ^ "_" ^ string_of_int (Oo.id self)
-
+    val mutable name = name
+    method set_name n = name <- n
+    initializer id <- generate_id name
     method id = id
 
     method set_id ?(definitive = true) s =
-      (* ID musn't contain "." *)
-      let s = Pcre.substitute ~pat:"\\." ~subst:(fun _ -> "(dot)") s in
       let s = Pcre.substitute ~pat:"[ \t\n]" ~subst:(fun _ -> "_") s in
       if not definitive_id then (
         id <- s;
-        definitive_id <- definitive );
+        definitive_id <- definitive);
+
       (* Sometimes the ID is changed during initialization,
        * in order to make it equal to the server name,
        * which is only registered at initialization time in order
@@ -282,12 +403,15 @@ class virtual operator ?(name = "src") content_kind sources =
     if debug then
       Gc.finalise (fun s -> source_log#info "Garbage collected %s." s#id) self
 
+    val mutex = Mutex.create ()
+    method mutexify : 'a 'b. ('a -> 'b) -> 'a -> 'b = Tutils.mutexify mutex
+
     (** Is the source infallible, i.e. is it always guaranteed that there
     * will be always be a next track immediately available. *)
     method virtual stype : source_t
 
     (** Is the source active *)
-    method is_output = false
+    method is_active = false
 
     (** Children sources *)
     val mutable sources : operator list = sources
@@ -302,13 +426,31 @@ class virtual operator ?(name = "src") content_kind sources =
      * be overridden, and it needs to be called at initialization:
      * #wake_up is too late since it's the clock who initiates it. *)
     val clock : active_operator var = create_unknown ~sources:[] ~sub_clocks:[]
-
     method clock = clock
+    method virtual self_sync : self_sync
 
     method private set_clock =
       List.iter (fun s -> unify self#clock s#clock) sources
 
     initializer self#set_clock
+    method kind = out_kind
+    initializer List.iter (fun s -> Kind.unify s#kind in_kind) sources
+    val mutable ctype = None
+
+    (* Content type. *)
+    method ctype =
+      match ctype with
+        | Some ctype -> ctype
+        | None ->
+            let ct = Kind.content_type self#kind in
+            self#log#debug "Content kind: %s, content type: %s"
+              (Kind.to_string self#kind)
+              (Frame.string_of_content_type ct);
+            ctype <- Some ct;
+            ct
+
+    method private audio_channels =
+      Frame_content.Audio.channels_of_format self#ctype.Frame.audio
 
     (** Startup/shutdown.
     *
@@ -341,30 +483,14 @@ class virtual operator ?(name = "src") content_kind sources =
     * so the activation management API is not thread-safe. *)
 
     val mutable caching = false
-
     val mutable dynamic_activations : operator list list = []
-
     val mutable static_activations : operator list list = []
 
     (* List of callbacks executed when source shuts down. *)
     val mutable on_shutdown = []
 
-    val on_shutdown_m = Mutex.create ()
-
-    method on_shutdown fn =
-      (Tutils.mutexify on_shutdown_m (fun () ->
-           on_shutdown <- fn :: on_shutdown))
-        ()
-
-    (* contains: (ns,descr,usage,name,f) *)
-    val mutable commands = []
-
-    val mutable ns_kind = "unknown"
-
-    val mutable ns = []
-
-    method register_command ~descr ?usage name f =
-      commands <- (descr, usage, name, f) :: commands
+    method on_shutdown =
+      self#mutexify (fun fn -> on_shutdown <- fn :: on_shutdown)
 
     method private update_caching_mode =
       let string_of activations =
@@ -373,12 +499,13 @@ class virtual operator ?(name = "src") content_kind sources =
              (fun l -> String.concat ":" (List.map (fun s -> s#id) l))
              activations)
       in
-      self#log#info "Activations changed: static=[%s], dynamic=[%s]."
+      self#log#debug "Activations changed: static=[%s], dynamic=[%s]."
         (string_of static_activations)
         (string_of dynamic_activations);
+
       (* Decide whether caching mode is needed, and why *)
       match
-        if self#is_output then Some "active source"
+        if self#is_active then Some "active source"
         else (
           match static_activations with
             | [] -> None
@@ -389,18 +516,21 @@ class virtual operator ?(name = "src") content_kind sources =
                     dynamic_activations
                 then Some "possible dynamic activation"
                 else None
-            | _ -> Some "two static activations" )
+            | _ -> Some "two static activations")
       with
         | None ->
-            if caching then begin
+            if caching then (
               caching <- false;
-              self#log#info "Disabling caching mode."
-            end
+              self#log#debug "Disabling caching mode.")
         | Some msg ->
-            if not caching then begin
+            if not caching then (
               caching <- true;
-              self#log#info "Enabling caching mode: %s." msg
-            end
+              self#log#debug "Enabling caching mode: %s." msg)
+
+    val mutable on_get_ready = []
+
+    method on_get_ready =
+      self#mutexify (fun fn -> on_get_ready <- on_get_ready @ [fn])
 
     (* Ask for initialization.
      * The current implementation makes it dangerous to call #get_ready from
@@ -408,28 +538,37 @@ class virtual operator ?(name = "src") content_kind sources =
      * forbidden. *)
     method get_ready ?(dynamic = false) (activation : operator list) =
       if log == source_log then self#create_log;
-      if static_activations = [] && dynamic_activations = [] then begin
-        source_log#info "Source %s gets up." id;
-        self#wake_up activation;
-        if commands <> [] then begin
-          assert (ns = []);
-          ns <- Server.register [self#id] ns_kind;
-          self#set_id (Server.to_string ns);
-          List.iter
-            (fun (descr, usage, name, f) -> Server.add ~ns ~descr ?usage name f)
-            commands
-        end
-      end;
+      if static_activations = [] && dynamic_activations = [] then (
+        source_log#info "Source %s gets up with content kind: %s." id
+          (Kind.to_string self#kind);
+        self#wake_up activation);
+      Server.register_op self#id name;
       if dynamic then dynamic_activations <- activation :: dynamic_activations
       else static_activations <- activation :: static_activations;
-      self#update_caching_mode
+      self#update_caching_mode;
+      let clock_id, clock_sync_mode =
+        match deref self#clock with
+          | Known c -> (c#id, (c#sync_mode :> clock_sync_mode))
+          | _ -> ("unknown", `Unknown)
+      in
+      self#iter_watchers (fun w ->
+          w.get_ready ~stype:self#stype ~is_active:self#is_active ~id:self#id
+            ~ctype:self#ctype ~clock_id ~clock_sync_mode);
+      let on_get_ready = self#mutexify (fun () -> on_get_ready) () in
+      List.iter (fun fn -> fn ()) on_get_ready
+
+    val mutable on_leave = []
+    method on_leave = self#mutexify (fun fn -> on_leave <- on_leave @ [fn])
 
     (* Release the source, which will shutdown if possible.
      * The current implementation makes it dangerous to call #leave from
      * another thread than the Root one, as interleaving with #get is
      * forbidden. *)
-    method leave ?(dynamic = false) src =
+    method leave ?(failed_to_start = true) ?(dynamic = false) src =
+      let on_leave = self#mutexify (fun () -> on_leave) () in
+      List.iter (fun fn -> fn ()) on_leave;
       let rec remove acc = function
+        | [] when failed_to_start -> []
         | [] ->
             self#log#critical "Got ill-balanced activations (from %s)!" src#id;
             assert false
@@ -439,41 +578,46 @@ class virtual operator ?(name = "src") content_kind sources =
       if dynamic then dynamic_activations <- remove [] dynamic_activations
       else static_activations <- remove [] static_activations;
       self#update_caching_mode;
-      if static_activations = [] && dynamic_activations = [] then begin
-        source_log#info "Source %s gets down." id;
-        (Tutils.mutexify on_shutdown_m (fun () ->
-             List.iter (fun fn -> try fn () with _ -> ()) on_shutdown;
-             on_shutdown <- []))
+      if static_activations = [] && dynamic_activations = [] then (
+        source_log#debug "Source %s gets down." id;
+        self#mutexify
+          (fun () ->
+            List.iter (fun fn -> try fn () with _ -> ()) on_shutdown;
+            on_shutdown <- [])
           ();
-        self#sleep;
-        List.iter (fun (_, _, name, _) -> Server.remove ~ns name) commands;
-        if ns <> [] then begin
-          Server.unregister ns;
-          ns <- []
-        end
-      end
+        self#sleep);
+      self#iter_watchers (fun w -> w.leave ())
 
     method is_up = static_activations <> [] || dynamic_activations <> []
 
     (** Two methods called for initialization and shutdown of the source *)
     method private wake_up activation =
-      self#log#info "Content kind is %s."
-        (Frame.string_of_content_kind content_kind);
+      self#log#debug "Clock is %s." (variable_to_string self#clock);
+      self#log#info "Content type is %s."
+        (Frame.string_of_content_type self#ctype);
       let activation = (self :> operator) :: activation in
       List.iter (fun s -> s#get_ready ?dynamic:None activation) sources
 
     method private sleep =
-      List.iter (fun s -> s#leave ?dynamic:None (self :> operator)) sources
+      List.iter
+        (fun s ->
+          s#leave ?failed_to_start:None ?dynamic:None (self :> operator))
+        sources
 
     (** Streaming *)
 
-    method kind = content_kind
-
     (* Number of frames left in the current track:
      * -1 means Infinity, time unit is the frame. *)
     method virtual remaining : int
+    val mutable elapsed = 0
+    method elapsed = elapsed
 
-    (* [self#seek x] skips [x] master ticks.
+    method duration =
+      let r = self#remaining in
+      let e = self#elapsed in
+      if r < 0 || e < 0 then -1 else e + r
+
+    (* [self#seek x] skips [x] main ticks.
      * returns the number of ticks actually skipped.
      * By default it always returns 0, refusing to seek at all. *)
     method seek (_ : int) =
@@ -490,10 +634,72 @@ class virtual operator ?(name = "src") content_kind sources =
      * to be played if there's anything like a file. *)
     method virtual abort_track : unit
 
-    (* In caching mode, remember what has been given during the current tick *)
-    val memo = Frame.create content_kind
+    (* In caching mode, remember what has been given during the current
+       tick. The generation is deferred until we actually have computed the kind
+       by unfication. *)
+    val mutable memo = None
 
-    method get_memo = memo
+    method memo =
+      match memo with
+        | Some memo -> memo
+        | None ->
+            let m = Frame.create self#ctype in
+            memo <- Some m;
+            m
+
+    val mutable last_metadata = None
+    method last_metadata = self#mutexify (fun () -> last_metadata) ()
+    val mutable on_metadata : (Frame.metadata -> unit) list = []
+
+    method on_metadata =
+      self#mutexify (fun fn -> on_metadata <- on_metadata @ [fn])
+
+    val mutable on_track : (Frame.metadata -> unit) list = []
+
+    (* We want to notify of new tracks on the next call after a
+       partial frame. *)
+    val mutable was_partial = true
+    method on_track = self#mutexify (fun fn -> on_track <- on_track @ [fn])
+
+    method private instrumented_get_frame buf =
+      let start_time = Unix.gettimeofday () in
+      let start_position = Frame.position buf in
+      self#get_frame buf;
+      let end_time = Unix.gettimeofday () in
+      let end_position = Frame.position buf in
+      let is_partial = Frame.is_partial buf in
+      if is_partial then elapsed <- 0
+      else elapsed <- elapsed + end_position - start_position;
+      let metadata =
+        List.filter
+          (fun (pos, _) -> start_position <= pos)
+          (Frame.get_all_metadata buf)
+      in
+      let on_metadata = self#mutexify (fun () -> on_metadata) () in
+      List.iter
+        (fun (i, m) ->
+          self#log#debug "Got metadata at position %d: calling handlers..." i;
+          List.iter (fun fn -> fn m) on_metadata)
+        metadata;
+      (match List.rev metadata with
+        | (_, m) :: _ ->
+            self#mutexify (fun () -> last_metadata <- Some (Hashtbl.copy m)) ()
+        | [] -> ());
+      self#mutexify
+        (fun () ->
+          if was_partial then (
+            was_partial <- false;
+            let m =
+              match Frame.get_metadata buf start_position with
+                | None -> Hashtbl.create 0
+                | Some m -> m
+            in
+            List.iter (fun fn -> fn m) on_track);
+          was_partial <- is_partial)
+        ();
+      self#iter_watchers (fun w ->
+          w.get_frame ~start_time ~start_position ~end_time ~end_position
+            ~is_partial ~metadata)
 
     (* [#get buf] completes the frame with the next data in the stream.
      * Depending on whether caching is enabled or not,
@@ -504,6 +710,7 @@ class virtual operator ?(name = "src") content_kind sources =
      * round ([#after_output]). *)
     method get buf =
       assert (Frame.is_partial buf);
+
       (* In some cases we can't avoid #get being called on a non-ready
        * source, for example:
        * - A starts pumping B, stops in the middle of the track
@@ -528,12 +735,16 @@ class virtual operator ?(name = "src") content_kind sources =
         if not self#is_ready then silent_end_track ()
         else (
           let b = Frame.breaks buf in
-          self#get_frame buf;
-          if List.length b + 1 <> List.length (Frame.breaks buf) then begin
-            self#log#severe "#get_frame didn't add exactly one break!";
-            assert false
-          end )
-      else begin
+          self#instrumented_get_frame buf;
+          if List.length b + 1 > List.length (Frame.breaks buf) then (
+            self#log#severe "#get_frame added too many breaks!";
+            assert false);
+          if List.length b + 1 < List.length (Frame.breaks buf) then (
+            self#log#severe
+              "#get_frame returned a buffer without enough breaks!";
+            assert false))
+      else (
+        let memo = self#memo in
         try Frame.get_chunk buf memo
         with Frame.No_chunk ->
           if not self#is_ready then silent_end_track ()
@@ -541,33 +752,32 @@ class virtual operator ?(name = "src") content_kind sources =
             (* [memo] has nothing new for [buf]. Feed [memo] and try again *)
             let b = Frame.breaks memo in
             let p = Frame.position memo in
-            self#get_frame memo;
-            if List.length b + 1 <> List.length (Frame.breaks memo) then begin
+            self#instrumented_get_frame memo;
+            if List.length b + 1 <> List.length (Frame.breaks memo) then (
               self#log#severe "#get_frame didn't add exactly one break!";
-              assert false
-            end
-            else if p < Frame.position memo then self#get buf
-            else Frame.add_break buf (Frame.position buf) )
-      end
+              assert false)
+            else if Frame.is_partial buf then
+              if p < Frame.position memo then self#get buf
+              else Frame.add_break buf (Frame.position buf)))
 
     (* That's the way the source produces audio data.
      * It cannot be called directly, but [#get] should be used instead, for
      * dealing with caching if needed. *)
     method virtual private get_frame : Frame.t -> unit
 
-    (* End the current output round.
-     * The default task is to clear the cache and propagate the call
-     * to children sources.
-     * It MUST be called at every frame for every source, even those who
-     * aren't played. This clock may be used for consistency of
-     * #is_ready/#get_frame for example, stopping the clock for one source
-     * can freeze its state in an unwanted way. *)
+    (* Prepare for output round. *)
+    method before_output =
+      List.iter (fun s -> s#before_output) sources;
+      self#iter_watchers (fun w -> w.before_output ())
+
+    (* Cleanup after output round. *)
     method after_output =
       List.iter (fun s -> s#after_output) sources;
-      self#advance
+      self#advance;
+      self#iter_watchers (fun w -> w.after_output ())
 
     (* Reset the cache frame *)
-    method advance = Frame.advance memo
+    method advance = Frame.advance self#memo
 
     (** Utils. *)
 
@@ -575,13 +785,14 @@ class virtual operator ?(name = "src") content_kind sources =
     * in the metadatas of the request. *)
     method private create_request ?(metadata = []) =
       let metadata = ("source", self#id) :: metadata in
-      Request.create ~metadata ~kind:content_kind
+      Request.create ~metadata
   end
 
 (** Entry-point sources, which need to actively perform some task. *)
-and virtual active_operator ?name content_kind sources =
+and virtual active_operator ?name ?audio_in ?video_in ?midi_in content_kind
+  sources =
   object (self)
-    inherit operator ?name content_kind sources
+    inherit operator ?name ?audio_in ?video_in ?midi_in content_kind sources
 
     initializer
     has_outputs := true;
@@ -590,33 +801,25 @@ and virtual active_operator ?name content_kind sources =
       (unify self#clock
          (create_unknown ~sources:[(self :> active_operator)] ~sub_clocks:[]))
 
-    method is_output = true
+    method is_active = true
 
     (** Start a new output round, may trigger the computation of a frame. *)
     method virtual output : unit
 
     (** Do whatever needed when the latency gets too big and is reset. *)
-    method virtual output_reset : unit
-
-    (** Is the source active ? *)
-    method virtual is_active : bool
-
-    (** Special init phase for outputs. This method is called by Root after the
-    * standard get_ready propagation, after the Root clock is started.
-    * It allows enhancements of the initial latency. *)
-    method virtual output_get_ready : unit
+    method virtual reset : unit
   end
 
 (** Shortcuts for defining sources with no children *)
 
-class virtual source ?name content_kind =
+class virtual source ?name ?audio_in ?video_in ?midi_in content_kind =
   object
-    inherit operator ?name content_kind []
+    inherit operator ?name ?audio_in ?video_in ?midi_in content_kind []
   end
 
-class virtual active_source ?name content_kind =
+class virtual active_source ?name ?audio_in ?video_in ?midi_in content_kind =
   object
-    inherit active_operator ?name content_kind []
+    inherit active_operator ?name ?audio_in ?video_in ?midi_in content_kind []
   end
 
 (** Specialized shortcuts *)
@@ -626,21 +829,14 @@ type clock_variable = active_source var
 class type clock =
   object
     method id : string
-
+    method sync_mode : sync
     method attach : active_source -> unit
-
     method detach : (active_source -> bool) -> unit
-
     method attach_clock : clock_variable -> unit
-
     method detach_clock : clock_variable -> unit
-
     method sub_clocks : clock_variable list
-
     method start_outputs : (active_source -> bool) -> unit -> active_source list
-
     method get_tick : int
-
     method end_tick : unit
   end
 
@@ -648,6 +844,12 @@ module Clock_variables = struct
   let to_string = variable_to_string
   let create_unknown = create_unknown
   let create_known = create_known
+
+  let subclocks v =
+    match deref v with
+      | Link { contents = Unknown (_, sc) } -> sc
+      | _ -> assert false
+
   let unify = unify
   let forget = forget
   let get v = match deref v with Known c -> c | _ -> assert false
