@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,60 +20,66 @@
 
  *****************************************************************************)
 
+open Mm
+
 (** Generate a square *)
 
 open Source
 
 class gen ~kind ~seek name g freq duration ampl =
-  let channels = (Frame.type_of_kind kind).Frame.audio in
-  let g = g freq ampl in
+  let g = g (freq ()) in
   object
     inherit Synthesized.source ~seek ~name kind duration
 
     method private synthesize frame off len =
-      let off = Frame.audio_of_master off in
-      let len = Frame.audio_of_master len in
-      let buf = AFrame.content_of_type ~channels frame off in
-      g#fill buf off len
+      let off = Frame.audio_of_main off in
+      let len = Frame.audio_of_main len in
+      let buf = AFrame.pcm frame in
+      g#set_frequency (freq ());
+      g#set_volume (ampl ());
+      g#fill (Audio.sub buf off len)
   end
 
 let add name g =
-  Lang.add_operator name ~category:Lang.Input
+  let kind = Lang.internal in
+  let return_t = Lang.kind_type_of_kind_format kind in
+  Lang.add_operator name ~category:`Input
     ~descr:("Generate a " ^ name ^ " wave.")
-    ~kind:Lang.audio_any
+    ~return_t
     [
       ( "duration",
-        Lang.float_t,
-        Some (Lang.float 0.),
-        Some "Duration in seconds (0. means infinite)." );
+        Lang.nullable_t Lang.float_t,
+        Some Lang.null,
+        Some "Duration in seconds (`null` means infinite)." );
       ( "amplitude",
-        Lang.float_t,
+        Lang.getter_t Lang.float_t,
         Some (Lang.float 1.),
         Some "Maximal value of the waveform." );
       ( "",
-        Lang.float_t,
+        Lang.getter_t Lang.float_t,
         Some (Lang.float 440.),
         Some ("Frequency of the " ^ name ^ ".") );
     ]
-    (fun p kind ->
-      ( new gen
-          ~seek:true ~kind name g
-          (Lang.to_float (List.assoc "" p))
-          (Lang.to_float (List.assoc "duration" p))
-          (Lang.to_float (List.assoc "amplitude" p))
-        :> source ))
+    (fun p ->
+      let kind = Source.Kind.of_kind kind in
+      (new gen
+         ~seek:true ~kind name g
+         (Lang.to_float_getter (List.assoc "" p))
+         (Lang.to_valued_option Lang.to_float (List.assoc "duration" p))
+         (Lang.to_float_getter (List.assoc "amplitude" p))
+        :> source))
 
-let sine f volume =
+let sine f =
   new Audio.Generator.of_mono
-    (new Audio.Mono.Generator.sine (Lazy.force Frame.audio_rate) ~volume f)
+    (new Audio.Mono.Generator.sine (Lazy.force Frame.audio_rate) f)
 
-let square f volume =
+let square f =
   new Audio.Generator.of_mono
-    (new Audio.Mono.Generator.square (Lazy.force Frame.audio_rate) ~volume f)
+    (new Audio.Mono.Generator.square (Lazy.force Frame.audio_rate) f)
 
-let saw f volume =
+let saw f =
   new Audio.Generator.of_mono
-    (new Audio.Mono.Generator.saw (Lazy.force Frame.audio_rate) ~volume f)
+    (new Audio.Mono.Generator.saw (Lazy.force Frame.audio_rate) f)
 
 let () =
   add "sine" sine;
