@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,51 +20,46 @@
 
  *****************************************************************************)
 
-open Source
+open Mm
 
 let log = Log.make ["input"; "jack"]
-let bjack_clock = Tutils.lazy_cell (fun () -> new Clock.self_sync "bjack")
+let bjack_clock = Tutils.lazy_cell (fun () -> new Clock.clock "bjack")
 
-class jack_in ~kind ~clock_safe ~nb_blocks ~server =
-  let channels = (Frame.type_of_kind kind).Frame.audio in
+class jack_in ~kind ~clock_safe ~on_start ~on_stop ~fallible ~autostart
+  ~nb_blocks ~server =
   let samples_per_frame = AFrame.size () in
   let samples_per_second = Lazy.force Frame.audio_rate in
   let seconds_per_frame = float samples_per_frame /. float samples_per_second in
   let bytes_per_sample = 2 in
-  let blank () =
-    Bytes.make (samples_per_frame * channels * bytes_per_sample) '0'
-  in
+
   object (self)
-    inherit active_source ~name:"input.jack" kind as active_source
+    inherit
+      Start_stop.active_source
+        ~name:"input.jack" ~content_kind:kind ~clock_safe ~on_start ~on_stop
+          ~fallible ~autostart () as active_source
 
-    inherit [Bytes.t] IoRing.input ~nb_blocks ~blank as ioring
-
-    method set_clock =
-      active_source#set_clock;
-      if clock_safe then
-        Clock.unify self#clock
-          (Clock.create_known (bjack_clock () :> Clock.clock))
+    inherit [Bytes.t] IoRing.input ~nb_blocks as ioring
 
     method private wake_up l =
       active_source#wake_up l;
-      if clock_safe then (bjack_clock ())#register_blocking_source
+      (* We need to know the number of channels to initialize the ioring. We
+           defer this until the kind is known. *)
+      let blank () =
+        Bytes.make
+          (samples_per_frame * self#audio_channels * bytes_per_sample)
+          '0'
+      in
+      ioring#init blank
 
     method private sleep =
       active_source#sleep;
-      ioring#sleep;
-      if clock_safe then (bjack_clock ())#unregister_blocking_source
-
-    method stype = Infallible
-
-    method is_ready = true
+      ioring#sleep
 
     method abort_track = ()
-
     method remaining = -1
-
     val mutable sample_freq = samples_per_second
-
     val mutable device = None
+    method self_sync = (`Dynamic, device <> None)
 
     method close =
       match device with
@@ -78,12 +73,16 @@ class jack_in ~kind ~clock_safe ~nb_blocks ~server =
         | None ->
             let server_name = match server with "" -> None | s -> Some s in
             let dev =
-              Bjack.open_t ~rate:samples_per_second
-                ~bits_per_sample:(bytes_per_sample * 8) ~input_channels:channels
-                ~output_channels:0 ~flags:[] ?server_name
-                ~ringbuffer_size:
-                  (nb_blocks * samples_per_frame * bytes_per_sample)
-                ~client_name:self#id ()
+              try
+                Bjack.open_t ~rate:samples_per_second
+                  ~bits_per_sample:(bytes_per_sample * 8)
+                  ~input_channels:self#audio_channels ~output_channels:0
+                  ~flags:[] ?server_name
+                  ~ringbuffer_size:
+                    (nb_blocks * samples_per_frame * bytes_per_sample)
+                  ~client_name:self#id ()
+              with Bjack.Open ->
+                failwith "Could not open JACK device: is the server running?"
             in
             Bjack.set_all_volume dev 100;
             device <- Some dev;
@@ -105,40 +104,49 @@ class jack_in ~kind ~clock_safe ~nb_blocks ~server =
     method private get_frame buf =
       assert (0 = AFrame.position buf);
       let buffer = ioring#get_block in
-      let fbuf = AFrame.content_of_type ~channels buf 0 in
+      let fbuf = AFrame.pcm buf in
       Audio.S16LE.to_audio
         (Bytes.unsafe_to_string buffer)
-        0 fbuf 0 samples_per_frame;
+        0
+        (Audio.sub fbuf 0 samples_per_frame);
       AFrame.add_break buf samples_per_frame
 
-    method output = if AFrame.is_partial memo then self#get_frame memo
-
-    method output_reset = ()
-
-    method is_active = true
+    method reset = ()
   end
 
 let () =
-  let k = Lang.kind_type_of_kind_format ~fresh:1 Lang.audio_any in
+  let kind = Lang.audio_pcm in
+  let return_t = Lang.kind_type_of_kind_format kind in
   Lang.add_operator "input.jack"
-    [
-      ( "clock_safe",
-        Lang.bool_t,
-        Some (Lang.bool true),
-        Some "Force the use of the dedicated bjack clock." );
-      ( "buffer_size",
-        Lang.int_t,
-        Some (Lang.int 2),
-        Some "Set buffer size, in frames. Must be >= 1." );
-      ( "server",
-        Lang.string_t,
-        Some (Lang.string ""),
-        Some "Jack server to connect to." );
-    ]
-    ~kind:(Lang.Unconstrained k) ~category:Lang.Input
+    (Start_stop.active_source_proto ~clock_safe:true ~fallible_opt:(`Yep false)
+    @ [
+        ( "buffer_size",
+          Lang.int_t,
+          Some (Lang.int 2),
+          Some "Set buffer size, in frames. Must be >= 1." );
+        ( "server",
+          Lang.string_t,
+          Some (Lang.string ""),
+          Some "Jack server to connect to." );
+      ])
+    ~meth:(Start_stop.meth ()) ~return_t ~category:`Input
     ~descr:"Get stream from jack."
-    (fun p kind ->
+    (fun p ->
       let clock_safe = Lang.to_bool (List.assoc "clock_safe" p) in
+      let fallible = Lang.to_bool (List.assoc "fallible" p) in
+      let autostart = Lang.to_bool (List.assoc "start" p) in
+      let on_start =
+        let f = List.assoc "on_start" p in
+        fun () -> ignore (Lang.apply f [])
+      in
+      let on_stop =
+        let f = List.assoc "on_stop" p in
+        fun () -> ignore (Lang.apply f [])
+      in
       let nb_blocks = Lang.to_int (List.assoc "buffer_size" p) in
       let server = Lang.to_string (List.assoc "server" p) in
-      (new jack_in ~kind ~clock_safe ~nb_blocks ~server :> Source.source))
+      let kind = Source.Kind.of_kind kind in
+      (new jack_in
+         ~kind ~clock_safe ~nb_blocks ~server ~fallible ~on_start ~on_stop
+         ~autostart
+        :> Start_stop.active_source))
