@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 
  *****************************************************************************)
 
+open Mm
 open Extralib
 module Generator = Generator.From_audio_video_plus
 module Generated = Generated.From_audio_video_plus
@@ -28,21 +29,23 @@ module Generated = Generated.From_audio_video_plus
 
 exception Finished of string * bool
 
-class external_input ~name ~kind ~restart ~bufferize ~restart_on_error ~max
-  ~converter ?read_header command =
+class external_input ~name ~kind ~restart ~bufferize ~log_overfull
+  ~restart_on_error ~max ~converter ?read_header command =
   let abg_max_len = Frame.audio_of_seconds max in
   (* We need a temporary log until the source has an id *)
   let log_ref = ref (fun _ -> ()) in
   let log x = !log_ref x in
-  let abg = Generator.create ~log ~kind `Audio in
-  let buflen = 1024 in
+  let abg = Generator.create ~log ~log_overfull `Audio in
+  let buflen = Utils.pagesize in
   let buf = Bytes.create buflen in
   let on_data reader =
     let ret = reader buf 0 buflen in
     let data = converter (Bytes.sub_string buf 0 ret) in
-    let len = Array.length data.(0) in
+    let len = Audio.length data in
     let buffered = Generator.length abg in
-    Generator.put_audio abg data 0 (Array.length data.(0));
+    Generator.put_audio abg
+      (Frame_content.Audio.lift_data data)
+      0 (Frame.main_of_audio len);
     if abg_max_len < buffered + len then
       `Delay (Frame.seconds_of_audio (buffered + len - (3 * abg_max_len / 4)))
     else `Continue
@@ -70,6 +73,10 @@ let proto =
       Lang.float_t,
       Some (Lang.float 10.),
       Some "Maximum duration of the buffered data." );
+    ( "log_overfull",
+      Lang.bool_t,
+      Some (Lang.bool true),
+      Some "Log when the source's buffer is overfull." );
     ( "restart",
       Lang.bool_t,
       Some (Lang.bool true),
@@ -82,60 +89,78 @@ let proto =
   ]
 
 let () =
-  Lang.add_operator "input.external.rawaudio" ~category:Lang.Input
-    ~descr:"Stream raw PCM data from an external application."
-    ( proto
+  let kind = Lang.audio_pcm in
+  let return_t = Lang.kind_type_of_kind_format kind in
+  Lang.add_operator "input.external.rawaudio" ~category:`Input
+    ~descr:
+      "Stream raw PCM data (interleaved signed 16 bits little endian integers) \
+       from an external application."
+    (proto
     @ [
         ("channels", Lang.int_t, Some (Lang.int 2), Some "Number of channels.");
         ("samplerate", Lang.int_t, Some (Lang.int 44100), Some "Samplerate.");
-      ] )
-    ~kind:Lang.audio_any
-    (fun p kind ->
+      ])
+    ~return_t
+    (fun p ->
       let command = Lang.to_string (List.assoc "" p) in
       let bufferize = Lang.to_float (List.assoc "buffer" p) in
-      let channels = Lang.to_int (List.assoc "channels" p) in
-      if not (Frame.mul_eq_int kind.Frame.audio channels) then
-        raise
-          (Lang_errors.Invalid_value
-             ( List.assoc "channels" p,
-               "Incompatible number of channels, please use a conversion \
-                operator." ));
-      let audio_src_rate = float (Lang.to_int (List.assoc "samplerate" p)) in
-      let converter =
-        Rutils.create_from_iff ~format:`Wav ~channels ~samplesize:16
-          ~audio_src_rate
+      let log_overfull = Lang.to_bool (List.assoc "log_overfull" p) in
+      let channels_v = List.assoc "channels" p in
+      let channels = Lang.to_int channels_v in
+      let channel_layout =
+        try Audio_converter.Channel_layout.layout_of_channels channels
+        with _ ->
+          raise
+            (Error.Invalid_value (channels_v, "unsupported number of channels"))
       in
+      let kind =
+        Lang.audio_params
+          { Frame_content.Contents.channel_layout = lazy channel_layout }
+      in
+      let kind = Source.Kind.of_kind kind in
+      let samplerate = Lang.to_int (List.assoc "samplerate" p) in
+      let resampler = Decoder_utils.samplerate_converter () in
+      let convert =
+        Decoder_utils.from_iff ~format:`Wav ~channels ~samplesize:16
+      in
+      let converter data = resampler ~samplerate (convert data) in
       let restart = Lang.to_bool (List.assoc "restart" p) in
       let restart_on_error = Lang.to_bool (List.assoc "restart_on_error" p) in
       let max = Lang.to_float (List.assoc "max" p) in
-      ( new external_input
-          ~kind ~restart ~bufferize ~restart_on_error ~max
-          ~name:"input.external.rawaudio" ~converter command
-        :> Source.source ))
+      (new external_input
+         ~kind ~restart ~bufferize ~log_overfull ~restart_on_error ~max
+         ~name:"input.external.rawaudio" ~converter command
+        :> Source.source))
 
 let () =
-  Lang.add_operator "input.external.wav" ~category:Lang.Input
-    ~descr:"Stream WAV data from an external application." proto
-    ~kind:Lang.audio_any (fun p kind ->
+  let kind = Lang.audio_pcm in
+  let return_t = Lang.kind_type_of_kind_format kind in
+  Lang.add_operator "input.external.wav" ~category:`Input
+    ~descr:"Stream WAV data from an external application." proto ~return_t
+    (fun p ->
       let command = Lang.to_string (List.assoc "" p) in
       let bufferize = Lang.to_float (List.assoc "buffer" p) in
+      let log_overfull = Lang.to_bool (List.assoc "log_overfull" p) in
       let converter_ref = ref (fun _ -> assert false) in
       let converter data = !converter_ref data in
       let read_header read =
         let header = Wav_aiff.read_header Wav_aiff.callback_ops read in
         let channels = Wav_aiff.channels header in
-        let audio_src_rate = float (Wav_aiff.sample_rate header) in
+        let samplerate = Wav_aiff.sample_rate header in
         let samplesize = Wav_aiff.sample_size header in
         Wav_aiff.close header;
-        converter_ref :=
-          Rutils.create_from_iff ~format:`Wav ~channels ~samplesize
-            ~audio_src_rate;
+        let resampler = Decoder_utils.samplerate_converter () in
+        let convert =
+          Decoder_utils.from_iff ~format:`Wav ~channels ~samplesize
+        in
+        (converter_ref := fun data -> resampler ~samplerate (convert data));
         `Reschedule Tutils.Non_blocking
       in
       let restart = Lang.to_bool (List.assoc "restart" p) in
       let restart_on_error = Lang.to_bool (List.assoc "restart_on_error" p) in
       let max = Lang.to_float (List.assoc "max" p) in
-      ( new external_input
-          ~kind ~restart ~bufferize ~read_header ~restart_on_error ~max
-          ~name:"input.external.wav" ~converter command
-        :> Source.source ))
+      let kind = Source.Kind.of_kind kind in
+      (new external_input
+         ~kind ~restart ~bufferize ~log_overfull ~read_header ~restart_on_error
+         ~max ~name:"input.external.wav" ~converter command
+        :> Source.source))
