@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,16 +24,14 @@
 class virtual source =
   object (self)
     val mutable latest_metadata = Hashtbl.create 0
-
     method virtual private on_new_metadata : unit
 
     method private save_latest_metadata frame =
       let compare x y = -compare (fst x) (fst y) in
       let l = List.sort compare (Frame.get_all_metadata frame) in
-      if List.length l > 0 then begin
+      if List.length l > 0 then (
         latest_metadata <- Hashtbl.copy (snd (List.hd l));
-        self#on_new_metadata
-      end
+        self#on_new_metadata)
 
     method private clear_latest_metadata = latest_metadata <- Hashtbl.create 0
   end
