@@ -1,7 +1,7 @@
 (*****************************************************************************
 
    Liquidsoap, a programmable stream generator.
-   Copyright 2003-2019 Savonet team
+   Copyright 2003-2021 Savonet team
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,6 +29,101 @@ type handler = {
   close : unit -> unit;
 }
 
+let log_failed_request (log : Log.t) request ans =
+  log#important "Could not resolve request %s: %s."
+    (Request.initial_uri request)
+    (match ans with
+      | Request.Failed -> "failed"
+      | Request.Timeout -> "timeout"
+      | Request.Resolved -> assert false)
+
+(** Play a request once and become unavailable. *)
+class once ~kind ~name ~timeout request =
+  object (self)
+    inherit source kind ~name as super
+    method self_sync = (`Static, false)
+    method stype = Fallible
+
+    (* True means that the request has already been played or could not be
+       resolved. *)
+    val mutable over = false
+
+    (* We must send metadata at beginning. *)
+    val mutable send_metadata = true
+
+    (* We need to insert a track at next frame. *)
+    val mutable must_fail = false
+    val mutable remaining = 0
+    method remaining = remaining
+    val mutable decoder = None
+    method request = request
+
+    method resolve =
+      Request.resolve ~ctype:(Some self#ctype) request timeout
+      = Request.Resolved
+
+    method private wake_up activation =
+      super#wake_up activation;
+      if not over then (
+        (* Ensure that the request is resolved. *)
+        (match Request.resolve ~ctype:(Some self#ctype) request timeout with
+          | Request.Resolved -> ()
+          | ans -> log_failed_request self#log request ans);
+        if not (Request.is_ready request) then (
+          over <- true;
+          self#log#critical "Failed to prepare track: request not ready.";
+          Request.destroy request)
+        else (
+          (match Request.ctype request with
+            | Some ctype -> assert (Frame.compatible ctype self#ctype)
+            | None -> ());
+          let file = Option.get (Request.get_filename request) in
+          decoder <- Request.get_decoder request;
+          assert (decoder <> None);
+          remaining <- -1;
+          self#log#important "Prepared %s (RID %d)." (Utils.quote_string file)
+            (Request.get_id request)))
+
+    method private end_track forced =
+      if not over then (
+        (match Request.get_filename request with
+          | None ->
+              self#log#severe
+                "Finished with a non-existent file?! Something may have been \
+                 moved or destroyed during decoding. It is VERY dangerous, \
+                 avoid it!"
+          | Some f -> self#log#info "Finished with %S." f);
+        let decoder = Option.get decoder in
+        decoder.Decoder.close ();
+        Request.destroy request;
+        remaining <- 0;
+        if forced then must_fail <- true else over <- true)
+
+    method is_ready = not over
+
+    method private get_frame buf =
+      if must_fail then (
+        must_fail <- false;
+        over <- true;
+        Frame.add_break buf (Frame.position buf))
+      else (
+        if send_metadata then (
+          Request.on_air request;
+          let m = Request.get_all_metadata request in
+          Frame.set_metadata buf (Frame.position buf) m;
+          send_metadata <- false);
+        let decoder = Option.get decoder in
+        remaining <- decoder.Decoder.fill buf;
+        if Frame.is_partial buf then self#end_track false)
+
+    method seek len =
+      let decoder = Option.get decoder in
+      decoder.Decoder.fseek len
+
+    method abort_track = self#end_track true
+    method private sleep = self#end_track false
+  end
+
 (** Class [unqueued] plays the file given by method [get_next_file] as a request
     which is ready, i.e. has been resolved. On the top of it we define [queued],
     which manages a queue of files, feed by resolving in an other thread requests
@@ -42,39 +137,38 @@ class virtual unqueued ~kind ~name =
     method virtual get_next_file : Request.t option
 
     val mutable remaining = 0
-
     val mutable must_fail = false
 
     (** These values are protected by [plock]. *)
     val mutable send_metadata = false
 
     val mutable current = None
-
+    method current = current
     val plock = Mutex.create ()
+    method self_sync = (`Static, false)
 
     (** How to unload a request. *)
-    method private end_track forced =
-      Mutex.lock plock;
-      begin
-        match current with
-        | None -> ()
-        | Some cur ->
-            begin
-              match Request.get_filename cur.req with
-              | None ->
-                  self#log#severe
-                    "Finished with a non-existent file?! Something may have \
-                     been moved or destroyed during decoding. It is VERY \
-                     dangerous, avoid it!"
-              | Some f -> self#log#info "Finished with %S." f
-            end;
-            cur.close ();
-            Request.destroy cur.req;
-            must_fail <- forced
-      end;
-      current <- None;
-      remaining <- 0;
-      Mutex.unlock plock
+    method private end_track =
+      Tutils.mutexify plock (fun forced ->
+          begin
+            match current with
+            | None -> ()
+            | Some cur ->
+                begin
+                  match Request.get_filename cur.req with
+                  | None ->
+                      self#log#severe
+                        "Finished with a non-existent file?! Something may \
+                         have been moved or destroyed during decoding. It is \
+                         VERY dangerous, avoid it!"
+                  | Some f -> self#log#info "Finished with %S." f
+                end;
+                cur.close ();
+                Request.destroy cur.req;
+                must_fail <- forced
+          end;
+          current <- None;
+          remaining <- 0)
 
     (** Load a request. Should be called within critical section, when there is no
       ready request. *)
@@ -86,12 +180,15 @@ class virtual unqueued ~kind ~name =
             self#log#debug "Failed to prepare track: no file.";
             false
         | Some req when Request.is_ready req ->
-            assert (Frame.kind_sub_kind (Utils.get_some (Request.kind req)) kind);
+            assert (Frame.compatible (Option.get (Request.ctype req)) self#ctype);
+
             (* [Request.is_ready] ensures that we can get a filename from the request,
                and it can be decoded. *)
-            let file = Utils.get_some (Request.get_filename req) in
-            let decoder = Utils.get_some (Request.get_decoder req) in
-            self#log#important "Prepared %S (RID %d)." file (Request.get_id req);
+            let file = Option.get (Request.get_filename req) in
+            let decoder = Option.get (Request.get_decoder req) in
+            self#log#important "Prepared %s (RID %d)." (Utils.quote_string file)
+              (Request.get_id req);
+
             (* We use this mutex to avoid seeking and filling at the same time.. *)
             let m = Mutex.create () in
             current <-
@@ -109,7 +206,7 @@ class virtual unqueued ~kind ~name =
             send_metadata <- true;
             true
         | Some req ->
-            (* We got an unresolved request.. this shoudn't actually happen *)
+            (* We got an unresolved request.. this shouldn't actually happen *)
             self#log#critical "Failed to prepare track: request not ready.";
             Request.destroy req;
             false
@@ -117,115 +214,102 @@ class virtual unqueued ~kind ~name =
     (** Now we can write the source's methods. *)
 
     method is_ready =
-      Mutex.lock plock;
-      let ans = current <> None || must_fail || self#begin_track in
-      Mutex.unlock plock;
-      ans
+      Tutils.mutexify plock
+        (fun () -> current <> None || must_fail || self#begin_track)
+        ()
 
     method remaining = remaining
 
     method private get_frame buf =
-      if must_fail then begin
-        must_fail <- false;
-        Frame.add_break buf (Frame.position buf)
-      end
-      else begin
-        let try_get () =
-          match current with
-            | None ->
-                (* We're supposed to be ready so this shouldn't be reached. *)
-                assert false
-            | Some cur ->
-                if send_metadata then begin
-                  Request.on_air cur.req;
-                  let m = Request.get_all_metadata cur.req in
-                  Frame.set_metadata buf (Frame.position buf) m;
-                  send_metadata <- false
-                end;
-                cur.fill buf
-        in
-        Tutils.mutexify plock try_get ();
-        if Frame.is_partial buf then self#end_track false
-      end
+      let end_track =
+        Tutils.mutexify plock
+          (fun () ->
+            if must_fail then (
+              must_fail <- false;
+              Frame.add_break buf (Frame.position buf);
+              false)
+            else (
+              match current with
+                | None ->
+                    (* We're supposed to be ready so this shouldn't be reached. *)
+                    assert false
+                | Some cur ->
+                    if send_metadata then (
+                      Request.on_air cur.req;
+                      let m = Request.get_all_metadata cur.req in
+                      Frame.set_metadata buf (Frame.position buf) m;
+                      send_metadata <- false);
+                    cur.fill buf;
+                    Frame.is_partial buf))
+          ()
+      in
+      if end_track then self#end_track false
 
     method seek x = match current with None -> 0 | Some cur -> cur.seek x
-
     method abort_track = self#end_track true
-
     method private sleep = self#end_track false
-
-    method copy_queue = match current with None -> [] | Some cur -> [cur.req]
   end
 
 type queue_item = {
   request : Request.t;
-  duration : float;
   (* in seconds *)
   mutable expired : bool;
 }
 
-(* Private types for request resolutions *)
-type resolution = Empty | Retry | Finished
-
 (* Scheduler priority for request resolutions. *)
 let priority = Tutils.Maybe_blocking
 
 (** Same thing, with a queue in which we prefetch files, which requests are
     given by [get_next_request]. Heuristical settings determining how the source
     feeds its queue:
-    - the source tries to have more than [length] seconds in queue
-    - if the duration of a file is unknown we use [default_duration] seconds
+    - the source tries to have more than [prefetch] requests in queue
     - downloading a file is required to take less than [timeout] seconds
    *)
-class virtual queued ~kind ~name ?(length = 10.) ?(default_duration = 30.)
-  ?(conservative = false) ?(timeout = 20.) () =
+class virtual queued ~kind ~name ?(prefetch = 1) ?(timeout = 20.) () =
   object (self)
     inherit unqueued ~kind ~name as super
-
     method stype = Fallible
-
     method virtual get_next_request : Request.t option
-
-    (** Management of the queue of files waiting to be played. *)
-    val min_queue_length = length
-
     val qlock = Mutex.create ()
-
     val retrieved : queue_item Queue.t = Queue.create ()
 
-    val mutable queue_length = 0.
+    method private queue_size =
+      self#mutexify (fun () -> Queue.length retrieved) ()
+
+    method queue = self#mutexify (fun () -> Queue.copy retrieved) ()
+
+    method set_queue =
+      self#mutexify (fun q ->
+          Queue.clear retrieved;
+          Queue.iter
+            (fun i ->
+              match
+                Request.resolve ~ctype:(Some self#ctype) i.request timeout
+              with
+                | Request.Resolved -> Queue.push i retrieved
+                | ans -> log_failed_request self#log i.request ans)
+            q)
+
+    method add =
+      self#mutexify (fun i ->
+          match Request.resolve ~ctype:(Some self#ctype) i.request timeout with
+            | Request.Resolved ->
+                Queue.push i retrieved;
+                true
+            | ans ->
+                log_failed_request self#log i.request ans;
+                false)
 
     (* Seconds *)
     val mutable resolving = None
 
-    (** [available_length] is the length of the queue plus the remaining time in
-      the current track (0 in conservative mode). Although [queue_length]
-      should be accessed under [qlock] we don't do it here, and more
-      importantly, all the instances of "if self#available_length <
-      min_queue_length then ..." are not run within [qlock]'s critical
-      section. The potential race condition (there seems to be enough, then the
-      length decreases but we have already decided to take no action) is
-      harmless because all the portions of the code that decrease the length
-      trigger the appropriate action (feeding) themselves. *)
-    method private available_length =
-      if conservative then queue_length
-      else (
-        let remaining = self#remaining in
-        if remaining < 0 then
-          (* There is a track available but we don't know its duration at this
-             point. Hence, using default_duration. *)
-          queue_length +. default_duration
-        else queue_length +. Frame.seconds_of_master remaining )
-
     (** State should be `Sleeping on awakening, and is then turned to `Running.
       Eventually #sleep puts it to `Tired, then waits for it to be `Sleeping,
       meaning that the feeding task exited. *)
     val mutable state = `Sleeping
 
     val state_lock = Mutex.create ()
-
     val state_cond = Condition.create ()
-
     val mutable task = None
 
     method private wake_up activation =
@@ -253,12 +337,14 @@ class virtual queued ~kind ~name ?(length = 10.) ?(default_duration = 30.)
           assert (state = `Running);
           state <- `Tired)
         ();
+
       (* Make sure the task is awake so that it can see our signal. *)
-      Duppy.Async.wake_up (Utils.get_some task);
+      Duppy.Async.wake_up (Option.get task);
       self#log#info "Waiting for feeding task to stop...";
       Tutils.wait state_cond state_lock (fun () -> state = `Sleeping);
-      Duppy.Async.stop (Utils.get_some task);
+      Duppy.Async.stop (Option.get task);
       task <- None;
+
       (* No more feeding task, we can go to sleep. *)
       super#sleep;
       self#log#info "Cleaning up request queue...";
@@ -280,12 +366,12 @@ class virtual queued ~kind ~name ?(length = 10.) ?(default_duration = 30.)
          exception. *)
       Tutils.mutexify state_lock
         (fun () ->
-          if state = `Running then Duppy.Async.wake_up (Utils.get_some task))
+          if state = `Running then Duppy.Async.wake_up (Option.get task))
         ()
 
     (** A function that returns delays for tasks, making sure that these tasks
       don't repeat too fast. The current scheme is to return 0. as long as there
-      are no more than [max] consecutive occurences separated by less than
+      are no more than [max] consecutive occurrences separated by less than
       [delay], otherwise return [delay]. *)
     val adaptative_delay =
       let last = ref 0. in
@@ -333,60 +419,49 @@ class virtual queued ~kind ~name ?(length = 10.) ?(default_duration = 30.)
                    *    emptied, but Sleeping saves us). *)
                   false)
           ()
-        && self#available_length < min_queue_length
+        && self#queue_size < prefetch
       then (
-        match self#prefetch with
-          | Finished -> 0.
-          | Retry -> adaptative_delay ()
-          | Empty -> -1. )
+        match self#fetch with
+          | `Finished ->
+              (* Retry again in order to make sure that we have enough data *)
+              0.5
+          | `Retry -> adaptative_delay ()
+          | `Empty -> -1.)
       else -1.
 
     (** Try to feed the queue with a new request. Return a resolution status:
       Empty if there was no new request to try,
       Retry if there was a new one but it failed to be resolved,
       Finished if all went OK. *)
-    method private prefetch =
+    method fetch =
       match self#get_next_request with
-        | None -> Empty
+        | None -> `Empty
         | Some req -> (
             resolving <- Some req;
-            match Request.resolve req timeout with
+            match Request.resolve ~ctype:(Some self#ctype) req timeout with
               | Request.Resolved ->
-                  let len =
-                    match Request.get_metadata req "duration" with
-                      | Some f -> (
-                          try float_of_string f with _ -> default_duration )
-                      | None -> default_duration
-                  in
                   let rec remove_expired n =
                     if n = 0 then ()
                     else (
                       let r = Queue.take retrieved in
-                      if r.expired then begin
+                      if r.expired then (
                         self#log#info "Dropping expired request.";
-                        Request.destroy r.request
-                      end
+                        Request.destroy r.request)
                       else Queue.add r retrieved;
-                      remove_expired (n - 1) )
+                      remove_expired (n - 1))
                   in
                   Mutex.lock qlock;
                   remove_expired (Queue.length retrieved);
-                  Queue.add
-                    { request = req; duration = len; expired = false }
-                    retrieved;
-                  self#log#info
-                    "Remaining: %.1fs, queued: %.1fs, adding: %.1fs (RID %d)"
-                    (Frame.seconds_of_master self#remaining)
-                    queue_length len (Request.get_id req);
-                  queue_length <- queue_length +. len;
+                  Queue.add { request = req; expired = false } retrieved;
+                  self#log#info "Queued %d requests" self#queue_size;
                   Mutex.unlock qlock;
                   resolving <- None;
-                  Finished
+                  `Finished
               | Request.Failed (* Failure of resolving or decoding *)
               | Request.Timeout ->
                   resolving <- None;
                   Request.destroy req;
-                  Retry )
+                  `Retry)
 
     (** Provide the unqueued [super] with resolved requests. *)
     method private get_next_file =
@@ -394,74 +469,47 @@ class virtual queued ~kind ~name ?(length = 10.) ?(default_duration = 30.)
       let ans =
         try
           let r = Queue.take retrieved in
-          self#log#info "Remaining: %.1fs, queued: %.1fs, taking: %.1fs"
-            (Frame.seconds_of_master self#remaining)
-            queue_length r.duration;
-          if not r.expired then queue_length <- queue_length -. r.duration;
+          self#log#info "Remaining %d requests" self#queue_size;
           Some r.request
         with Queue.Empty ->
           self#log#debug "Queue is empty!";
           None
       in
       Mutex.unlock qlock;
+
       (* A request has been taken off the queue, there is a chance that the
          queue should be refilled: awaken the feeding task. However, we can wait
-         that this file is played, and this need will be noticed in #get_frame.
-         This is critical in non-conservative mode because at this point
-         remaining is still 0 (we're inside #begin_track) which would lead to
-         too early prefetching; if we wait that a frame has been produced, we'll
-         get the first non-infinite remaining time estimations. *)
+         that this file is played, and this need will be noticed in #get_frame. *)
       ans
 
     method private expire test =
-      let already_short = self#available_length < min_queue_length in
+      let already_short = self#queue_size < prefetch in
       Mutex.lock qlock;
       Queue.iter
-        (fun r ->
-          if test r.request && not r.expired then begin
-            r.expired <- true;
-            queue_length <- queue_length -. r.duration
-          end)
+        (fun r -> if test r.request && not r.expired then r.expired <- true)
         retrieved;
       Mutex.unlock qlock;
-      if self#available_length < min_queue_length then begin
+      if self#queue_size < prefetch then (
         if not already_short then
           self#log#info "Expirations made the queue too short, feeding...";
+
         (* Notify in any case, notifying twice never hurts. *)
-        self#notify_new_request
-      end
+        self#notify_new_request)
 
     method private get_frame ab =
       super#get_frame ab;
+
       (* At an end of track, we always have unqueued#remaining=0, so there's
          nothing special to do. *)
-      if self#available_length < min_queue_length then self#notify_new_request
-
-    method copy_queue =
-      Mutex.lock qlock;
-      let q = match current with None -> [] | Some cur -> [cur.req] in
-      let q = match resolving with None -> q | Some r -> r :: q in
-      let q = Queue.fold (fun l r -> r.request :: l) q retrieved in
-      Mutex.unlock qlock;
-      q
+      if self#queue_size < prefetch then self#notify_new_request
   end
 
 let queued_proto =
   [
-    ( "length",
-      Lang.float_t,
-      Some (Lang.float 40.),
-      Some "How much audio (in sec.) should be queued in advance." );
-    ( "default_duration",
-      Lang.float_t,
-      Some (Lang.float 30.),
-      Some "When unknown, assume this duration (in sec.) for files." );
-    ( "conservative",
-      Lang.bool_t,
-      Some (Lang.bool false),
-      Some
-        "If true, estimated remaining time on the current track is not \
-         considered when computing queue length." );
+    ( "prefetch",
+      Lang.int_t,
+      Some (Lang.int 1),
+      Some "How many requests should be queued in advance." );
     ( "timeout",
       Lang.float_t,
       Some (Lang.float 20.),
@@ -469,8 +517,6 @@ let queued_proto =
   ]
 
 let extract_queued_params p =
-  let l = Lang.to_float (List.assoc "length" p) in
-  let d = Lang.to_float (List.assoc "default_duration" p) in
+  let l = Lang.to_int (List.assoc "prefetch" p) in
   let t = Lang.to_float (List.assoc "timeout" p) in
-  let c = Lang.to_bool (List.assoc "conservative" p) in
-  (l, d, t, c)
+  (l, t)
