@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,41 +20,35 @@
 
  *****************************************************************************)
 
-(* For some synthetized function (e.g. noise, blank, sine),
- * we can pretend we support seek by doing nothing. However, for
- * other, seek should be disabled. Thus, if [seek] is [true],
- * the seek function is [fun x -> x] otherwise it is
- * [fun _ -> 0] *)
+(* For some synthetized function (e.g. noise, blank, sine), we can pretend we
+   support seek by doing nothing. However, for other, seek should be
+   disabled. Thus, if [seek] is [true], the seek function is [fun x -> x]
+   otherwise it is [fun _ -> 0] *)
 class virtual source ?name ~seek kind duration =
-  let track_size =
-    if duration <= 0. then None else Some (Frame.master_of_seconds duration)
-  in
+  let track_size = Option.map Frame.main_of_seconds duration in
   object (self)
     inherit Source.source ?name kind
 
-    method stype = Source.Infallible
-
-    method is_ready = true
-
-    method seek x = if seek then x else 0
+    method stype =
+      if track_size = None then Source.Infallible else Source.Fallible
 
     val mutable remaining = track_size
+    method is_ready = remaining <> Some 0
+    method seek x = if seek then x else 0
+    method self_sync = (`Static, false)
 
     method remaining =
       match remaining with None -> -1 | Some remaining -> remaining
 
     val mutable must_fail = false
-
     method abort_track = must_fail <- true
-
     method virtual private synthesize : Frame.t -> int -> int -> unit
 
     method private get_frame frame =
-      if must_fail then begin
+      if must_fail then (
         Frame.add_break frame (Frame.position frame);
-        remaining <- track_size;
-        must_fail <- false
-      end
+        must_fail <- false;
+        if track_size <> None then remaining <- Some 0)
       else (
         let off = Frame.position frame in
         let len =
@@ -66,9 +60,5 @@ class virtual source ?name ~seek kind duration =
                 len
         in
         self#synthesize frame off len;
-        Frame.add_break frame (off + len);
-        if VFrame.is_partial frame then begin
-          assert (remaining = Some 0);
-          remaining <- track_size
-        end )
+        Frame.add_break frame (off + len))
   end
