@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@ module type S = sig
   val clear : t -> unit
   val fill : t -> Frame.t -> unit
   val remove : t -> int -> unit
-  val add_metadata : t -> Frame.metadata -> unit
+  val add_metadata : ?pos:int -> t -> Frame.metadata -> unit
 end
 
 module type S_Asio = sig
@@ -46,10 +46,10 @@ module type S_Asio = sig
 
   val clear : t -> unit
   val fill : t -> Frame.t -> unit
-  val add_metadata : t -> Frame.metadata -> unit
-  val add_break : ?sync:[ `Strict | `Ignore | `Drop ] -> t -> unit
-  val put_audio : t -> Frame.audio_t array -> int -> int -> unit
-  val put_video : t -> Frame.video_t array -> int -> int -> unit
+  val add_metadata : ?pos:int -> t -> Frame.metadata -> unit
+  val add_break : ?sync:bool -> ?pos:int -> t -> unit
+  val put_audio : ?pts:int64 -> t -> Frame_content.data -> int -> int -> unit
+  val put_video : ?pts:int64 -> t -> Frame_content.data -> int -> int -> unit
   val set_mode : t -> [ `Audio | `Video | `Both | `Undefined ] -> unit
 end
 
@@ -58,8 +58,6 @@ module Generator = struct
   (** A chunk with given offset and length. *)
   type 'a chunk = 'a * int * int
 
-  let chunk_data ((buf, _, _) : 'a chunk) = buf
-
   (** A buffer is a queue of chunks. *)
   type 'a buffer = 'a chunk Queue.t
 
@@ -86,60 +84,28 @@ module Generator = struct
       let _, _, b_len = Queue.peek g.buffers in
       (* Is it enough to advance in the first buffer?
        * Or do we need to consume it completely and go farther in the queue? *)
-      if g.offset + len < b_len then begin
+      if g.offset + len < b_len then (
         g.length <- g.length - len;
-        g.offset <- g.offset + len
-      end
+        g.offset <- g.offset + len)
       else (
         let removed = b_len - g.offset in
         ignore (Queue.take g.buffers);
         g.length <- g.length - removed;
         g.offset <- 0;
-        remove g (len - removed) ) )
+        remove g (len - removed)))
 
-  (** Remove data at the end of the generator: this is not a natural operation
-  * for Generators, it's done in linear time. *)
-  let remove_end g remove_len =
-    (* Remove length [l] at the beginning of the buffers,
-     * should correspond exactly to some last [n] chunks. *)
-    let rec remove l =
-      if l > 0 then (
-        let _, _, len = Queue.take g.buffers in
-        assert (l >= len);
-        remove (l - len) )
-    in
-    (* Go through the beginning of length [l] of the queue,
-     * possibly cut some element in half, remove the rest.
-     * The parsed elements are put back at the end
-     * of the queue. *)
-    let rec cut l =
-      let c, ofs, len = Queue.take g.buffers in
-      if len < l then begin
-        Queue.push (c, ofs, len) g.buffers;
-        cut (l - len)
-      end
-      else begin
-        Queue.push (c, ofs, l) g.buffers;
-        remove (remove_len - (len - l))
-      end
-    in
-    let new_len = g.length - remove_len in
-    assert (remove_len > 0 && new_len >= 0);
-    cut new_len;
-    g.length <- new_len
-
-  (** Feed an item into a generator.
-  * The item is put as such, not copied. *)
+  (* Feed an item into a generator.
+     The item is put as such, not copied. *)
   let put g content ofs len =
     g.length <- g.length + len;
     Queue.add (content, ofs, len) g.buffers
 
   (* Get [size] amount of data from [g].
-     * Returns a list where each element will typically be passed to a blit:
-     * its elements are of the form [b,o,o',l] where [o] is the offset of data
-     * in the block [b], [o'] is the position at which it should be written
-     * (the first position [o'] will always be [0]), and [l] is the length
-     * of data to be taken from that block. *)
+     Returns a list where each element will typically be passed to a blit:
+     its elements are of the form [b,o,o',l] where [o] is the offset of data
+     in the block [b], [o'] is the position at which it should be written
+     (the first position [o'] will always be [0]), and [l] is the length
+     of data to be taken from that block. *)
   let get g size =
     (* The main loop takes the current offset in the output buffer,
      * and iterates on input buffer chunks. *)
@@ -152,15 +118,14 @@ module Generator = struct
       let copied = min needed block_len in
       let chunks = (block, block_ofs + g.offset, offset, copied) :: chunks in
       (* Update buffer data -- did we consume a full block? *)
-      if block_len <= needed then begin
+      if block_len <= needed then (
         ignore (Queue.take g.buffers);
         g.length <- g.length - block_len;
-        g.offset <- 0
-      end
-      else begin
+        g.offset <- 0)
+      else (
         g.length <- g.length - needed;
-        g.offset <- g.offset + needed
-      end;
+        g.offset <- g.offset + needed);
+
       (* Add more data by recursing on the next block, or finish. *)
       if block_len < needed then aux chunks (offset + block_len)
       else List.rev chunks
@@ -232,6 +197,7 @@ module Metadata = struct
       (fun (p, m) -> if p < needed then Frame.set_metadata frame (offset + p) m)
       g.metadata;
     advance g needed;
+
     (* Mark the end of this filling. If the frame is partial it must be because
      * of a break in the generator, or because the generator is emptying.
      * Conversely, each break in the generator must cause a partial frame, so
@@ -263,8 +229,10 @@ module From_frames = struct
   (** Duration of data (in ticks) before the next break, -1 if there's none. *)
   let remaining fg = match fg.breaks with a :: _ -> a | _ -> -1
 
-  let add_metadata fg m = fg.metadata <- fg.metadata @ [(length fg, m)]
-  let add_break fg = fg.breaks <- fg.breaks @ [length fg]
+  let add_metadata ?(pos = 0) fg m =
+    fg.metadata <- fg.metadata @ [(length fg + pos, m)]
+
+  let add_break ?(pos = 0) fg = fg.breaks <- fg.breaks @ [length fg + pos]
 
   (* Advance metadata and breaks by [len] ticks. *)
   let advance fg len =
@@ -280,20 +248,28 @@ module From_frames = struct
   (** Only the breaks and metadata in the considered portion of the
     * content will be taken into account. This includes position
     * ofs but excludes ofs+len for metadata and the opposite for breaks. *)
-  let feed fg ?(copy = true) ?(breaks = []) ?(metadata = []) content ofs len =
+  let feed fg ?(copy : [ `None | `Audio | `Video | `Both ] = `Both)
+      ?(breaks = []) ?(metadata = []) content ofs len =
     let breaks = List.filter (fun p -> ofs < p && p <= ofs + len) breaks in
     let metadata =
       List.filter (fun (p, _) -> ofs <= p && p < ofs + len) metadata
     in
-    let content = if copy then Frame.copy content else content in
+    let content =
+      match copy with
+        | `None -> content
+        | `Audio -> Frame.copy_audio content
+        | `Video -> Frame.copy_video content
+        | `Both -> Frame.copy content
+    in
     fg.breaks <- fg.breaks @ List.map (fun p -> length fg + p - ofs) breaks;
     fg.metadata <-
       fg.metadata @ List.map (fun (p, m) -> (length fg + p - ofs, m)) metadata;
     Generator.put fg.generator content ofs len
 
   (** Take all data from a frame: breaks, metadata and available content. *)
-  let feed_from_frame fg frame =
-    let size = Lazy.force Frame.size in
+  let feed_from_frame ?(copy : [ `None | `Audio | `Video | `Both ] = `Both) fg
+      frame =
+    let position = Frame.position frame in
     fg.metadata <-
       fg.metadata
       @ List.map
@@ -305,15 +281,18 @@ module From_frames = struct
           (fun p -> length fg + p)
           (* Filter out the last break, which only marks the end
            * of frame, not a track limit (doesn't mean is_partial). *)
-          (List.filter (fun x -> x < size) (Frame.breaks frame));
+          (List.filter (fun x -> x < position) (Frame.breaks frame));
+
     (* Feed all content layers into the generator. *)
-    let rec feed_all ofs = function
-      | (cstop, content) :: contents ->
-          Generator.put fg.generator (Frame.copy content) ofs cstop;
-          if cstop < size then feed_all cstop contents
-      | [] -> assert false
+    let content = frame.Frame.content in
+    let content =
+      match copy with
+        | `None -> content
+        | `Audio -> Frame.copy_audio content
+        | `Video -> Frame.copy_video content
+        | `Both -> Frame.copy content
     in
-    feed_all 0 frame.Frame.contents
+    Generator.put fg.generator content 0 position
 
   (* Fill a frame from the generator's data. *)
   let fill fg frame =
@@ -327,14 +306,14 @@ module From_frames = struct
     let blocks = Generator.get fg.generator needed in
     List.iter
       (fun (block, o, o', size) ->
-        let ctype = Frame.type_of_content block in
-        let dst = Frame.content_of_type frame (offset + o') ctype in
-        Frame.blit_content block o dst (offset + o') size)
+        let dst = frame.Frame.content in
+        Frame.fill_content block o dst (offset + o') size)
       blocks;
     List.iter
       (fun (p, m) -> if p < needed then Frame.set_metadata frame (offset + p) m)
       fg.metadata;
     advance fg needed;
+
     (* Mark the end of this filling.
      * If the frame is partial it must be because of a break in the
      * generator, or because the generator is emptying.
@@ -345,16 +324,43 @@ module From_frames = struct
       match fg.breaks with
         | 0 :: tl -> fg.breaks <- tl
         | [] -> () (* end of stream / underrun ... *)
-        | _ -> assert false )
+        | _ -> assert false)
 end
 
+(** In [`Both] mode, the buffer is always kept in sync as follows:
+    - PTS in audio and video is expected to be tracked when submitting
+      data. User with no knowledge of PTS (typically all decoders except
+      ffmpeg) should be able to submit without having to deal with PTS.
+    - The buffer is always in sync except for potentially one type of
+      data and always at the end of the buffer. Typically:
+         0----1----2--> audio
+         0----1----2----3----4----> video
+    - When a new chunk is added, any gap in data from the other type
+      is removed. So, for instance, when adding an audio chunk of type
+      4--(5)----(6)- one gets:
+         0----1----4----5----6-> audio
+         0----1----4----> video
+      Note: video frame being usually one frame long, we unfortunately cannot
+      keep partially filled frames.
+    - Current chunk length and PTS are tracked so that new chunk are split
+      by increment of frame size. Typically, if we have:
+         0----1----2--> audio
+         0----1----2----3----4----5> video
+      Adding a chunk of audio of type: 2--(3)----(4)- should
+      result in  adding a first chunk of 4-- then a chunk of 3----
+      and, finally, a chunk of 4-. *)
 module From_audio_video = struct
   type mode = [ `Audio | `Video | `Both | `Undefined ]
+  type 'a content = { pts : int64; data : 'a }
 
   type t = {
     mutable mode : mode;
-    audio : Frame.audio_t array Generator.t;
-    video : Frame.video_t array Generator.t;
+    mutable current_audio_pts : int64;
+    current_audio : Frame_content.data content Generator.t;
+    audio : Frame_content.data content Generator.t;
+    mutable current_video_pts : int64;
+    current_video : Frame_content.data content Generator.t;
+    video : Frame_content.data content Generator.t;
     mutable metadata : (int * Frame.metadata) list;
     mutable breaks : int list;
   }
@@ -362,44 +368,29 @@ module From_audio_video = struct
   let create m =
     {
       mode = m;
+      current_audio_pts = 0L;
+      current_audio = Generator.create ();
       audio = Generator.create ();
+      current_video_pts = 0L;
+      current_video = Generator.create ();
       video = Generator.create ();
       metadata = [];
       breaks = [];
     }
 
   (** Audio length, in ticks. *)
-  let audio_length t = Generator.length t.audio
+  let audio_length t =
+    Generator.length t.audio + Generator.length t.current_audio
 
   (** Video length, in ticks. *)
-  let video_length t = Generator.length t.video
+  let video_length t =
+    Generator.length t.video + Generator.length t.current_video
 
   (** Total length. *)
   let length t = min (Generator.length t.audio) (Generator.length t.video)
 
-  let audio_size t =
-    let float_size = 8 in
-    let track_size (t : Frame.audio_t) = Array.length t * float_size in
-    let audio_size = Array.fold_left (fun n t -> n + track_size t) 0 in
-    Queue.fold
-      (fun n a -> n + audio_size (Generator.chunk_data a))
-      0 t.audio.Generator.buffers
-
-  let video_size t =
-    let image_size (i : Image.RGBA32.t) =
-      Bigarray.Array1.size_in_bytes (Image.RGBA32.data i)
-    in
-    let video_size (v : Frame.video_t) =
-      Array.fold_left (fun n i -> n + image_size i) 0 v
-    in
-    let buffer_size (b : Frame.video_t array) =
-      Array.fold_left (fun n v -> n + video_size v) 0 b
-    in
-    Queue.fold
-      (fun n a -> n + buffer_size (Generator.chunk_data a))
-      0 t.video.Generator.buffers
-
-  let size t = audio_size t + video_size t
+  (** Total buffered length. *)
+  let buffered_length t = max (audio_length t) (video_length t)
 
   (** Duration of data (in ticks) before the next break, -1 if there's none. *)
   let remaining t = match t.breaks with a :: _ -> a | _ -> -1
@@ -407,21 +398,15 @@ module From_audio_video = struct
   (** Add metadata at the minimum position of audio and video.
     * You probably want to call this when there is as much
     * audio as video. *)
-  let add_metadata t m = t.metadata <- t.metadata @ [(length t, m)]
+  let add_metadata ?(pos = 0) t m =
+    t.metadata <- t.metadata @ [(length t + pos, m)]
 
   (** Add a track limit. Audio and video length should be equal. *)
-  let add_break ?(sync = `Strict) t =
-    begin
-      match sync with
-      | `Strict -> assert (audio_length t = video_length t)
-      | `Ignore -> ()
-      | `Drop ->
-          let alen = audio_length t in
-          let vlen = video_length t in
-          if alen > vlen then Generator.remove_end t.audio (alen - vlen);
-          if vlen > alen then Generator.remove_end t.video (vlen - alen)
-    end;
-    t.breaks <- t.breaks @ [length t]
+  let add_break ?(sync = false) ?(pos = 0) t =
+    if sync then (
+      Generator.clear t.current_audio;
+      Generator.clear t.current_video);
+    t.breaks <- t.breaks @ [length t + pos]
 
   let clear t =
     t.metadata <- [];
@@ -436,35 +421,166 @@ module From_audio_video = struct
 
   (** Change the generator mode. Only allowed when there is as much audio as video.  *)
   let set_mode t m =
-    if t.mode <> m then begin
+    if t.mode <> m then (
       assert (audio_length t = video_length t);
-      t.mode <- m
-    end
+      t.mode <- m)
 
-  (** Add some audio content. Offset and length are given in audio samples. *)
-  let put_audio t content o l =
-    assert (content = [||] || Array.length content.(0) >= o + l);
-    let o = Frame.master_of_audio o in
-    let l = Frame.master_of_audio l in
-    Generator.put t.audio content o l;
-    match t.mode with
-      | `Audio -> Generator.put t.video [||] 0 l
-      | `Both -> ()
-      | `Video | `Undefined -> assert false
+  (** Check for pending synced A/V content *)
+  let sync_content t =
+    let s = Lazy.force Frame.size in
 
-  (** Add some video content. Offset and length are given in video samples. *)
-  let put_video t content o l =
-    assert (content = [||] || Array.length content.(0) <= o + l);
-    let o = Frame.master_of_video o in
-    let l = Frame.master_of_video l in
-    Generator.put t.video content o l;
-    match t.mode with
-      | `Video -> Generator.put t.audio [||] 0 l
+    let audio = Queue.copy t.current_audio.Generator.buffers in
+    let initial_audio_offset = t.current_audio.Generator.offset in
+
+    let video = Queue.copy t.current_video.Generator.buffers in
+    let initial_video_offset = t.current_video.Generator.offset in
+
+    (* First buffer offset comes from the generator! *)
+    let rec pick ~offset ~picked ~pos ~chunks pts =
+      match Queue.peek_opt chunks with
+        | Some (chunk, _, l) when chunk.pts = pts ->
+            Queue.add (Queue.take chunks) picked;
+            pick ~offset:0 ~picked ~pos:(pos + l - offset) ~chunks pts
+        | Some (chunk, _, _) when pts < chunk.pts -> (pos, true)
+        (* Prevent user from submitting non-monotonic PTS. *)
+        | Some (chunk, _, _) when chunk.pts < pts ->
+            pick ~offset:0 ~picked ~pos ~chunks pts
+        | None -> (pos, false)
+        | _ -> assert false
+    in
+
+    let add_audio ~picked ~pos () =
+      Queue.iter (fun (chunk, o, l) -> Generator.put t.audio chunk o l) picked;
+      Generator.remove t.current_audio pos
+    in
+
+    let add_video ~picked ~pos () =
+      Queue.iter (fun (chunk, o, l) -> Generator.put t.video chunk o l) picked;
+      Generator.remove t.current_video pos
+    in
+
+    let rec f () =
+      match (Queue.peek_opt audio, Queue.peek_opt video) with
+        | Some ({ pts = audio_pts }, _, _), Some ({ pts = video_pts }, _, _)
+          when audio_pts < video_pts ->
+            let picked_audio = Queue.create () in
+            let audio_len, _ =
+              pick ~offset:initial_audio_offset ~picked:picked_audio ~pos:0
+                ~chunks:audio audio_pts
+            in
+            Generator.remove t.current_audio audio_len;
+            f ()
+        | Some ({ pts = audio_pts }, _, _), Some ({ pts = video_pts }, _, _)
+          when video_pts < audio_pts ->
+            let picked_video = Queue.create () in
+            let video_len, _ =
+              pick ~offset:initial_video_offset ~picked:picked_video ~pos:0
+                ~chunks:video video_pts
+            in
+            Generator.remove t.current_video video_len;
+            f ()
+        | Some ({ pts }, _, _), Some _ ->
+            let picked_audio = Queue.create () in
+            let picked_video = Queue.create () in
+            let audio_len, more_audio =
+              pick ~offset:initial_audio_offset ~picked:picked_audio ~pos:0
+                ~chunks:audio pts
+            in
+            let video_len, more_video =
+              pick ~offset:initial_audio_offset ~picked:picked_video ~pos:0
+                ~chunks:video pts
+            in
+            (match (audio_len, video_len) with
+              (* Full frame sync. Take it! *)
+              | _ when audio_len = video_len && video_len = s ->
+                  add_audio ~picked:picked_audio ~pos:audio_len ();
+                  add_video ~picked:picked_video ~pos:video_len ()
+              (* Partial audio or video frame. Can it! *)
+              | _
+                when (audio_len < s && more_audio)
+                     || (video_len < s && more_video) ->
+                  Generator.remove t.current_audio audio_len;
+                  Generator.remove t.current_video video_len
+              | _ -> ());
+            f ()
+        | _ -> ()
+    in
+
+    f ()
+
+  let put_frames ~pts ~current_pts gen data o l =
+    let s = Lazy.force Frame.size in
+
+    let current_chunk = Generator.length gen mod s in
+
+    let put ~pts o l =
+      if current_pts <= pts && 0 < l then Generator.put gen { pts; data } o l
+    in
+
+    (* First complete the previous chunk. *)
+    let r = min l (s - current_chunk) in
+    put ~pts o r;
+
+    let pts = if r + current_chunk = s then Int64.succ pts else pts in
+    let l = l - r in
+    let o = o + r in
+
+    (* Now fill out the rest of the frames in [l] *)
+    let frames = l / s in
+    let rem = l mod s in
+
+    (* Add data by increment one one pts/frame.size *)
+    for i = 0 to frames - 1 do
+      let pts = Int64.add pts (Int64.of_int i) in
+      put ~pts (o + (i * s)) s
+    done;
+
+    let pts = Int64.add pts (Int64.of_int frames) in
+
+    put ~pts (o + (frames * s)) rem;
+
+    pts
+
+  (** Add some audio content. Offset and length are given in main ticks. *)
+  let put_audio ?pts t data o l =
+    let pts = match pts with Some pts -> pts | None -> t.current_audio_pts in
+    t.current_audio_pts <-
+      put_frames ~pts ~current_pts:t.current_audio_pts t.current_audio data o l;
+    begin
+      match t.mode with
+      (* The buffer's logic is all synchronous so we keep
+         constant empty content for the other type when using
+         the buffer with one single type. *)
+      | `Audio ->
+          t.current_video_pts <-
+            put_frames ~pts ~current_pts:t.current_video_pts t.current_video
+              Frame_content.None.data 0 l
       | `Both -> ()
-      | `Audio | `Undefined -> assert false
+      | _ -> assert false
+    end;
+    sync_content t
+
+  (** Add some video content. Offset and length are given in main ticks. *)
+  let put_video ?pts t data o l =
+    let pts = match pts with Some pts -> pts | None -> t.current_video_pts in
+    t.current_video_pts <-
+      put_frames ~pts ~current_pts:t.current_video_pts t.current_video data o l;
+    begin
+      match t.mode with
+      (* The buffer's logic is all synchronous so we keep
+         constant empty content for the other type when using
+         the buffer with one single type. *)
+      | `Video ->
+          t.current_audio_pts <-
+            put_frames ~pts ~current_pts:t.current_audio_pts t.current_audio
+              Frame_content.None.data 0 l
+      | _ -> ()
+    end;
+    sync_content t
 
   (** Take all data from a frame: breaks, metadata and available content. *)
-  let feed_from_frame t frame =
+  let feed_from_frame ?(copy : [ `None | `Audio | `Video | `Both ] = `Both)
+      ?mode t frame =
     let size = Lazy.force Frame.size in
     t.metadata <-
       t.metadata
@@ -478,21 +594,39 @@ module From_audio_video = struct
           (* Filter out the last break, which only marks the end
            * of frame, not a track limit (doesn't mean is_partial). *)
           (List.filter (fun x -> x < size) (Frame.breaks frame));
+
     (* Feed all content layers into the generator. *)
-    let rec feed_all ofs = function
-      | (cstop, content) :: contents ->
-          let content = Frame.copy content in
-          ( match mode t with
-            | `Audio -> put_audio t content.Frame.audio ofs cstop
-            | `Video -> put_video t content.Frame.video ofs cstop
-            | `Both ->
-                put_audio t content.Frame.audio ofs cstop;
-                put_video t content.Frame.video ofs cstop
-            | `Undefined -> () );
-          if cstop < size then feed_all cstop contents
-      | [] -> assert false
-    in
-    feed_all 0 frame.Frame.contents
+    let pts = Frame.pts frame in
+    let mode = match mode with Some mode -> mode | None -> t.mode in
+    let pos = Frame.position frame in
+    let content = frame.Frame.content in
+
+    match mode with
+      | `Audio ->
+          let content =
+            match copy with
+              | `Audio | `Both -> Frame.copy_audio content
+              | _ -> content
+          in
+          put_audio ~pts t content.Frame.audio 0 pos
+      | `Video ->
+          let content =
+            match copy with
+              | `Video | `Both -> Frame.copy_video content
+              | _ -> content
+          in
+          put_video ~pts t content.Frame.video 0 pos
+      | `Both ->
+          let content =
+            match copy with
+              | `None -> content
+              | `Audio -> Frame.copy_audio content
+              | `Video -> Frame.copy_video content
+              | `Both -> Frame.copy content
+          in
+          put_audio ~pts t content.Frame.audio 0 pos;
+          put_video ~pts t content.Frame.video 0 pos
+      | `Undefined -> ()
 
   (* Advance metadata and breaks by [len] ticks. *)
   let advance t len =
@@ -503,69 +637,79 @@ module From_audio_video = struct
 
   let remove t len =
     Generator.remove t.audio len;
+    Generator.clear t.current_audio;
     Generator.remove t.video len;
+    Generator.clear t.current_video;
     advance t len
 
+  let remove_buffered t len =
+    let audio_length = Generator.length t.current_audio in
+    let video_length = Generator.length t.current_video in
+    let top_generator, bottom_generator, top_length, diff_len =
+      if audio_length > video_length then
+        ( t.current_audio,
+          t.current_video,
+          audio_length,
+          audio_length - video_length )
+      else
+        ( t.current_video,
+          t.current_audio,
+          audio_length,
+          video_length - audio_length )
+    in
+    let remaining =
+      if len <= diff_len then (
+        Generator.remove top_generator len;
+        0)
+      else if len <= top_length then (
+        Generator.remove top_generator len;
+        Generator.remove bottom_generator (len - diff_len);
+        0)
+      else (
+        Generator.clear top_generator;
+        Generator.clear bottom_generator;
+        len - top_length)
+    in
+    if remaining > 0 then (
+      Generator.remove t.audio remaining;
+      Generator.remove t.video remaining;
+      advance t remaining)
+
   let fill t frame =
+    let mode = mode t in
     let fpos = Frame.position frame in
     let size = Lazy.force Frame.size in
+
     let remaining =
       let l = remaining t in
       if l = -1 then length t else l
     in
+
     let l = min (size - fpos) remaining in
+
     let audio = Generator.get t.audio l in
     let video = Generator.get t.video l in
-    (* We got equal durations of audio and video, but segmented differently.
-     * We walk through them and blit them into the frame, possibly creating
-     * several content layers of different types as the numbers of channels
-     * vary. *)
-    let rec blit audio video =
-      match (audio, video) with
-        | [], [] -> Frame.add_break frame (fpos + l)
-        | (ablk, apos, apos', al) :: audio, (vblk, vpos, vpos', vl) :: video ->
-            (* Audio and video destination positions are the same,
-             * however they need be aligned.
-             * At the beginning they are aligned (and zero), but then
-             * we might advance from a few audio samples, which might
-             * not correspond to an integer number of video samples,
-             * after which vpos' is not the position of a video frame. *)
-            assert (apos' = vpos');
-            let fpos = fpos + apos' in
-            let ctype =
-              {
-                Frame.audio = Array.length ablk;
-                video = Array.length vblk;
-                midi = 0;
-              }
-            in
-            let dst = Frame.content_of_type frame fpos ctype in
-            let l = min al vl in
-            Utils.array_iter2 vblk dst.Frame.video (fun v v' ->
-                let ( ! ) = Frame.video_of_master in
-                (* How many samples should we output:
-                 * the number of samples expected in total after this
-                 * blit round, minus those that have been outputted before.
-                 * When everything is aligned, this is the same as [!l]. *)
-                let l = !(vpos' + l) - !vpos' in
-                Video.blit v !vpos v' !fpos l);
-            Utils.array_iter2 ablk dst.Frame.audio (fun a a' ->
-                let ( ! ) = Frame.audio_of_master in
-                (* Same as above, even if in practice everything
-                 * will always be aligned on the audio side. *)
-                let l = !(apos' + l) - !apos' in
-                Audio.Mono.blit a !apos a' !fpos l);
-            if al = vl then blit audio video
-            else if al > vl then
-              blit ((ablk, apos + vl, apos' + vl, al - vl) :: audio) video
-            else blit audio ((vblk, vpos + al, vpos' + al, vl - al) :: video)
-        | _, _ -> assert false
-    in
-    blit audio video;
+
+    if mode = `Both || mode = `Audio then
+      List.iter
+        (fun ({ data }, apos, apos', al) ->
+          Frame_content.blit data apos (AFrame.content frame) (fpos + apos') al)
+        audio;
+
+    if mode = `Both || mode = `Video then
+      List.iter
+        (fun ({ data }, vpos, vpos', vl) ->
+          Frame_content.blit data vpos (VFrame.content frame) (fpos + vpos') vl)
+        video;
+
+    Frame.add_break frame (fpos + l);
+
     List.iter
       (fun (p, m) -> if p < l then Frame.set_metadata frame (fpos + p) m)
       t.metadata;
+
     advance t l;
+
     (* If the frame is partial it must be because of a break in the
      * generator, or because the generator is emptying.
      * Conversely, each break in the generator must cause a partial frame,
@@ -574,7 +718,7 @@ module From_audio_video = struct
       match t.breaks with
         | 0 :: tl -> t.breaks <- tl
         | [] -> () (* end of stream / underrun ... *)
-        | _ -> assert false )
+        | _ -> assert false)
 end
 
 module From_audio_video_plus = struct
@@ -594,46 +738,53 @@ module From_audio_video_plus = struct
 
   type t = {
     lock : Mutex.t;
-    (* The generator knows what content kind it is expected to produce
-     * Because of the async put_audio/video calls, we only detect the
-     * error upon [fill]. When an error is detected, the error flag is
-     * set, which makes put_audio/video fail and hence kills the feeding
-     * process. *)
-    kind : Frame.content_kind;
+    (* The type of fed data must always be the same. This is fixed by the first
+       filled frame. *)
+    mutable ctype : Frame.content_type option;
     mutable error : bool;
     overfull : overfull option;
     gen : Super.t;
     log : string -> unit;
+    log_overfull : bool;
     (* Metadata rewriting, in place modification allowed *)
     mutable map_meta : Frame.metadata -> Frame.metadata;
   }
 
-  let create ?(lock = Mutex.create ()) ?overfull ~kind ~log mode =
+  let create ?(lock = Mutex.create ()) ?overfull ~log ~log_overfull mode =
     {
       lock;
-      kind;
+      ctype = None;
       error = false;
       overfull;
       log;
+      log_overfull;
       gen = Super.create mode;
       map_meta = (fun x -> x);
     }
 
+  let content_type t = Tutils.mutexify t.lock (fun () -> Option.get t.ctype) ()
+
+  let set_content_type t =
+    Tutils.mutexify t.lock (fun ctype ->
+        assert (t.ctype = None);
+        t.ctype <- Some ctype)
+
   let mode t = Tutils.mutexify t.lock Super.mode t.gen
   let set_mode t mode = Tutils.mutexify t.lock (Super.set_mode t.gen) mode
   let audio_length t = Tutils.mutexify t.lock Super.audio_length t.gen
   let video_length t = Tutils.mutexify t.lock Super.video_length t.gen
   let length t = Tutils.mutexify t.lock Super.length t.gen
   let remaining t = Tutils.mutexify t.lock Super.remaining t.gen
-  let audio_size t = Tutils.mutexify t.lock Super.audio_size t.gen
-  let video_size t = Tutils.mutexify t.lock Super.video_size t.gen
-  let size t = Tutils.mutexify t.lock Super.size t.gen
   let set_rewrite_metadata t f = t.map_meta <- f
 
-  let add_metadata t m =
-    Tutils.mutexify t.lock (fun m -> Super.add_metadata t.gen (t.map_meta m)) m
+  let add_metadata ?pos t m =
+    Tutils.mutexify t.lock
+      (fun m -> Super.add_metadata ?pos t.gen (t.map_meta m))
+      m
+
+  let add_break ?sync ?pos t =
+    Tutils.mutexify t.lock (Super.add_break ?sync ?pos) t.gen
 
-  let add_break ?sync t = Tutils.mutexify t.lock (Super.add_break ?sync) t.gen
   let clear t = Tutils.mutexify t.lock Super.clear t.gen
 
   let fill t frame =
@@ -642,14 +793,16 @@ module From_audio_video_plus = struct
         let p = Frame.position frame in
         let breaks = Frame.breaks frame in
         Super.fill t.gen frame;
-        let _, c = Frame.content frame p in
-        if not (Frame.type_has_kind (Frame.type_of_content c) t.kind) then begin
-          t.log "Incorrect stream type!";
-          t.error <- true;
-          Super.clear t.gen;
-          Frame.clear_from frame p;
-          Frame.set_breaks frame (p :: breaks)
-        end)
+        let c = frame.Frame.content in
+        match t.ctype with
+          | None -> t.ctype <- Some (Frame.type_of_content c)
+          | Some ctype ->
+              if not (Frame.compatible (Frame.type_of_content c) ctype) then (
+                t.log "Incorrect stream type!";
+                t.error <- true;
+                Super.clear t.gen;
+                Frame.clear_from frame p;
+                Frame.set_breaks frame (p :: breaks)))
       ()
 
   let remove t len = Tutils.mutexify t.lock (Super.remove t.gen) len
@@ -657,42 +810,57 @@ module From_audio_video_plus = struct
   let check_overfull t extra =
     assert (Tutils.seems_locked t.lock);
     match t.overfull with
-      | Some (`Drop_old len) when Super.length t.gen + extra > len ->
-          let len = Super.length t.gen + extra - len in
-          let len_time = Frame.seconds_of_master len in
-          t.log
-            (Printf.sprintf
-               "Buffer overrun: Dropping %.2fs. Consider increasing the max \
-                buffer size!"
-               len_time);
-          Super.remove t.gen len
+      | Some (`Drop_old len) when Super.buffered_length t.gen + extra > len ->
+          let len = Super.buffered_length t.gen + extra - len in
+          let len_time = Frame.seconds_of_main len in
+          if t.log_overfull then
+            t.log
+              (Printf.sprintf
+                 "Buffer overrun: Dropping %.2fs. Consider increasing the max \
+                  buffer size!"
+                 len_time);
+          Super.remove_buffered t.gen len
       | _ -> ()
 
-  let put_audio t buf off len =
+  let put_audio ?pts t buf off len =
     Tutils.mutexify t.lock
       (fun () ->
-        if t.error then begin
+        if t.error then (
           Super.clear t.gen;
           t.error <- false;
-          raise Incorrect_stream_type
-        end
-        else begin
-          check_overfull t (Frame.master_of_audio len);
-          Super.put_audio t.gen buf off len
-        end)
+          raise Incorrect_stream_type)
+        else (
+          check_overfull t len;
+          Super.put_audio ?pts t.gen buf off len))
       ()
 
-  let put_video t buf off len =
+  let put_video ?pts t buf off len =
     Tutils.mutexify t.lock
       (fun () ->
-        if t.error then begin
+        if t.error then (
           Super.clear t.gen;
           t.error <- false;
-          raise Incorrect_stream_type
-        end
-        else begin
-          check_overfull t (Frame.master_of_video len);
-          Super.put_video t.gen buf off len
-        end)
+          raise Incorrect_stream_type)
+        else (
+          check_overfull t len;
+          Super.put_video ?pts t.gen buf off len))
+      ()
+
+  let feed_from_frame ?mode t frame =
+    Tutils.mutexify t.lock
+      (fun () ->
+        (match t.ctype with
+          | None -> t.ctype <- Some (Frame.content_type frame)
+          | Some ctype ->
+              if Frame.content_type frame <> ctype then (
+                t.log "Incorrect stream type!";
+                t.error <- true));
+        if t.error then (
+          Super.clear t.gen;
+          t.error <- false;
+          raise Incorrect_stream_type)
+        else (
+          check_overfull t (Lazy.force Frame.size);
+          Super.feed_from_frame ?mode t.gen frame))
       ()
 end
