@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -21,16 +21,20 @@
  *****************************************************************************)
 
 (** Operations on generators, which are like FIFO for multimedia data. They are
-    efficiently handling small chunks of data (as found in frames) without copy,
-    and also metadata. *)
+    efficiently handling small chunks of data (as found in frames) with minimal copy,
+    and also metadata.
+
+    Data coming from frames is copied when added to the buffer because frame content
+    are reused during each streaming loop. However, data is only assigned exiting the
+    buffer. This means that we expect the buffer to feed a single source on the way
+    out. *)
 
 (** Raised when trying to feed a generator with data of incorrect type (wrong
     number of audio channels, etc.). *)
 exception Incorrect_stream_type
 
 (** Signature for a generator. *)
-module type S =
-sig
+module type S = sig
   type t
 
   (** Length of the generator in ticks. *)
@@ -49,12 +53,11 @@ sig
   val remove : t -> int -> unit
 
   (** Add metadata. *)
-  val add_metadata : t -> Frame.metadata -> unit
+  val add_metadata : ?pos:int -> t -> Frame.metadata -> unit
 end
 
 (** Content-agnostic generator. *)
-module Generator :
-sig
+module Generator : sig
   (** A generator. *)
   type 'a t
 
@@ -100,7 +103,7 @@ module Metadata : sig
   (** Time until next break, or -1 if there is none. *)
   val remaining : t -> int
 
-  (** Drop a portion at the begining. *)
+  (** Drop a portion at the beginning. *)
   val advance : t -> int -> unit
 
   (** Drop break at the beginning. This should be called after filling a partial
@@ -119,8 +122,7 @@ module Metadata : sig
 end
 
 (** A generator that consumes frames (or frame content) and produces frames. *)
-module From_frames :
-sig
+module From_frames : sig
   (** A generator. *)
   type t
 
@@ -137,37 +139,70 @@ sig
   val remaining : t -> int
 
   (** Add metadata at the current position. *)
-  val add_metadata : t -> Frame.metadata -> unit
+  val add_metadata : ?pos:int -> t -> Frame.metadata -> unit
 
   (** Add a break at the current position. *)
-  val add_break : t -> unit
+  val add_break : ?pos:int -> t -> unit
 
   (** Remove data. *)
   val remove : t -> int -> unit
 
   (** Feed the generator with data. *)
-  val feed : t ->
-    ?copy:bool ->
-    ?breaks:(int list) -> ?metadata:((int*Frame.metadata) list) ->
-    Frame.content -> int -> int -> unit
+  val feed :
+    t ->
+    ?copy:[ `None | `Audio | `Video | `Both ] ->
+    ?breaks:int list ->
+    ?metadata:(int * Frame.metadata) list ->
+    Frame.content ->
+    int ->
+    int ->
+    unit
 
   (** Feed the generator with the contents of a frame (the contents is
       copied). *)
-  val feed_from_frame : t -> Frame.t -> unit
+  val feed_from_frame :
+    ?copy:[ `None | `Audio | `Video | `Both ] -> t -> Frame.t -> unit
 
   (** Fill a frame from the generator. *)
   val fill : t -> Frame.t -> unit
 end
 
+(** Generator not only with output but also with asynchronous input. *)
+module type S_Asio = sig
+  type t
+
+  val length : t -> int (* ticks *)
+
+  val audio_length : t -> int
+  val video_length : t -> int
+  val remaining : t -> int (* ticks *)
+
+  val clear : t -> unit
+  val fill : t -> Frame.t -> unit
+  val add_metadata : ?pos:int -> t -> Frame.metadata -> unit
+  val add_break : ?sync:bool -> ?pos:int -> t -> unit
+  val put_audio : ?pts:int64 -> t -> Frame_content.data -> int -> int -> unit
+  val put_video : ?pts:int64 -> t -> Frame_content.data -> int -> int -> unit
+  val set_mode : t -> [ `Audio | `Video | `Both | `Undefined ] -> unit
+end
+
 (** Generator that consumes audio and video asynchronously, and produces
     frames. *)
-module From_audio_video :
-sig
+module From_audio_video : sig
   type t
 
   (** In [`Audio] mode, only audio can be put in the buffer, and similarly for
-      the [`Video] mode. In [`Both] mode, both types of content can be fed into
-      the generator, asynchronously, and they exit the buffer synchronously.
+      the [`Video] mode.
+
+      In [`Both] mode, both types of content can be fed into the generator,
+      asynchronously, and they exit the buffer synchronously. PTS are only
+      used in this mode and are used to make sure that all audio and video
+      content is synchronized when exported. Typically, during muxing, the
+      audio source may not be ready for a while while the video source keeps
+      filling up the buffer. In this case, when the audio source starts
+      filling up the buffer as well, we can filter out all the video content
+      that has no corresponding audio.
+
       [`Undefined] forbids any feeding, it's useful to make sure a meaningful
       mode is assigned before any use. *)
   type mode = [ `Audio | `Video | `Both | `Undefined ]
@@ -192,65 +227,52 @@ sig
   (** Length of data available in both audio and video. *)
   val length : t -> int
 
-  (** Size of audio data in bytes. *)
-  val audio_size : t -> int
-
-  (** Size of video data in bytes. *)
-  val video_size : t -> int
-
-  (** Size of data in bytes. *)
-  val size : t -> int
+  (** Length of buffered data, taking into account buffered
+      unsynced data. *)
+  val buffered_length : t -> int
 
   (** Duration of data (in ticks) before the next break, -1 if there's none. *)
   val remaining : t -> int
 
-  (** Add metadata at the minimum position of audio and video. You probably want
-      to call this when there is as much audio as video. *)
-  val add_metadata : t -> Frame.metadata -> unit
+  (** Add metadata at the minimum position of audio and video. [pos] is an
+      offset from the current buffer's length and defaults to [0]. *)
+  val add_metadata : ?pos:int -> t -> Frame.metadata -> unit
 
-  (** Add a track limit. Audio and video length should be equal. *)
-  val add_break : ?sync:[`Strict|`Ignore|`Drop] -> t -> unit
+  (** Add a track limit. [pos] is an offset from the current buffer's length and
+      defaults to [0]. *)
+  val add_break : ?sync:bool -> ?pos:int -> t -> unit
 
-  (* [put_audio buffer data offset length]: offset and length
-   * are in samples ! *)
-  val put_audio : t -> Frame.audio_t array -> int -> int -> unit
+  (* [put_audio ?pts buffer data offset length]: offset and length
+   * are in main ticks ! *)
+  val put_audio : ?pts:int64 -> t -> Frame_content.data -> int -> int -> unit
 
-  (* [put_video buffer data offset length]: offset and length
-   * are in samples ! *)
-  val put_video : t -> Frame.video_t array -> int -> int -> unit
+  (* [put_video ?pts buffer data offset length]: offset and length
+   * are in main ticks ! *)
+  val put_video : ?pts:int64 -> t -> Frame_content.data -> int -> int -> unit
 
-  (** Feed from a frame, only copying data according to the mode. *)
-  val feed_from_frame : t -> Frame.t -> unit
+  (** Feed from a frame, only copying data according to the mode.
+      Defaults to the generator's mode. *)
+  val feed_from_frame :
+    ?copy:[ `None | `Audio | `Video | `Both ] ->
+    ?mode:mode ->
+    t ->
+    Frame.t ->
+    unit
 
   (** Fill a frame from the generator. *)
   val fill : t -> Frame.t -> unit
 
+  (* Remove from synced content, clear unsynced content. *)
   val remove : t -> int -> unit
 
+  (* Remove from buffered content (synced & unsynced), starting with top buffer. *)
+  val remove_buffered : t -> int -> unit
   val clear : t -> unit
 end
 
-(** Generator not only with Output but also with ASynchronous Input. *)
-module type S_Asio =
-sig
-  type t
-  val length : t -> int (* ticks *)
-  val audio_length : t -> int
-  val video_length : t -> int
-  val remaining : t -> int (* ticks *)
-  val clear : t -> unit
-  val fill : t -> Frame.t -> unit
-  val add_metadata : t -> Frame.metadata -> unit
-  val add_break : ?sync:[`Strict|`Ignore|`Drop] -> t -> unit
-  val put_audio : t -> Frame.audio_t array -> int -> int -> unit
-  val put_video : t -> Frame.video_t array -> int -> int -> unit
-  val set_mode : t -> [ `Audio | `Video | `Both | `Undefined ] -> unit
-end
-
 (** Same as [From_audio_video] but with two extra features useful for streaming
     decoders: it is thread safe and supports overfull buffer management. *)
-module From_audio_video_plus :
-sig
+module From_audio_video_plus : sig
   type t
 
   (** Same as [From_audio_video]. *)
@@ -260,34 +282,35 @@ sig
     * drop old data, keeping at most [len] ticks. *)
   type overfull = [ `Drop_old of int ]
 
-  val create : ?lock:Mutex.t -> ?overfull:overfull ->
-               kind:Frame.content_kind ->
-               log:(string -> unit) -> mode -> t
+  val create :
+    ?lock:Mutex.t ->
+    ?overfull:overfull ->
+    log:(string -> unit) ->
+    log_overfull:bool ->
+    mode ->
+    t
 
+  val content_type : t -> Frame.content_type
+  val set_content_type : t -> Frame.content_type -> unit
   val mode : t -> From_audio_video.mode
   val set_mode : t -> From_audio_video.mode -> unit
-
   val audio_length : t -> int
   val video_length : t -> int
   val length : t -> int
   val remaining : t -> int
-
-  val audio_size : t -> int
-  val video_size : t -> int
-  val size : t -> int
-
   val set_rewrite_metadata : t -> (Frame.metadata -> Frame.metadata) -> unit
-  val add_metadata : t -> Frame.metadata -> unit
-  val add_break : ?sync:[`Strict|`Ignore|`Drop] -> t -> unit
+  val add_metadata : ?pos:int -> t -> Frame.metadata -> unit
+  val add_break : ?sync:bool -> ?pos:int -> t -> unit
+
+  (* [put_audio ?pts buffer data offset length]:
+   * offset and length are in main ticks! *)
+  val put_audio : ?pts:int64 -> t -> Frame_content.data -> int -> int -> unit
 
-  (* [put_audio buffer data offset length]:
-   * offset and length are in audio samples! *)
-  val put_audio : t -> Frame.audio_t array -> int -> int -> unit
   (* [put_video buffer data offset length]:
-   * offset and length are in video samples! *)
-  val put_video : t -> Frame.video_t array -> int -> int -> unit
+   * offset and length are in main ticks! *)
+  val put_video : ?pts:int64 -> t -> Frame_content.data -> int -> int -> unit
+  val feed_from_frame : ?mode:mode -> t -> Frame.t -> unit
   val fill : t -> Frame.t -> unit
-
   val remove : t -> int -> unit
   val clear : t -> unit
 end
