@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,13 +24,15 @@ open Frame
 
 type t = Frame.t
 
-let tov = Frame.master_of_video
+let tov = Frame.main_of_video
 
 let vot ?round x =
   match round with
-    | None | Some `Down -> Frame.video_of_master x
-    | Some `Up -> Frame.video_of_master (x + Lazy.force Frame.video_rate - 1)
+    | None | Some `Down -> Frame.video_of_main x
+    | Some `Up -> Frame.video_of_main (x + Lazy.force Frame.video_rate - 1)
 
+let content b = b.content.video
+let yuva420p b = Frame_content.Video.get_data (content b)
 let size _ = vot (Lazy.force size)
 let next_sample_position t = vot ~round:`Up (Frame.position t)
 let add_break t i = add_break t (tov i)
@@ -45,13 +47,4 @@ let get_content frame source =
   in
   let v0 = vot ~round:`Up p0 in
   let v1 = vot ~round:`Down p1 in
-  if v0 < v1 then (
-    let stop, content = Frame.content frame p0 in
-    assert (stop = Lazy.force Frame.size);
-    Some (content.video, v0, v1 - v0) )
-  else None
-
-let content_of_type ~channels b =
-  let ctype = { audio = 0; video = channels; midi = 0 } in
-  let content = content_of_type b (Frame.position b) ctype in
-  content.video
+  if v0 < v1 then Some (frame.content.video, v0, v1 - v0) else None
