@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -56,10 +56,11 @@ val add_break : t -> int -> unit
   * It is possible that a successful filling produced audio samples
   * but no video sample. *)
 val get_content :
-      Frame.t -> Source.source -> (Video.buffer array * int * int) option
+  Frame.t -> Source.source -> (Frame_content.data * int * int) option
 
-(** Create a new video-only content layer for [channels] video channels,
-  * at the current position in the frame, i.e., suitable for the next
-  * filling operation.
-  * To choose the position, use Frame directly, and be careful. *)
-val content_of_type : channels:int -> t -> Video.buffer array
+(** Get video contents. Raises [Not_found] is frame has no video *)
+val content : t -> Frame_content.data
+
+(** Get yuva420p video content. Raises [Frame_content.Invalid] if video
+  * content is not yuva420p and [Not_found] if frame has no video content. *)
+val yuva420p : t -> Frame_content.Video.data
