@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 
  *****************************************************************************)
 
+open Mm
 open Source
 open Dssi
 
@@ -49,14 +50,11 @@ let all_chans = 16
 (* chan = None means synth all channels *)
 class dssi ~kind ?chan plugin descr outputs params source =
   object
-    inherit operator ~name:"dssi" kind [source]
-
+    inherit operator ~name:"dssi" (Source.Kind.of_kind kind) [source]
     method stype = source#stype
-
     method remaining = source#remaining
-
     method is_ready = source#is_ready
-
+    method self_sync = source#self_sync
     method abort_track = source#abort_track
 
     val di =
@@ -67,8 +65,7 @@ class dssi ~kind ?chan plugin descr outputs params source =
           (if chan <> None then 1 else all_chans)
           (fun _ ->
             Ladspa.Descriptor.instantiate (Descriptor.ladspa d)
-              (Lazy.force Frame.audio_rate)
-              (AFrame.size ())) )
+              (Lazy.force Frame.audio_rate)) )
 
     initializer
     Array.iter (fun inst -> Ladspa.Descriptor.activate inst) (snd di)
@@ -80,16 +77,15 @@ class dssi ~kind ?chan plugin descr outputs params source =
         source#get buf;
         Frame.position buf
       in
-      let _, content = Frame.content buf offset in
-      let b = content.Frame.audio in
-      let evs = content.Frame.midi in
+      let b = AFrame.pcm buf in
+      let evs = MFrame.midi buf in
       (* Now convert everything to audio samples. *)
-      let offset = Frame.audio_of_master offset in
-      let position = Frame.audio_of_master position in
+      let offset = Frame.audio_of_main offset in
+      let position = Frame.audio_of_main position in
       let len = position - offset in
       let evs =
         let dssi_of_midi (t, e) =
-          let t = Frame.audio_of_master (Frame.master_of_midi t) in
+          let t = Frame.audio_of_main (Frame.main_of_midi t) in
           match e with
             | MIDI.Note_on (n, v) ->
                 Some (t, Dssi.Event_note_on (0, n, int_of_float (v *. 127.)))
@@ -100,21 +96,20 @@ class dssi ~kind ?chan plugin descr outputs params source =
         match chan with
           | Some chan ->
               let evs = MIDI.data evs.(chan) in
-              [| Array.of_list (Utils.may_map dssi_of_midi evs) |]
+              [| Array.of_list (List.filter_map dssi_of_midi evs) |]
           | None ->
               Array.init all_chans (fun chan ->
                   Array.of_list
-                    (Utils.may_map dssi_of_midi (MIDI.data evs.(chan))))
+                    (List.filter_map dssi_of_midi (MIDI.data evs.(chan))))
       in
       Array.iter
         (fun inst ->
           List.iter
-            (fun (p, v) ->
-              Ladspa.Descriptor.connect_control_port_in inst p (v ()))
+            (fun (p, v) -> Ladspa.Descriptor.set_control_port inst p (v ()))
             params;
-          Ladspa.Descriptor.set_samples inst len;
           for c = 0 to Array.length outputs - 1 do
-            Ladspa.Descriptor.connect_audio_port inst outputs.(c) b.(c) offset
+            Ladspa.Descriptor.connect_port inst outputs.(c)
+              (Audio.Mono.sub b.(c) offset len)
           done)
         inst;
       assert (Array.length outputs = Array.length b);
@@ -125,55 +120,51 @@ class dssi ~kind ?chan plugin descr outputs params source =
         done
   end
 
+let () =
+  Lang.add_module "dssi";
+  Lang.add_module "synth.dssi";
+  Lang.add_module "synth.all.dssi"
+
 let register_descr plugin_name descr_n descr outputs =
   let ladspa_descr = Descriptor.ladspa descr in
   let liq_params, params = Ladspa_op.params_of_descr ladspa_descr in
   let chans = Array.length outputs in
-  let k =
-    Lang.kind_type_of_kind_format ~fresh:1
-      (Lang.Constrained
-         {
-           Frame.audio = Lang.Fixed chans;
-           video = Lang.Any_fixed 0;
-           midi = Lang.Fixed 1;
-         })
+  let kind =
+    { Frame.audio = Frame.audio_n chans; video = `Any; midi = Frame.midi_n 1 }
   in
+  let k = Lang.kind_type_of_kind_format kind in
   let liq_params = liq_params in
   Lang.add_operator
-    ( "synth.dssi."
-    ^ Utils.normalize_parameter_string (Ladspa.Descriptor.label ladspa_descr) )
-    ( [
-        ( "channel",
-          Lang.int_t,
-          Some (Lang.int 0),
-          Some "MIDI channel to handle." );
-      ]
+    ("synth.dssi."
+    ^ Utils.normalize_parameter_string (Ladspa.Descriptor.label ladspa_descr))
+    ([
+       ("channel", Lang.int_t, Some (Lang.int 0), Some "MIDI channel to handle.");
+     ]
     @ liq_params
-    @ [("", Lang.source_t k, None, None)] )
-    ~kind:(Lang.Unconstrained k) ~category:Lang.SoundSynthesis ~flags:[]
+    @ [("", Lang.source_t k, None, None)])
+    ~return_t:k ~category:`Synthesis ~flags:[`Extra]
     ~descr:(Ladspa.Descriptor.name ladspa_descr ^ ".")
-    (fun p kind ->
+    (fun p ->
       let f v = List.assoc v p in
       let chan = Lang.to_int (f "channel") in
       let source = Lang.to_source (f "") in
       let params = params p in
       new dssi ~kind plugin_name descr_n outputs params ~chan source);
-  let k =
-    Lang.kind_type_of_kind_format ~fresh:1
-      (Lang.Constrained
-         {
-           Frame.audio = Lang.Fixed chans;
-           video = Lang.Any_fixed 0;
-           midi = Lang.Fixed all_chans;
-         })
+  let kind =
+    {
+      Frame.audio = Frame.audio_n chans;
+      video = `Any;
+      midi = Frame.midi_n all_chans;
+    }
   in
+  let k = Lang.kind_type_of_kind_format kind in
   Lang.add_operator
-    ( "synth.all.dssi."
-    ^ Utils.normalize_parameter_string (Ladspa.Descriptor.label ladspa_descr) )
+    ("synth.all.dssi."
+    ^ Utils.normalize_parameter_string (Ladspa.Descriptor.label ladspa_descr))
     (liq_params @ [("", Lang.source_t k, None, None)])
-    ~kind:(Lang.Unconstrained k) ~category:Lang.SoundSynthesis ~flags:[]
+    ~return_t:k ~category:`Synthesis ~flags:[`Extra]
     ~descr:(Ladspa.Descriptor.name ladspa_descr ^ ".")
-    (fun p kind ->
+    (fun p ->
       let f v = List.assoc v p in
       let source = Lang.to_source (f "") in
       let params = params p in
@@ -192,7 +183,7 @@ let register_plugin ?(log_errors = false) pname =
           register_descr pname n d o;
           if log_errors then
             log#important "Registered DSSI plugin: %s."
-              (Ladspa.Descriptor.label ladspa_descr) )
+              (Ladspa.Descriptor.label ladspa_descr))
         else if log_errors then
           log#important
             "Plugin %s has inputs, don't know how to handle them for now."
@@ -230,20 +221,21 @@ let dssi_init =
     if !inited then ()
     else (
       Dssi.init ();
-      inited := true )
+      inited := true)
 
 let () =
   if dssi_enable then (
     dssi_init ();
-    register_plugins () );
+    register_plugins ());
   List.iter (register_plugin ~log_errors:true) dssi_load
 
 let () =
   Lang.add_builtin "dssi.register"
-    ~category:(Lang.string_of_category Lang.SoundSynthesis)
-    ~descr:"Resgister a DSSI plugin."
+    ~category:(`Source `Synthesis)
+    ~descr:"Register a DSSI plugin."
     [("", Lang.string_t, None, Some "Path of the DSSI plugin file.")]
-    Lang.unit_t (fun p _ ->
+    Lang.unit_t
+    (fun p ->
       dssi_init ();
       let fname = Lang.to_string (List.assoc "" p) in
       register_plugin ~log_errors:true fname;
