@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,6 +20,9 @@
 
  *****************************************************************************)
 
+open Mm
+open Tsdl
+
 let knotes2 =
   [|
     '&';
@@ -71,66 +74,75 @@ let array_index a x =
   done;
   match !ans with Some i -> i | None -> raise Not_found
 
+let char_of_key k =
+  let c = Sdl.get_key_name k in
+  if c = "" then raise Not_found;
+  c.[0]
+
 let note_of_char c =
   try array_index knotes2 c + 71 with Not_found -> array_index knotes1 c + 59
 
 class keyboard ~kind velocity =
-  let () = Sdl_utils.init [`EVENTTHREAD; `VIDEO] in
+  let () = Sdl_utils.init [Sdl.Init.events; Sdl.Init.video] in
   object (self)
-    inherit Source.active_source ~name:"input.keyboard.sdl" kind
+    inherit
+      Source.active_source ~name:"input.keyboard.sdl" (Source.Kind.of_kind kind)
 
     method stype = Source.Infallible
-
     method is_ready = true
-
     method remaining = -1
-
     method abort_track = ()
+    method self_sync = (`Static, false)
 
-    method output = if AFrame.is_partial memo then self#get_frame memo
+    method output =
+      if self#is_ready && AFrame.is_partial self#memo then self#get self#memo
 
-    method output_get_ready =
-      Sdlevent.disable_events Sdlevent.all_events_mask;
-      Sdlevent.enable_events
-        (Sdlevent.make_mask
-           [Sdlevent.KEYDOWN_EVENT; Sdlevent.KEYUP_EVENT; Sdlevent.QUIT_EVENT]);
-      ignore (Sdlvideo.set_video_mode ~w:640 ~h:480 ~bpp:16 [])
+    val mutable window = None
+
+    method wake_up _ =
+      window <-
+        Some
+          (Sdl_utils.check
+             (fun () ->
+               Sdl.create_window "Liquidsoap" ~w:640 ~h:480 Sdl.Window.windowed)
+             ())
 
     method private sleep = Sdl.quit ()
-
     val mutable reader = None
-
     val mutable velocity = velocity
-
-    method output_reset = ()
-
-    method is_active = true
+    method reset = ()
 
     method get_frame frame =
       assert (0 = MFrame.position frame);
-      let m = Frame.content_of_type frame 0 (Frame.type_of_kind kind) in
-      let m = m.Frame.midi in
+      let m = MFrame.midi frame in
       let t =
         let ans = MIDI.create (MFrame.size ()) in
-        Sdlevent.pump ();
-        while Sdlevent.has_event () do
+        Sdl.pump_events ();
+        while
+          Sdl.has_event Sdl.Event.key_down || Sdl.has_event Sdl.Event.key_up
+        do
           try
-            match Sdlevent.poll () with
-              | Some (Sdlevent.KEYDOWN k) ->
-                  let c = Sdlkey.char_of_key k.Sdlevent.keysym in
-                  if c = '+' || c = '*' then velocity <- min 1. (velocity +. 0.1)
-                  else if c = '-' || c = '/' then
-                    velocity <- max 0. (velocity -. 0.1)
-                  else (
+            let e = Sdl.Event.create () in
+            if Sdl.poll_event (Some e) then (
+              match Sdl.Event.(enum (get e typ)) with
+                | `Key_down ->
+                    let k = Sdl.Event.(get e keyboard_keycode) in
+                    let c = char_of_key k in
+                    if c = '+' || c = '*' then
+                      velocity <- min 1. (velocity +. 0.1)
+                    else if c = '-' || c = '/' then
+                      velocity <- max 0. (velocity -. 0.1)
+                    else (
+                      let n = note_of_char c in
+                      (* Printf.printf "Playing note %d.\n%!" n; *)
+                      MIDI.insert ans (0, MIDI.Note_on (n, velocity)))
+                | `Key_up ->
+                    let k = Sdl.Event.(get e keyboard_keycode) in
+                    let c = char_of_key k in
                     let n = note_of_char c in
-                    (* Printf.printf "Playing note %d.\n%!" n; *)
-                    MIDI.insert ans (0, MIDI.Note_on (n, velocity)) )
-              | Some (Sdlevent.KEYUP k) ->
-                  let c = Sdlkey.char_of_key k.Sdlevent.keysym in
-                  let n = note_of_char c in
-                  (* Printf.printf "Stopping note %d.\n%!" n; *)
-                  MIDI.insert ans (0, MIDI.Note_off (n, velocity))
-              | _ -> ()
+                    (* Printf.printf "Stopping note %d.\n%!" n; *)
+                    MIDI.insert ans (0, MIDI.Note_off (n, velocity))
+                | _ -> ())
           with Not_found | Invalid_argument _ -> ()
         done;
         ans
@@ -143,6 +155,8 @@ class keyboard ~kind velocity =
   end
 
 let () =
+  let kind = Lang.midi in
+  let k = Lang.kind_type_of_kind_format kind in
   Lang.add_operator "input.keyboard.sdl"
     [
       ( "velocity",
@@ -150,16 +164,9 @@ let () =
         Some (Lang.float 0.8),
         Some "Velocity of notes." );
     ]
-    ~kind:
-      (Lang.Constrained
-         {
-           Frame.audio = Lang.Any_fixed 0;
-           video = Lang.Fixed 0;
-           midi = Lang.Any_fixed 1;
-         })
-    ~category:Lang.Input ~flags:[Lang.Experimental]
+    ~return_t:k ~category:`Input ~flags:[`Experimental]
     ~descr:"Play notes from the keyboard."
-    (fun p kind ->
+    (fun p ->
       let f v = List.assoc v p in
       let velocity = Lang.to_float (f "velocity") in
       (new keyboard ~kind velocity :> Source.source))
