@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,44 +20,41 @@
 
  *****************************************************************************)
 
+open Mm
 open Source
 
 class synth ~kind (synth : Synth.synth) (source : source) chan volume =
   object (self)
     inherit operator ~name:"synth" kind [source]
-
     initializer synth#set_volume volume
-
     method stype = source#stype
-
+    method self_sync = source#self_sync
     method remaining = source#remaining
-
     method is_ready = source#is_ready
-
     method abort_track = source#abort_track
 
     method private get_frame buf =
       let offset = AFrame.position buf in
-      let midi = MFrame.content buf (MFrame.position buf) in
+      let midi = MFrame.midi buf in
       if chan >= Array.length midi then (
         self#log#important
           "Cannot read MIDI channel %d, stream only has %d channels." chan
           (Array.length midi);
-        source#get buf )
+        source#get buf)
       else (
         let evs = midi.(chan) in
         source#get buf;
-        let b = AFrame.content buf offset in
+        let b = AFrame.pcm buf in
         let position = AFrame.position buf in
         let len = position - offset in
-        synth#play evs offset b offset len )
+        synth#play evs offset (Audio.sub b offset len))
   end
 
 let register obj name descr =
-  let k =
-    Lang.kind_type_of_kind_format ~fresh:1
-      (Lang.any_fixed_with ~audio:1 ~midi:1 ())
+  let kind =
+    { Frame.audio = Frame.audio_mono; video = `Any; midi = Frame.midi_n 1 }
   in
+  let k = Lang.kind_type_of_kind_format kind in
   Lang.add_operator ("synth." ^ name)
     [
       ("channel", Lang.int_t, Some (Lang.int 0), Some "MIDI channel to handle.");
@@ -81,8 +78,8 @@ let register obj name descr =
         Some "Envelope release (in seconds)." );
       ("", Lang.source_t k, None, None);
     ]
-    ~kind:(Lang.Unconstrained k) ~category:Lang.SoundSynthesis ~descr
-    (fun p kind ->
+    ~return_t:k ~category:`Synthesis ~descr
+    (fun p ->
       let f v = List.assoc v p in
       let chan = Lang.to_int (f "channel") in
       let volume = Lang.to_float (f "volume") in
@@ -98,11 +95,12 @@ let register obj name descr =
         else None
       in
       let src = Lang.to_source (f "") in
-      new synth ~kind (obj adsr) src chan volume);
-  let k =
-    Lang.kind_type_of_kind_format ~fresh:1
-      (Lang.any_fixed_with ~audio:1 ~midi:16 ())
+      let kind = Source.Kind.of_kind kind in
+      (new synth ~kind (obj adsr) src chan volume :> Source.source));
+  let kind =
+    { Frame.audio = Frame.audio_mono; video = `Any; midi = Frame.midi_n 16 }
   in
+  let k = Lang.kind_type_of_kind_format kind in
   Lang.add_operator ("synth.all." ^ name)
     [
       ("envelope", Lang.bool_t, Some (Lang.bool true), Some "Use envelope.");
@@ -124,9 +122,9 @@ let register obj name descr =
         Some "Envelope release (in seconds)." );
       ("", Lang.source_t k, None, None);
     ]
-    ~kind:(Lang.Unconstrained k) ~category:Lang.SoundSynthesis
+    ~return_t:k ~category:`Synthesis
     ~descr:(descr ^ " It creates one synthesizer for each channel.")
-    (fun p kind ->
+    (fun p ->
       let f v = List.assoc v p in
       let src = Lang.to_source (f "") in
       let adsr =
@@ -141,14 +139,21 @@ let register obj name descr =
         else None
       in
       let synths =
-        Array.init (Frame.type_of_kind kind).Frame.midi (fun c ->
-            (1, new synth ~kind (obj adsr) src c 1.))
+        Array.init (Lazy.force Frame.midi_channels) (fun c ->
+            ( (fun () -> 1.),
+              let kind = Source.Kind.of_kind kind in
+              new synth ~kind (obj adsr) src c 1. ))
       in
       let synths = Array.to_list synths in
-      new Add.add
-        ~kind ~renorm:false synths
-        (fun _ -> ())
-        (fun _ buf tmp -> Image.RGBA32.add buf tmp))
+      let kind = Source.Kind.of_kind kind in
+      (new Add.add
+         ~kind
+         ~renorm:(fun () -> false)
+         ~power:(fun () -> false)
+         synths
+         (fun _ -> ())
+         (fun _ buf tmp -> Video.Image.add buf tmp)
+        :> Source.source))
 
 let () =
   register
