@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -41,6 +41,23 @@ let chunk id data =
   let ans = id ^ dword n ^ data in
   if n mod 2 = 0 then ans else ans ^ "\000"
 
+(* Audio in 16LE *)
+let audio_chunk b = chunk "01wb" b
+
+(* Video in RGB. *)
+let video_chunk b = chunk "00db" b
+
+(* TODO: this could be merged with the above with low cost *)
+
+(* The original data is cleared. *)
+let chunk_strings id data =
+  let n = Strings.Mutable.length data in
+  let ans = Strings.of_list [id; dword n] in
+  let ans = Strings.append ans (Strings.Mutable.to_strings data) in
+  if n mod 2 <> 0 then Strings.add ans "\000" else ans
+
+let audio_chunk_strings b = chunk_strings "01wb" b
+let video_chunk_strings b = chunk_strings "00db" b
 let list = chunk "LIST"
 
 let header ~channels ~samplerate () =
@@ -52,7 +69,7 @@ let header ~channels ~samplerate () =
   let height = Lazy.force Frame.video_height in
   let avi_header =
     chunk "avih"
-      ( dword (1000000 / video_rate) (* microsec per frame *)
+      (dword (1000000 / video_rate) (* microsec per frame *)
       ^ dword 0 (* maximum bytes per second *)
       ^ dword 0 (* reserved *)
       ^ dword 0x0100 (* flags (interleaved) *)
@@ -61,16 +78,18 @@ let header ~channels ~samplerate () =
       ^ dword 2 (* number of streams *)
       ^ dword 0 (* suggested buffer size *)
       ^ dword width (* width *) ^ dword height (* height *)
-      ^ dword 0 (* reserved *) ^ dword 0
-      (* reserved *) ^ dword 0 (* reserved *)
-      ^ dword 0 (* reserved *) )
+      ^ dword 0 (* reserved *) ^ dword 0 (* reserved *)
+      ^ dword 0 (* reserved *) ^ dword 0)
+    (* reserved *)
   in
   let video_header =
     let stream_header =
       chunk "strh"
-        ( "vids" (* stream type *) ^ dword 0 (* stream codec *)
+        ("vids" (* stream type *)
+        ^ dword 0x30323449 (* fourcc (codec) *)
         ^ dword 0 (* flags *) ^ word 0
-        (* priority *) ^ word 0 (* language *)
+        (* priority *)
+        ^ word 0 (* language *)
         ^ dword 0 (* initial frames *)
         ^ dword 1 (* scale *)
         ^ dword video_rate (* rate *)
@@ -79,29 +98,32 @@ let header ~channels ~samplerate () =
         ^ dword 0 (* suggested buffer size *)
         ^ dword_max () (* quality *) ^ dword 0 (* sample size *)
         ^ word 0 (* left *) ^ word 0
-        (* top *) ^ word width (* right *)
-        ^ word height (* bottom *) )
+        (* top *)
+        ^ word width (* right *)
+        ^ word height)
+      (* bottom *)
     in
     let stream_format =
       (* see BITMAPINFO *)
       chunk "strf"
-        ( dword 40 (* size of this structure *)
+        (dword 40 (* size of this structure *)
         ^ dword width (* width *) ^ dword height (* height *)
-        ^ word 1 (* panes *) ^ word 24 (* depth *)
-        ^ dword 0 (* RGB uncompressed format *)
-        ^ dword 0 (* image size *)
+        ^ word 1 (* panes *) ^ word 12 (* depth *)
+        ^ dword 0x30323449 (* codec: I420 *)
+        ^ dword (width * height * 6 / 4) (* image size *)
         ^ dword 0 (* pixels / x meter *)
         ^ dword 0 (* pixels / y meter *)
-        ^ dword 0 (* colors used *)
-        ^ dword 0 (* important colors *) )
+        ^ dword 0 (* colors used *) ^ dword 0)
+      (* important colors *)
     in
     list ("strl" ^ stream_header ^ stream_format)
   in
   let audio_header =
     let stream_header =
       chunk "strh"
-        ( "auds" (* stream type *) ^ dword 0
-        (* stream *) ^ dword 0 (* flags *)
+        ("auds" (* stream type *) ^ dword 0
+        (* stream *)
+        ^ dword 0 (* flags *)
         ^ word 0 (* priority *) ^ word 0 (* language *)
         ^ dword 0 (* initial frames *)
         ^ dword 1 (* scale *)
@@ -112,18 +134,22 @@ let header ~channels ~samplerate () =
         ^ dword_max () (* quality *)
         ^ dword (2 * channels) (* sample size *)
         ^ word 0 (* left *) ^ word 0
-        (* top *) ^ word 0 (* right *)
-        ^ word 0 (* bottom *) )
+        (* top *)
+        ^ word 0
+        (* right *)
+        ^ word 0)
+      (* bottom *)
     in
     let stream_format =
       chunk "strf"
-        ( word 1 (* stream type (PCM) *)
+        (word 1 (* stream type (PCM) *)
         ^ word channels (* channels *)
         ^ dword samplerate (* rate *)
         ^ dword (2 * channels * samplerate) (* byte rate *)
         ^ word (2 * channels) (* block align *)
         ^ word 16 (* bits per sample *)
-        ^ word 0 (* size of extra information *) )
+        ^ word 0)
+      (* size of extra information *)
     in
     list ("strl" ^ stream_header ^ stream_format)
   in
@@ -136,12 +162,6 @@ let header ~channels ~samplerate () =
   ^ dword_max () (* file size *)
   ^ "AVI " ^ headers ^ info ^ "LIST" ^ dword_max () ^ "movi"
 
-(* Audio in 16LE *)
-let audio_chunk b = chunk "01wb" b
-
-(* Video in RGB. *)
-let video_chunk b = chunk "00db" b
-
 module Read = struct
   let read n f =
     let s = Bytes.create n in
@@ -185,8 +205,9 @@ module Read = struct
                 rem := !rem - l;
                 ll := c :: !ll
               done;
+
               (* Printf.printf ">>\n%!"; *)
-              `LIST (subtag, List.rev !ll) )
+              `LIST (subtag, List.rev !ll))
         | "avih" ->
             (* microsec_per_frame *)
             let _ = dword f in
@@ -196,6 +217,7 @@ module Read = struct
             let _ = read 4 f in
             let flags = dword f in
             must "Not interleaved." (flags land 0x0100 <> 0);
+
             (* interleaved *)
             (* total_frame *)
             let _ = dword f in
@@ -209,6 +231,7 @@ module Read = struct
             let height = dword f in
             let scale = dword f in
             must "Non-zero scale." (scale = 0);
+
             (* rate *)
             let _ = dword f in
             (* start *)
@@ -222,16 +245,18 @@ module Read = struct
             let fourcc = dword f in
             if
               not
-                ( stream_type <> "vids" || fourcc = 0 || fourcc = 0x52474218
-                (* RGB24 *) || fourcc = 0x30323449 (* I420 *) )
+                (stream_type <> "vids" || fourcc = 0 || fourcc = 0x52474218
+               (* RGB24 *) || fourcc = 0x30323449)
+              (* I420 *)
             then (
               let err =
                 Printf.sprintf "Wrong %s fourcc: 0x%x." stream_type fourcc
               in
-              must err false );
+              must err false);
             must "Wrong auds fourcc." (stream_type <> "auds" || fourcc = 1);
             let flags = dword f in
             must "Wrong strh flags." (flags = 0);
+
             (* priority *)
             let _ = word f in
             (* language *)
@@ -282,6 +307,7 @@ module Read = struct
 
   let headers f =
     must "Not a RIFF file." (read 4 f = "RIFF");
+
     (* filesize *)
     let _ = dword f in
     must "Not an AVI file." (read 4 f = "AVI ");
@@ -338,15 +364,15 @@ module Read = struct
                         | 0x30323449 -> `I420
                         | _ -> assert false
                     in
-                    streams := `Video (fourcc, width, height, fps) :: !streams )
+                    streams := `Video (fourcc, width, height, fps) :: !streams)
                   else if stream_type = "auds" then (
                     let codec = word 0 in
                     must "Wrong audio codec." (codec = 1 || codec = 255);
                     let channels = word 2 in
                     let sample_rate = dword 4 in
-                    streams := `Audio (channels, sample_rate) :: !streams )
+                    streams := `Audio (channels, sample_rate) :: !streams)
                   else raise (Invalid "Unhandled stream type.")
-              | _ -> () )
+              | _ -> ())
         | _ -> ())
       h;
     let streams = List.rev !streams in
