@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -34,3 +34,5 @@ type text_style =
   | `white ]
 
 val colorize : text_style list -> string -> string
+val start_color : text_style list -> string
+val stop_color : unit -> string
