@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -54,12 +54,11 @@ let load_dynlinks () =
               check_list
                 (fun file -> Sys.file_exists (get_file file))
                 dynload.files
-            then begin
+            then (
               List.iter
                 (fun file -> Dynlink.loadfile (get_file file))
                 dynload.files;
-              raise (Done path)
-            end
+              raise (Done path))
           with Dynlink.Error e ->
             dyn_log#important "Error while loading dynamic %s at %s" name path;
             dyn_log#important "%s" (Dynlink.error_message e))
