@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@ let handlers = ref []
 let m = Mutex.create ()
 
 let rec watchdog () =
-  let fd = Utils.get_some !fd in
+  let fd = Option.get !fd in
   let handler =
     Tutils.mutexify m (fun _ ->
         let events = Inotify.read fd in
@@ -42,14 +42,13 @@ let rec watchdog () =
 
 let watch : File_watcher.watch =
  fun e file f ->
-  if not (Sys.file_exists file) then raise Not_found;
+  if not (Sys.file_exists file) then Runtime_error.error "not_found";
   Tutils.mutexify m
     (fun () ->
-      if !fd = None then begin
+      if !fd = None then (
         fd := Some (Inotify.create ());
-        Duppy.Task.add Tutils.scheduler (watchdog ())
-      end;
-      let fd = Utils.get_some !fd in
+        Duppy.Task.add Tutils.scheduler (watchdog ()));
+      let fd = Option.get !fd in
       let event_conv = function
         | `Modify ->
             [
