@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,16 +22,22 @@
 
 (** Producer/consumer source utils. *)
 
-class virtual ['a] base ~nb_blocks ~blank =
+class virtual ['a] base ~nb_blocks =
   let () =
     if nb_blocks < 1 then
       failwith "Buffered I/O requires a non-zero buffer length."
   in
   object
-    val buffer = Array.init nb_blocks (fun _ -> blank ())
+    val mutable buffer = None
+
+    method buffer =
+      match buffer with Some buffer -> buffer | None -> assert false
+
+    method init (blank : unit -> 'a) =
+      assert (buffer = None);
+      buffer <- Some (Array.init nb_blocks (fun _ -> blank ()))
 
     val mutable read = 0
-
     val mutable write = 0
 
     (* First, consider read and write as positions on an infinite string:
@@ -56,7 +62,6 @@ class virtual ['a] base ~nb_blocks ~blank =
 
     (* Accesses to read/write must be protected by wait_m. *)
     val wait_m = Mutex.create ()
-
     val wait_c = Condition.create ()
 
     (* State of the I/O process:
@@ -64,41 +69,39 @@ class virtual ['a] base ~nb_blocks ~blank =
      *   `Running (thread ID) when running;
      *   `Crashed when process has crashed.
      *   `Tired while shutting down. *)
-    val mutable state = `Idle
+    val mutable io_state = `Idle
 
     method sourcering_stop =
-      match state with
+      match io_state with
         | `Running id ->
             Mutex.lock wait_m;
-            state <- `Tired;
+            io_state <- `Tired;
             Mutex.unlock wait_m;
+
             (* One signal is enough since there is only one half of
              * the process waiting for us, the other half cannot be
              * called concurrently with this method. *)
             Condition.signal wait_c;
             Thread.join id;
-            state <- `Idle
+            io_state <- `Idle
         | `Tired | `Idle -> assert false
         | `Crashed -> ()
   end
 
-class virtual ['a] input ~nb_blocks ~blank =
+class virtual ['a] input ~nb_blocks =
   object (self)
-    inherit ['a] base ~nb_blocks ~blank
-
+    inherit ['a] base ~nb_blocks
     method virtual pull_block : 'a -> unit
-
     method virtual id : string
-
     method virtual close : unit
-
     method private sleep = self#sourcering_stop
+    method private stop = self#sourcering_stop
 
-    method output_get_ready =
-      assert (state = `Idle);
+    method private start =
+      assert (io_state = `Idle);
       read <- 0;
       write <- 0;
-      state <- `Running (Tutils.create (fun _ -> self#writer) () self#id)
+      io_state <- `Running (Tutils.create (fun _ -> self#writer) () self#id)
 
     method private writer =
       try
@@ -106,17 +109,17 @@ class virtual ['a] input ~nb_blocks ~blank =
           (* Wait for the reader to read the block we fancy, or for shutdown. *)
           Mutex.lock wait_m;
           if
-            state <> `Tired
+            io_state <> `Tired
             && read <> write
             && write mod nb_blocks = read mod nb_blocks
           then Condition.wait wait_c wait_m;
           Mutex.unlock wait_m;
 
           (* Exit or... *)
-          if state = `Tired then raise Exit;
+          if io_state = `Tired then raise Exit;
 
           (* ...write a block. *)
-          self#pull_block buffer.(write mod nb_blocks);
+          self#pull_block self#buffer.(write mod nb_blocks);
           Mutex.lock wait_m;
           write <- (write + 1) mod (2 * nb_blocks);
           Mutex.unlock wait_m;
@@ -130,13 +133,14 @@ class virtual ['a] input ~nb_blocks ~blank =
              * which is performed outside of critical section, so there's
              * not need to unlock. *)
             self#close;
+
             (* It is possible that the reader is waiting for us,
              * hence blocking the streaming thread, and consequently
              * the possibility of going to sleep peacefully.
              * Let's resume it, even though he'll get an arbitrary block. *)
             Mutex.lock wait_m;
             write <- (write + 1) mod (2 * nb_blocks);
-            state <- `Crashed;
+            io_state <- `Crashed;
             Mutex.unlock wait_m;
             Condition.signal wait_c;
             raise e
@@ -145,48 +149,46 @@ class virtual ['a] input ~nb_blocks ~blank =
      * so it makes sense to require that #sleep hasn't been called
      * and won't be called before #get_block returns. *)
     method private get_block =
-      assert (match state with `Running _ | `Crashed -> true | _ -> false);
+      assert (match io_state with `Running _ | `Crashed -> true | _ -> false);
+
       (* Check that the writer still has an advance. *)
       Mutex.lock wait_m;
-      if write = read && state <> `Crashed then Condition.wait wait_c wait_m;
-      let b = buffer.(read mod nb_blocks) in
+      if write = read && io_state <> `Crashed then Condition.wait wait_c wait_m;
+      let b = self#buffer.(read mod nb_blocks) in
       read <- (read + 1) mod (2 * nb_blocks);
       Mutex.unlock wait_m;
       Condition.signal wait_c;
       b
   end
 
-class virtual ['a] output ~nb_blocks ~blank =
+class virtual ['a] output ~nb_blocks =
   object (self)
-    inherit ['a] base ~nb_blocks ~blank
-
+    inherit ['a] base ~nb_blocks
     method virtual id : string
-
     method virtual push_block : 'a -> unit
-
     method virtual close : unit
+    method stop = self#sourcering_stop
 
-    method output_stop = self#sourcering_stop
-
-    method output_start =
-      assert (state = `Idle);
+    method start =
+      assert (io_state = `Idle);
       read <- 0;
       write <- 0;
-      state <- `Running (Tutils.create (fun () -> self#reader) () self#id)
+      io_state <- `Running (Tutils.create (fun () -> self#reader) () self#id)
 
     method reader =
       try
         while true do
           (* Wait for the writer to emit the block we fancy, or for shutdown. *)
           Mutex.lock wait_m;
-          if state <> `Tired && read = write then Condition.wait wait_c wait_m;
+          if io_state <> `Tired && read = write then
+            Condition.wait wait_c wait_m;
           Mutex.unlock wait_m;
 
           (* Exit or... *)
-          if state = `Tired then raise Exit;
+          if io_state = `Tired then raise Exit;
 
           (* ...read a block. *)
-          self#push_block buffer.(read mod nb_blocks);
+          self#push_block self#buffer.(read mod nb_blocks);
           Mutex.lock wait_m;
           read <- (read + 1) mod (2 * nb_blocks);
           Mutex.unlock wait_m;
@@ -200,30 +202,31 @@ class virtual ['a] output ~nb_blocks ~blank =
              * which is performed outside of critical section, so there's
              * not need to unlock. *)
             self#close;
+
             (* It is possible that the reader is waiting for us,
              * hence blocking the streaming thread, and consequently
              * the possibility of going to sleep peacefully.
              * Let's resume it, even though he'll get an arbitrary block. *)
             Mutex.lock wait_m;
             read <- (read + 1) mod (2 * nb_blocks);
-            state <- `Crashed;
+            io_state <- `Crashed;
             Mutex.unlock wait_m;
             Condition.signal wait_c;
             raise e
 
-    (* This is meant to be called from #output_send,
+    (* This is meant to be called from #send_frame,
      * so it makes sense to require that #sleep hasn't been called
      * and won't be called before #put_block returns. *)
     method put_block (f : 'a -> unit) =
-      assert (match state with `Running _ | `Crashed -> true | _ -> false);
+      assert (match io_state with `Running _ | `Crashed -> true | _ -> false);
       Mutex.lock wait_m;
       if
         read <> write
         && write mod nb_blocks = read mod nb_blocks
-        && state <> `Crashed
+        && io_state <> `Crashed
       then Condition.wait wait_c wait_m;
       Mutex.unlock wait_m;
-      f buffer.(write mod nb_blocks);
+      f self#buffer.(write mod nb_blocks);
       Mutex.lock wait_m;
       write <- (write + 1) mod (2 * nb_blocks);
       Mutex.unlock wait_m;
