@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,9 @@
 
  *****************************************************************************)
 
-val conf_rw      : string list Dtools.Conf.t
-val conf_ro      : string list Dtools.Conf.t
-val conf_network : bool Dtools.Conf.t 
-val cmd          : ?rw:(string list) -> ?ro:(string list) -> ?network:bool -> string -> string
+val conf_rw : string list Dtools.Conf.t
+val conf_ro : string list Dtools.Conf.t
+val conf_network : bool Dtools.Conf.t
+
+val cmd :
+  ?rw:string list -> ?ro:string list -> ?network:bool -> string -> string
