@@ -1,8 +1,7 @@
-(* -*- mode: tuareg; -*- *)
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -114,42 +113,29 @@ let conf_telnet_revdns =
     "Perform reverse DNS lookup to get the client's hostname from its IP."
 
 let log = Log.make ["server"]
+let conf_log = Dtools.Conf.void ~p:(conf#plug "log") "Log configuration."
+
+let conf_log_level =
+  Dtools.Conf.int ~p:(conf_log#plug "level") ~d:3
+    "Default log level for messages."
 
 exception Duppy of Duppy.Io.failure
 
-(* {1 Manage available commands and namespaces}
- *
- * This needs to be thread safe: (un)registering is done in wakeup/sleep
- * which can be called from arbitrary clock threads. *)
+(* {1 Manage available commands} *)
+
 type namespace = string list
 
+(* This needs to be thread safe: (un)registering is done in wakeup/sleep which
+   can be called from arbitrary clock threads. *)
 let lock = Mutex.create ()
-let namespaces = Hashtbl.create 10
 
 let commands : (string, (string -> string) * string * string) Hashtbl.t =
   Hashtbl.create 50
 
-(* First, get a fresh namespace *)
-let register ns kind =
-  let c = ref 0 in
-  let mkns () = if !c = 0 then ns else ns @ [string_of_int !c] in
-  let ns () =
-    while Hashtbl.mem namespaces (mkns ()) do
-      incr c
-    done;
-    mkns ()
-  in
-  Tutils.mutexify lock
-    (fun () ->
-      let ns = ns () in
-      Hashtbl.add namespaces ns kind;
-      ns)
-    ()
-
 let to_string = String.concat "."
 let prefix_ns cmd ns = to_string (ns @ [cmd])
 
-(* Then add your commands to that namespace *)
+(* Add your commands. *)
 let add ~ns ?usage ~descr cmd handler =
   let usage = match usage with None -> cmd | Some u -> u in
   let usage = prefix_ns usage ns in
@@ -157,17 +143,25 @@ let add ~ns ?usage ~descr cmd handler =
     (fun () ->
       let name = prefix_ns cmd ns in
       if Hashtbl.mem commands name then
-        log#severe
-          "Server command %s already registered! Previous definition replaced.."
+        log#f (conf_log_level#get - 1)
+          "Server command %s already registered! Previous definition replaced."
           name
       else ();
       Hashtbl.replace commands (prefix_ns cmd ns) (handler, usage, descr))
     ()
 
-(* ... maybe remove them *)
+(* ... and maybe remove them. *)
 let remove ~ns cmd =
   Tutils.mutexify lock (fun () -> Hashtbl.remove commands (prefix_ns cmd ns)) ()
 
+(* Maintain a list of known operators. *)
+let operators = ref []
+
+let register_op id kind =
+  Tutils.mutexify lock
+    (fun () -> operators := (id, kind) :: List.remove_assoc id !operators)
+    ()
+
 (* That's if you want to have your command wait. *)
 type condition = {
   wait : (unit -> string) -> unit;
@@ -224,23 +218,6 @@ exception Read of read
 
 let read ~after payload = raise (Read { payload; after })
 
-let unregister ns =
-  Tutils.mutexify lock
-    (fun () ->
-      let ns_str = to_string ns in
-      let ns_len = String.length ns_str in
-      let is_prefix cmd =
-        String.length cmd > ns_len && ns_str = String.sub cmd 0 ns_len
-      in
-      let to_remove =
-        Hashtbl.fold
-          (fun cmd _ l -> if is_prefix cmd then cmd :: l else l)
-          commands []
-      in
-      List.iter (Hashtbl.remove commands) to_remove;
-      Hashtbl.remove namespaces ns)
-    ()
-
 (* The usage string sums up all the commands... *)
 let usage () =
   let l =
@@ -269,16 +246,17 @@ let () =
         (if args <> "" then "No such command: " ^ args ^ "\r\n" else "")
         ^ "Available commands:" ^ usage () ^ "\r\n\r\n"
         ^ "Type \"help <command>\" for more information.");
-  add "list" ~descr:"Get the list of available operators with their interfaces."
+  add "list" ~descr:"Get the list of available operators with their kind."
     (fun _ ->
       Tutils.mutexify lock
         (fun () ->
           String.concat "\r\n"
-            (Hashtbl.fold
-               (fun k v s -> Printf.sprintf "%s : %s" (to_string k) v :: s)
-               namespaces []))
+            (List.fold_left
+               (fun s (id, kind) -> Printf.sprintf "%s : %s" id kind :: s)
+               [] !operators))
         ())
 
+(** Execute a command. *)
 let exec s =
   let s, args =
     try
@@ -300,13 +278,16 @@ let exec s =
 
 let handle_client socket ip =
   let on_error e =
-    ( match e with
+    (match e with
       | Duppy.Io.Io_error -> ()
       | Duppy.Io.Timeout ->
-          log#info "Timeout reached while communicating to client %s." ip
+          log#f (conf_log_level#get + 1)
+            "Timeout reached while communicating to client %s." ip
       | Duppy.Io.Unix (c, p, m) ->
-          log#info "%s" (Printexc.to_string (Unix.Unix_error (c, p, m)))
-      | Duppy.Io.Unknown e -> log#important "%s" (Printexc.to_string e) );
+          log#f (conf_log_level#get + 1) "%s"
+            (Printexc.to_string (Unix.Unix_error (c, p, m)))
+      | Duppy.Io.Unknown e ->
+          log#f conf_log_level#get "%s" (Printexc.to_string e));
     Duppy e
   in
   let h =
@@ -369,7 +350,8 @@ let handle_client socket ip =
       | (Exit | Duppy Duppy.Io.Timeout) as e ->
           let on_error e =
             ignore (on_error e);
-            log#important "Client %s disconnected while saying goodbye..!" ip;
+            log#f conf_log_level#get
+              "Client %s disconnected while saying goodbye..!" ip;
             close ()
           in
           let msg =
@@ -379,14 +361,15 @@ let handle_client socket ip =
               | _ -> assert false
           in
           let exec () =
-            log#important "Client %s disconnected." ip;
+            log#f conf_log_level#get "Client %s disconnected." ip;
             close ()
           in
           Duppy.Io.write ~timeout:(get_timeout ()) ~priority:Tutils.Non_blocking
             ~on_error ~exec Tutils.scheduler ~string:(Bytes.of_string msg)
             socket
       | _ ->
-          log#important "Client %s disconnected without saying goodbye..!" ip;
+          log#f conf_log_level#get
+            "Client %s disconnected without saying goodbye..!" ip;
           close ()
     in
     Duppy.Monad.run ~return:run ~raise process
@@ -401,17 +384,16 @@ let start_socket () =
   let bind_addr = Unix.ADDR_UNIX socket_path in
   let rights = conf_socket_perms#get in
   let max_conn = 10 in
-  let sock = Unix.socket Unix.PF_UNIX Unix.SOCK_STREAM 0 in
+  let sock = Unix.socket ~cloexec:true Unix.PF_UNIX Unix.SOCK_STREAM 0 in
   let rec incoming _ =
-    ( try
-        let socket, caller = Unix.accept sock in
-        let ip =
-          Utils.name_of_sockaddr ~rev_dns:conf_telnet_revdns#get caller
-        in
-        log#important "New client %s." ip;
-        handle_client socket ip
-      with e ->
-        log#severe "Failed to accept new client: %S" (Printexc.to_string e) );
+    (try
+       let socket, caller = Unix.accept ~cloexec:true sock in
+       let ip = Utils.name_of_sockaddr ~rev_dns:conf_telnet_revdns#get caller in
+       log#f conf_log_level#get "New client %s." ip;
+       handle_client socket ip
+     with e ->
+       log#f (conf_log_level#get - 1) "Failed to accept new client: %S"
+         (Printexc.to_string e));
     [
       {
         Duppy.Task.priority = Tutils.Non_blocking;
@@ -430,11 +412,11 @@ let start_socket () =
     try Unix.bind sock bind_addr
     with exn -> raise (Bind_error (Printexc.to_string exn))
   end;
+  log#f conf_log_level#get "Socket created at %s." socket_path;
   Unix.listen sock max_conn;
-  ignore
-    (Dtools.Init.make ~after:[Tutils.scheduler_shutdown_atom] (fun () ->
-         log#important "Unlink %s" socket_name;
-         Unix.unlink socket_path));
+  Lifecycle.after_scheduler_shutdown (fun () ->
+      log#f conf_log_level#get "Unlink %s" socket_name;
+      Unix.unlink socket_path);
   Unix.chmod socket_path rights;
   Duppy.Task.add Tutils.scheduler
     {
@@ -448,28 +430,25 @@ let start_telnet () =
   let bind_addr_inet = Unix.inet_addr_of_string conf_telnet_bind_addr#get in
   let bind_addr = Unix.ADDR_INET (bind_addr_inet, port) in
   let max_conn = 10 in
-  let sock = Unix.socket Unix.PF_INET Unix.SOCK_STREAM 0 in
+  let sock = Unix.socket ~cloexec:true Unix.PF_INET Unix.SOCK_STREAM 0 in
   let () =
     (* The socket has to be closed for restart to work, and this has to be
        done after duppy has stopped using it. *)
-    ignore
-      (Dtools.Init.make ~after:[Tutils.scheduler_shutdown_atom]
-         ~name:"Server shutdown" (fun () ->
-           log#important "Closing socket.";
-           Unix.close sock))
+    Lifecycle.after_scheduler_shutdown (fun () ->
+        log#f conf_log_level#get "Closing socket.";
+        Unix.close sock)
   in
   (* Set TCP_NODELAY on the socket *)
   Unix.setsockopt sock Unix.TCP_NODELAY true;
   let rec incoming _ =
-    ( try
-        let socket, caller = Unix.accept sock in
-        let ip =
-          Utils.name_of_sockaddr ~rev_dns:conf_telnet_revdns#get caller
-        in
-        log#important "New client: %s." ip;
-        handle_client socket ip
-      with e ->
-        log#severe "Failed to accept new client: %S" (Printexc.to_string e) );
+    (try
+       let socket, caller = Unix.accept ~cloexec:true sock in
+       let ip = Utils.name_of_sockaddr ~rev_dns:conf_telnet_revdns#get caller in
+       log#f conf_log_level#get "New client: %s." ip;
+       handle_client socket ip
+     with e ->
+       log#f (conf_log_level#get - 1) "Failed to accept new client: %S"
+         (Printexc.to_string e));
     [
       {
         Duppy.Task.priority = Tutils.Non_blocking;
@@ -496,10 +475,10 @@ let start () =
   let socket = conf_socket#get in
   if telnet || socket then (
     if telnet then start_telnet () else ();
-    if socket then start_socket () else () )
+    if socket then start_socket () else ())
   else ()
 
-let () = ignore (Dtools.Init.at_start start)
+let () = Lifecycle.on_start start
 
 (* Re-wrap exec for external use *)
 let exec s = try exec s with Exit -> "ERROR: Attempt to exit!"
