@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -50,30 +50,21 @@ val conf_telnet_port : int Conf.t
 
 (** {2 Commands of the server} *)
 
-(** Namespaces are used to avoid name conflicts. Any entity which wants to add
-  * commands to the server should first get a new unique namespace, in which the
-  * commands will be added. *)
+(** Namespaces are used to avoid name conflicts. *)
 type namespace = string list
 
 (** Get a string representation of a namespace. *)
 val to_string : namespace -> string
 
-(** [register hint kind] creates a new namespace and registers it.
-  * If [hint] is not used, it will be used as the new namespace. Otherwise,
-  * some numeral suffix will be appended to it in order to distinguish it.
-  * The [kind] is meant to provide information about what kind of namespace
-  * this is. In liquidsoap, it tells which kind of operator owns the namespace,
-  * and hence which commands should be expected in it. *)
-val register : namespace -> string -> namespace
-
-(** Release a namespace, deleting all associated commands. *)
-val unregister : namespace -> unit
+(** Register an operator with given identifier and kind. This is only used in
+   the "list" telnet command.  *)
+val register_op : string -> string -> unit
 
 (** Specialized implementation of conditions to use in server commands. *)
 type condition = {
-  wait:      (unit -> string) -> unit;
-  signal:    unit -> unit;
-  broadcast: unit -> unit
+  wait : (unit -> string) -> unit;
+  signal : unit -> unit;
+  broadcast : unit -> unit;
 }
 
 (** [condition ()] instantiates a server command condition. *)
@@ -89,8 +80,13 @@ val read : after:(string -> string) -> Duppy.Io.marker -> unit
   * When the command is called, the function [f] is executed with the argument of
   * the command as parameter. The return value of [f] is then displayed. ~descr is
   * the command description. *)
-val add : ns:namespace -> ?usage:string -> descr:string ->
-          string -> (string -> string) -> unit
+val add :
+  ns:namespace ->
+  ?usage:string ->
+  descr:string ->
+  string ->
+  (string -> string) ->
+  unit
 
 (** Remove a command from the server. *)
 val remove : ns:namespace -> string -> unit
