@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -30,9 +30,12 @@
   the needed threads: that function will sleep until a thread
   raises an exception. *)
 val create : ('a -> unit) -> 'a -> string -> Thread.t
+
 val main : unit -> unit
+val start : unit -> unit
 val has_started : unit -> bool
-val shutdown : unit -> unit
+val shutdown : int -> unit
+val exit_code : unit -> int
 
 (** Special exception allowed for "clean" termination of Tutils threads.
   * All other exceptions are reported as bugs. *)
@@ -46,14 +49,16 @@ val join_all : unit -> unit
 
 (** Priorities for the different scheduler usages. *)
 type priority =
-  | Blocking       (** For example a last.fm submission. *)
-  | Maybe_blocking (** Request resolutions vary a lot. *)
-  | Non_blocking   (** Non-blocking tasks like the server. *)
+  | Blocking  (** For example a last.fm submission. *)
+  | Maybe_blocking  (** Request resolutions vary a lot. *)
+  | Non_blocking  (** Non-blocking tasks like the server. *)
 
 (** task scheduler *)
 val scheduler : priority Duppy.scheduler
 
-val scheduler_shutdown_atom : Dtools.Init.t
+(** Register queue error handlers. Should return [true] if 
+    the exception was handler by the given callback. *)
+val error_handlers : (bt:string -> name:string -> exn -> bool) Stack.t
 
 (** {1 Misc} *)
 
@@ -62,16 +67,15 @@ val scheduler_shutdown_atom : Dtools.Init.t
   * and after the call. *)
 val wait : Condition.t -> Mutex.t -> (unit -> bool) -> unit
 
-(** Make a function work in critical section, protected by a given lock. *) 
-val mutexify : Mutex.t -> ('a -> 'b) -> ('a -> 'b)
+(** Make a function work in critical section, protected by a given lock. *)
+val mutexify : Mutex.t -> ('a -> 'b) -> 'a -> 'b
 
 exception Timeout of float
 
-type event = [
-  | `Read of Unix.file_descr
+type event =
+  [ `Read of Unix.file_descr
   | `Write of Unix.file_descr
-  | `Both of Unix.file_descr
-]
+  | `Both of Unix.file_descr ]
 
 (* Wait some events: [`Read socket], [`Write socket] or [`Both timeout]
  * Raises [Timeout elapsed_time] if timeout is reached. *)
@@ -88,17 +92,4 @@ val finalize : k:(unit -> unit) -> (unit -> 'a) -> 'a
 val seems_locked : Mutex.t -> bool
 
 (** Thread-safe equivalent to Lazy.from_fun. *)
-val lazy_cell : (unit -> 'a) -> (unit -> 'a)
-
-(** Preemptive stoppable thread.
-  *
-  * The thread function receives a [should_stop,has_stop] pair on startup.
-  * It should regularly poll the [should_stop] and stop when asked to.
-  * Before stopping it should call [has_stopped].
-  *
-  * The function returns a [kill,wait] pair. The first function should be
-  * called to request that the thread stops, and the second to wait
-  * that it has effectively stopped. *)
-val stoppable_thread :
-     (((unit -> bool) * (unit -> unit)) -> unit) -> string ->
-     (unit -> unit) * (unit -> unit)
+val lazy_cell : (unit -> 'a) -> unit -> 'a
