@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,14 +20,14 @@
 
  *****************************************************************************)
 
-let pi = 4. *. atan 1.
-let get_some = function Some x -> x | None -> assert false
-let maybe f = function Some x -> Some (f x) | None -> None
-let maydo f = function Some x -> f x | None -> ()
-let some_or none = function Some x -> x | None -> none
-
 (* Force locale to C *)
-external force_locale : unit -> unit = "liquidsoap_set_locale"
+external force_locale : unit -> unit = "liquidsoap_set_locale" [@@noalloc]
+
+(** Get page size. *)
+external pagesize : unit -> int = "liquidsoao_get_pagesize"
+  [@@noalloc]
+
+let pagesize = pagesize ()
 
 (** General configuration *)
 let conf = Dtools.Conf.void "Liquidsoap configuration"
@@ -47,8 +47,6 @@ let getenv_opt value = try Some (Sys.getenv value) with Not_found -> None
 
 (* Several list utilities *)
 
-let rec make_list n v = if n = 0 then [] else v :: make_list (n - 1) v
-
 let rec prefix p l =
   match (p, l) with
     | [], _ -> true
@@ -67,52 +65,6 @@ let list_of_metadata m =
 let hashtbl_get : ('a, 'b) Hashtbl.t -> 'a -> 'b option =
  fun h k -> try Some (Hashtbl.find h k) with Not_found -> None
 
-(** Unescape a string. *)
-let unescape s = try Scanf.sscanf s "%S" (fun u -> u) with _ -> s
-
-(* Unescape a given char in a string, i.e. \\c -> c *)
-let unescape_char c s =
-  let len = String.length s in
-  let rec f ~escaped cur pos =
-    if pos >= len then if escaped then Printf.sprintf "%s\\" cur else cur
-    else (
-      let d = s.[pos] in
-      let pos = pos + 1 in
-      match d with
-        | '\\' when not escaped -> f ~escaped:true cur pos
-        | x when x = c -> f ~escaped:false (Printf.sprintf "%s%c" cur c) pos
-        | x when escaped -> f ~escaped:false (Printf.sprintf "%s\\%c" cur x) pos
-        | x -> f ~escaped:false (Printf.sprintf "%s%c" cur x) pos )
-  in
-  f ~escaped:false "" 0
-
-(** Parse strings of the form foo <sep> bar,
- *  trying to be clever on escaping <sep> in foo
- *  and bar.. *)
-let split ~sep s =
-  let rec split cur s =
-    let len = String.length s in
-    let rec get_index escaped pos =
-      if pos == len then pos
-      else (
-        match s.[pos] with
-          | '\\' when not escaped -> get_index true (pos + 1)
-          | x when x == sep && not escaped -> pos
-          | _ -> get_index false (pos + 1) )
-    in
-    let pos = get_index false 0 in
-    if pos == len then s :: cur
-    else if pos == len - 1 then String.sub s 0 pos :: cur
-    else
-      split (String.sub s 0 pos :: cur) (String.sub s (pos + 1) (len - pos - 1))
-  in
-  List.map (unescape_char sep) (List.rev (split [] s))
-
-(** Remove trailing and leading spaces. *)
-let trim s =
-  Pcre.replace ~rex:(Pcre.regexp "\\s+$")
-    (Pcre.replace ~rex:(Pcre.regexp "^\\s+") s)
-
 (** Remove the first element satisfying a predicate, raising Not_found
   * if none is found. *)
 let remove_one f l =
@@ -122,11 +74,6 @@ let remove_one f l =
   in
   aux [] l
 
-let rec may_map f = function
-  | h :: t -> (
-      match f h with Some h -> h :: may_map f t | None -> may_map f t )
-  | [] -> []
-
 (* Read all data from a given filename.
  * We cannot use really_input with the 
  * reported length of the file because
@@ -137,19 +84,17 @@ let rec may_map f = function
  * different than its reported length.. *)
 let read_all filename =
   let channel = open_in filename in
-  let buflen = 1024 in
-  let tmp = Bytes.create 1024 in
-  let contents = Buffer.create buflen in
+  let tmp = Bytes.create pagesize in
+  let contents = Strings.Mutable.empty () in
   let rec read () =
-    let ret = input channel tmp 0 1024 in
-    if ret > 0 then begin
-      Buffer.add_subbytes contents tmp 0 ret;
-      read ()
-    end
+    let ret = input channel tmp 0 pagesize in
+    if ret > 0 then (
+      Strings.Mutable.add_subbytes contents tmp 0 ret;
+      read ())
   in
   read ();
   close_in channel;
-  Buffer.contents contents
+  Strings.Mutable.to_string contents
 
 (* Drop the first [len] bytes. *)
 let buffer_drop buffer len =
@@ -159,7 +104,7 @@ let buffer_drop buffer len =
   else (
     let tmp = Buffer.sub buffer len (size - len) in
     Buffer.clear buffer;
-    Buffer.add_string buffer tmp )
+    Buffer.add_string buffer tmp)
 
 let unix_translator = function
   | Unix.Unix_error (code, name, param) ->
@@ -182,84 +127,188 @@ let randomize a =
       permute i (i + Random.int (l - i))
     done
 
-let array_iter2 a b f =
-  assert (Array.length a = Array.length b);
-  for i = 0 to Array.length a - 1 do
-    f a.(i) b.(i)
-  done
-
-let special_char = function
-  | '"' | '\\'
-  (* DEL *)
-  | '\x7F'
-  (* Control chars *)
-  | '\x00' .. '\x1F' ->
-      true
-  | _ -> false
-
-let escape_char c = if c <> '"' then Char.escaped c else "\\\""
-
 (* Generic escaping function *)
-let escape ?(special_char = special_char) ?(next = fun _ i -> i + 1)
-    ?(escape_char = escape_char) f s =
-  let out = Format.pp_print_char f in
-  let outs = Format.pp_print_string f in
+let escape ~special_char ~next ~escape_char f s =
+  let out = Format.pp_print_string f in
   let len = String.length s in
   let rec f pos =
-    let c = String.unsafe_get s pos in
-    if special_char c then outs (escape_char c) else out c;
-    let new_pos = next s pos in
-    for i = pos + 1 to min (new_pos - 1) (len - 1) do
-      out (String.unsafe_get s i)
-    done;
+    let new_pos = next s (pos : int) in
+    let s = String.sub s pos (new_pos - pos) in
+    if special_char s then out (escape_char s) else out s;
     if new_pos < len then f new_pos
   in
-  out '"';
-  if len > 0 then f 0;
-  out '"'
-
-(* These two functions are taken from Extlib's module UTF8
- * Copyright (c) 2002, 2003 Yamagata Yoriyuki *)
-let rec utf8_search_head s i =
-  if i >= String.length s then i
-  else (
-    let n = Char.code (String.unsafe_get s i) in
-    if n < 0x80 || n >= 0xc2 then i else utf8_search_head s (i + 1) )
+  if len > 0 then f 0
 
 let utf8_next s i =
-  let n = Char.code s.[i] in
-  if n < 0x80 then i + 1
-  else if n < 0xc0 then utf8_search_head s (i + 1)
-  else if n <= 0xdf then i + 2
-  else if n <= 0xef then i + 3
-  else if n <= 0xf7 then i + 4
-  else if n <= 0xfb then i + 5
-  else if n <= 0xfd then i + 6
-  else failwith "Utils.utf_8.next"
+  match s.[i] with
+    | '\000' .. '\127' -> i + 1
+    | '\192' .. '\223' -> i + 2
+    | '\224' .. '\239' -> i + 3
+    | '\240' .. '\247' -> i + 4
+    | _ -> i + 1
+
+(* Return the utf8 char code of the first utf8 character in the given
+   string. *)
+let utf8_char_code s =
+  match s.[0] with
+    | '\000' .. '\127' as c -> Char.code c
+    | '\192' .. '\223' as c ->
+        let n1 = Char.code c in
+        let n2 = Char.code s.[1] in
+        if n2 lsr 6 != 0b10 then failwith "invalid utf8";
+        ((n1 land 0x1f) lsl 6) lor (n2 land 0x3f)
+    | '\224' .. '\239' as c ->
+        let n1 = Char.code c in
+        let n2 = Char.code s.[1] in
+        let n3 = Char.code s.[2] in
+        if n2 lsr 6 != 0b10 || n3 lsr 6 != 0b10 then failwith "invalid utf8";
+        let p =
+          ((n1 land 0x0f) lsl 12) lor ((n2 land 0x3f) lsl 6) lor (n3 land 0x3f)
+        in
+        if p >= 0xd800 && p <= 0xdf00 then failwith "invalid utf8";
+        p
+    | '\240' .. '\247' as c ->
+        let n1 = Char.code c in
+        let n2 = Char.code s.[1] in
+        let n3 = Char.code s.[2] in
+        let n4 = Char.code s.[3] in
+        if n2 lsr 6 != 0b10 || n3 lsr 6 != 0b10 || n4 lsr 6 != 0b10 then
+          failwith "invalid utf8";
+        ((n1 land 0x07) lsl 18)
+        lor ((n2 land 0x3f) lsl 12)
+        lor ((n3 land 0x3f) lsl 6)
+        lor (n4 land 0x3f)
+    | _ -> failwith "invalid utf8"
 
 (* End of Extlib code *)
 
-let escape_utf8_char = function
-  | '"' -> "\\\""
-  | '\t' -> "\\t"
-  | '\r' -> "\\r"
-  | '\b' -> "\\b"
-  | '\n' -> "\\n"
-  | '\012' -> "\\f"
-  | '\\' -> "\\\\"
-  | '/' -> "\\/"
-  | c -> Printf.sprintf "\\u%04X" (int_of_char c)
+let ascii_special_char = function
+  | "'" | "\"" | "\\"
+  (* DEL *)
+  | "\x7F" ->
+      true
+  (* Control chars *)
+  | s when String.length s = 1 && Char.code s.[0] <= 0x1F -> true
+  | s when String.length s = 1 && Char.code s.[0] > 0x7E -> true
+  | _ -> false
 
-let escape_utf8 ?special_char ?(escape_char = escape_utf8_char) =
-  escape ?special_char ~escape_char ~next:utf8_next
+let utf8_special_char s = String.length s = 1 && ascii_special_char s
+let ascii_next _ i = i + 1
 
-let escape_string escape s =
-  let b = Buffer.create (String.length s) in
-  let f = Format.formatter_of_buffer b in
-  escape f s;
-  Format.pp_print_flush f ();
+let escape_char ~escape_fun = function
+  | "\b" -> "\\b"
+  | "\n" -> "\\n"
+  | "\r" -> "\\r"
+  | "\t" -> "\\t"
+  | "\\" -> "\\\\"
+  | "\"" -> "\\\""
+  | "\'" -> "\\'"
+  | c -> escape_fun c
+
+let escape_utf8_char =
+  let utf8_char_code s =
+    try utf8_char_code s with _ -> Uchar.to_int Uchar.rep
+  in
+  escape_char ~escape_fun:(fun s -> Printf.sprintf "\\u%04X" (utf8_char_code s))
+
+let escape_utf8_formatter ?(special_char = utf8_special_char) =
+  escape ~special_char ~escape_char:escape_utf8_char ~next:utf8_next
+
+let escape_hex_char =
+  escape_char ~escape_fun:(fun s ->
+      Printf.sprintf "\\x%02X" (int_of_char s.[0]))
+
+let escape_octal_char =
+  escape_char ~escape_fun:(fun s -> Printf.sprintf "\\%03o" (int_of_char s.[0]))
+
+(* We use the \xhh syntax to make sure the resulting string is also consistently readable in
+   OCaml. \nnn can be tricky since they are octal sequences in liquidsoap and digital sequences
+   in OCaml. *)
+let escape_ascii_formatter ?(special_char = ascii_special_char) =
+  escape ~special_char ~escape_char:escape_hex_char ~next:ascii_next
+
+let escape_string = Format.asprintf "%a"
+
+let escape_utf8_string ?special_char =
+  escape_string (escape_utf8_formatter ?special_char)
+
+let escape_ascii_string ?special_char =
+  escape_string (escape_ascii_formatter ?special_char)
+
+let quote_utf8_string s =
+  Printf.sprintf "\"%s\""
+    (escape_utf8_string
+       ~special_char:(function "'" -> false | x -> utf8_special_char x)
+       s)
+
+let quote_ascii_string s =
+  Printf.sprintf "\"%s\""
+    (escape_ascii_string
+       ~special_char:(function "'" -> false | x -> ascii_special_char x)
+       s)
+
+let quote_string s = try quote_utf8_string s with _ -> quote_ascii_string s
+let unescape_utf8_pattern = "\\\\u[0-9a-fA-F][0-9a-fA-F][0-9a-fA-F][0-9a-fA-F]"
+let unescape_hex_pattern = "\\\\x[0-9a-fA-F][0-9a-fA-F]"
+let unescape_octal_pattern = "\\\\[0-9][0-9][0-9]"
+
+let unescape_patterns =
+  [
+    "\\\\a";
+    "\\\\b";
+    "\\\\e";
+    "\\\\f";
+    "\\\\n";
+    "\\\\r";
+    "\\\\t";
+    "\\\\v";
+    "\\\\/";
+    "\\\\\\\\";
+    "\\\\\"";
+    "\\\\\'";
+    "\\\\\\?";
+    unescape_octal_pattern;
+    unescape_hex_pattern;
+    unescape_utf8_pattern;
+  ]
+
+let unescape_octal_char s =
+  let s = String.sub s 1 3 in
+  Printf.sprintf "%c" (Char.chr (int_of_string ("0o" ^ s)))
+
+let unescape_hex_char s =
+  let s = String.sub s 2 2 in
+  Printf.sprintf "%c" (Char.chr (int_of_string ("0x" ^ s)))
+
+let unescape_utf8_char s =
+  let b = Buffer.create 1 in
+  let s = String.sub s 2 4 in
+  Buffer.add_utf_8_uchar b (Uchar.of_int (int_of_string ("0x" ^ s)));
   Buffer.contents b
 
+let unescape_char = function
+  | "\\a" -> "\x07"
+  | "\\b" -> "\b"
+  | "\\e" -> "\x1b"
+  | "\\f" -> "\x0c"
+  | "\\n" -> "\n"
+  | "\\r" -> "\r"
+  | "\\t" -> "\t"
+  | "\\v" -> "\x0b"
+  | "\\/" -> "/"
+  | "\\\\" -> "\\"
+  | "\\\"" -> "\""
+  | "\\'" -> "'"
+  | "\\?" -> "\x3f"
+  | s when String.length s = 6 -> unescape_utf8_char s
+  | s when String.length s = 4 && s.[1] = 'x' -> unescape_hex_char s
+  | s when String.length s = 4 -> unescape_octal_char s
+  | _ -> assert false
+
+let unescape_string =
+  let rex = Pcre.regexp_or unescape_patterns in
+  Pcre.substitute ~rex ~subst:unescape_char
+
 (** Remove line breaks from markdown text. This is useful for reflowing markdown such as when printing doc. *)
 let unbreak_md md =
   let must_break = function
@@ -338,6 +387,7 @@ let which ~path s =
       true
     with _ -> false
   in
+  if s = "" then raise Not_found;
   if test s then s
   else List.find test (List.map (fun d -> Filename.concat d s) path)
 
@@ -345,6 +395,7 @@ let which_opt ~path s = try Some (which ~path s) with Not_found -> None
 
 (** Get current timezone. *)
 external timezone : unit -> int = "liquidsoap_get_timezone"
+  [@@noalloc]
 
 let string_of_timezone tz =
   (* TODO: not sure about why we need this... *)
@@ -391,7 +442,7 @@ let rec mkdir ~perm dir =
   else (
     let up = Filename.dirname dir in
     if up = "." then () else mkdir ~perm up;
-    Unix.mkdir dir perm )
+    Unix.mkdir dir perm)
 
 (** Expand ~ notation in filenames. *)
 let home_unrelate =
@@ -411,8 +462,8 @@ let home_unrelate =
             try
               let home = (Unix.getpwnam user).Unix.pw_dir in
               Filename.concat home (String.sub s index (len - index))
-            with Not_found -> s )
-        | _ -> s )
+            with Not_found -> s)
+        | _ -> s)
   in
   unrel
 
@@ -481,14 +532,13 @@ let encode64 s =
     2 := (c1 lsl 2) land 63 lor (c2 lsr 6);
     3 := c2 land 63
   done;
-  if extra = 1 then begin
+  if extra = 1 then (
     Bytes.set dst ((4 * (n / 3)) - 2) '=';
-    Bytes.set dst ((4 * (n / 3)) - 1) '='
-  end
+    Bytes.set dst ((4 * (n / 3)) - 1) '=')
   else if extra = 2 then Bytes.set dst ((4 * (n / 3)) - 1) '=';
   Bytes.unsafe_to_string dst
 
-(* This is not garanteed to work 100% but should
+(* This is not guaranteed to work 100% but should
  * be ok on reasonable cases. A problematic cases
  * is for instance: http://bla.com/foo.mp3?gni=bla.truc *)
 
@@ -500,6 +550,8 @@ let get_ext s =
     String.lowercase_ascii (Pcre.get_substring ret 1)
   with _ -> raise Not_found
 
+let get_ext_opt s = try Some (get_ext s) with Not_found -> None
+
 let name_of_sockaddr ?(rev_dns = false) ?(show_port = false) a =
   match a with
     | Unix.ADDR_UNIX s -> Printf.sprintf "unix socket %S" s
@@ -541,15 +593,15 @@ let normalize_parameter_string s =
 let reopen_out outchan filename =
   flush outchan;
   let fd1 = Unix.descr_of_out_channel outchan in
-  let fd2 = Unix.openfile filename [Unix.O_WRONLY] 0o666 in
-  Unix.dup2 fd2 fd1;
+  let fd2 = Unix.openfile filename [Unix.O_WRONLY; Unix.O_CLOEXEC] 0o666 in
+  Unix.dup2 ~cloexec:true fd2 fd1;
   Unix.close fd2
 
 (** The same for inchan *)
 let reopen_in inchan filename =
   let fd1 = Unix.descr_of_in_channel inchan in
-  let fd2 = Unix.openfile filename [Unix.O_RDONLY] 0o666 in
-  Unix.dup2 fd2 fd1;
+  let fd2 = Unix.openfile filename [Unix.O_RDONLY; Unix.O_CLOEXEC] 0o666 in
+  Unix.dup2 ~cloexec:true fd2 fd1;
   Unix.close fd2
 
 (* See: http://www.onicos.com/staff/iz/formats/ieee.c *)
@@ -580,12 +632,11 @@ let float_of_extended_float bytes =
   in
   if expon = 0 && hiMant = Int32.zero && loMant = Int32.zero then 0.
   else if expon = 0x7fff then nan
-  else begin
+  else (
     let expon = expon - 16383 - 31 in
     let f = ldexp (float_of_unsigned hiMant) expon in
     let f = f +. ldexp (float_of_unsigned loMant) (expon - 32) in
-    if int_of_char bytes.[0] land 0x80 <> 0 then -1. *. f else f
-  end
+    if int_of_char bytes.[0] land 0x80 <> 0 then -1. *. f else f)
 
 (* From OCaml *)
 let file_extension_len ~dir_sep name =
@@ -611,26 +662,9 @@ let file_extension ?(leading_dot = true) ?(dir_sep = Filename.dir_sep) name =
       | _ -> s
   with Invalid_argument _ -> s
 
-let quote s =
-  let quote = '"' in
-  let quotequote = "\\\"" in
-  let escp = '\\' in
-  let quoteescp = "\\\\" in
-  let l = String.length s in
-  let b = Buffer.create (l + 20) in
-  Buffer.add_char b quote;
-  for i = 0 to l - 1 do
-    match s.[i] with
-      | c when c = quote -> Buffer.add_string b quotequote
-      | c when c = escp -> Buffer.add_string b quoteescp
-      | c -> Buffer.add_char b c
-  done;
-  Buffer.add_char b quote;
-  Buffer.contents b
-
 let environment () =
   let l = Unix.environment () in
-  (* Split at first occurence of '='. Return v,"" if
+  (* Split at first occurrence of '='. Return v,"" if
    * no '=' could be found. *)
   let split s =
     try
@@ -652,3 +686,154 @@ let string_of_size n =
   else if n < 1 lsl 30 then
     Printf.sprintf "%.02f MiB" (float_of_int n /. float_of_int (1 lsl 20))
   else Printf.sprintf "%.02f GiB" (float_of_int n /. float_of_int (1 lsl 30))
+
+(* From dbuenzli/cmdliner *)
+let find_cmd cmds =
+  let test, null =
+    match Sys.os_type with
+      | "Win32" -> ("where", " NUL")
+      | _ -> ("type", "/dev/null")
+  in
+  let rec cmd = function
+    | (c, args) :: l ->
+        if Sys.command (Printf.sprintf "%s %s 1>%s 2>%s" test c null null) = 0
+        then Some (if args = "" then c else c ^ " " ^ args)
+        else cmd l
+    | [] -> None
+  in
+  cmd cmds
+
+let print_strings ?(pager = false) s =
+  let pager = if Sys.getenv_opt "PAGER" = Some "none" then false else pager in
+  let default s = Strings.iter (output_substring stdout) s in
+  let cmd =
+    let cmds = [("less", "-F -X"); ("more", "")] in
+    let cmds = try (Sys.getenv "PAGER", "") :: cmds with Not_found -> cmds in
+    let cmds =
+      try (Sys.getenv "MANPAGER", "") :: cmds with Not_found -> cmds
+    in
+    find_cmd cmds
+  in
+  match (pager, cmd) with
+    | false, _ | _, None -> default s
+    | true, Some pager -> (
+        let fname, oc = Filename.open_temp_file "liquidsoap" ".txt" in
+        try
+          Strings.iter (output_substring oc) s;
+          flush oc;
+          if Sys.command (Printf.sprintf "%s %s" pager fname) <> 0 then
+            default s;
+          raise Exit
+        with _ ->
+          close_out oc;
+          Unix.unlink fname)
+
+let print_string ?(pager = false) s =
+  if not pager then print_string s
+  else print_strings ~pager (Strings.of_string s)
+
+let kprint_string ?(pager = false) f =
+  if not pager then f (print_string ~pager)
+  else (
+    let ans = Strings.Mutable.empty () in
+    f (Strings.Mutable.add ans);
+    print_strings ~pager (Strings.Mutable.to_strings ans))
+
+(** String representation of a matrix of strings. *)
+let string_of_matrix a =
+  let height = Array.length a in
+  let width = Array.fold_left (fun h a -> max h (Array.length a)) 0 a in
+  let len = Array.make width 0 in
+  for j = 0 to height - 1 do
+    for i = 0 to Array.length a.(j) - 1 do
+      len.(i) <- max len.(i) (String.length a.(j).(i))
+    done
+  done;
+  let ans = Strings.Mutable.empty () in
+  for j = 0 to height - 1 do
+    for i = 0 to Array.length a.(j) - 1 do
+      let s = a.(j).(i) in
+      if i <> 0 then Strings.Mutable.add ans " ";
+      Strings.Mutable.add ans s;
+      Strings.Mutable.add ans (String.make (len.(i) - String.length s) ' ')
+    done;
+    Strings.Mutable.add ans "\n"
+  done;
+  Strings.Mutable.to_string ans
+
+(* Util to log exception and backtrace together
+   when log level is set to info and just exception
+   as severe otherwise. Backtrace should be captured as early
+   as possible. *)
+let log_exception ~(log : Log.t) ~bt msg =
+  if log#active 4 (* info *) then log#info "%s\n%s" msg bt
+  else log#severe "%s" msg
+
+(** Operations on versions of Liquidsoap. *)
+module Version = struct
+  type t = int list * string
+
+  (* We assume something like, 2.0.0+git@7e211ffd *)
+  let of_string s : t =
+    let rex = Pcre.regexp "([\\.\\d]+)([^\\.]+)?" in
+    let sub = Pcre.exec ~rex s in
+    let num = Pcre.get_substring sub 1 in
+    let str = try Pcre.get_substring sub 2 with Not_found -> "" in
+    let num = String.split_on_char '.' num |> List.map int_of_string in
+    (num, str)
+
+  (** Number part. *)
+  let num (v : t) = fst v
+
+  (** String part. *)
+  let str (v : t) = snd v
+
+  (** Compare two versions. 2.0.0~foo is less than 2.0.0 *)
+  let compare v w =
+    let compare_suffixes =
+      match (str v, str w) with
+        | v, w
+          when String.length v > 1
+               && String.length w > 1
+               && v.[0] = '~'
+               && w.[0] = '~' ->
+            String.compare v w
+        | v, w
+          when String.length v > 1
+               && String.length w > 1
+               && v.[0] = '~'
+               && w.[0] <> '~' ->
+            -1
+        | v, w
+          when String.length v > 1
+               && String.length w > 1
+               && v.[0] <> '~'
+               && w.[0] = '~' ->
+            1
+        | v, "" when String.length v > 1 && v.[0] = '~' -> -1
+        | "", w when String.length w > 1 && w.[0] = '~' -> 1
+        | v, w -> String.compare v w
+    in
+    let rec aux v w =
+      match (v, w) with
+        | m :: _, n :: _ when m < n -> -1
+        | m :: _, n :: _ when m > n -> 1
+        | _ :: v, _ :: w -> aux v w
+        | [], [] -> compare_suffixes
+        | [], _ -> -1
+        | _, [] -> 1
+    in
+    aux (num v) (num w)
+end
+
+let self_sync_type sources =
+  lazy
+    (fst
+       (List.fold_left
+          (fun cur s ->
+            match (cur, s#self_sync) with
+              | (`Static, None), (`Static, v) -> (`Static, Some v)
+              | (`Static, Some v), (`Static, v') when v = v' -> (`Static, Some v)
+              | _ -> (`Dynamic, None))
+          (`Static, None)
+          sources))
