@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,24 +20,21 @@
 
  *****************************************************************************)
 
+open Mm
 open Source
 
 class midimeter ~kind source =
   object
-    inherit operator ~name:"midimeter" kind [source]
-
+    inherit operator ~name:"midi.inspect" kind [source]
     method stype = source#stype
-
     method is_ready = source#is_ready
-
     method remaining = source#remaining
-
     method abort_track = source#abort_track
+    method self_sync = source#self_sync
 
     method get_frame buf =
-      let offset = MFrame.position buf in
       source#get buf;
-      let m = MFrame.content buf offset in
+      let m = MFrame.midi buf in
       (* Printf.printf "len: %d\n%!" (List.length (MIDI.data m.(0))); *)
       for c = 0 to Array.length m - 1 do
         List.iter
@@ -58,13 +55,14 @@ class midimeter ~kind source =
   end
 
 let () =
-  let k = Lang.kind_type_of_kind_format ~fresh:1 Lang.any_fixed in
-  Lang.add_operator "midimeter"
+  let kind = Lang.any in
+  let k = Lang.kind_type_of_kind_format kind in
+  Lang.add_operator "midi.inspect"
     [("", Lang.source_t k, None, None)]
-    ~kind:(Lang.Unconstrained k) ~category:Lang.Visualization
-    ~flags:[Lang.Hidden; Lang.Experimental]
-    ~descr:"Display midi events."
-    (fun p kind ->
+    ~return_t:k ~category:`Visualization
+    ~descr:"Display midi events on standard output."
+    (fun p ->
       let f v = List.assoc v p in
       let src = Lang.to_source (f "") in
+      let kind = Source.Kind.of_kind kind in
       (new midimeter ~kind src :> Source.source))
