@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2019 Savonet team
+  Copyright 2003-2021 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -29,65 +29,62 @@ let group_size = 1764
 let f_group_size = float group_size
 
 class vumeter ~kind source =
-  let channels = (Frame.type_of_kind kind).Frame.audio in
   object (self)
     inherit operator ~name:"visu.volume" kind [source] as super
-
     method stype = source#stype
-
     method is_ready = source#is_ready
-
     method remaining = source#remaining
-
     method abort_track = source#abort_track
-
-    method private wake_up act =
-      super#wake_up act;
-      Graphics.open_graph "";
-      Graphics.set_window_title "Liquidsoap's volume";
-      Graphics.auto_synchronize false
+    method self_sync = source#self_sync
 
     method private sleep =
       super#sleep;
       Graphics.close_graph ()
 
     (* Ringbuffer for previous values, with its current position *)
-    val vol = Array.init channels (fun _ -> Array.make backpoints 0.)
-
+    val mutable vol = [||]
     val mutable pos = 0
 
     (* Another buffer for accumulating RMS over [group_size] samples,
      * with its current position. *)
-    val mutable cur_rms = Array.make channels 0.
-
+    val mutable cur_rms = [||]
     val mutable group = 0
 
+    method private wake_up act =
+      super#wake_up act;
+      vol <- Array.init self#audio_channels (fun _ -> Array.make backpoints 0.);
+      cur_rms <- Array.make self#audio_channels 0.;
+      Graphics.open_graph "";
+      Graphics.set_window_title "Liquidsoap's volume";
+      Graphics.auto_synchronize false
+
     method private add_vol v =
+      let channels = self#audio_channels in
       for c = 0 to channels - 1 do
         cur_rms.(c) <- cur_rms.(c) +. v.(c)
       done;
       group <- (group + 1) mod group_size;
-      if group = 0 then begin
+      if group = 0 then (
         for c = 0 to channels - 1 do
           vol.(c).(pos) <- sqrt (cur_rms.(c) /. f_group_size);
           cur_rms.(c) <- 0.
         done;
-        pos <- (pos + 1) mod backpoints
-      end
+        pos <- (pos + 1) mod backpoints)
 
     method private get_frame buf =
+      let channels = self#audio_channels in
       let offset = AFrame.position buf in
       let end_pos =
         source#get buf;
         AFrame.position buf
       in
       if offset < end_pos then (
-        let content = AFrame.content buf offset in
+        let content = AFrame.pcm buf in
         for i = offset to AFrame.position buf - 1 do
           self#add_vol
             (Array.map
                (fun c ->
-                 let x = c.(i) in
+                 let x = c.{i} in
                  x *. x)
                content)
         done;
@@ -110,20 +107,22 @@ class vumeter ~kind source =
           for i = 1 to backpoints - 1 do
             Graphics.lineto
               (int_of_float (volwidth *. float i))
-              ( chan_height
-              + int_of_float (volheight *. vol.((i + pos) mod backpoints)) )
+              (chan_height
+              + int_of_float (volheight *. vol.((i + pos) mod backpoints)))
           done
         done;
-        Graphics.synchronize () )
+        Graphics.synchronize ())
   end
 
 let () =
-  let k = Lang.kind_type_of_kind_format ~fresh:1 Lang.audio_any in
+  let kind = Lang.audio_pcm in
+  let k = Lang.kind_type_of_kind_format Lang.audio_pcm in
   Lang.add_operator "visu.volume"
     [("", Lang.source_t k, None, None)]
-    ~kind:(Lang.Unconstrained k) ~category:Lang.Visualization
+    ~return_t:k ~category:`Visualization
     ~descr:"Graphical visualization of the volume."
-    (fun p kind ->
+    (fun p ->
       let f v = List.assoc v p in
       let src = Lang.to_source (f "") in
+      let kind = Source.Kind.of_kind kind in
       new vumeter ~kind src)
